Package: stepssurvey
Title: Analyse WHO STEPS Survey Data
Version: 0.1.0
Authors@R: c(
    person("Abhijit", "Pakhare", email = "abhijit.cfm@aiimsbhopal.edu.in",
           role = c("aut", "cre")),
    person("Ankur", "Joshi", role = "aut"),
    person("Lena", "Charlette", role = "aut"),
    person("WHO STEPS R Pipeline Contributors", role = "ctb")
  )
Description: Provides a complete analysis pipeline for the WHO STEPwise
    Approach to NCD Risk Factor Surveillance (STEPS) as described in
    Riley et al. (2016) <doi:10.2105/AJPH.2015.302962>. Imports raw
    survey data ('CSV', 'Excel', 'Stata', 'SPSS'), applies WHO-standard
    cleaning and recoding, sets up complex survey designs, computes all
    standard NCD indicators (tobacco, alcohol, diet, physical activity,
    anthropometry, blood pressure, biochemical), and generates
    publication-ready tables, visualisations, and 'Word'/'HTML' reports
    (fact sheet, data book, country report).
License: MIT + file LICENSE
URL: https://github.com/drpakhare/stepssurvey
BugReports: https://github.com/drpakhare/stepssurvey/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: bslib, dplyr, DT, flextable, ggplot2, glue, haven, janitor,
        patchwork, purrr, readr, readxl, rmarkdown, shiny, survey,
        tools
Suggests: knitr, remotes, testthat (>= 3.0.0), withr
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-06 08:20:10 UTC; drpakhare
Author: Abhijit Pakhare [aut, cre],
  Ankur Joshi [aut],
  Lena Charlette [aut],
  WHO STEPS R Pipeline Contributors [ctb]
Maintainer: Abhijit Pakhare <abhijit.cfm@aiimsbhopal.edu.in>
Repository: CRAN
Date/Publication: 2026-05-11 19:10:03 UTC
Built: R 4.6.0; ; 2026-05-11 19:43:24 UTC; unix

# Generated by roxygen2: do not edit by hand

export(build_all_tables)
export(build_forest_plot)
export(build_radar_plot)
export(build_steps_plots)
export(build_steps_tables)
export(build_table)
export(clean_steps_data)
export(compute_alcohol_indicators)
export(compute_all_indicators)
export(compute_all_tables)
export(compute_anthropometry_indicators)
export(compute_biochemical_indicators)
export(compute_bp_indicators)
export(compute_diet_pa_indicators)
export(compute_table)
export(compute_tobacco_indicators)
export(detect_col)
export(detect_steps_columns)
export(generate_test_data)
export(get_registry_by_section)
export(get_registry_by_step)
export(import_steps_data)
export(list_registry_sections)
export(plot_completeness)
export(plot_digit_preference)
export(plot_weights)
export(read_column_mapping)
export(recode_yn)
export(render_country_report)
export(render_data_book)
export(render_fact_sheet)
export(run_app)
export(run_steps_pipeline)
export(save_steps_plots)
export(setup_survey_design)
export(steps_colors)
export(steps_config)
export(steps_data_quality)
export(steps_table_registry)
export(svymn)
export(svyprop)
export(theme_steps)
importFrom(ggplot2,"%+replace%")
importFrom(stats,as.formula)
importFrom(stats,confint)
importFrom(stats,rbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(utils,head)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 5,
  fig.align  = "center",
  eval = TRUE
)

## ----install, eval = FALSE----------------------------------------------------
# # install.packages("pak")
# pak::pak("drpakhare/stepssurvey")

## ----install-devtools, eval = FALSE-------------------------------------------
# # install.packages("devtools")
# devtools::install_github("drpakhare/stepssurvey")

## ----library------------------------------------------------------------------
library(stepssurvey)

## ----generate-----------------------------------------------------------------
raw <- generate_test_data(n = 3000, seed = 42)
dim(raw)
names(raw)

## ----import, eval = FALSE-----------------------------------------------------
# # CSV
# raw <- import_steps_data("data/raw/steps_survey_2024.csv")
# 
# # Excel
# raw <- import_steps_data("data/raw/steps_survey_2024.xlsx")
# 
# # Stata (.dta) -- common for STEPS exports
# raw <- import_steps_data("data/raw/steps_survey_2024.dta")
# 
# # SPSS (.sav)
# raw <- import_steps_data("data/raw/steps_survey_2024.sav")

## ----detect-------------------------------------------------------------------
cols <- detect_steps_columns(raw)

## ----detect-inspect-----------------------------------------------------------
# Which column was matched for fasting glucose?
cols$fasting_glucose

# Which column for SBP reading 1?
cols$sbp1

# How many columns were detected?
sum(!sapply(cols, is.null))

## ----override, eval = FALSE---------------------------------------------------
# cols$fasting_glucose <- "my_custom_glucose_variable"

## ----mapping-template, eval = FALSE-------------------------------------------
# # Copy the template to your working directory
# file.copy(
#   system.file("templates", "column_mapping_template.xlsx", package = "stepssurvey"),
#   file.path(tempdir(), "my_mapping.xlsx")
# )

## ----mapping-read, eval = FALSE-----------------------------------------------
# cols <- read_column_mapping("my_mapping.xlsx", data = raw)

## ----mapping-pipeline, eval = FALSE-------------------------------------------
# result <- run_steps_pipeline(
#   "my_data.dta",
#   country_name = "My Country",
#   survey_year  = 2024,
#   mapping_file = "my_mapping.xlsx"
# )

## ----clean--------------------------------------------------------------------
clean <- clean_steps_data(raw, cols, age_min = 18, age_max = 69)
dim(clean)

## ----thresholds, eval = FALSE-------------------------------------------------
# clean <- clean_steps_data(raw, cols,
#   bp_sbp_threshold          = 130,   # SBP threshold (default 140)
#   bp_dbp_threshold          = 80,    # DBP threshold (default 90)
#   bmi_overweight             = 25.0,  # BMI overweight (default 25)
#   bmi_obese                  = 30.0,  # BMI obese (default 30)
#   glucose_threshold          = 7.0,   # Raised glucose mmol/L (default 7.0)
#   glucose_impaired_threshold = 6.1,   # Impaired glucose mmol/L (default 6.1)
#   chol_threshold             = 5.0    # Raised cholesterol mmol/L (default 5.0)
# )

## ----clean-inspect------------------------------------------------------------
# BMI categories
table(clean$bmi_category, clean$sex)

# Blood pressure staging
table(clean$bp_stage)

# Physical activity levels
table(clean$pa_category, clean$sex)

## ----quality, eval = FALSE----------------------------------------------------
# quality <- steps_data_quality(clean)
# names(quality)
# # [1] "digit_preference" "completeness" "plausibility" "weights"

## ----digit-plot, eval = FALSE-------------------------------------------------
# plot_digit_preference(quality, measure = "sbp")

## ----completeness-plot, eval = FALSE------------------------------------------
# plot_completeness(quality)

## ----weights-plot, eval = FALSE-----------------------------------------------
# plot_weights(quality)

## ----design-------------------------------------------------------------------
designs <- setup_survey_design(clean)

## ----indicators---------------------------------------------------------------
result <- compute_all_indicators(designs)

## ----key----------------------------------------------------------------------
result$key_indicators

## ----domain, eval = FALSE-----------------------------------------------------
# tob  <- compute_tobacco_indicators(designs$step1)
# alc  <- compute_alcohol_indicators(designs$step1)
# diet <- compute_diet_pa_indicators(designs$step1)
# anth <- compute_anthropometry_indicators(designs$step2)
# bp   <- compute_bp_indicators(designs$step2)
# bio  <- compute_biochemical_indicators(designs$step3)

## ----tob-example--------------------------------------------------------------
tob <- compute_tobacco_indicators(designs$step1)
names(tob)

# Overall prevalence of current tobacco use
tob$current_tobacco_total

# Prevalence by sex
tob$current_tobacco_by_sex

## ----custom-------------------------------------------------------------------
# Weighted proportion with 95% CI (raised_bp is a Step 2 variable)
svyprop(~raised_bp, designs$step2)

# Stratified by sex
svyprop(~raised_bp, designs$step2, by = ~sex)

# Weighted mean with 95% CI
svymn(~mean_sbp, designs$step2, by = ~sex)

## ----tables-------------------------------------------------------------------
tables <- build_steps_tables(result$results)
names(tables)

## ----table-show, eval = FALSE-------------------------------------------------
# # Display the raised blood pressure table
# tables$raised_bp
# 
# # Export to Word
# flextable::save_as_docx(tables$raised_bp, path = file.path(tempdir(), "bp_table.docx"))

## ----detailed-tables, eval = FALSE--------------------------------------------
# # Step 1: Compute raw results from the table registry
# computed <- compute_all_tables(designs)
# 
# # Step 2: Format into flextable objects with WHO styling
# detailed <- build_all_tables(computed)
# names(detailed)  # e.g. "T_current_smokers", "M_bp_mean", "B_glucose_raised"

## ----registry, eval = FALSE---------------------------------------------------
# # Browse the full table registry
# registry <- steps_table_registry()
# 
# # Get all tables for a specific section
# bp_entries <- get_registry_by_section("Blood Pressure")
# 
# # Get all Step 2 tables
# step2_entries <- get_registry_by_step(2)
# 
# # List available sections
# list_registry_sections()

## ----plots, fig.width = 8, fig.height = 5-------------------------------------
plots <- build_steps_plots(
  indicators    = result$results,
  key_indicators = result$key_indicators,
  country_name  = "Exampleland",
  survey_year   = 2024
)
names(plots)

## ----overview, fig.width = 8, fig.height = 5----------------------------------
plots$overview

## ----dashboard, fig.width = 10, fig.height = 7, eval = FALSE------------------
# plots$sex_dashboard

## ----age-trend, fig.width = 8, fig.height = 4.5-------------------------------
plots$bp_by_age

## ----forest, fig.width = 8, fig.height = 6, eval = FALSE----------------------
# plots$forest
# # Or build standalone:
# build_forest_plot(result$key_indicators, "Exampleland", 2024)

## ----radar, fig.width = 7, fig.height = 7, eval = FALSE-----------------------
# plots$radar
# # Or build standalone:
# build_radar_plot(result$key_indicators, "Exampleland", 2024)

## ----save-plots, eval = FALSE-------------------------------------------------
# save_steps_plots(plots, output_dir = file.path(tempdir(), "figures"))
# # Creates:
# #   outputs/figures/01_overview_indicators.png
# #   outputs/figures/02_by_sex_dashboard.png
# #   outputs/figures/03_bp_by_age.png
# #   outputs/figures/04_obesity_by_age.png
# #   outputs/figures/05_forest_plot.png
# #   outputs/figures/06_radar_plot.png

## ----theme, fig.width = 6, fig.height = 3.5-----------------------------------
pal <- steps_colors()
str(pal)

library(ggplot2)
ggplot(clean, aes(x = age_group, fill = sex)) +
  geom_bar(position = "dodge") +
  scale_fill_manual(values = c(Male = pal$male, Female = pal$female)) +
  theme_steps() +
  labs(title = "Sample distribution by age and sex")

## ----reports, eval = FALSE----------------------------------------------------
# cfg <- steps_config(
#   data_path    = "data/raw/steps_survey_2024.csv",
#   country_name = "Exampleland",
#   survey_year  = 2024,
#   age_min      = 18,
#   age_max      = 69
# )
# 
# # Fact sheet -- one-page overview (HTML for sharing, Word for print)
# render_fact_sheet(cfg, output_dir = "outputs", format = "html")
# render_fact_sheet(cfg, output_dir = "outputs", format = "word")
# 
# # Summary report -- narrative with key findings, charts, recommendations
# render_country_report(cfg, output_dir = "outputs")
# 
# # Data book -- detailed WHO 3-panel tables by domain
# render_data_book(cfg, output_dir = "outputs")

## ----pipeline, eval = FALSE---------------------------------------------------
# out <- run_steps_pipeline(
#   data_path      = "data/raw/steps_survey_2024.csv",
#   country_name   = "Exampleland",
#   survey_year    = 2024,
#   age_min        = 18,
#   age_max        = 69,
#   output_dir     = "outputs",
#   render_reports = TRUE
# )
# 
# # Access any intermediate result
# out$raw_data
# out$clean_data
# out$design
# out$indicators
# out$key_indicators
# out$tables
# out$plots

## ----pipeline-mapping, eval = FALSE-------------------------------------------
# out <- run_steps_pipeline(
#   "my_data.dta",
#   country_name = "My Country",
#   survey_year  = 2024,
#   mapping_file = "my_mapping.xlsx"
# )

## ----custom-names, eval = FALSE-----------------------------------------------
# raw  <- import_steps_data("my_steps_data.csv")
# cols <- detect_steps_columns(raw)
# cols$fasting_glucose <- "blood_sugar_fasting"
# cols$sbp1 <- "systolic_bp_1"
# clean <- clean_steps_data(raw, cols)

## ----age-range, eval = FALSE--------------------------------------------------
# # Include up to age 79
# clean <- clean_steps_data(raw, cols, age_min = 18, age_max = 79)

## ----worked, fig.width = 8, fig.height = 5------------------------------------
library(stepssurvey)

# 1. Generate a realistic test dataset
raw <- generate_test_data(n = 3000, seed = 42)

# 2. Detect standard STEPS variable columns
cols <- detect_steps_columns(raw)

# 3. Clean data and derive all indicators
clean <- clean_steps_data(raw, cols, age_min = 18, age_max = 69)

# 4. Create the complex survey design
designs <- setup_survey_design(clean)

# 5. Compute all NCD risk factor indicators
result <- compute_all_indicators(designs)

# 6. View headline estimates
result$key_indicators

# 7. Build formatted tables
tables <- build_steps_tables(result$results)

# 8. Build visualisations
plots <- build_steps_plots(
  indicators     = result$results,
  key_indicators = result$key_indicators,
  country_name   = "Exampleland",
  survey_year    = 2024
)

# 9. Display the overview chart
plots$overview

## ----shiny, eval = FALSE------------------------------------------------------
# library(stepssurvey)
# run_app()

## ----cite, eval = FALSE-------------------------------------------------------
# citation("stepssurvey")

## ----session------------------------------------------------------------------
sessionInfo()

#!/usr/bin/env Rscript
# =============================================================================
# Validation Script: Moldova STEPS 2021
# Compares stepssurvey package outputs against official WHO fact sheet values
#
# Data source: WHO NCD Microdata Repository (licensed access)
# Reference: Moldova 2021 STEPS Fact Sheet (WHO European Region, 2022)
# =============================================================================

library(stepssurvey)
library(dplyr)
library(survey)

# -- Configuration -----------------------------------------------------------
# Resolve paths: work from project root or from inst/validation/
.pkg_root <- tryCatch(rprojroot::find_package_root_file(), error = function(e) getwd())
DATA_PATH <- file.path(.pkg_root, "../STEPS Licensed Datasets/Republic of Moldova 2021/MDA_2021_STEPS.dta")
REF_PATH  <- file.path(.pkg_root, "inst/validation/moldova_2021_reference.csv")
# Fallback: try relative paths (when sourced from inst/validation/)
if (!file.exists(DATA_PATH)) DATA_PATH <- "../../../STEPS Licensed Datasets/Republic of Moldova 2021/MDA_2021_STEPS.dta"
if (!file.exists(REF_PATH))  REF_PATH  <- "moldova_2021_reference.csv"

cat("=== Moldova STEPS 2021 Validation ===\n\n")

# -- Step 1: Import ----------------------------------------------------------
cat("--- Step 1: Import ---\n")
raw <- import_steps_data(DATA_PATH)
cat(sprintf("  Imported: %d rows x %d cols\n\n", nrow(raw), ncol(raw)))

# -- Step 2: Detect columns --------------------------------------------------
cat("--- Step 2: Detect columns ---\n")
cols <- detect_steps_columns(raw)

# Show key detections
key_cols <- c("age", "sex", "weight_step1", "weight_step2", "weight_step3",
              "strata", "psu", "agerange", "tobacco_current", "alcohol_current",
              "fasting_glucose", "total_chol", "sbp1", "height", "weight")
for (k in key_cols) {
  val <- if (is.null(cols[[k]])) "NOT FOUND" else cols[[k]]
  cat(sprintf("  %-20s -> %s\n", k, val))
}

# -- Step 3: Clean -----------------------------------------------------------
cat("\n--- Step 3: Clean ---\n")
cleaned <- clean_steps_data(.strip_haven_labels(raw), cols)
cat(sprintf("  Cleaned: %d rows\n", nrow(cleaned)))

# Verify sex coding
cat("  Sex distribution:\n")
print(table(cleaned$sex, useNA = "ifany"))

# Verify age groups
if ("agerange" %in% names(cleaned)) {
  cat("  Age range distribution:\n")
  print(table(cleaned$agerange, useNA = "ifany"))
}

# -- Step 4: Survey design ---------------------------------------------------
cat("\n--- Step 4: Survey design ---\n")
designs <- setup_survey_design(cleaned)
cat("  Design class:", class(designs), "\n")

# -- Step 5: Compute key indicators and compare ------------------------------
cat("\n--- Step 5: Computing indicators and comparing ---\n\n")

# Load reference values
ref <- read.csv(REF_PATH, stringsAsFactors = FALSE)

# Helper: compute a proportion estimate (both sexes, 18-69)
compute_prop <- function(var_name, design, label = var_name) {
  if (!var_name %in% names(design$variables)) {
    return(data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA))
  }

  tryCatch({
    formula <- as.formula(paste0("~", var_name))
    est <- svymean(formula, design, na.rm = TRUE)
    ci <- confint(est)
    # svymean on logical variables returns 2 rows (FALSE, TRUE) — pick TRUE
    vals <- as.numeric(est)
    if (length(vals) == 2) {
      data.frame(
        indicator = label,
        our_est = round(vals[2] * 100, 1),
        our_ci_low = round(ci[2, 1] * 100, 1),
        our_ci_high = round(ci[2, 2] * 100, 1)
      )
    } else {
      data.frame(
        indicator = label,
        our_est = round(vals[1] * 100, 1),
        our_ci_low = round(ci[1, 1] * 100, 1),
        our_ci_high = round(ci[1, 2] * 100, 1)
      )
    }
  }, error = function(e) {
    data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA)
  })
}

# Helper: compute a mean estimate
compute_mean <- function(var_name, design, label = var_name) {
  if (!var_name %in% names(design$variables)) {
    return(data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA))
  }

  tryCatch({
    formula <- as.formula(paste0("~", var_name))
    est <- svymean(formula, design, na.rm = TRUE)
    ci <- confint(est)
    data.frame(
      indicator = label,
      our_est = round(as.numeric(est), 1),
      our_ci_low = round(ci[1, 1], 1),
      our_ci_high = round(ci[1, 2], 1)
    )
  }, error = function(e) {
    data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA)
  })
}

# -- Compute indicators using step-specific designs --------------------------
# Step 1 indicators (behavioural)
d1 <- designs$step1

# Step 2 indicators (physical measurements)
d2 <- designs$step2

# Step 3 indicators (biochemical)
d3 <- designs$step3

# List of proportion indicators to check
# NOTE: Variable names must match what clean_steps_data() produces:
#   current_tobacco_any = smoking OR smokeless (fact sheet "current tobacco use")
#   current_tobacco     = current smoking only (fact sheet "current smoking")
#   current_alcohol     = drank in past 30 days
#   overweight_obese    = BMI >= 25 (fact sheet "overweight" = includes obese)
#   raised_bp           = SBP>=140 OR DBP>=90 OR on meds
#   raised_glucose      = FBG>=7.0 OR on meds
#   raised_chol_or_meds = TC>=5.0 OR on meds

results <- bind_rows(
  # Tobacco
  compute_prop("current_tobacco_any", d1, "current_tobacco_use"),
  compute_prop("current_tobacco", d1, "current_smoking"),
  compute_prop("secondhand_home", d1, "secondhand_smoke_home"),
  compute_prop("secondhand_work", d1, "secondhand_smoke_work"),

  # Alcohol
  compute_prop("current_alcohol", d1, "current_alcohol_30days"),
  compute_prop("heavy_episodic", d1, "heavy_episodic_drinking"),

  # Diet
  compute_prop("low_fruit_veg", d1, "insufficient_fruit_veg"),

  # Physical Activity
  compute_prop("insufficient_pa", d1, "insufficient_pa"),

  # Physical Measurements
  compute_prop("overweight_obese", d2, "overweight_bmi25"),
  compute_prop("obese", d2, "obese_bmi30"),
  compute_prop("raised_bp", d2, "raised_bp_or_meds"),

  # Biochemical
  compute_prop("raised_glucose", d3, "raised_glucose_or_meds"),
  compute_prop("raised_chol_or_meds", d3, "raised_cholesterol_or_meds"),
  compute_prop("impaired_glucose", d3, "impaired_fasting_glucose"),

  # Means
  compute_mean("bmi", d2, "mean_bmi"),
  compute_mean("mean_sbp", d2, "mean_sbp"),
  compute_mean("total_chol", d3, "mean_total_cholesterol")
)

# Merge with reference values
comparison <- results %>%
  left_join(
    ref %>% select(indicator, ref_est = both, ref_ci_low = both_ci_low, ref_ci_high = both_ci_high),
    by = "indicator"
  ) %>%
  mutate(
    diff = our_est - ref_est,
    match = abs(diff) < 1.0  # Within 1 percentage point
  )

# -- Display results ---------------------------------------------------------
cat("VALIDATION RESULTS: Moldova STEPS 2021\n")
cat("Our package estimates vs. WHO official fact sheet (Both Sexes, 18-69)\n")
cat(sprintf("%-35s %8s %8s %8s %8s %8s\n",
            "Indicator", "Ours", "WHO", "Diff", "Match?", "CI overlap?"))
cat(paste(rep("-", 85), collapse = ""), "\n")

for (i in seq_len(nrow(comparison))) {
  r <- comparison[i, ]
  ci_overlap <- ""
  if (!is.na(r$our_ci_low) && !is.na(r$ref_ci_low)) {
    # Check if our CI overlaps with WHO CI
    overlaps <- r$our_ci_high >= r$ref_ci_low && r$our_ci_low <= r$ref_ci_high
    ci_overlap <- ifelse(overlaps, "YES", "NO")
  }

  cat(sprintf("%-35s %8s %8s %8s %8s %8s\n",
              r$indicator,
              ifelse(is.na(r$our_est), "N/A", sprintf("%.1f", r$our_est)),
              ifelse(is.na(r$ref_est), "N/A", sprintf("%.1f", r$ref_est)),
              ifelse(is.na(r$diff), "N/A", sprintf("%+.1f", r$diff)),
              ifelse(is.na(r$match), "N/A", ifelse(r$match, "YES", "NO")),
              ci_overlap))
}

# Summary
n_compared <- sum(!is.na(comparison$diff))
n_match <- sum(comparison$match, na.rm = TRUE)
n_ci_overlap <- sum(
  comparison$our_ci_high >= comparison$ref_ci_low &
  comparison$our_ci_low <= comparison$ref_ci_high,
  na.rm = TRUE
)

cat(paste(rep("-", 85), collapse = ""), "\n")
cat(sprintf("SUMMARY: %d/%d indicators within 1pp | %d/%d CIs overlap\n",
            n_match, n_compared, n_ci_overlap, n_compared))
cat(sprintf("Indicators not computed: %d\n", sum(is.na(comparison$our_est))))

# -- Save results ------------------------------------------------------------
write.csv(comparison, "moldova_2021_validation_results.csv", row.names = FALSE)
cat("\nResults saved to moldova_2021_validation_results.csv\n")

# -- Print available derived variables for debugging -------------------------
cat("\n--- Available derived variables in cleaned data ---\n")
derived_vars <- c("current_tobacco", "current_smoker", "tobacco_daily",
                  "smokeless_current", "secondhand_home", "secondhand_work",
                  "current_drinker", "heavy_episodic",
                  "low_fruit_veg", "insufficient_pa", "met_total",
                  "bmi", "overweight", "obese", "mean_sbp", "raised_bp",
                  "fasting_glucose", "raised_glucose", "impaired_glucose",
                  "total_chol", "raised_chol",
                  "age_group", "agerange", "sex")

for (v in derived_vars) {
  if (v %in% names(cleaned)) {
    if (is.logical(cleaned[[v]])) {
      cat(sprintf("  %-25s logical  TRUE=%d FALSE=%d NA=%d\n", v,
                  sum(cleaned[[v]], na.rm = TRUE),
                  sum(!cleaned[[v]], na.rm = TRUE),
                  sum(is.na(cleaned[[v]]))))
    } else if (is.numeric(cleaned[[v]])) {
      cat(sprintf("  %-25s numeric  mean=%.2f  NA=%d\n", v,
                  mean(cleaned[[v]], na.rm = TRUE),
                  sum(is.na(cleaned[[v]]))))
    } else {
      cat(sprintf("  %-25s %s  levels=%d  NA=%d\n", v, class(cleaned[[v]])[1],
                  length(unique(cleaned[[v]])),
                  sum(is.na(cleaned[[v]]))))
    }
  } else {
    cat(sprintf("  %-25s *** NOT FOUND ***\n", v))
  }
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)

## ----inspect------------------------------------------------------------------
# raw  <- import_steps_data("my_data.sav")
# cols <- detect_steps_columns(raw)
# 
# # See all detected columns
# str(cols[!sapply(cols, is.null)])
# 
# # See what was NOT detected
# names(cols[sapply(cols, is.null)])

## ----convert------------------------------------------------------------------
# # Glucose: mg/dL to mmol/L (divide by 18)
# raw$b5 <- raw$b5 / 18
# 
# # Cholesterol: mg/dL to mmol/L (divide by 38.67)
# raw$b8 <- raw$b8 / 38.67

## ----merge--------------------------------------------------------------------
# step1 <- haven::read_spss("step1_interview.sav")
# step2 <- haven::read_spss("step2_measurements.sav")
# step3 <- haven::read_spss("step3_biochemistry.sav")
# 
# combined <- dplyr::left_join(step1, step2, by = "pid") |>
#   dplyr::left_join(step3, by = "pid")
# 
# # Save combined file
# haven::write_sav(combined, file.path(tempdir(), "steps_combined.sav"))
# 
# # Or import directly
# raw <- combined |> janitor::clean_names()

## ----diagnostic---------------------------------------------------------------
# library(stepssurvey)
# 
# raw  <- import_steps_data("my_data.sav")
# cols <- detect_steps_columns(raw)
# 
# # Summary
# cat("Rows:", nrow(raw), "\n")
# cat("Columns:", ncol(raw), "\n")
# cat("Detected:", sum(!sapply(cols, is.null)), "/", length(cols), "\n")
# 
# # Check key variables
# if (!is.null(cols$age)) {
#   cat("\nAge range:", range(raw[[cols$age]], na.rm = TRUE), "\n")
#   cat("Age NAs:", sum(is.na(raw[[cols$age]])), "\n")
# }
# if (!is.null(cols$sex)) {
#   cat("\nSex distribution:\n")
#   print(table(raw[[cols$sex]], useNA = "ifany"))
# }
# if (!is.null(cols$weight_step1)) {
#   wt <- raw[[cols$weight_step1]]
#   cat("\nWeight range:", round(range(wt, na.rm = TRUE), 3), "\n")
#   cat("Weight NAs:", sum(is.na(wt)), "\n")
# }
# 
# # List undetected variables
# missing <- names(cols[sapply(cols, is.null)])
# cat("\nUndetected variables (", length(missing), "):\n")
# cat(paste(" ", missing, collapse = "\n"), "\n")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)

## ----summary-table, eval=TRUE, echo=FALSE, results='asis'---------------------
summary_data <- data.frame(
  Country = c("Moldova 2021", "Mongolia 2019", "Georgia 2016",
              "Afghanistan 2018", "Algeria 2016", "Ukraine 2019",
              "Ecuador 2018", "Cabo Verde 2020", "Bahamas 2019",
              "**TOTAL**"),
  Compared = c(17, 15, 15, 14, 14, 14, 14, 15, 10, 128),
  Within_1pp = c(16, 15, 15, 14, 14, 13, 13, 14, 9, 123),
  CI_Overlap = c(17, 15, 15, 14, 14, 14, 14, 15, 10, 128),
  Match_Rate = c("94%", "100%", "100%", "100%", "100%", "93%",
                 "93%", "93%", "90%", "**96%**")
)

knitr::kable(
  summary_data,
  col.names = c("Country", "Indicators compared", "Within 1 pp",
                "CI overlap", "Match rate"),
  align = c("l", "c", "c", "c", "c"),
  caption = "Validation summary: stepssurvey vs. WHO fact sheets (9 countries)"
)

## ----moldova, eval=TRUE, echo=FALSE, results='asis'---------------------------
mda <- data.frame(
  Indicator = c(
    "Current tobacco use", "Current smoking", "Second-hand smoke (home)",
    "Second-hand smoke (work)", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(27.7, 27.6, 23.3, 26.4, 63.2, 13.8, 63.2, 9.1,
           63.9, 22.7, 35.0, 6.3, 28.4, 9.9, 26.9, 129.2, 4.4),
  WHO = c(29.9, 27.6, 23.2, 26.4, 63.2, 13.8, 63.4, 9.1,
          63.9, 22.7, 34.8, 6.3, 27.7, 9.9, 26.9, 129.2, 4.4),
  Diff = c(-2.2, 0.0, 0.1, 0.0, 0.0, 0.0, -0.2, 0.0,
           0.0, 0.0, 0.2, 0.0, 0.7, 0.0, 0.0, 0.0, 0.0),
  Match = c("No", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes"),
  CI_Overlap = c("Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes",
                 "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes")
)

knitr::kable(mda, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Moldova 2021: 16/17 within 1 pp, 17/17 CI overlap")

## ----mongolia, eval=TRUE, echo=FALSE, results='asis'--------------------------
mng <- data.frame(
  Indicator = c(
    "Current tobacco use", "Current smoking (daily)",
    "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds (130/80)",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(25.0, 21.6, 34.8, 20.2, 83.2, 22.5,
           49.3, 18.5, 44.3, 8.3, 27.8, 17.4, 25.6, 120.5, 4.4),
  WHO = c(24.2, 21.6, 34.8, 19.8, 83.4, 21.9,
          49.4, 18.5, 44.0, 8.3, 27.8, 17.4, 25.5, 120.5, 4.4),
  Diff = c(0.8, 0.0, 0.0, 0.4, -0.2, 0.6,
           -0.1, 0.0, 0.3, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0),
  Match = rep("Yes", 15),
  CI_Overlap = rep("Yes", 15)
)

knitr::kable(mng, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Mongolia 2019: 15/15 within 1 pp, 15/15 CI overlap")

## ----georgia, eval=TRUE, echo=FALSE, results='asis'---------------------------
geo <- data.frame(
  Indicator = c(
    "Current tobacco use", "Current smoking (daily)",
    "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(31.1, 28.0, 39.0, 18.7, 62.9, 18.2,
           64.6, 33.4, 37.7, 4.5, 27.7, 2.0, 28.2, 129.4, 4.3),
  WHO = c(31.0, 28.0, 39.1, 18.3, 63.0, 17.4,
          64.6, 33.2, 37.7, 4.5, 27.7, 2.0, 28.1, 129.4, 4.3),
  Diff = c(0.1, 0.0, -0.1, 0.4, -0.1, 0.8,
           0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0),
  Match = rep("Yes", 15),
  CI_Overlap = rep("Yes", 15)
)

knitr::kable(geo, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Georgia 2016: 15/15 within 1 pp, 15/15 CI overlap")

## ----afghanistan, eval=TRUE, echo=FALSE, results='asis'-----------------------
afg <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(8.6, 0.2, 0.1, 97.3, 26.6,
           42.7, 17.2, 29.2, 9.2, 17.8, 4.9,
           25.2, 125.5, 3.8),
  WHO = c(8.6, 0.2, 0.1, 97.3, 26.5,
          42.7, 17.0, 29.2, 9.2, 18.1, 4.7,
          25.1, 125.5, 3.8),
  Diff = c(0.0, 0.0, 0.0, 0.0, 0.1,
           0.0, 0.2, 0.0, 0.0, -0.3, 0.2,
           0.1, 0.0, 0.0),
  Match = rep("Yes", 14),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(afg, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Afghanistan 2018: 14/14 within 1 pp, 14/14 CI overlap")

## ----algeria, eval=TRUE, echo=FALSE, results='asis'---------------------------
dza <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(16.4, 2.1, 1.3, 85.2, 23.7,
           55.5, 21.9, 23.7, 8.8, 23.5, 8.6,
           26.4, 126.4, 4.2),
  WHO = c(16.5, 2.1, 1.3, 85.3, 23.7,
          55.6, 21.8, 23.6, 9.0, 24.0, 8.2,
          26.4, 126.3, 4.2),
  Diff = c(-0.1, 0.0, 0.0, -0.1, 0.0,
           -0.1, 0.1, 0.1, -0.2, -0.5, 0.4,
           0.0, 0.1, 0.0),
  Match = rep("Yes", 14),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(dza, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Algeria 2016: 14/14 within 1 pp, 14/14 CI overlap")

## ----ukraine, eval=TRUE, echo=FALSE, results='asis'---------------------------
ukr <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(33.7, 55.5, 20.3, 66.1, 10.7,
           59.1, 24.9, 36.7, 7.1, 40.7, 9.0,
           26.9, 129.2, 4.7),
  WHO = c(33.9, 55.6, 19.7, 66.4, 10.0,
          59.0, 24.8, 34.8, 7.1, 40.7, 8.8,
          26.8, 129.1, 4.7),
  Diff = c(-0.2, -0.1, 0.6, -0.3, 0.7,
           0.1, 0.1, 1.9, 0.0, 0.0, 0.2,
           0.1, 0.1, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "No", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(ukr, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Ukraine 2019: 13/14 within 1 pp, 14/14 CI overlap")

## ----ecuador, eval=TRUE, echo=FALSE, results='asis'---------------------------
ecu <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(13.7, 39.3, 24.1, 94.6, 17.8,
           63.6, 25.7, 20.5, 6.9, 33.7, 8.3,
           27.2, 119.7, 4.4),
  WHO = c(13.7, 39.3, 23.8, 94.6, 17.8,
          63.6, 25.7, 19.8, 7.1, 34.7, 7.8,
          27.2, 119.7, 4.4),
  Diff = c(0.0, 0.0, 0.3, 0.0, 0.0,
           0.0, 0.0, 0.7, -0.2, -1.0, 0.5,
           0.0, 0.0, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes", "Yes", "No", "Yes",
            "Yes", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(ecu, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Ecuador 2018: 13/14 within 1 pp, 14/14 CI overlap")

## ----caboverde, eval=TRUE, echo=FALSE, results='asis'-------------------------
cpv <- data.frame(
  Indicator = c(
    "Current smoking", "Second-hand smoke (work)",
    "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(9.6, 15.0, 45.0, 17.5, 77.3, 31.6,
           44.2, 14.3, 31.1, 3.6, 17.9, 2.4,
           25.1, 128.8, 4.0),
  WHO = c(9.6, 15.0, 45.0, 17.5, 79.0, 31.8,
          44.2, 14.3, 30.8, 3.7, 18.8, 2.3,
          25.1, 128.8, 4.0),
  Diff = c(0.0, 0.0, 0.0, 0.0, -1.7, -0.2,
           0.0, 0.0, 0.3, -0.1, -0.9, 0.1,
           0.0, 0.0, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "No", "Yes",
            "Yes", "Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 15)
)

knitr::kable(cpv, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Cabo Verde 2020: 14/15 within 1 pp, 15/15 CI overlap")

## ----bahamas, eval=TRUE, echo=FALSE, results='asis'---------------------------
bhs <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Mean BMI", "Mean SBP"
  ),
  Ours = c(17.4, 49.5, 18.1, 85.0, 30.1,
           71.7, 44.5, 35.3, 30.6, 125.4),
  WHO = c(17.4, 49.6, 17.6, 85.3, 30.2,
          71.6, 43.6, 36.7, 29.8, 125.4),
  Diff = c(0.0, -0.1, 0.5, -0.3, -0.1,
           0.1, 0.9, -1.4, 0.8, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "No", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 10)
)

knitr::kable(bhs, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Bahamas 2019: 9/10 within 1 pp, 10/10 CI overlap")

#!/usr/bin/env Rscript
# =============================================================================
# Multi-Country Validation Script
# Runs stepssurvey package on all available STEPS datasets and compares
# against official WHO fact sheet values
# =============================================================================

library(stepssurvey)
library(dplyr)
library(survey)

# -- Configuration -----------------------------------------------------------
DATASETS_ROOT <- "../STEPS Licensed Datasets"

countries <- list(
  list(
    name     = "Moldova 2021",
    data     = file.path(DATASETS_ROOT, "Republic of Moldova 2021/MDA_2021_STEPS.dta"),
    ref      = "inst/validation/moldova_2021_reference.csv",
    age_min  = 18,
    age_max  = 69
  ),
  # Jordan 2019 excluded: non-standard instrument coding produces systematic

  # discrepancies in current_smoking (+5.4pp) and raised_bp (+4.0pp).
  # Kept for future investigation once Jordan-specific coding is understood.
  # list(
  #   name     = "Jordan 2019",
  #   data     = file.path(DATASETS_ROOT, "Jordan 2019/jor2019.dta"),
  #   ref      = "inst/validation/jordan_2019_reference.csv",
  #   age_min  = 18,
  #   age_max  = 69
  # ),
  list(
    name     = "Mongolia 2019",
    data     = file.path(DATASETS_ROOT, "Mongolia 2019/mng2019.dta"),
    ref      = "inst/validation/mongolia_2019_reference.csv",
    age_min  = 15,
    age_max  = 69,
    bp_sbp   = 130,
    bp_dbp   = 80,
    # Mongolia fact sheet reports daily smoking under "current smoking"
    smoking_var = "daily_smoker"
  ),
  list(
    name     = "Georgia 2016",
    data     = file.path(DATASETS_ROOT, "Geogrgia 2016/geo2016.dta"),
    ref      = "inst/validation/georgia_2016_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Georgia fact sheet reports daily smoking under "current smoking"
    smoking_var = "daily_smoker"
  ),
  list(
    name     = "Afghanistan 2018",
    data     = file.path(DATASETS_ROOT, "Afganistan 2018/afg2018.dta"),
    ref      = "inst/validation/afghanistan_2018_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Fact sheet reports "currently smoke tobacco" (any frequency, smoking only,
    # not including smokeless). Maps to current_tobacco in the package.
    smoking_var = "current_tobacco",
    # Cholesterol and glucose reported in mg/dl in fact sheet
    units_note = "cholesterol_mgdl"
  ),
  list(
    name     = "Algeria 2016",
    data     = file.path(DATASETS_ROOT, "Algeria 2016/dza2016.dta"),
    ref      = "inst/validation/algeria_2016_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Fact sheet reports "currently smoke tobacco" (any frequency)
    smoking_var = "current_tobacco",
    units_note = "cholesterol_mgdl"
  ),
  list(
    name     = "Ukraine 2019",
    data     = file.path(DATASETS_ROOT, "Ukraine 2019/ukr2019.dta"),
    ref      = "inst/validation/ukraine_2019_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Fact sheet reports "currently smoke tobacco" (any frequency)
    smoking_var = "current_tobacco"
    # Ukraine fact sheet uses mmol/L for cholesterol (same as package default)
  ),
  list(
    name     = "Ecuador 2018",
    data     = file.path(DATASETS_ROOT, "Ecuador 2018/ecu2018.dta"),
    ref      = "inst/validation/ecuador_2018_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Fact sheet reports "current tobacco smokers" (any frequency)
    smoking_var = "current_tobacco",
    units_note = "cholesterol_mgdl"
  ),
  list(
    name     = "Cabo Verde 2020",
    data     = file.path(DATASETS_ROOT, "Cabo Verde/cpv2020.dta"),
    ref      = "inst/validation/cabo_verde_2020_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Fact sheet reports "currently smoke tobacco" (any frequency)
    smoking_var = "current_tobacco",
    units_note = "cholesterol_mgdl"
  ),
  list(
    name     = "Bahamas 2019",
    data     = file.path(DATASETS_ROOT, "Bahmas 2019/bhs2019.dta"),
    ref      = "inst/validation/bahamas_2019_reference.csv",
    age_min  = 18,
    age_max  = 69,
    # Fact sheet reports "currently smoke tobacco" (any frequency)
    smoking_var = "current_tobacco",
    # Bahamas Step 3 biochemical measurements are UNWEIGHTED in fact sheet
    # (response rate for Step 3 was < 60%). Biochemical comparisons will
    # differ because our package uses weighted estimates.
    skip_step3 = TRUE
  )
)

# -- Helpers (same as Moldova script) ----------------------------------------
compute_prop <- function(var_name, design, label = var_name) {
  if (!var_name %in% names(design$variables)) {
    return(data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA))
  }
  tryCatch({
    formula <- as.formula(paste0("~", var_name))
    est <- svymean(formula, design, na.rm = TRUE)
    ci <- confint(est)
    vals <- as.numeric(est)
    if (length(vals) == 2) {
      data.frame(indicator = label, our_est = round(vals[2] * 100, 1),
                 our_ci_low = round(ci[2, 1] * 100, 1), our_ci_high = round(ci[2, 2] * 100, 1))
    } else {
      data.frame(indicator = label, our_est = round(vals[1] * 100, 1),
                 our_ci_low = round(ci[1, 1] * 100, 1), our_ci_high = round(ci[1, 2] * 100, 1))
    }
  }, error = function(e) {
    data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA)
  })
}

compute_mean <- function(var_name, design, label = var_name) {
  if (!var_name %in% names(design$variables)) {
    return(data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA))
  }
  tryCatch({
    formula <- as.formula(paste0("~", var_name))
    est <- svymean(formula, design, na.rm = TRUE)
    ci <- confint(est)
    data.frame(indicator = label, our_est = round(as.numeric(est), 1),
               our_ci_low = round(ci[1, 1], 1), our_ci_high = round(ci[1, 2], 1))
  }, error = function(e) {
    data.frame(indicator = label, our_est = NA, our_ci_low = NA, our_ci_high = NA)
  })
}

# -- Run validation for each country -----------------------------------------
all_results <- list()

for (country in countries) {
  cat("\n", paste(rep("=", 80), collapse = ""), "\n")
  cat("  VALIDATING:", country$name, "\n")
  cat(paste(rep("=", 80), collapse = ""), "\n\n")

  # Check files exist
  if (!file.exists(country$data)) {
    cat("  *** DATA FILE NOT FOUND:", country$data, "-- SKIPPING ***\n")
    next
  }
  if (!file.exists(country$ref)) {
    cat("  *** REFERENCE FILE NOT FOUND:", country$ref, "-- SKIPPING ***\n")
    next
  }

  # Step 1: Import
  raw <- import_steps_data(country$data)
  cat(sprintf("  Imported: %d rows x %d cols\n", nrow(raw), ncol(raw)))

  # Step 2: Detect columns
  cols <- detect_steps_columns(raw)

  # Step 3: Clean
  bp_sbp <- if (!is.null(country$bp_sbp)) country$bp_sbp else 140
  bp_dbp <- if (!is.null(country$bp_dbp)) country$bp_dbp else 90
  # When source data is in mg/dl, the package auto-converts to mmol/L.
  # WHO uses 190 mg/dl threshold (= 4.914 mmol/L). Use 4.914 instead of
  # default 5.0 to match the fact sheet definition exactly.
  chol_thr <- if (!is.null(country$units_note) &&
                   country$units_note == "cholesterol_mgdl") 4.914 else 5.0
  cleaned <- clean_steps_data(.strip_haven_labels(raw), cols,
                               age_min = country$age_min,
                               age_max = country$age_max,
                               bp_sbp_threshold = bp_sbp,
                               bp_dbp_threshold = bp_dbp,
                               chol_threshold = chol_thr)
  cat(sprintf("  Cleaned: %d rows\n", nrow(cleaned)))

  # Step 4: Survey design
  designs <- setup_survey_design(cleaned)

  # Step 5: Compute indicators
  d1 <- designs$step1
  d2 <- designs$step2
  d3 <- designs$step3

  # Some WHO fact sheets report daily smoking under "current smoking"
  smoking_var <- if (!is.null(country$smoking_var)) country$smoking_var else "current_tobacco"

  # Step 1 and Step 2 indicators (always computed)
  results <- bind_rows(
    compute_prop("current_tobacco_any", d1, "current_tobacco_use"),
    compute_prop(smoking_var, d1, "current_smoking"),
    compute_prop("secondhand_home", d1, "secondhand_smoke_home"),
    compute_prop("secondhand_work", d1, "secondhand_smoke_work"),
    compute_prop("current_alcohol", d1, "current_alcohol_30days"),
    compute_prop("heavy_episodic", d1, "heavy_episodic_drinking"),
    compute_prop("low_fruit_veg", d1, "insufficient_fruit_veg"),
    compute_prop("insufficient_pa", d1, "insufficient_pa"),
    compute_prop("overweight_obese", d2, "overweight_bmi25"),
    compute_prop("obese", d2, "obese_bmi30"),
    compute_prop("raised_bp", d2, "raised_bp_or_meds"),
    compute_mean("bmi", d2, "mean_bmi"),
    compute_mean("mean_sbp", d2, "mean_sbp")
  )

  # Step 3 biochemical indicators (skip if fact sheet reports unweighted)
  skip_step3 <- !is.null(country$skip_step3) && country$skip_step3
  if (!skip_step3) {
    step3_results <- bind_rows(
      compute_prop("raised_glucose", d3, "raised_glucose_or_meds"),
      compute_prop("raised_chol_or_meds", d3, "raised_cholesterol_or_meds"),
      compute_prop("impaired_glucose", d3, "impaired_fasting_glucose"),
      compute_mean("total_chol", d3, "mean_total_cholesterol")
    )
    results <- bind_rows(results, step3_results)
  } else {
    cat("  NOTE: Step 3 biochemical comparisons skipped (fact sheet reports unweighted)\n")
  }

  # Load reference and compare
  ref <- read.csv(country$ref, stringsAsFactors = FALSE)
  comparison <- results %>%
    left_join(
      ref %>% select(indicator, ref_est = both, ref_ci_low = both_ci_low, ref_ci_high = both_ci_high),
      by = "indicator"
    ) %>%
    mutate(
      diff = our_est - ref_est
    )

  # Determine tolerance: for cholesterol in mg/dl, use 2.0 unit tolerance
  is_mgdl <- !is.null(country$units_note) && country$units_note == "cholesterol_mgdl"
  comparison$tolerance <- ifelse(
    comparison$indicator == "mean_total_cholesterol" & is_mgdl, 2.0, 1.0
  )
  comparison$match <- abs(comparison$diff) < comparison$tolerance

  # Display
  cat(sprintf("\nVALIDATION RESULTS: %s\n", country$name))
  cat("Our package vs. WHO official fact sheet (Both Sexes)\n")
  cat(sprintf("%-35s %8s %8s %8s %8s %8s\n",
              "Indicator", "Ours", "WHO", "Diff", "Match?", "CI overlap?"))
  cat(paste(rep("-", 85), collapse = ""), "\n")

  for (i in seq_len(nrow(comparison))) {
    r <- comparison[i, ]
    ci_overlap <- ""
    if (!is.na(r$our_ci_low) && !is.na(r$ref_ci_low)) {
      overlaps <- r$our_ci_high >= r$ref_ci_low && r$our_ci_low <= r$ref_ci_high
      ci_overlap <- ifelse(overlaps, "YES", "NO")
    }
    cat(sprintf("%-35s %8s %8s %8s %8s %8s\n",
                r$indicator,
                ifelse(is.na(r$our_est), "N/A", sprintf("%.1f", r$our_est)),
                ifelse(is.na(r$ref_est), "N/A", sprintf("%.1f", r$ref_est)),
                ifelse(is.na(r$diff), "N/A", sprintf("%+.1f", r$diff)),
                ifelse(is.na(r$match), "N/A", ifelse(r$match, "YES", "NO")),
                ci_overlap))
  }

  n_compared <- sum(!is.na(comparison$diff))
  n_match <- sum(comparison$match, na.rm = TRUE)
  n_ci_overlap <- sum(
    comparison$our_ci_high >= comparison$ref_ci_low &
    comparison$our_ci_low <= comparison$ref_ci_high,
    na.rm = TRUE
  )

  cat(paste(rep("-", 85), collapse = ""), "\n")
  cat(sprintf("SUMMARY: %d/%d within 1pp | %d/%d CIs overlap\n",
              n_match, n_compared, n_ci_overlap, n_compared))

  # Save per-country results
  out_file <- sprintf("inst/validation/%s_validation_results.csv",
                       gsub(" ", "_", tolower(country$name)))
  write.csv(comparison, out_file, row.names = FALSE)
  cat(sprintf("Results saved to %s\n", out_file))

  all_results[[country$name]] <- list(
    n_compared = n_compared,
    n_match = n_match,
    n_ci_overlap = n_ci_overlap,
    comparison = comparison
  )
}

# -- Grand summary -----------------------------------------------------------
cat("\n\n", paste(rep("=", 80), collapse = ""), "\n")
cat("  GRAND SUMMARY: All Countries\n")
cat(paste(rep("=", 80), collapse = ""), "\n\n")

cat(sprintf("%-25s %12s %12s %12s\n", "Country", "Within 1pp", "CI Overlap", "Match Rate"))
cat(paste(rep("-", 65), collapse = ""), "\n")
total_compared <- 0
total_match <- 0
total_ci <- 0

for (nm in names(all_results)) {
  r <- all_results[[nm]]
  cat(sprintf("%-25s %6d/%-5d %6d/%-5d %10.0f%%\n",
              nm, r$n_match, r$n_compared, r$n_ci_overlap, r$n_compared,
              r$n_match / r$n_compared * 100))
  total_compared <- total_compared + r$n_compared
  total_match    <- total_match    + r$n_match
  total_ci       <- total_ci       + r$n_ci_overlap
}

cat(paste(rep("-", 65), collapse = ""), "\n")
cat(sprintf("%-25s %6d/%-5d %6d/%-5d %10.0f%%\n",
            "TOTAL", total_match, total_compared, total_ci, total_compared,
            total_match / total_compared * 100))

#' Compute Blood Pressure Indicators
#'
#' Calculates prevalence of raised blood pressure and mean systolic and
#' diastolic blood pressure from a survey design object.
#'
#' @param design A survey design object from [setup_survey_design()].
#'
#' @return A named list of survey estimates. Each element contains estimates
#'   (as tibble with columns: estimate, lower, upper, etc.) for:
#'   - `raised_bp_total`: raised blood pressure, overall
#'   - `raised_bp_by_sex`: raised blood pressure, by sex
#'   - `raised_bp_by_age`: raised blood pressure, by age group
#'   - `mean_sbp_mean_total`: mean systolic BP, overall
#'   - `mean_sbp_mean_by_sex`: mean systolic BP, by sex
#'   - `mean_sbp_mean_by_age`: mean systolic BP, by age group
#'   - `mean_dbp_mean_total`: mean diastolic BP, overall
#'   - `mean_dbp_mean_by_sex`: mean diastolic BP, by sex
#'   - `mean_dbp_mean_by_age`: mean diastolic BP, by age group
#'   (if the corresponding variables exist in design)
#'
#' @seealso [compute_all_indicators()]
#'
#' @export
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   bp_results <- compute_bp_indicators(design)
#' }
compute_bp_indicators <- function(design) {
  message("  Computing blood pressure indicators...")
  results <- list()

  bin_vars <- c("raised_bp")
  bin_vars <- bin_vars[bin_vars %in% names(design$variables)]
  for (v in bin_vars) {
    f <- as.formula(paste0("~as.numeric(", v, ")"))
    results[[paste0(v, "_total")]]  <- svyprop(f, design)
    results[[paste0(v, "_by_sex")]] <- svyprop(f, design, by = ~sex)
    results[[paste0(v, "_by_age")]] <- svyprop(f, design, by = ~age_group)
  }

  cont_vars <- c("mean_sbp", "mean_dbp")
  cont_vars <- cont_vars[cont_vars %in% names(design$variables)]
  for (v in cont_vars) {
    f <- as.formula(paste0("~", v))
    results[[paste0(v, "_mean_total")]]  <- svymn(f, design)
    results[[paste0(v, "_mean_by_sex")]] <- svymn(f, design, by = ~sex)
    results[[paste0(v, "_mean_by_age")]] <- svymn(f, design, by = ~age_group)
  }

  return(results)
}

#' About Page Module UI
#'
#' Displays information about the stepssurvey package, authors,
#' and institutional affiliations.
#'
#' @param id Module namespace ID.
#' @return A Shiny UI element.
#' @keywords internal
mod_about_ui <- function(id) {
  ns <- shiny::NS(id)

  shiny::div(
    style = "max-width: 860px; margin: 0 auto; padding: 24px 20px;",

    # Logo and title
    shiny::div(
      style = "text-align: center; margin-bottom: 30px;",
      shiny::uiOutput(ns("logo_img")),
      shiny::h2("stepssurvey",
                 style = "color: #00427A; margin-top: 10px; font-weight: bold;"),
      shiny::h4("WHO STEPS NCD Survey Analysis Pipeline",
                 style = "color: #0072BC; margin-bottom: 4px;"),
      shiny::p(
        paste0("Version ", utils::packageVersion("stepssurvey")),
        style = "color: #888; font-size: 0.9em;"
      )
    ),

    # --- About the Package ---
    shiny::h4(
      shiny::icon("box-open"), " About the Package",
      style = "color: #00427A; border-bottom: 2px solid #008DC9; padding-bottom: 6px; margin-bottom: 14px;"
    ),
    shiny::p(
      "The WHO STEPwise Approach to NCD Risk Factor Surveillance (STEPS) is ",
      "the global standard for monitoring non-communicable disease risk factors. ",
      "Adopted by over 130 countries, it provides a standardised framework for ",
      "collecting behavioural, physical, and biochemical data across three steps: ",
      "questionnaire-based assessment of tobacco, alcohol, diet, and physical activity ",
      "(Step 1); physical measurements including blood pressure and anthropometry ",
      "(Step 2); and biochemical measurements of blood glucose and lipids (Step 3)."
    ),
    shiny::p(
      "Despite well-defined WHO protocols for data collection, ",
      "the analysis of STEPS data remains a significant bottleneck. Country teams ",
      "often rely on manual Epi Info or SPSS scripts that require substantial ",
      "statistical expertise, are error-prone, and produce inconsistent outputs. ",
      "This is particularly challenging for low- and middle-income countries (LMICs) ",
      "where trained biostatisticians are scarce and surveillance cycles are infrequent."
    ),
    shiny::p(
      shiny::tags$strong("stepssurvey"), " addresses this gap by providing a complete, ",
      "end-to-end analysis pipeline in R that automates the entire workflow ",
      "from raw survey data to publication-ready reports, faithfully implementing ",
      "WHO-standard methodology for all ten headline NCD risk factor indicators. ",
      "It supports CSV, Excel, Stata, and SPSS formats; implements WHO-standard ",
      "cleaning rules (smk_cln/smkless_cln, GPAQ P_clean); handles complex survey ",
      "designs with step-specific weights; and produces formatted tables, ",
      "WHO-branded visualisations, and three Word report templates."
    ),
    shiny::p(
      "The package has been validated against published WHO fact sheets from ",
      "nine countries across six WHO regions, achieving ",
      shiny::tags$strong("96% concordance"),
      " (123 of 128 indicators within 1 percentage point of published values) with ",
      "100% overlapping 95% confidence intervals.",
      style = "margin-bottom: 28px;"
    ),

    # --- About AIIMS Bhopal ---
    shiny::h4(
      shiny::icon("university"), " About AIIMS Bhopal",
      style = "color: #00427A; border-bottom: 2px solid #008DC9; padding-bottom: 6px; margin-bottom: 14px;"
    ),
    shiny::p(
      "The All India Institute of Medical Sciences (AIIMS), Bhopal is an Institute ",
      "of National Importance established in 2012, serving as a centre of excellence ",
      "in medical education, research, and tertiary healthcare for central India. ",
      "The Department of Community and Family Medicine has extensive experience in ",
      "population-based NCD surveillance, having conducted STEPS-aligned surveys ",
      "and large-scale epidemiological studies across Madhya Pradesh.",
      style = "margin-bottom: 28px;"
    ),

    # --- Authors ---
    shiny::h4(
      shiny::icon("users"), " Authors",
      style = "color: #00427A; border-bottom: 2px solid #008DC9; padding-bottom: 6px; margin-bottom: 14px;"
    ),

    # Dr Abhijit Pakhare
    shiny::p(
      shiny::tags$strong("Dr Abhijit Pakhare"),
      " \u2014 Professor, Department of Community and Family Medicine, AIIMS Bhopal"
    ),

    # Dr Ankur Joshi
    shiny::p(
      shiny::tags$strong("Dr Ankur Joshi"),
      " \u2014 Additional Professor, Department of Community and Family Medicine, AIIMS Bhopal"
    ),

    # Dr Lena Charlette
    shiny::p(
      shiny::tags$strong("Dr Lena Charlette"),
      " \u2014 Assistant Professor, Department of Community and Family Medicine, AIIMS Madurai",
      style = "margin-bottom: 20px;"
    ),

    # AI acknowledgement
    shiny::p(
      shiny::tags$strong("AI-Assisted Development: "),
      "Substantial portions of this package were developed with the assistance of ",
      "Claude (Anthropic), an AI coding assistant. This includes the Shiny application, ",
      "automated test suite, visualisation functions, report templates, and the ",
      "multi-country validation pipeline. The domain expertise, epidemiological ",
      "methodology, and analytical decisions are entirely the authors'; Claude ",
      "served as a development accelerator that enabled three researchers to build ",
      "and validate a tool that would otherwise require a dedicated software ",
      "engineering team. All AI-generated code was reviewed, tested, and validated ",
      "by the authors against published WHO outputs.",
      style = "color: #666; font-style: italic; margin-bottom: 28px;"
    ),

    # --- Citation ---
    shiny::h4(
      shiny::icon("quote-left"), " Citation",
      style = "color: #00427A; border-bottom: 2px solid #008DC9; padding-bottom: 6px; margin-bottom: 14px;"
    ),
    shiny::p("If you use stepssurvey in your research or surveillance work, please cite:"),
    shiny::tags$blockquote(
      style = paste0(
        "background: #f8f9fa; padding: 12px 16px; border-left: 4px solid #0072BC; ",
        "font-family: monospace; font-size: 0.9em; white-space: pre-wrap; ",
        "margin-bottom: 14px;"
      ),
      paste0(
        "Pakhare A, Joshi A, Charlette L (2026). stepssurvey: Analyse WHO STEPS Survey Data.\n",
        "R package version 0.1.0.\n",
        "https://github.com/drpakhare/stepssurvey"
      )
    ),
    shiny::p(
      shiny::tags$strong("License: "), "MIT",
      " | ",
      shiny::tags$strong("Source: "),
      shiny::tags$a(href = "https://github.com/drpakhare/stepssurvey",
                    "GitHub", target = "_blank"),
      " | ",
      shiny::tags$strong("Docs: "),
      shiny::tags$a(href = "https://drpakhare.github.io/stepssurvey/",
                    "Package website", target = "_blank"),
      " | ",
      shiny::tags$strong("Issues: "),
      shiny::tags$a(href = "https://github.com/drpakhare/stepssurvey/issues",
                    "Bug reports", target = "_blank"),
      style = "font-size: 0.9em;"
    )
  )
}

#' About Page Module Server
#'
#' @param id Module namespace ID.
#' @keywords internal
mod_about_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    output$logo_img <- shiny::renderUI({
      logo_path <- system.file("www", "logo.png", package = "stepssurvey")
      if (nzchar(logo_path)) {
        shiny::img(src = "www/logo.png", height = "120px",
                   alt = "stepssurvey logo")
      }
    })
  })
}

#' Recode a yes/no variable to logical
#'
#' Converts various representations of yes/no (numeric, text, case-insensitive)
#' to logical TRUE/FALSE values.
#'
#' @param x A vector of yes/no values (numeric or character).
#' @return A logical vector, with NA for unrecognized values.
#' @keywords internal
#' @export
#' @examples
#' recode_yn(c(1, 2, "yes", "no", NA))
recode_yn <- function(x) {
  dplyr::case_when(
    x %in% c(1, "1", "yes", "Yes", "YES", "y", "Y") ~ TRUE,
    x %in% c(0, "0", 2, "2", "no", "No", "NO", "n", "N") ~ FALSE,
    TRUE ~ NA
  )
}

#' Clean and recode WHO STEPS data
#'
#' Processes raw STEPS survey data: renames columns, coerces types,
#' derives standard indicators, handles missing values, and applies
#' plausibility checks.
#'
#' @param data A data frame (typically from [import_steps_data()]).
#' @param cols A named list of column names, as returned by [detect_steps_columns()].
#' @param age_min Minimum age for inclusion (default 18).
#' @param age_max Maximum age for inclusion (default 69).
#' @param bp_sbp_threshold SBP threshold for raised BP (default 140; Mongolia uses 130).
#' @param bp_dbp_threshold DBP threshold for raised BP (default 90; Mongolia uses 80).
#' @param bmi_overweight BMI threshold for overweight (default 25.0).
#' @param bmi_obese BMI threshold for obesity (default 30.0).
#' @param glucose_threshold Fasting glucose threshold for raised glucose / diabetes
#'   in mmol/L (default 7.0).
#' @param glucose_impaired_threshold Fasting glucose threshold for impaired fasting
#'   glucose in mmol/L (default 6.1).
#' @param chol_threshold Total cholesterol threshold for raised cholesterol
#'   in mmol/L (default 5.0).
#'
#' @return A data frame with standardised and derived variables, ready for
#' survey design setup.
#'
#' @details
#' The function performs the following transformations:
#' - Renames columns to standard names (age, sex, wt_final, etc.)
#' - Converts numeric strings to appropriate types
#' - Restricts age to \code{[age_min, age_max]}
#' - Creates WHO standard age groups (18-24, 25-34, etc.)
#' - Harmonises sex coding to Male/Female
#' - Derives body mass index (BMI) and categories
#' - Averages blood pressure readings (last 2 of 3)
#' - Recodes yes/no variables to logical
#' - Creates derived risk indicators (raised BP, diabetes, etc.)
#' - Applies plausibility checks to measurements
#' - Drops records with missing age or sex
#'
#' @export
clean_steps_data <- function(data, cols, age_min = 18, age_max = 69,
                             bp_sbp_threshold = 140, bp_dbp_threshold = 90,
                             bmi_overweight = 25.0, bmi_obese = 30.0,
                             glucose_threshold = 7.0,
                             glucose_impaired_threshold = 6.1,
                             chol_threshold = 5.0) {

  d <- data

  # Helper: TRUE only if the column mapping exists AND the column is in the data
  has <- function(col_name) {
    !is.null(cols[[col_name]]) && cols[[col_name]] %in% names(d)
  }

  # -- Filter to valid/consented records (WHO scripts: valid==1) ---------------
  if (has("valid")) {
    n_before <- nrow(d)
    d <- d |> dplyr::filter(.data[[cols$valid]] == 1)
    n_dropped <- n_before - nrow(d)
    if (n_dropped > 0) {
      message(glue::glue("    \u2713 Filtered to valid records: dropped {n_dropped}, kept {nrow(d)}"))
    }
  }

  # -- Age & Sex --------------------------------------------------------------
  if (has("age")) {
    d <- d |>
      dplyr::rename(age = dplyr::all_of(cols$age)) |>
      dplyr::mutate(
        age = as.numeric(age),
        # Restrict to survey age range
        age = dplyr::if_else(age < age_min | age > age_max, NA_real_, age),
        # WHO standard age groups for STEPS
        age_group = cut(age,
          breaks = c(18, 25, 35, 45, 55, 65, Inf),
          labels = c("18-24", "25-34", "35-44", "45-54", "55-64", "65+"),
          right  = FALSE
        )
      )
  }

  # Use pre-computed agerange from WHO-processed datasets when available.
  # Country surveys often use different age groupings (e.g. 18-29/30-44/45-59/60-69
  # or 18-44/45-69), so the dataset's own agerange is authoritative for matching
  # published outputs. We keep our computed age_group as well for standardisation.
  if (has("agerange")) {
    agerange_col <- cols$agerange
    if (agerange_col != "agerange") {
      d <- d |> dplyr::rename(agerange = dplyr::all_of(agerange_col))
    }
    d$agerange <- as.character(d$agerange)
    d$agerange <- factor(d$agerange, levels = unique(sort(d$agerange)))
    message("    \u2713 Using dataset's pre-computed agerange: ",
            paste(levels(d$agerange), collapse = ", "))
  }

  if (has("sex")) {
    d <- d |>
      dplyr::rename(sex = dplyr::all_of(cols$sex)) |>
      dplyr::mutate(
        # Harmonise sex coding: 1=Male, 2=Female (WHO STEPS standard)
        sex = dplyr::case_when(
          sex %in% c(1, "1", "m", "M", "male", "Male", "Men")   ~ "Male",
          sex %in% c(2, "2", "f", "F", "female", "Female", "Women") ~ "Female",
          TRUE ~ NA_character_
        ),
        sex = factor(sex, levels = c("Male", "Female"))
      )
  }

  # -- Sampling design variables (step-specific weights per WHO STEPS) --------

  # WHO STEPS toolkit uses WStep1/WStep2/WStep3 -- one analysis weight per Step.
  # We rename to wt_step1/wt_step2/wt_step3 internally.
  # If only Step 1 weight is found, we copy it to Step 2 and Step 3.
  # If no weight is found at all, all three are set to 1 (unweighted).
  if (has("weight_step1")) {
    d <- d |> dplyr::rename(wt_step1 = dplyr::all_of(cols$weight_step1)) |>
      dplyr::mutate(wt_step1 = as.numeric(wt_step1))
  }
  if (has("weight_step2")) {
    d <- d |> dplyr::rename(wt_step2 = dplyr::all_of(cols$weight_step2)) |>
      dplyr::mutate(wt_step2 = as.numeric(wt_step2))
  }
  if (has("weight_step3")) {
    d <- d |> dplyr::rename(wt_step3 = dplyr::all_of(cols$weight_step3)) |>
      dplyr::mutate(wt_step3 = as.numeric(wt_step3))
  }

  # Fill in missing step weights: copy from step 1 if available, else unweighted
  if (!"wt_step1" %in% names(d)) {
    if (!"wt_step2" %in% names(d) && !"wt_step3" %in% names(d)) {
      message("  \u26a0 No weight variable found - using equal weights (unweighted)")
      d <- d |> dplyr::mutate(wt_step1 = 1, wt_step2 = 1, wt_step3 = 1)
    } else {
      # Use whichever step weight we found as fallback
      fallback <- if ("wt_step2" %in% names(d)) "wt_step2" else "wt_step3"
      message(glue::glue("  \u26a0 No Step 1 weight - falling back to '{fallback}'"))
      d$wt_step1 <- d[[fallback]]
    }
  }
  if (!"wt_step2" %in% names(d)) {
    message("  \u26a0 No Step 2 weight found - copying Step 1 weight")
    d$wt_step2 <- d$wt_step1
  }
  if (!"wt_step3" %in% names(d)) {
    message("  \u26a0 No Step 3 weight found - copying Step 1 weight")
    d$wt_step3 <- d$wt_step1
  }

  # Keep wt_final as alias for backward compatibility (points to Step 1 weight)
  d$wt_final <- d$wt_step1

  if (has("strata")) d <- d |> dplyr::rename(stratum = dplyr::all_of(cols$strata))
  if (has("psu"))    d <- d |> dplyr::rename(psu     = dplyr::all_of(cols$psu))

  # -- Tobacco ---------------------------------------------------------------
  if (has("tobacco_current")) {
    d <- d |>
      dplyr::rename(tobacco_current_raw = dplyr::all_of(cols$tobacco_current)) |>
      dplyr::mutate(current_tobacco = recode_yn(tobacco_current_raw))
  }
  if (has("tobacco_daily")) {
    d <- d |>
      dplyr::rename(tobacco_daily_raw = dplyr::all_of(cols$tobacco_daily)) |>
      dplyr::mutate(daily_tobacco = recode_yn(tobacco_daily_raw))
  }
  if (has("smokeless_current")) {
    d <- d |>
      dplyr::rename(smokeless_current_raw = dplyr::all_of(cols$smokeless_current)) |>
      dplyr::mutate(current_smokeless = recode_yn(smokeless_current_raw))
  }
  if (has("smokeless_daily")) {
    d <- d |>
      dplyr::rename(smokeless_daily_raw = dplyr::all_of(cols$smokeless_daily)) |>
      dplyr::mutate(daily_smokeless = recode_yn(smokeless_daily_raw))
  }

  # -- WHO data quality filters (smk_cln / smkless_cln) -----------------------
  # WHO official scripts exclude logically inconsistent tobacco respondents
  # from the denominator.  smk_cln==1 means "valid smoking record":
  #   • current smoker AND t2 answered (daily/not daily) → valid
  #   • non-smoker AND (t2 is NA or says not daily)      → valid
  #   • current smoker but past-use says "never smoked"  → INVALID
  #   • current smoker but t2 is not answered             → INVALID
  #
  # NOTE: We use the already-decoded logical current_tobacco rather than raw
  # codes, because raw coding varies (0/1 in some datasets, 1/2 in WHO STEPS).
  # recode_yn() has already normalised this.
  if ("current_tobacco" %in% names(d)) {
    ct <- d$current_tobacco  # logical: TRUE = smoker, FALSE = non-smoker
    # daily_tobacco is logical if decoded, else we look at raw
    dt <- if ("daily_tobacco" %in% names(d)) {
      d$daily_tobacco
    } else {
      rep(NA, nrow(d))
    }
    # Past tobacco use (t8) — raw value; "2" means "never smoked" in WHO coding
    t8_raw <- if (has("tobacco_past")) as.numeric(d[[cols$tobacco_past]]) else rep(NA_real_, nrow(d))

    # Only apply strict validation when past-use variable (t8) is available
    has_t8 <- any(!is.na(t8_raw))

    smk_cln <- dplyr::case_when(
      # Smoker who says they never smoked in the past → inconsistent
      !is.na(ct) & ct == TRUE & !is.na(t8_raw) & t8_raw == 2   ~ 2L,
      # Smoker with daily/nondaily answer → valid
      !is.na(ct) & ct == TRUE  & !is.na(dt)                     ~ 1L,
      # Non-smoker → always valid (t2 doesn't matter)
      !is.na(ct) & ct == FALSE                                   ~ 1L,
      # Smoker but daily question unanswered → invalid only if t8 is in the dataset
      !is.na(ct) & ct == TRUE  & is.na(dt) & has_t8             ~ 2L,
      # Default: valid (don't penalise datasets without t2/t8)
      TRUE                                                       ~ 1L
    )

    n_invalid_smk <- sum(smk_cln == 2, na.rm = TRUE)
    if (n_invalid_smk > 0) {
      d$current_tobacco[smk_cln == 2] <- NA
      message(glue::glue("    \u2713 WHO smk_cln filter: {n_invalid_smk} inconsistent smoking records set to NA"))
    }
  }

  if ("current_smokeless" %in% names(d)) {
    cs <- d$current_smokeless  # logical
    ds <- if ("daily_smokeless" %in% names(d)) {
      d$daily_smokeless
    } else {
      rep(NA, nrow(d))
    }
    t15_raw <- if (has("smokeless_past")) as.numeric(d[[cols$smokeless_past]]) else rep(NA_real_, nrow(d))

    has_t15 <- any(!is.na(t15_raw))

    smkless_cln <- dplyr::case_when(
      !is.na(cs) & cs == TRUE  & !is.na(t15_raw) & t15_raw == 2 ~ 2L,
      !is.na(cs) & cs == TRUE  & !is.na(ds)                      ~ 1L,
      !is.na(cs) & cs == FALSE                                    ~ 1L,
      !is.na(cs) & cs == TRUE  & is.na(ds) & has_t15             ~ 2L,
      TRUE                                                        ~ 1L
    )

    n_invalid_smkless <- sum(smkless_cln == 2, na.rm = TRUE)
    if (n_invalid_smkless > 0) {
      d$current_smokeless[smkless_cln == 2] <- NA
      message(glue::glue("    \u2713 WHO smkless_cln filter: {n_invalid_smkless} inconsistent smokeless records set to NA"))
    }
  }

  # Derived smoking variables
  # current_smoker = smokes any tobacco product (cigarettes, pipes, etc.)
  if ("current_tobacco" %in% names(d)) {
    d$current_smoker <- d$current_tobacco
  }
  # Smoking status categories
  if (has("tobacco_past")) {
    d <- d |>
      dplyr::rename(tobacco_past_raw = dplyr::all_of(cols$tobacco_past)) |>
      dplyr::mutate(former_smoker = recode_yn(tobacco_past_raw))
  }
  if (all(c("current_tobacco", "daily_tobacco") %in% names(d))) {
    d <- d |> dplyr::mutate(
      nondaily_smoker = current_tobacco & !dplyr::if_else(is.na(daily_tobacco), FALSE, daily_tobacco),
      daily_smoker    = dplyr::if_else(is.na(daily_tobacco), FALSE, daily_tobacco)
    )
    # Smoking status factor
    if ("former_smoker" %in% names(d)) {
      d <- d |> dplyr::mutate(
        never_smoker = !current_tobacco & !dplyr::if_else(is.na(former_smoker), FALSE, former_smoker),
        smoking_status = dplyr::case_when(
          daily_smoker    ~ "Daily",
          nondaily_smoker ~ "Non-daily",
          dplyr::if_else(is.na(former_smoker), FALSE, former_smoker) ~ "Former",
          TRUE ~ "Never"
        )
      )
    }
  }
  # Past daily smoker
  if (has("tobacco_past_daily")) {
    d <- d |>
      dplyr::rename(tobacco_past_daily_raw = dplyr::all_of(cols$tobacco_past_daily)) |>
      dplyr::mutate(former_daily_smoker = recode_yn(tobacco_past_daily_raw))
  }
  # Age started smoking and duration
  if (has("tobacco_start_age")) {
    d <- d |> dplyr::mutate(
      smoking_start_age = as.numeric(.data[[cols$tobacco_start_age]])
    )
  }
  if (has("tobacco_duration")) {
    d <- d |> dplyr::mutate(
      smoking_duration = as.numeric(.data[[cols$tobacco_duration]])
    )
  }
  # Tobacco products: manufactured cigarettes per day
  if (has("tobacco_cig_day")) {
    d <- d |> dplyr::mutate(
      cig_per_day_manufactured = as.numeric(.data[[cols$tobacco_cig_day]])
    )
    if ("daily_smoker" %in% names(d)) {
      d <- d |> dplyr::mutate(
        smokes_manufactured_cig = dplyr::if_else(
          daily_smoker & !is.na(cig_per_day_manufactured) & cig_per_day_manufactured > 0,
          TRUE, FALSE
        )
      )
    }
  }
  if (has("tobacco_hand_day")) {
    d <- d |> dplyr::mutate(
      cig_per_day_handrolled = as.numeric(.data[[cols$tobacco_hand_day]])
    )
  }
  # Quit attempt and advice
  if (has("tobacco_quit_attempt")) {
    d <- d |> dplyr::mutate(quit_attempt = recode_yn(.data[[cols$tobacco_quit_attempt]]))
  }
  if (has("tobacco_quit_advice")) {
    d <- d |> dplyr::mutate(quit_advice = recode_yn(.data[[cols$tobacco_quit_advice]]))
  }
  # Second-hand smoke
  if (has("secondhand_home")) {
    d <- d |> dplyr::mutate(secondhand_home = recode_yn(.data[[cols$secondhand_home]]))
  }
  if (has("secondhand_work")) {
    d <- d |> dplyr::mutate(secondhand_work = recode_yn(.data[[cols$secondhand_work]]))
  }

  # Fix: non-tobacco users should be FALSE for smokeless, not NA
  if ("daily_smokeless" %in% names(d) && "current_tobacco" %in% names(d)) {
    d <- d |> dplyr::mutate(
      daily_smokeless = dplyr::if_else(
        current_tobacco == FALSE & is.na(daily_smokeless), FALSE, daily_smokeless
      )
    )
  }
  if ("current_smokeless" %in% names(d) && "current_tobacco" %in% names(d)) {
    d <- d |> dplyr::mutate(
      current_smokeless = dplyr::if_else(
        current_tobacco == FALSE & is.na(current_smokeless), FALSE, current_smokeless
      )
    )
  }

  # Combined: any current tobacco use (smoking OR smokeless)
  has_smoker    <- "current_smoker" %in% names(d)
  has_smokeless <- "current_smokeless" %in% names(d) || "daily_smokeless" %in% names(d)
  if (has_smoker && has_smokeless) {
    sl_var <- if ("current_smokeless" %in% names(d)) "current_smokeless" else "daily_smokeless"
    d <- d |> dplyr::mutate(
      current_tobacco_any = dplyr::if_else(
        is.na(current_smoker) & is.na(.data[[sl_var]]), NA,
        dplyr::if_else(
          dplyr::coalesce(current_smoker, FALSE) | dplyr::coalesce(.data[[sl_var]], FALSE),
          TRUE, FALSE
        )
      )
    )
  }

  # -- Alcohol ---------------------------------------------------------------
  if (has("alcohol_current")) {
    d <- d |>
      dplyr::rename(alcohol_current_raw = dplyr::all_of(cols$alcohol_current))
    # A5-style columns may have 3+ coded values (1=current, 2=past 12m, 3+=abstainer)
    # recode_yn handles 1->TRUE, 2->FALSE, but other values (3,4,...) -> NA.
    # For "current alcohol", any non-1 response means "not current" = FALSE.
    raw_vals <- unique(stats::na.omit(as.numeric(d$alcohol_current_raw)))
    if (length(raw_vals) > 0 && max(raw_vals, na.rm = TRUE) > 2) {
      # Multi-value column: only value 1 = current drinker
      message("  \u2192 Multi-value alcohol column detected (values: ",
              paste(sort(raw_vals), collapse = ", "), ") -- coding 1=current, rest=not current")
      d <- d |>
        dplyr::mutate(current_alcohol = as.numeric(alcohol_current_raw) == 1)
    } else {
      d <- d |>
        dplyr::mutate(current_alcohol = recode_yn(alcohol_current_raw))
    }
  }

  if (has("heavy_episode")) {
    d <- d |>
      dplyr::rename(heavy_episode_raw = dplyr::all_of(cols$heavy_episode))

    # a9 in WHO STEPS v3.2 is a COUNT variable (number of occasions with 6+ drinks).
    # Values: 0 = never, 1+ = yes, 77 = don't know, 88 = refused.
    # For yes/no binary columns, recode_yn handles it. For count columns,
    # we derive: HED = TRUE if count >= 1 (excluding 77/88).
    raw_vals <- unique(stats::na.omit(as.numeric(d$heavy_episode_raw)))
    if (length(raw_vals) > 0 && max(raw_vals, na.rm = TRUE) > 2) {
      # Count variable: 0 = no HED, >=1 = HED, 77/88 = unknown
      message("  \u2192 Heavy episodic drinking: count variable detected (values: ",
              paste(sort(raw_vals[raw_vals <= 10]), collapse = ", "),
              if (any(raw_vals > 10)) paste0(", ..., ", max(raw_vals)) else "", ")")
      d <- d |>
        dplyr::mutate(
          heavy_episode_raw_n = as.numeric(heavy_episode_raw),
          heavy_episodic = dplyr::case_when(
            heavy_episode_raw_n %in% c(77, 88, 99) ~ NA,
            heavy_episode_raw_n >= 1 ~ TRUE,
            heavy_episode_raw_n == 0 ~ FALSE,
            TRUE ~ NA
          )
        ) |>
        dplyr::select(-heavy_episode_raw_n)
    } else {
      d <- d |>
        dplyr::mutate(heavy_episodic = recode_yn(heavy_episode_raw))
    }

  }

  # Alcohol status categories (ever / 12m / current)
  if (has("alcohol_ever")) {
    d <- d |> dplyr::mutate(alcohol_ever = recode_yn(.data[[cols$alcohol_ever]]))
  }
  if (has("alcohol_12m")) {
    d <- d |> dplyr::mutate(alcohol_12m = recode_yn(.data[[cols$alcohol_12m]]))
  }

  # WHO STEPS denominator fix: respondents who skipped the "past 30 days"

  # question because they are lifetime abstainers (a1=No) or didn't drink
  # in the past 12 months (a2=No) should be FALSE for current_alcohol,
  # NOT NA. Otherwise svymean with na.rm=TRUE inflates the proportion
  # by excluding non-drinkers from the denominator.
  if ("current_alcohol" %in% names(d)) {
    if ("alcohol_ever" %in% names(d)) {
      d <- d |> dplyr::mutate(
        current_alcohol = dplyr::if_else(
          is.na(current_alcohol) & !is.na(alcohol_ever) & alcohol_ever == FALSE,
          FALSE, current_alcohol
        )
      )
    }
    if ("alcohol_12m" %in% names(d)) {
      d <- d |> dplyr::mutate(
        current_alcohol = dplyr::if_else(
          is.na(current_alcohol) & !is.na(alcohol_12m) & alcohol_12m == FALSE,
          FALSE, current_alcohol
        )
      )
    }
    # Also: if alcohol_ever exists and is FALSE, but current_alcohol is still NA
    # (no alcohol_12m column), set to FALSE
    message("    \u2713 Non-drinkers coded as FALSE for current_alcohol (not NA)")
  }

  # WHO STEPS standard: HED denominator = total population, not just drinkers.
  # Non-drinkers (by any upstream skip) must be coded FALSE (not NA) for heavy
  # episodic drinking.  This block runs AFTER the current_alcohol denominator
  # fix so that all non-drinker skip patterns have already been resolved.
  if ("heavy_episodic" %in% names(d)) {
    if ("current_alcohol" %in% names(d)) {
      d <- d |> dplyr::mutate(
        heavy_episodic = dplyr::if_else(
          current_alcohol == FALSE & is.na(heavy_episodic), FALSE, heavy_episodic
        )
      )
    }
    if ("alcohol_12m" %in% names(d)) {
      d <- d |> dplyr::mutate(
        heavy_episodic = dplyr::if_else(
          alcohol_12m == FALSE & is.na(heavy_episodic), FALSE, heavy_episodic
        )
      )
    }
    if ("alcohol_ever" %in% names(d)) {
      d <- d |> dplyr::mutate(
        heavy_episodic = dplyr::if_else(
          alcohol_ever == FALSE & is.na(heavy_episodic), FALSE, heavy_episodic
        )
      )
    }
    message("    \u2713 Non-drinkers coded as FALSE for heavy_episodic (total population denominator)")
  }

  # Alcohol consumption status (4-level category)
  if (all(c("alcohol_ever", "alcohol_12m") %in% names(d))) {
    d <- d |> dplyr::mutate(
      lifetime_abstainer  = dplyr::if_else(is.na(alcohol_ever), NA, !alcohol_ever),
      past_12m_abstainer  = dplyr::if_else(alcohol_ever & !dplyr::if_else(is.na(alcohol_12m), FALSE, alcohol_12m), TRUE, FALSE),
      past_12m_not_current = dplyr::if_else(
        dplyr::if_else(is.na(alcohol_12m), FALSE, alcohol_12m) &
        !dplyr::if_else(is.na(current_alcohol), FALSE, current_alcohol), TRUE, FALSE
      )
    )
  }
  # Alcohol frequency past 12 months
  if (has("alcohol_freq_12m")) {
    d <- d |> dplyr::mutate(
      alcohol_freq_12m = as.numeric(.data[[cols$alcohol_freq_12m]])
    )
  }
  # Drinking occasions and drinks per occasion (continuous means)
  if (has("alcohol_occasions")) {
    d <- d |> dplyr::mutate(
      drinking_occasions_30d = as.numeric(.data[[cols$alcohol_occasions]])
    )
  }
  if (has("alcohol_drinks_occasion")) {
    d <- d |> dplyr::mutate(
      drinks_per_occasion = as.numeric(.data[[cols$alcohol_drinks_occasion]])
    )
  }
  # Drinking level categories (from High_End_Drinking or derived)
  if (has("drinking_level")) {
    d <- d |> dplyr::mutate(
      drinking_level_raw = as.numeric(.data[[cols$drinking_level]])
    )
  }

  # -- Physical activity (MET-based) -----------------------------------------

  # WHO STEPS special code cleaning for GPAQ:
  # 77 = don't know, 88 = refused → recode to NA before any computation.
  # Days should be 0-7, hours 0-23, minutes 0-59.
  # Also: if respondent said "No" to the activity (e.g. p1=2 for vig work),
  # the skip pattern means days/hours/mins should be 0 (not NA).
  .clean_gpaq_val <- function(x, max_valid = NULL) {
    x <- as.numeric(x)
    x[x %in% c(77, 88, 99)] <- NA
    if (!is.null(max_valid)) x[!is.na(x) & x > max_valid] <- NA
    x
  }

  # Clean GPAQ days (max 7), hours (max 23), minutes (max 59)
  gpaq_days_cols <- c("pa_work_vig_days", "pa_work_mod_days", "pa_transport_days",
                      "pa_rec_vig_days", "pa_rec_mod_days")
  gpaq_hrs_cols  <- c("pa_work_vig_hrs", "pa_work_mod_hrs", "pa_transport_hrs",
                      "pa_rec_vig_hrs", "pa_rec_mod_hrs")
  gpaq_min_cols  <- c("pa_work_vig_min", "pa_work_mod_min", "pa_transport_min",
                      "pa_rec_vig_min", "pa_rec_mod_min")

  # Pre-clean: recode 77/88 to NA and cap at valid ranges
  for (col_key in gpaq_days_cols) {
    if (has(col_key)) {
      d[[cols[[col_key]]]] <- .clean_gpaq_val(d[[cols[[col_key]]]], max_valid = 7)
    }
  }
  for (col_key in gpaq_hrs_cols) {
    if (has(col_key)) {
      d[[cols[[col_key]]]] <- .clean_gpaq_val(d[[cols[[col_key]]]], max_valid = 23)
    }
  }
  for (col_key in gpaq_min_cols) {
    if (has(col_key)) {
      d[[cols[[col_key]]]] <- .clean_gpaq_val(d[[cols[[col_key]]]], max_valid = 59)
    }
  }
  # Also clean sedentary hours/minutes
  if (has("pa_sedentary_hrs")) d[[cols$pa_sedentary_hrs]] <- .clean_gpaq_val(d[[cols$pa_sedentary_hrs]], max_valid = 23)
  if (has("pa_sedentary_min")) d[[cols$pa_sedentary_min]] <- .clean_gpaq_val(d[[cols$pa_sedentary_min]], max_valid = 59)

  message("    \u2713 GPAQ special codes (77/88/99) cleaned, values capped at valid ranges")

  if (has("met_total")) {
    d <- d |>
      dplyr::rename(met_total = dplyr::all_of(cols$met_total)) |>
      dplyr::mutate(
        met_total            = as.numeric(met_total),
        insufficient_pa      = met_total < 600,    # WHO threshold: <600 MET-min/week
        low_pa               = met_total < 3000,   # Moderate activity threshold
        pa_category = dplyr::case_when(
          met_total >= 3000 ~ "High",
          met_total >= 600  ~ "Moderate",
          TRUE              ~ "Low"
        )
      )
  }

  # PA domain-specific: minutes per day for work, transport, recreation
  # WHO STEPS cleaning rules:
  #  1. When a screening question (P1/P4/P7/P10/P13) = "No", the follow-up
  #     days/time questions are skipped (NA).  The domain contribution is 0
  #     (not missing).  We must set it explicitly so these respondents are
  #     included in the MET denominator.
  #  2. Cap daily time at 16 hours (960 min) per domain.  If exceeded,
  #     the WHO GPAQ guide says ALL PA data for that person should be removed.

  # Helper: recode screening question to logical (TRUE = does activity)
  .screen_yes <- function(x) {
    v <- as.numeric(x)
    dplyr::if_else(v == 1, TRUE, dplyr::if_else(v == 2, FALSE, NA))
  }

  # Track respondents with implausible PA (>960 min/day in any domain).

  # WHO GPAQ guide: if daily time exceeds 16 hours in ANY single domain,
  # ALL physical activity data for that respondent should be set to NA.
  pa_implausible <- rep(FALSE, nrow(d))

  # Helper to compute domain min/day, respecting screening question
  .domain_min_day <- function(screen_col, days_col, hrs_col, min_col) {
    # If screening question is available and answered "No" → 0
    screen <- if (!is.null(screen_col) && screen_col %in% names(d)) .screen_yes(d[[screen_col]]) else NULL
    days <- as.numeric(d[[days_col]])
    hrs  <- if (!is.null(hrs_col) && hrs_col %in% names(d)) as.numeric(d[[hrs_col]]) else 0
    mins <- if (!is.null(min_col) && min_col %in% names(d)) as.numeric(d[[min_col]]) else 0

    daily_time <- hrs * 60 + mins  # time per session (= per day)
    result <- (days * daily_time) / 7  # average min/day over the week

    # WHO cleaning: if daily time > 960 min in this domain, flag the
    # respondent for removal from ALL PA computations (applied after
    # all domains are computed)
    pa_implausible <<- pa_implausible | (!is.na(daily_time) & daily_time > 960)

    # Screening = No → domain contribution is 0
    if (!is.null(screen)) {
      result <- dplyr::if_else(!is.na(screen) & screen == FALSE, 0, result)
    }
    result
  }

  # Work vigorous: P1 screens P2/P3
  if (has("pa_work_vig_days") && (has("pa_work_vig_hrs") || has("pa_work_vig_min"))) {
    d$pa_work_vig_min_day <- .domain_min_day(
      if (has("pa_work_vig")) cols$pa_work_vig else NULL,
      cols$pa_work_vig_days,
      if (has("pa_work_vig_hrs")) cols$pa_work_vig_hrs else NULL,
      if (has("pa_work_vig_min")) cols$pa_work_vig_min else NULL
    )
  }
  # Work moderate: P4 screens P5/P6
  if (has("pa_work_mod_days") && (has("pa_work_mod_hrs") || has("pa_work_mod_min"))) {
    d$pa_work_mod_min_day <- .domain_min_day(
      if (has("pa_work_mod")) cols$pa_work_mod else NULL,
      cols$pa_work_mod_days,
      if (has("pa_work_mod_hrs")) cols$pa_work_mod_hrs else NULL,
      if (has("pa_work_mod_min")) cols$pa_work_mod_min else NULL
    )
  }
  # Sum work domain
  work_cols <- intersect(c("pa_work_vig_min_day", "pa_work_mod_min_day"), names(d))
  if (length(work_cols) > 0) {
    d$pa_work_min_day <- rowSums(d[work_cols], na.rm = TRUE)
    d$pa_work_min_day[rowSums(!is.na(d[work_cols])) == 0] <- NA
  }

  # Transport: P7 screens P8/P9
  if (has("pa_transport_days") && (has("pa_transport_hrs") || has("pa_transport_min"))) {
    d$pa_transport_min_day <- .domain_min_day(
      if (has("pa_transport")) cols$pa_transport else NULL,
      cols$pa_transport_days,
      if (has("pa_transport_hrs")) cols$pa_transport_hrs else NULL,
      if (has("pa_transport_min")) cols$pa_transport_min else NULL
    )
  }

  # Recreation vigorous: P10 screens P11/P12
  if (has("pa_rec_vig_days") && (has("pa_rec_vig_hrs") || has("pa_rec_vig_min"))) {
    d$pa_rec_vig_min_day <- .domain_min_day(
      if (has("pa_rec_vig")) cols$pa_rec_vig else NULL,
      cols$pa_rec_vig_days,
      if (has("pa_rec_vig_hrs")) cols$pa_rec_vig_hrs else NULL,
      if (has("pa_rec_vig_min")) cols$pa_rec_vig_min else NULL
    )
  }
  # Recreation moderate: P13 screens P14/P15
  if (has("pa_rec_mod_days") && (has("pa_rec_mod_hrs") || has("pa_rec_mod_min"))) {
    d$pa_rec_mod_min_day <- .domain_min_day(
      if (has("pa_rec_mod")) cols$pa_rec_mod else NULL,
      cols$pa_rec_mod_days,
      if (has("pa_rec_mod_hrs")) cols$pa_rec_mod_hrs else NULL,
      if (has("pa_rec_mod_min")) cols$pa_rec_mod_min else NULL
    )
  }
  rec_cols <- intersect(c("pa_rec_vig_min_day", "pa_rec_mod_min_day"), names(d))
  if (length(rec_cols) > 0) {
    d$pa_recreation_min_day <- rowSums(d[rec_cols], na.rm = TRUE)
    d$pa_recreation_min_day[rowSums(!is.na(d[rec_cols])) == 0] <- NA
  }

  # Total PA minutes per day (sum of all domains)
  all_pa_cols <- intersect(c("pa_work_min_day", "pa_transport_min_day", "pa_recreation_min_day"), names(d))
  if (length(all_pa_cols) > 0) {
    d$pa_minutes_per_day <- rowSums(d[all_pa_cols], na.rm = TRUE)
    d$pa_minutes_per_day[rowSums(!is.na(d[all_pa_cols])) == 0] <- NA
  }

  # WHO GPAQ cleaning rule 1: if ANY domain had daily time > 960 min, set ALL
  # PA data for that respondent to NA (implausible data)
  if (any(pa_implausible, na.rm = TRUE)) {
    n_implausible <- sum(pa_implausible, na.rm = TRUE)
    pa_domain_vars <- intersect(
      c("pa_work_vig_min_day", "pa_work_mod_min_day", "pa_work_min_day",
        "pa_transport_min_day",
        "pa_rec_vig_min_day", "pa_rec_mod_min_day", "pa_recreation_min_day",
        "pa_minutes_per_day"),
      names(d)
    )
    for (pv in pa_domain_vars) {
      d[[pv]][pa_implausible] <- NA
    }
    # Also invalidate pre-provided met_total and derived indicators
    if ("met_total" %in% names(d)) {
      d$met_total[pa_implausible] <- NA
      d$insufficient_pa[pa_implausible] <- NA
      if ("low_pa" %in% names(d)) d$low_pa[pa_implausible] <- NA
      if ("pa_category" %in% names(d)) d$pa_category[pa_implausible] <- NA
    }
    message(sprintf("    \u2713 %d respondent(s) with >960 min/day in a PA domain: all PA set to NA", n_implausible))
  }

  # WHO GPAQ cleaning rule 2 (P_clean): if a respondent answered "Yes" to a
  # screening question but has NA follow-up data for that domain, their PA
  # data is inconsistent. WHO excludes such respondents from ALL PA analysis.
  # Without this, rowSums(na.rm=TRUE) silently drops the invalid domain,
  # underestimating total PA and inflating insufficient_pa.
  pa_inconsistent <- rep(FALSE, nrow(d))
  screen_domain_pairs <- list(
    list(screen = "pa_work_vig",  domain = "pa_work_vig_min_day"),
    list(screen = "pa_work_mod",  domain = "pa_work_mod_min_day"),
    list(screen = "pa_transport", domain = "pa_transport_min_day"),
    list(screen = "pa_rec_vig",   domain = "pa_rec_vig_min_day"),
    list(screen = "pa_rec_mod",   domain = "pa_rec_mod_min_day")
  )
  for (pair in screen_domain_pairs) {
    sc <- pair$screen
    dm <- pair$domain
    if (has(sc) && dm %in% names(d)) {
      screen_yes <- .screen_yes(d[[cols[[sc]]]])
      # Inconsistent: screening = Yes but domain result is NA
      is_incon <- !is.na(screen_yes) & screen_yes == TRUE & is.na(d[[dm]])
      pa_inconsistent <- pa_inconsistent | is_incon
    }
  }
  if (any(pa_inconsistent, na.rm = TRUE)) {
    n_incon <- sum(pa_inconsistent, na.rm = TRUE)
    pa_all_vars <- intersect(
      c("pa_work_vig_min_day", "pa_work_mod_min_day", "pa_work_min_day",
        "pa_transport_min_day",
        "pa_rec_vig_min_day", "pa_rec_mod_min_day", "pa_recreation_min_day",
        "pa_minutes_per_day"),
      names(d)
    )
    for (pv in pa_all_vars) {
      d[[pv]][pa_inconsistent] <- NA
    }
    if ("met_total" %in% names(d)) {
      d$met_total[pa_inconsistent] <- NA
      d$insufficient_pa[pa_inconsistent] <- NA
      if ("low_pa" %in% names(d)) d$low_pa[pa_inconsistent] <- NA
      if ("pa_category" %in% names(d)) d$pa_category[pa_inconsistent] <- NA
    }
    message(sprintf("    \u2713 %d respondent(s) with inconsistent GPAQ data (screen=Yes but domain=NA): all PA set to NA", n_incon))
  }

  message("    \u2713 GPAQ screening questions used to set non-active domains to 0")

  # Compute MET-min/week from raw GPAQ domain variables if met_total not already set
  # WHO GPAQ MET formula: vigorous activities × 8 MET, moderate/transport × 4 MET
  if (!"met_total" %in% names(d)) {
    met_components <- list(
      pa_work_vig_min_day    = 8,   # vigorous work: 8 MET
      pa_work_mod_min_day    = 4,   # moderate work: 4 MET
      pa_transport_min_day   = 4,   # transport: 4 MET
      pa_rec_vig_min_day     = 8,   # vigorous recreation: 8 MET
      pa_rec_mod_min_day     = 4    # moderate recreation: 4 MET
    )
    available_met <- names(met_components)[names(met_components) %in% names(d)]
    if (length(available_met) > 0) {
      met_matrix <- do.call(cbind, lapply(available_met, function(col) {
        d[[col]] * met_components[[col]] * 7  # convert min/day to MET-min/week
      }))
      d$met_total <- rowSums(met_matrix, na.rm = TRUE)
      d$met_total[rowSums(!is.na(met_matrix)) == 0] <- NA
      d$insufficient_pa <- d$met_total < 600
      d$low_pa          <- d$met_total < 3000
      d$pa_category <- dplyr::case_when(
        d$met_total >= 3000 ~ "High",
        d$met_total >= 600  ~ "Moderate",
        TRUE                ~ "Low"
      )
      message("    \u2713 Computed met_total and insufficient_pa from GPAQ domain variables")
    }
  }

  # No vigorous activity
  if (has("pa_work_vig") && has("pa_rec_vig")) {
    d <- d |> dplyr::mutate(
      no_vigorous_pa = recode_yn(.data[[cols$pa_work_vig]]) == FALSE &
                       recode_yn(.data[[cols$pa_rec_vig]]) == FALSE
    )
  }

  # Sedentary time (P16a hours + P16b minutes)
  if (has("pa_sedentary_hrs") || has("pa_sedentary_min")) {
    hrs <- if (has("pa_sedentary_hrs")) as.numeric(d[[cols$pa_sedentary_hrs]]) else 0
    mins <- if (has("pa_sedentary_min")) as.numeric(d[[cols$pa_sedentary_min]]) else 0
    d$sedentary_min_day <- hrs * 60 + mins
  }

  # -- Diet ------------------------------------------------------------------
  # WHO STEPS diet cleaning:
  #  1. Special codes (77=don't know, 88=refused) → NA
  #  2. When days=0, servings question is skipped (NA) — contribution is 0, not missing
  #  3. Cap servings at reasonable max (WHO uses no explicit cap, but >30 is implausible)
  .clean_diet_val <- function(x) {
    x <- as.numeric(x)
    x[x %in% c(77, 88, 99)] <- NA
    x
  }

  if (has("fruit_days") & has("fruit_servings")) {
    d <- d |>
      dplyr::rename(fruit_days = dplyr::all_of(cols$fruit_days),
             fruit_serv = dplyr::all_of(cols$fruit_servings)) |>
      dplyr::mutate(
        fruit_days = .clean_diet_val(fruit_days),
        fruit_serv = .clean_diet_val(fruit_serv),
        # When days=0, servings is skipped (NA) — avg is 0, not missing
        fruit_serv = dplyr::if_else(fruit_days == 0 & is.na(fruit_serv), 0, fruit_serv),
        avg_fruit_servings = (fruit_days * fruit_serv) / 7
      )
  }
  if (has("veg_days") & has("veg_servings")) {
    d <- d |>
      dplyr::rename(veg_days = dplyr::all_of(cols$veg_days),
             veg_serv = dplyr::all_of(cols$veg_servings)) |>
      dplyr::mutate(
        veg_days = .clean_diet_val(veg_days),
        veg_serv = .clean_diet_val(veg_serv),
        veg_serv = dplyr::if_else(veg_days == 0 & is.na(veg_serv), 0, veg_serv),
        avg_veg_servings = (veg_days * veg_serv) / 7
      )
  }
  message("    \u2713 Diet special codes (77/88) cleaned; zero-days \u2192 0 servings")

  # Combined fruit+veg servings per day and <5 threshold
  if (all(c("avg_fruit_servings", "avg_veg_servings") %in% names(d))) {
    d <- d |>
      dplyr::mutate(
        avg_fruit_veg_servings = avg_fruit_servings + avg_veg_servings,
        low_fruit_veg = avg_fruit_veg_servings < 5
      )
  }

  # Salt-related variables
  if (has("salt_table")) {
    d <- d |> dplyr::mutate(
      always_add_salt = as.numeric(.data[[cols$salt_table]]) %in% c(1, 2)  # 1=Always, 2=Often
    )
  }
  if (has("processed_salt")) {
    d <- d |> dplyr::mutate(
      always_processed_salt = as.numeric(.data[[cols$processed_salt]]) %in% c(1, 2)
    )
  }
  if (has("salt_perception")) {
    d <- d |> dplyr::mutate(
      perceived_excess_salt = as.numeric(.data[[cols$salt_perception]]) %in% c(1, 2)  # Far too much / Too much
    )
  }

  # -- Anthropometry ---------------------------------------------------------
  if (has("height") & has("weight")) {
    d <- d |>
      dplyr::rename(height_cm = dplyr::all_of(cols$height),
             weight_kg = dplyr::all_of(cols$weight)) |>
      dplyr::mutate(
        height_cm = as.numeric(height_cm),
        weight_kg = as.numeric(weight_kg),
        # Plausibility checks
        height_cm = dplyr::if_else(height_cm < 100 | height_cm > 250, NA_real_, height_cm),
        weight_kg = dplyr::if_else(weight_kg < 20  | weight_kg > 300, NA_real_, weight_kg),
        bmi       = weight_kg / (height_cm / 100)^2,
        bmi_category = dplyr::case_when(
          bmi < 18.5              ~ "Underweight",
          bmi < bmi_overweight    ~ "Normal",
          bmi < bmi_obese         ~ "Overweight",
          bmi >= bmi_obese        ~ "Obese",
          TRUE                    ~ NA_character_
        ),
        overweight_obese = bmi >= bmi_overweight,
        obese            = bmi >= bmi_obese
      )
  }

  if (has("waist")) {
    d <- d |>
      dplyr::rename(waist_cm = dplyr::all_of(cols$waist)) |>
      dplyr::mutate(
        waist_cm = as.numeric(waist_cm),
        waist_cm = dplyr::if_else(waist_cm < 40 | waist_cm > 200, NA_real_, waist_cm),
        # WHO central obesity thresholds
        central_obesity = dplyr::case_when(
          sex == "Male"   & waist_cm >= 102 ~ TRUE,
          sex == "Female" & waist_cm >= 88  ~ TRUE,
          TRUE ~ FALSE
        )
      )
  }

  if (has("hip")) {
    d <- d |>
      dplyr::rename(hip_cm = dplyr::all_of(cols$hip)) |>
      dplyr::mutate(
        hip_cm = as.numeric(hip_cm),
        hip_cm = dplyr::if_else(hip_cm < 50 | hip_cm > 200, NA_real_, hip_cm)
      )
  }
  # Waist-to-hip ratio (if both available)
  if (all(c("waist_cm", "hip_cm") %in% names(d))) {
    d <- d |> dplyr::mutate(waist_hip_ratio = waist_cm / hip_cm)
  }

  # -- Blood pressure --------------------------------------------------------
  # Average last 2 of 3 readings (WHO STEPS protocol)
  bp_cols_sbp <- c(cols$sbp1, cols$sbp2, cols$sbp3)
  bp_cols_dbp <- c(cols$dbp1, cols$dbp2, cols$dbp3)
  bp_cols_sbp <- bp_cols_sbp[!sapply(bp_cols_sbp, is.null)]
  bp_cols_dbp <- bp_cols_dbp[!sapply(bp_cols_dbp, is.null)]

  if (length(bp_cols_sbp) >= 2) {
    d <- d |>
      dplyr::rename_with(~ paste0("sbp", seq_along(bp_cols_sbp)), dplyr::all_of(unlist(bp_cols_sbp))) |>
      dplyr::rename_with(~ paste0("dbp", seq_along(bp_cols_dbp)), dplyr::all_of(unlist(bp_cols_dbp)))

    if (length(bp_cols_sbp) == 3) {
      # WHO STEPS protocol: mean of last 2 of 3 readings.
      # If reading 3 is missing, fall back to reading 2 alone.
      d <- d |> dplyr::mutate(
        sbp1 = as.numeric(sbp1), sbp2 = as.numeric(sbp2), sbp3 = as.numeric(sbp3),
        dbp1 = as.numeric(dbp1), dbp2 = as.numeric(dbp2), dbp3 = as.numeric(dbp3),
        mean_sbp = dplyr::case_when(
          !is.na(sbp2) & !is.na(sbp3) ~ (sbp2 + sbp3) / 2,
          !is.na(sbp2) ~ sbp2,
          !is.na(sbp3) ~ sbp3,
          TRUE ~ as.numeric(sbp1)
        ),
        mean_dbp = dplyr::case_when(
          !is.na(dbp2) & !is.na(dbp3) ~ (dbp2 + dbp3) / 2,
          !is.na(dbp2) ~ dbp2,
          !is.na(dbp3) ~ dbp3,
          TRUE ~ as.numeric(dbp1)
        )
      )
    } else {
      d <- d |> dplyr::mutate(
        mean_sbp = as.numeric(sbp1),
        mean_dbp = as.numeric(dbp1)
      )
    }

    # Plausibility
    d <- d |> dplyr::mutate(
      mean_sbp = dplyr::if_else(mean_sbp < 60 | mean_sbp > 300, NA_real_, mean_sbp),
      mean_dbp = dplyr::if_else(mean_dbp < 30 | mean_dbp > 200, NA_real_, mean_dbp)
    )
  }

  if (has("bp_meds")) {
    d <- d |>
      dplyr::rename(bp_meds_raw = dplyr::all_of(cols$bp_meds)) |>
      dplyr::mutate(on_bp_meds = recode_yn(bp_meds_raw))
  }

  # Ensure on_bp_meds exists (FALSE if not detected) before deriving raised_bp
  if (!"on_bp_meds" %in% names(d)) d$on_bp_meds <- FALSE

  if (all(c("mean_sbp", "mean_dbp") %in% names(d))) {
    if (bp_sbp_threshold != 140 || bp_dbp_threshold != 90) {
      message(glue::glue("    \u26a0 Using custom BP threshold: SBP>={bp_sbp_threshold} / DBP>={bp_dbp_threshold}"))
    }
    d <- d |> dplyr::mutate(
      raised_bp = (mean_sbp >= bp_sbp_threshold | mean_dbp >= bp_dbp_threshold) | (dplyr::if_else(!is.na(on_bp_meds), on_bp_meds, FALSE)),
      bp_stage = dplyr::case_when(
        mean_sbp >= 180 | mean_dbp >= 110 ~ "Stage 3",
        mean_sbp >= 160 | mean_dbp >= 100 ~ "Stage 2",
        mean_sbp >= 140 | mean_dbp >= 90  ~ "Stage 1",
        mean_sbp >= 130 | mean_dbp >= 80  ~ "Elevated",
        TRUE                               ~ "Normal"
      )
    )
  }

  # -- Blood glucose ---------------------------------------------------------
  # WHO STEPS stores glucose in mmol/L (typical range 3-20), but some

  # country datasets use mg/dL (typical range 50-400).
  # Auto-detect units: if median > 35, assume mg/dL and convert to mmol/L.
  # Threshold aligned with WHO official scripts (median_b5 > 35).
  if (has("fasting_glucose")) {
    d <- d |>
      dplyr::rename(fasting_glucose = dplyr::all_of(cols$fasting_glucose)) |>
      dplyr::mutate(fasting_glucose = as.numeric(fasting_glucose))

    med_gluc <- stats::median(d$fasting_glucose, na.rm = TRUE)
    if (!is.na(med_gluc) && med_gluc > 35) {
      message(glue::glue("  \u26a0 Fasting glucose appears to be in mg/dL (median = {round(med_gluc, 1)}). Converting to mmol/L."))
      d <- d |> dplyr::mutate(fasting_glucose = fasting_glucose / 18.018)
    }

    # Plausibility check (mmol/L scale: 1-40)
    d <- d |> dplyr::mutate(
      fasting_glucose = dplyr::if_else(fasting_glucose < 1 | fasting_glucose > 40, NA_real_, fasting_glucose)
    )
  }
  if (has("dm_meds")) {
    d <- d |>
      dplyr::rename(dm_meds_raw = dplyr::all_of(cols$dm_meds)) |>
      dplyr::mutate(on_dm_meds = recode_yn(dm_meds_raw))
  }
  # Ensure on_dm_meds exists before deriving glucose indicators
  if (!"on_dm_meds" %in% names(d)) d$on_dm_meds <- FALSE

  if ("fasting_glucose" %in% names(d)) {
    d <- d |> dplyr::mutate(
      raised_glucose = fasting_glucose >= glucose_threshold | (dplyr::if_else(!is.na(on_dm_meds), on_dm_meds, FALSE)),
      diabetes       = fasting_glucose >= glucose_threshold | (dplyr::if_else(!is.na(on_dm_meds), on_dm_meds, FALSE)),
      impaired_glucose = fasting_glucose >= glucose_impaired_threshold & fasting_glucose < glucose_threshold
    )
  }

  # -- Cholesterol -----------------------------------------------------------
  # WHO STEPS stores cholesterol in mmol/L (typical range 2-12), but some
  # country datasets use mg/dL (typical range 80-400).
  # Auto-detect units: if median > 12, assume mg/dL and convert to mmol/L.
  # Threshold aligned with WHO official scripts (median_b8 > 12).
  if (has("total_chol")) {
    d <- d |>
      dplyr::rename(total_chol = dplyr::all_of(cols$total_chol)) |>
      dplyr::mutate(total_chol = as.numeric(total_chol))

    med_chol <- stats::median(d$total_chol, na.rm = TRUE)
    if (!is.na(med_chol) && med_chol > 12) {
      message(glue::glue("  \u26a0 Total cholesterol appears to be in mg/dL (median = {round(med_chol, 1)}). Converting to mmol/L."))
      d <- d |> dplyr::mutate(total_chol = total_chol / 38.67)
    }

    # Plausibility check (mmol/L scale: 1-20)
    d <- d |> dplyr::mutate(
      total_chol     = dplyr::if_else(total_chol < 1 | total_chol > 20, NA_real_, total_chol),
      raised_chol    = total_chol >= chol_threshold
    )
  }

  if (has("hdl_chol")) {
    d <- d |>
      dplyr::rename(hdl_chol = dplyr::all_of(cols$hdl_chol)) |>
      dplyr::mutate(hdl_chol = as.numeric(hdl_chol))

    med_hdl <- stats::median(d$hdl_chol, na.rm = TRUE)
    if (!is.na(med_hdl) && med_hdl > 10) {
      message(glue::glue("  \u26a0 HDL cholesterol appears to be in mg/dL (median = {round(med_hdl, 1)}). Converting to mmol/L."))
      d <- d |> dplyr::mutate(hdl_chol = hdl_chol / 38.67)
    }

    d <- d |> dplyr::mutate(
      hdl_chol = dplyr::if_else(hdl_chol < 0.1 | hdl_chol > 10, NA_real_, hdl_chol),
      low_hdl  = dplyr::case_when(
        sex == "Male"   & hdl_chol < 1.0 ~ TRUE,
        sex == "Female" & hdl_chol < 1.3 ~ TRUE,
        TRUE ~ FALSE
      )
    )
  }

  if (has("triglycerides")) {
    d <- d |>
      dplyr::rename(triglycerides = dplyr::all_of(cols$triglycerides)) |>
      dplyr::mutate(triglycerides = as.numeric(triglycerides))

    med_trig <- stats::median(d$triglycerides, na.rm = TRUE)
    if (!is.na(med_trig) && med_trig > 10) {
      message(glue::glue("  \u26a0 Triglycerides appears to be in mg/dL (median = {round(med_trig, 1)}). Converting to mmol/L."))
      d <- d |> dplyr::mutate(triglycerides = triglycerides / 88.57)
    }

    d <- d |> dplyr::mutate(
      triglycerides = dplyr::if_else(triglycerides < 0.1 | triglycerides > 30, NA_real_, triglycerides),
      raised_trig   = triglycerides >= 1.7   # WHO/IDF threshold: >=1.7 mmol/L
    )
  }

  # -- Additional cholesterol thresholds and medication-inclusive ---------------
  if ("total_chol" %in% names(d)) {
    d <- d |> dplyr::mutate(
      raised_chol_high = total_chol >= 6.2  # Higher threshold (240 mg/dL)
    )
  }
  if (has("chol_meds")) {
    d <- d |> dplyr::mutate(on_chol_meds = recode_yn(.data[[cols$chol_meds]]))
  }
  if (!"on_chol_meds" %in% names(d)) d$on_chol_meds <- FALSE
  if ("raised_chol" %in% names(d)) {
    d <- d |> dplyr::mutate(
      raised_chol_or_meds = raised_chol | dplyr::if_else(is.na(on_chol_meds), FALSE, on_chol_meds),
      raised_chol_high_or_meds = dplyr::if_else(is.na(raised_chol_high), FALSE, raised_chol_high) |
                                 dplyr::if_else(is.na(on_chol_meds), FALSE, on_chol_meds)
    )
  }

  # -- History variables (H-codes) -------------------------------------------
  # BP history
  if (has("bp_ever_measured")) {
    d <- d |> dplyr::mutate(bp_ever_measured = recode_yn(.data[[cols$bp_ever_measured]]))
  }
  if (has("bp_diagnosed")) {
    d <- d |> dplyr::mutate(bp_diagnosed = recode_yn(.data[[cols$bp_diagnosed]]))
  }
  if (has("bp_diagnosed_12m")) {
    d <- d |> dplyr::mutate(bp_diagnosed_12m = recode_yn(.data[[cols$bp_diagnosed_12m]]))
  }
  # BP controlled: among those treated, SBP below threshold
  if (all(c("on_bp_meds", "mean_sbp", "mean_dbp") %in% names(d))) {
    d <- d |> dplyr::mutate(
      bp_controlled = on_bp_meds & mean_sbp < bp_sbp_threshold & mean_dbp < bp_dbp_threshold
    )
  }

  # Diabetes history
  if (has("glucose_ever_measured")) {
    d <- d |> dplyr::mutate(glucose_ever_measured = recode_yn(.data[[cols$glucose_ever_measured]]))
  }
  if (has("dm_diagnosed")) {
    d <- d |> dplyr::mutate(dm_diagnosed = recode_yn(.data[[cols$dm_diagnosed]]))
  }
  if (has("dm_diagnosed_12m")) {
    d <- d |> dplyr::mutate(dm_diagnosed_12m = recode_yn(.data[[cols$dm_diagnosed_12m]]))
  }
  if (has("dm_insulin")) {
    d <- d |> dplyr::mutate(on_dm_insulin = recode_yn(.data[[cols$dm_insulin]]))
  }
  # New DM: raised glucose but NOT previously diagnosed
  if (all(c("raised_glucose", "dm_diagnosed") %in% names(d))) {
    d <- d |> dplyr::mutate(
      dm_new = dplyr::if_else(is.na(raised_glucose), FALSE, raised_glucose) &
               !dplyr::if_else(is.na(dm_diagnosed), FALSE, dm_diagnosed)
    )
  }

  # Cholesterol history
  if (has("chol_ever_measured")) {
    d <- d |> dplyr::mutate(chol_ever_measured = recode_yn(.data[[cols$chol_ever_measured]]))
  }
  if (has("chol_diagnosed") && !"chol_diagnosed" %in% names(d)) {
    d <- d |> dplyr::mutate(chol_diagnosed = recode_yn(.data[[cols$chol_diagnosed]]))
  }
  if (has("chol_diagnosed_12m")) {
    d <- d |> dplyr::mutate(chol_diagnosed_12m = recode_yn(.data[[cols$chol_diagnosed_12m]]))
  }

  # CVD history
  if (has("cvd_history")) {
    d <- d |> dplyr::mutate(cvd_history = recode_yn(.data[[cols$cvd_history]]))
  }
  if (has("cvd_aspirin")) {
    d <- d |> dplyr::mutate(on_aspirin = recode_yn(.data[[cols$cvd_aspirin]]))
  }
  if (has("cvd_statins")) {
    d <- d |> dplyr::mutate(on_statins = recode_yn(.data[[cols$cvd_statins]]))
  }

  # Lifestyle advice (H20a-f)
  if (has("adv_quit_tobacco")) d$advised_quit_tobacco  <- recode_yn(d[[cols$adv_quit_tobacco]])
  if (has("adv_reduce_salt"))  d$advised_reduce_salt   <- recode_yn(d[[cols$adv_reduce_salt]])
  if (has("adv_fruit_veg"))    d$advised_fruit_veg     <- recode_yn(d[[cols$adv_fruit_veg]])
  if (has("adv_reduce_fat"))   d$advised_reduce_fat    <- recode_yn(d[[cols$adv_reduce_fat]])
  if (has("adv_more_pa"))      d$advised_more_pa       <- recode_yn(d[[cols$adv_more_pa]])
  if (has("adv_healthy_wt"))   d$advised_healthy_weight <- recode_yn(d[[cols$adv_healthy_wt]])

  # Cervical cancer screening (female only)
  if (has("cervical_screen")) {
    d <- d |> dplyr::mutate(cervical_screened = recode_yn(.data[[cols$cervical_screen]]))
  }

  # -- Heart rate --------------------------------------------------------------
  if (has("mean_hr")) {
    d <- d |> dplyr::mutate(mean_heart_rate = as.numeric(.data[[cols$mean_hr]]))
  } else {
    # Compute mean from readings (same protocol as BP: mean of last 2 of 3)
    hr_cols <- c(cols$heart_rate1, cols$heart_rate2, cols$heart_rate3)
    hr_cols <- hr_cols[!sapply(hr_cols, is.null)]
    if (length(hr_cols) >= 2) {
      for (i in seq_along(hr_cols)) {
        d[[paste0(".hr", i)]] <- as.numeric(d[[hr_cols[[i]]]])
      }
      if (length(hr_cols) == 3) {
        d <- d |> dplyr::mutate(
          mean_heart_rate = dplyr::case_when(
            !is.na(.hr2) & !is.na(.hr3) ~ (.hr2 + .hr3) / 2,
            !is.na(.hr2) ~ .hr2,
            !is.na(.hr3) ~ .hr3,
            TRUE ~ .hr1
          )
        )
      } else {
        d$mean_heart_rate <- d$.hr1
      }
      # Clean up temp columns
      d <- d |> dplyr::select(-dplyr::starts_with(".hr"))
    }
  }

  # -- BMI: additional category variables for registry tables -----------------
  if ("bmi_category" %in% names(d)) {
    d <- d |> dplyr::mutate(
      underweight    = bmi < 18.5,
      normal_weight  = bmi >= 18.5 & bmi < 25,
      overweight_only = bmi >= 25 & bmi < 30
      # obese already exists from original BMI derivation
    )
  }

  # -- Drop incomplete cases for key variables --------------------------------
  n_before <- nrow(d)
  d <- d |> dplyr::filter(!is.na(age), !is.na(sex))
  n_after  <- nrow(d)
  if (n_before > n_after)
    message(glue::glue("  \u26a0 Removed {n_before - n_after} rows with missing age or sex"))

  message(glue::glue("\u2713 Cleaning complete. Final dataset: {nrow(d)} rows x {ncol(d)} columns"))
  return(d)
}

test_that("default thresholds produce expected raised_bp definition", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  # Default: SBP >= 140 or DBP >= 90 or on meds
  raised <- clean$raised_bp
  mean_sbp <- clean$mean_sbp
  mean_dbp <- clean$mean_dbp
  # clean_steps_data() renames bp_meds to bp_meds_raw and derives on_bp_meds
  on_meds <- if ("on_bp_meds" %in% names(clean)) clean$on_bp_meds else rep(FALSE, nrow(clean))

  # Every raised_bp == TRUE should have SBP >= 140 or DBP >= 90 or on meds
  for (i in which(!is.na(raised) & raised == TRUE)) {
    high_sbp <- !is.na(mean_sbp[i]) && mean_sbp[i] >= 140
    high_dbp <- !is.na(mean_dbp[i]) && mean_dbp[i] >= 90
    has_meds <- !is.na(on_meds[i]) && on_meds[i] == TRUE
    expect_true(high_sbp || high_dbp || has_meds,
      info = paste("Row", i, "raised_bp=TRUE but SBP=", mean_sbp[i],
                   "DBP=", mean_dbp[i], "on_bp_meds=", on_meds[i]))
  }
})

test_that("custom BP thresholds change raised_bp prevalence", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)

  # Default 140/90
  clean_default <- clean_steps_data(td, cols,
    bp_sbp_threshold = 140, bp_dbp_threshold = 90)

  # Stricter 130/80
  clean_strict <- clean_steps_data(td, cols,
    bp_sbp_threshold = 130, bp_dbp_threshold = 80)

  # Looser 160/100
  clean_loose <- clean_steps_data(td, cols,
    bp_sbp_threshold = 160, bp_dbp_threshold = 100)

  prev_default <- mean(clean_default$raised_bp, na.rm = TRUE)
  prev_strict <- mean(clean_strict$raised_bp, na.rm = TRUE)
  prev_loose <- mean(clean_loose$raised_bp, na.rm = TRUE)

  # Stricter threshold should give higher prevalence
 expect_true(prev_strict >= prev_default,
    info = paste("strict:", prev_strict, "default:", prev_default))

  # Looser threshold should give lower prevalence
  expect_true(prev_loose <= prev_default,
    info = paste("loose:", prev_loose, "default:", prev_default))
})

test_that("custom BMI thresholds change overweight/obese prevalence", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)

  # Default 25/30
  clean_default <- clean_steps_data(td, cols,
    bmi_overweight = 25, bmi_obese = 30)

  # Stricter Asian thresholds 23/27.5
  clean_asian <- clean_steps_data(td, cols,
    bmi_overweight = 23, bmi_obese = 27.5)

  prev_ow_default <- mean(clean_default$overweight_obese, na.rm = TRUE)
  prev_ow_asian <- mean(clean_asian$overweight_obese, na.rm = TRUE)

  # Lower threshold should capture more people
  expect_true(prev_ow_asian >= prev_ow_default,
    info = paste("asian:", prev_ow_asian, "default:", prev_ow_default))
})

test_that("custom glucose threshold changes raised_glucose prevalence", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)

  # Default 7.0
  clean_default <- clean_steps_data(td, cols, glucose_threshold = 7.0)

  # Stricter 6.5
  clean_strict <- clean_steps_data(td, cols, glucose_threshold = 6.5)

  if ("raised_glucose" %in% names(clean_default) &&
      "raised_glucose" %in% names(clean_strict)) {
    prev_default <- mean(clean_default$raised_glucose, na.rm = TRUE)
    prev_strict <- mean(clean_strict$raised_glucose, na.rm = TRUE)

    expect_true(prev_strict >= prev_default,
      info = paste("strict:", prev_strict, "default:", prev_default))
  }
})

test_that("custom cholesterol threshold changes raised_chol prevalence", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)

  clean_default <- clean_steps_data(td, cols, chol_threshold = 5.0)
  clean_strict <- clean_steps_data(td, cols, chol_threshold = 4.5)

  if ("raised_chol" %in% names(clean_default) &&
      "raised_chol" %in% names(clean_strict)) {
    prev_default <- mean(clean_default$raised_chol, na.rm = TRUE)
    prev_strict <- mean(clean_strict$raised_chol, na.rm = TRUE)

    expect_true(prev_strict >= prev_default,
      info = paste("strict:", prev_strict, "default:", prev_default))
  }
})

test_that("steps_config stores all threshold parameters", {
  cfg <- steps_config(
    data_path = "dummy.csv",
    bp_sbp_threshold = 130,
    bp_dbp_threshold = 80,
    bmi_overweight = 23,
    bmi_obese = 27.5,
    glucose_threshold = 6.5,
    glucose_impaired_threshold = 5.6,
    chol_threshold = 4.5
  )

  expect_equal(cfg$bp_sbp_threshold, 130)
  expect_equal(cfg$bp_dbp_threshold, 80)
  expect_equal(cfg$bmi_overweight, 23)
  expect_equal(cfg$bmi_obese, 27.5)
  expect_equal(cfg$glucose_threshold, 6.5)
  expect_equal(cfg$glucose_impaired_threshold, 5.6)
  expect_equal(cfg$chol_threshold, 4.5)
})

test_that("recode_yn handles various encodings", {
  expect_true(recode_yn(1))
  expect_true(recode_yn("Yes"))
  expect_true(recode_yn("yes"))
  expect_true(recode_yn("Y"))
  expect_false(recode_yn(2))
  expect_false(recode_yn("No"))
  expect_false(recode_yn("no"))
  expect_true(is.na(recode_yn(NA)))
  expect_true(is.na(recode_yn(99)))
})

test_that("clean_steps_data produces expected columns", {
  td <- generate_test_data(n = 200, seed = 1)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols, age_min = 18, age_max = 69)

  expect_s3_class(clean, "data.frame")
  expect_true("age_group" %in% names(clean))
  expect_true("sex" %in% names(clean))
  expect_s3_class(clean$sex, "factor")
  expect_equal(levels(clean$sex), c("Male", "Female"))

  # Check derived indicators exist
  expect_true("bmi" %in% names(clean))
  expect_true("raised_bp" %in% names(clean))
  expect_true("current_tobacco" %in% names(clean))
})

test_that("compute_all_indicators returns expected structure", {
  td <- generate_test_data(n = 300, seed = 1)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  design <- setup_survey_design(clean)
  result <- compute_all_indicators(design)

  expect_type(result, "list")
  expect_true("results" %in% names(result))
  expect_true("key_indicators" %in% names(result))

  # Check domains
  expect_true(all(c("tobacco", "alcohol", "diet_pa", "anthropometry",
                     "blood_pressure", "biochemical") %in% names(result$results)))

  # Check key_indicators is a tibble
  ki <- result$key_indicators
  expect_s3_class(ki, "data.frame")
  expect_true(nrow(ki) > 0)
  expect_true(all(c("domain", "indicator", "estimate", "lower", "upper") %in% names(ki)))

  # All estimates should be percentages between 0 and 100
  expect_true(all(ki$estimate >= 0 & ki$estimate <= 100))
})

# Edge case tests for robustness

test_that("clean_steps_data handles missing optional columns gracefully", {
  # Minimal dataset: only age, sex, weight, stratum, psu
  set.seed(42)
  n <- 100
  minimal <- data.frame(
    age = sample(18:69, n, replace = TRUE),
    sex = sample(1:2, n, replace = TRUE),
    wt_final = runif(n, 0.5, 2.0),
    stratum = rep("S1", n),
    psu = sample(paste0("P", 1:5), n, replace = TRUE)
  )

  cols <- detect_steps_columns(minimal)
  clean <- clean_steps_data(minimal, cols)

  expect_s3_class(clean, "data.frame")
  expect_true(nrow(clean) > 0)
  expect_true("age_group" %in% names(clean))
  expect_s3_class(clean$sex, "factor")
})

test_that("clean_steps_data handles age filtering correctly", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)

  # Narrow age range
  clean <- clean_steps_data(td, cols, age_min = 25, age_max = 44)

  expect_true(all(clean$age >= 25))
  expect_true(all(clean$age <= 44))
  expect_true(nrow(clean) < 500)
})

test_that("clean_steps_data handles extended age range", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)

  # Wider than default
  clean_wide <- clean_steps_data(td, cols, age_min = 15, age_max = 79)
  clean_default <- clean_steps_data(td, cols, age_min = 18, age_max = 69)

  expect_true(nrow(clean_wide) >= nrow(clean_default))
})

test_that("non-drinkers are coded FALSE not NA for alcohol indicators", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  if ("current_alcohol" %in% names(clean)) {
    # Non-drinkers should be FALSE, not NA
    n_false <- sum(clean$current_alcohol == FALSE, na.rm = TRUE)
    expect_true(n_false > 0,
      info = "No non-drinkers found -- all may be NA")
  }

  if ("heavy_episodic" %in% names(clean)) {
    # Non-drinkers should be FALSE for heavy episodic too
    n_false_hed <- sum(clean$heavy_episodic == FALSE, na.rm = TRUE)
    expect_true(n_false_hed > 0,
      info = "No FALSE values in heavy_episodic -- denominator may be wrong")
  }
})

test_that("GPAQ screening No sets domain to zero not NA", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  if ("met_total" %in% names(clean)) {
    # MET should have zeros (from screening No), not all NA or all positive
    n_zero <- sum(clean$met_total == 0, na.rm = TRUE)
    n_positive <- sum(clean$met_total > 0, na.rm = TRUE)

    expect_true(n_zero > 0 || n_positive > 0,
      info = "MET total is all NA")
  }
})

test_that("diet zero-days gives zero servings not NA", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  if ("avg_fruit_veg" %in% names(clean)) {
    # Should not be all NA
    expect_true(sum(!is.na(clean$avg_fruit_veg)) > 0)

    # Zero should be possible
    n_zero <- sum(clean$avg_fruit_veg == 0, na.rm = TRUE)
    # It's OK if there happen to be none, but avg should be reasonable
    avg <- mean(clean$avg_fruit_veg, na.rm = TRUE)
    expect_true(avg >= 0 & avg < 20,
      info = paste("Average fruit/veg servings:", avg))
  }
})

test_that("BMI is computed and categorised", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  expect_true("bmi" %in% names(clean))
  expect_true("bmi_category" %in% names(clean))

  # BMI should be reasonable (15-60)
  bmi_vals <- clean$bmi[!is.na(clean$bmi)]
  expect_true(all(bmi_vals > 10 & bmi_vals < 80))

  # Categories should include expected values (character, not factor)
  cats <- unique(clean$bmi_category[!is.na(clean$bmi_category)])
  expect_true("Normal" %in% cats || "Overweight" %in% cats)
})

test_that("BP averaging uses last two of three readings", {
  td <- generate_test_data(n = 200, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  expect_true("mean_sbp" %in% names(clean))
  expect_true("mean_dbp" %in% names(clean))

  # Mean SBP should be in reasonable range
  sbp_vals <- clean$mean_sbp[!is.na(clean$mean_sbp)]
  expect_true(all(sbp_vals > 60 & sbp_vals < 300))
})

test_that("setup_survey_design works without weights", {
  set.seed(42)
  n <- 100
  no_weight <- data.frame(
    age = sample(18:69, n, replace = TRUE),
    sex = sample(1:2, n, replace = TRUE),
    stratum = rep("S1", n),
    psu = sample(paste0("P", 1:5), n, replace = TRUE)
  )

  cols <- detect_steps_columns(no_weight)
  clean <- clean_steps_data(no_weight, cols)
  designs <- setup_survey_design(clean)

  # Should still produce a valid design (with weight = 1)
  expect_s3_class(designs$step1, "survey.design")
})

test_that("full pipeline runs end-to-end on demo data", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)
  result <- compute_all_indicators(designs)
  tables <- build_steps_tables(result$results)
  plots <- build_steps_plots(result$results, result$key_indicators,
                              "Test", 2024)

  expect_true(nrow(result$key_indicators) > 0)
  expect_true(length(tables) > 0)
  expect_true(length(plots) > 0)
})

# Regression tests for the tobacco 100% prevalence bug.
# The smk_cln filter was using raw WHO 1/2 codes on demo data that uses
# 0/1 codes, causing all non-smokers to be set to NA and producing 100%
# tobacco prevalence.

test_that("demo data does NOT produce 100% tobacco prevalence", {
  td <- generate_test_data(n = 1000, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  tobacco_prev <- mean(clean$current_tobacco, na.rm = TRUE)

  # Demo data is generated with ~25% smokers, should never be 100%
  expect_true(tobacco_prev < 0.80,
    info = paste("Tobacco prevalence:", round(tobacco_prev * 100, 1), "%"))
  expect_true(tobacco_prev > 0.05,
    info = paste("Tobacco prevalence:", round(tobacco_prev * 100, 1), "%"))
})

test_that("smk_cln filter preserves non-smokers with 0/1 coding", {
  # Simulate 0/1 coding (R/demo convention)
  td <- generate_test_data(n = 500, seed = 99)
  cols <- detect_steps_columns(td)

  # Verify demo data uses 0/1 coding for t1
  t1_vals <- unique(td[[cols$tobacco_current]])
  expect_true(all(t1_vals %in% c(0, 1, NA)),
    info = paste("t1 values:", paste(sort(t1_vals), collapse = ", ")))

  clean <- clean_steps_data(td, cols)

  # Non-smokers should NOT be NA after cleaning
  n_smokers <- sum(clean$current_tobacco == TRUE, na.rm = TRUE)
  n_nonsmokers <- sum(clean$current_tobacco == FALSE, na.rm = TRUE)

  expect_true(n_nonsmokers > 0,
    info = paste("Non-smokers:", n_nonsmokers, "Smokers:", n_smokers))
})

test_that("smk_cln filter works with 1/2 WHO coding", {
  # Create data with WHO standard 1=Yes, 2=No coding
  set.seed(123)
  n <- 200
  td <- data.frame(
    age = sample(18:69, n, replace = TRUE),
    sex = sample(1:2, n, replace = TRUE),
    wt_final = runif(n, 0.5, 2.0),
    stratum = sample(paste0("S", 1:3), n, replace = TRUE),
    psu = sample(paste0("P", 1:10), n, replace = TRUE),
    t1 = sample(c(1, 2), n, replace = TRUE, prob = c(0.3, 0.7)),
    t2 = sample(c(1, 2), n, replace = TRUE),
    m4a = rnorm(n, 125, 15), m5a = rnorm(n, 125, 15),
    m4b = rnorm(n, 80, 10), m5b = rnorm(n, 80, 10),
    m11 = rnorm(n, 170, 10), m12 = rnorm(n, 75, 15)
  )

  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  tobacco_prev <- mean(clean$current_tobacco, na.rm = TRUE)

  # Should be around 30%, definitely not 100%
  expect_true(tobacco_prev < 0.80,
    info = paste("WHO 1/2 coding tobacco prevalence:",
                 round(tobacco_prev * 100, 1), "%"))
})

test_that("daily_tobacco is derived correctly", {
  td <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  # Daily tobacco users should be a subset of current tobacco users
  if ("daily_tobacco" %in% names(clean)) {
    daily <- clean$daily_tobacco
    current <- clean$current_tobacco

    # Everyone who is daily should also be current
    daily_but_not_current <- sum(daily & !current, na.rm = TRUE)
    expect_equal(daily_but_not_current, 0,
      info = "Found daily users who are not current users")
  }
})

test_that("tobacco indicators compute without error", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)

  result <- compute_tobacco_indicators(designs$step1)
  expect_type(result, "list")
  expect_true("current_tobacco_total" %in% names(result))

  # Prevalence should be a reasonable number
  prev <- result$current_tobacco_total$estimate
  expect_true(prev >= 0 & prev <= 100,
    info = paste("Tobacco prevalence:", prev))
})

#' Compute Tobacco Use Indicators
#'
#' Calculates prevalence of tobacco use from a survey design object.
#' Computes proportions of current and daily tobacco use, stratified by
#' sex and age group where available.
#'
#' When both smoking and smokeless tobacco variables are present,
#' `current_tobacco_any` (either smoking or smokeless) is preferred
#' as the headline tobacco indicator.  The function also reports
#' `current_smoker` and `current_smokeless` separately if available.
#'
#' @param design A survey design object from [setup_survey_design()].
#'
#' @return A named list of survey estimates. Each element contains proportion
#'   estimates (as tibble with columns: estimate, lower, upper, etc.) for:
#'   - `current_tobacco_any_total/by_sex/by_age`: any current tobacco use
#'     (smoking or smokeless) -- preferred headline variable
#'   - `current_tobacco_total/by_sex/by_age`: current tobacco smoking
#'   - `current_smoker_total/by_sex/by_age`: current smoker
#'   - `current_smokeless_total/by_sex/by_age`: current smokeless tobacco
#'   - `daily_tobacco_total/by_sex/by_age`: daily tobacco use
#'   (only elements for variables present in design are returned)
#'
#' @seealso [compute_all_indicators()]
#'
#' @export
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   tobacco_results <- compute_tobacco_indicators(design)
#' }
compute_tobacco_indicators <- function(design) {
  message("  Computing tobacco indicators...")
  results <- list()

  # Look for all tobacco-related variables in the design
  candidate_vars <- c(
    "current_tobacco_any",   # combined smoking + smokeless (preferred headline)
    "current_tobacco",       # tobacco smoking (may be same as current_smoker)
    "current_smoker",        # current smoker
    "current_smokeless",     # current smokeless tobacco
    "daily_smokeless",       # daily smokeless (fallback if current_smokeless absent)
    "daily_tobacco"          # daily tobacco use
  )
  vars <- candidate_vars[candidate_vars %in% names(design$variables)]

  for (v in vars) {
    f <- as.formula(paste0("~as.numeric(", v, ")"))
    results[[paste0(v, "_total")]]  <- svyprop(f, design)
    results[[paste0(v, "_by_sex")]] <- svyprop(f, design, by = ~sex)
    results[[paste0(v, "_by_age")]] <- svyprop(f, design, by = ~age_group)
  }

  return(results)
}

#' Generate simulated STEPS test data
#'
#' Creates a realistic simulated dataset matching WHO STEPS survey structure.
#' Includes sampling design variables, demographics, and measures from all
#' three steps (behavioural, physical, biochemical).
#'
#' @param n Number of observations (default 3000).
#' @param seed Random seed for reproducibility (default 42).
#'
#' @return A data frame with `n` rows and the following columns:
#'   - `stratum`: Strata identifier (S1-S5)
#'   - `psu`: Primary sampling unit (PSU1-PSU40)
#'   - `wt_final`: Final analysis weight
#'   - `sex`: Sex (1=Male, 2=Female)
#'   - `age`: Age in years (18-69)
#'   - Step 1 (behavioural): `t1`, `t2` (tobacco), `a1`, `a5` (alcohol),
#'     `met_total` (physical activity), `d1`-`d4` (diet)
#'   - Step 2 (physical): `m1` (height), `m2` (weight), `m3` (waist),
#'     `b1`-`b6` (blood pressure), `b7` (BP medication)
#'   - Step 3 (biochemical): `c1_mmol` (glucose), `c5` (DM meds),
#'     `c6` (cholesterol), `c10` (cholesterol meds)
#'
#' @details
#' Simulation parameters are realistic for low-middle income settings:
#'   - Tobacco prevalence: 32% males, 8% females
#'   - Alcohol current use: 55% males, 28% females
#'   - Heavy episodic drinking: 35% of drinkers
#'   - Physical activity: MET-minutes/week, mean 1800, SD 1200
#'   - Diet: Fruit/veg days and servings per day (0-7, 1-5)
#'   - BP increases with age; medication prevalence 12%
#'   - Glucose: mean 5.2 mmol/L, increases with age
#'   - Total cholesterol: mean 4.8 mmol/L
#'
#' Use this function for:
#'   - Testing the STEPS pipeline
#'   - Developing reports before real data arrives
#'   - Training analysts on the analysis system
#'
#' @examples
#' \donttest{
#'   # Generate smaller dataset for quick testing
#'   test_data <- generate_test_data(n = 500, seed = 123)
#'   head(test_data)
#' }
#'
#' @export
generate_test_data <- function(n = 3000, seed = 42) {
  set.seed(seed)

  # - Sampling design
  strata  <- sample(paste0("S", 1:5), n, replace = TRUE)
  psu     <- sample(paste0("PSU", 1:40), n, replace = TRUE)
  wt      <- runif(n, 0.5, 2.5)

  # - Demographics
  sex <- sample(c(1, 2), n, replace = TRUE, prob = c(0.48, 0.52))
  age <- round(runif(n, 18, 69))

  age_grp <- cut(age, breaks = c(18, 25, 35, 45, 55, 65, Inf),
                 labels = c("18-24","25-34","35-44","45-54","55-64","65+"),
                 right = FALSE)

  # - Step 1: Behaviour
  tobacco_current <- rbinom(n, 1, ifelse(sex == 1, 0.32, 0.08))
  tobacco_daily   <- tobacco_current * rbinom(n, 1, 0.75)
  alcohol_current <- rbinom(n, 1, ifelse(sex == 1, 0.55, 0.28))
  heavy_episodic  <- alcohol_current * rbinom(n, 1, 0.35)

  # Physical activity (MET-minutes/week)
  met_total <- round(abs(rnorm(n, mean = 1800, sd = 1200)))

  # Diet
  fruit_days <- sample(0:7, n, replace = TRUE)
  fruit_serv <- sample(1:5, n, replace = TRUE)
  veg_days   <- sample(0:7, n, replace = TRUE)
  veg_serv   <- sample(1:5, n, replace = TRUE)

  # - Step 2: Physical measurements
  height <- round(rnorm(n, ifelse(sex == 1, 170, 158), 8), 1)
  weight <- round(rnorm(n, ifelse(sex == 1, 78, 68), 15), 1)
  waist  <- round(rnorm(n, ifelse(sex == 1, 90, 84), 14), 1)

  # Blood pressure - increases with age
  age_effect <- (age - 18) / 51
  sbp1 <- round(rnorm(n, 115 + 30 * age_effect, 18))
  sbp2 <- sbp1 + round(rnorm(n, -2, 4))
  sbp3 <- sbp1 + round(rnorm(n, -3, 4))
  dbp1 <- round(rnorm(n, 72 + 15 * age_effect, 12))
  dbp2 <- dbp1 + round(rnorm(n, -1, 3))
  dbp3 <- dbp1 + round(rnorm(n, -2, 3))
  bp_meds <- rbinom(n, 1, 0.12)

  # - Step 3: Biochemical
  fasting_glucose <- round(abs(rnorm(n, 5.2 + 0.8 * age_effect, 1.4)), 1)
  dm_meds         <- rbinom(n, 1, 0.06)
  total_chol      <- round(abs(rnorm(n, 4.8, 1.1)), 1)
  chol_meds       <- rbinom(n, 1, 0.04)

  # - Assemble data frame
  test_data <- data.frame(
    # Sampling
    stratum = strata, psu = psu, wt_final = wt,
    # Demographics
    sex = sex, age = age,
    # Step 1
    t1 = tobacco_current, t2 = tobacco_daily,
    a1 = alcohol_current, a5 = heavy_episodic,
    met_total = met_total,
    d1 = fruit_days, d2 = fruit_serv, d3 = veg_days, d4 = veg_serv,
    # Step 2
    m1 = height, m2 = weight, m3 = waist,
    b1 = sbp1, b2 = dbp1, b3 = sbp2, b4 = dbp2, b5 = sbp3, b6 = dbp3,
    b7 = bp_meds,
    # Step 3
    c1_mmol = fasting_glucose, c5 = dm_meds,
    c6 = total_chol, c10 = chol_meds
  )

  message(glue::glue("\u2713 Generated test data: {nrow(test_data)} rows \u00d7 {ncol(test_data)} columns"))

  return(test_data)
}

#' Compute Diet and Physical Activity Indicators
#'
#' Calculates prevalence of insufficient physical activity and low fruit &
#' vegetable intake, plus mean metabolic equivalent (MET) values, from a
#' survey design object.
#'
#' @param design A survey design object from [setup_survey_design()].
#'
#' @return A named list of survey estimates. Each element contains estimates
#'   (as tibble with columns: estimate, lower, upper, etc.) for:
#'   - `insufficient_pa_total`: insufficient physical activity, overall
#'   - `insufficient_pa_by_sex`: insufficient physical activity, by sex
#'   - `insufficient_pa_by_age`: insufficient physical activity, by age group
#'   - `low_fruit_veg_total`: low fruit & vegetable intake, overall
#'   - `low_fruit_veg_by_sex`: low fruit & vegetable intake, by sex
#'   - `low_fruit_veg_by_age`: low fruit & vegetable intake, by age group
#'   - `met_mean_total`: mean MET (if available)
#'   - `met_mean_by_sex`: mean MET by sex (if available)
#'   (if the corresponding variables exist in design)
#'
#' @seealso [compute_all_indicators()]
#'
#' @export
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   diet_pa_results <- compute_diet_pa_indicators(design)
#' }
compute_diet_pa_indicators <- function(design) {
  message("  Computing diet & physical activity indicators...")
  results <- list()

  bin_vars <- c("insufficient_pa", "low_fruit_veg")
  bin_vars <- bin_vars[bin_vars %in% names(design$variables)]
  for (v in bin_vars) {
    f <- as.formula(paste0("~as.numeric(", v, ")"))
    results[[paste0(v, "_total")]]  <- svyprop(f, design)
    results[[paste0(v, "_by_sex")]] <- svyprop(f, design, by = ~sex)
    results[[paste0(v, "_by_age")]] <- svyprop(f, design, by = ~age_group)
  }

  if ("met_total" %in% names(design$variables)) {
    results$met_mean_total  <- svymn(~met_total, design)
    results$met_mean_by_sex <- svymn(~met_total, design, by = ~sex)
  }

  return(results)
}

#' Build publication-ready STEPS visualizations
#'
#' Generates a list of ggplot2 plots showing key NCD risk factor prevalence
#' with 95% confidence intervals, stratified by sex and age group.
#'
#' @param indicators A list of indicator results from [compute_all_indicators()].
#' @param key_indicators A data frame with key indicators (domain, indicator, estimate, lower, upper).
#' @param country_name Country name for plot titles.
#' @param survey_year Survey year for plot titles.
#'
#' @return A named list of ggplot2 objects:
#'   - `overview`: Horizontal bar chart of key indicators
#'   - `tobacco_by_sex`: Sex-stratified tobacco use
#'   - `bp_by_sex`: Sex-stratified blood pressure
#'   - `obesity_by_sex`: Sex-stratified overweight/obesity
#'   - `glucose_by_sex`: Sex-stratified blood glucose
#'   - `bp_by_age`: Age-stratified blood pressure with ribbon CI
#'   - `obesity_by_age`: Age-stratified overweight/obesity with ribbon CI
#'   - `sex_dashboard`: Combined 2x2 dashboard of sex-stratified charts (if >=2 sex plots available)
#'   NULL entries are preserved in the list.
#'
#' @details
#' All plots use the WHO STEPS colour scheme and professional styling.
#' Error bars represent 95% confidence intervals.
#' Prevalence values are displayed on bars/points with light background text.
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   all_ind <- compute_all_indicators(design)
#'   plots <- build_steps_plots(all_ind$results, all_ind$key_indicators, "Test", 2023)
#'   names(plots)
#' }
#'
#' @export
build_steps_plots <- function(indicators, key_indicators, country_name, survey_year) {
  plots <- list()

  # Ensure theme functions are available
  theme_steps <- theme_steps()
  steps_colors <- steps_colors()
  config <- list(country_name = country_name, survey_year = survey_year)

  # - 1. Key indicators overview (horizontal bar chart)
  # Wrap long indicator labels so they fit in smaller plot areas
  overview_df <- key_indicators |>
    dplyr::arrange(estimate) |>
    dplyr::mutate(
      indicator = vapply(indicator, function(x) paste(strwrap(x, width = 30), collapse = "\n"), character(1)),
      indicator = factor(indicator, levels = indicator)
    )

  # Position labels beyond the CI upper bound so they never overlap
  overview_df$label_x <- pmax(overview_df$estimate, overview_df$upper) + 1.5
  x_max <- max(overview_df$upper, na.rm = TRUE) + 12

  plots$overview <- ggplot2::ggplot(overview_df,
      ggplot2::aes(x = estimate, y = indicator, fill = domain)) +
    ggplot2::geom_col(width = 0.6, alpha = 0.9) +
    ggplot2::geom_errorbar(ggplot2::aes(xmin = lower, xmax = upper), height = 0.25, color = "grey40", orientation = "y") +
    ggplot2::geom_text(ggplot2::aes(x = label_x, label = paste0(round(estimate, 1), "%")),
              hjust = 0, size = 3.2, color = "grey20") +
    ggplot2::scale_x_continuous(limits = c(0, x_max), labels = function(x) paste0(x, "%")) +
    ggplot2::scale_fill_brewer(palette = "Set2") +
    ggplot2::labs(
      title    = glue::glue("NCD Risk Factor Prevalence \u2014 {config$country_name} {config$survey_year}"),
      subtitle = "Weighted estimates with 95% confidence intervals",
      x        = "Prevalence (%)",
      y        = NULL,
      caption  = "Source: WHO STEPS Survey"
    ) +
    theme_steps +
    ggplot2::theme(
      legend.position = "right",
      plot.margin = ggplot2::margin(5, 15, 5, 5)
    )

  # - 2. By-sex comparison (grouped bar chart)
  # Prefer current_tobacco_any (smoking + smokeless) if available
  tob_by_sex <- indicators$tobacco[["current_tobacco_any_by_sex"]]
  if (is.null(tob_by_sex)) tob_by_sex <- indicators$tobacco[["current_tobacco_by_sex"]]
  plots$tobacco_by_sex <- make_sex_chart(
    tob_by_sex,
    "Current tobacco use by sex", "current_tobacco",
    steps_colors, theme_steps
  )
  plots$bp_by_sex <- make_sex_chart(
    indicators$blood_pressure$raised_bp_by_sex,
    "Raised blood pressure by sex", "raised_bp",
    steps_colors, theme_steps
  )
  plots$obesity_by_sex <- make_sex_chart(
    indicators$anthropometry$overweight_obese_by_sex,
    "Overweight or obese (BMI >=25) by sex", "overweight_obese",
    steps_colors, theme_steps
  )
  plots$glucose_by_sex <- make_sex_chart(
    indicators$biochemical$raised_glucose_by_sex,
    "Raised fasting blood glucose by sex", "raised_glucose",
    steps_colors, theme_steps
  )

  # - 3. By-age trend chart
  plots$bp_by_age <- make_age_chart(
    indicators$blood_pressure$raised_bp_by_age,
    "Raised BP by age group",
    steps_colors, theme_steps
  )
  plots$obesity_by_age <- make_age_chart(
    indicators$anthropometry$overweight_obese_by_age,
    "Overweight/Obese by age group",
    steps_colors, theme_steps
  )

  # - 4. Forest plot
  plots$forest <- build_forest_plot(key_indicators, config$country_name, config$survey_year)

  # - 5. Radar chart
  plots$radar <- build_radar_plot(key_indicators, config$country_name, config$survey_year)

  # - 6. Combined dashboard (patchwork)
  sex_plots <- Filter(Negate(is.null), list(
    plots$tobacco_by_sex, plots$bp_by_sex,
    plots$obesity_by_sex, plots$glucose_by_sex
  ))

  if (length(sex_plots) >= 2) {
    plots$sex_dashboard <- patchwork::wrap_plots(sex_plots, ncol = 2) +
      patchwork::plot_annotation(
        title   = glue::glue("{config$country_name} {config$survey_year} - NCD Risk Factors by Sex"),
        caption = "Source: WHO STEPS Survey",
        theme   = ggplot2::theme(plot.title = ggplot2::element_text(face = "bold", color = steps_colors$dark_blue))
      )
  }

  return(plots)
}

#' Create sex-stratified bar chart
#'
#' @param by_sex_df Data frame with `sex`, `estimate`, `lower`, `upper` columns.
#' @param title Plot title.
#' @param var_name Variable name (for internal use).
#' @param steps_colors Colour list from [steps_colors()].
#' @param theme_steps Theme from [theme_steps()].
#'
#' @return A ggplot2 object or NULL if input is NULL.
#'
#' @keywords internal
make_sex_chart <- function(by_sex_df, title, var_name, steps_colors, theme_steps) {
  if (is.null(by_sex_df)) return(NULL)

  # Position labels above the CI upper bound to avoid overlap
  by_sex_df$label_y <- pmax(by_sex_df$estimate, by_sex_df$upper) + 0.5

  ggplot2::ggplot(by_sex_df, ggplot2::aes(x = sex, y = estimate, fill = sex)) +
    ggplot2::geom_col(width = 0.5, alpha = 0.9) +
    ggplot2::geom_errorbar(ggplot2::aes(ymin = lower, ymax = upper), width = 0.15) +
    ggplot2::geom_text(ggplot2::aes(y = label_y, label = paste0(round(estimate, 1), "%")),
              vjust = -0.3, size = 3.5, fontface = "bold") +
    ggplot2::scale_fill_manual(values = c(Male = steps_colors$male, Female = steps_colors$female)) +
    ggplot2::scale_y_continuous(labels = function(x) paste0(x, "%"), expand = ggplot2::expansion(mult = c(0, 0.18))) +
    ggplot2::labs(title = title, x = NULL, y = "Prevalence (%)") +
    theme_steps +
    ggplot2::theme(legend.position = "none")
}

#' Create age-stratified trend chart
#'
#' @param by_age_df Data frame with `age_group`, `estimate`, `lower`, `upper` columns.
#' @param title Plot title.
#' @param steps_colors Colour list from [steps_colors()].
#' @param theme_steps Theme from [theme_steps()].
#'
#' @return A ggplot2 object or NULL if input is NULL.
#'
#' @keywords internal
make_age_chart <- function(by_age_df, title, steps_colors, theme_steps) {
  if (is.null(by_age_df)) return(NULL)

  ggplot2::ggplot(by_age_df, ggplot2::aes(x = age_group, y = estimate, group = 1)) +
    ggplot2::geom_ribbon(ggplot2::aes(ymin = lower, ymax = upper), fill = steps_colors$blue, alpha = 0.15) +
    ggplot2::geom_line(color = steps_colors$blue, linewidth = 1.2) +
    ggplot2::geom_point(color = steps_colors$dark_blue, size = 3, fill = "white", shape = 21, stroke = 2) +
    ggplot2::geom_text(ggplot2::aes(label = paste0(round(estimate, 1), "%")),
              vjust = -1, size = 3, color = steps_colors$dark_blue) +
    ggplot2::scale_y_continuous(labels = function(x) paste0(x, "%"), expand = ggplot2::expansion(mult = c(0, 0.2))) +
    ggplot2::labs(title = title, x = "Age group (years)", y = "Prevalence (%)") +
    theme_steps
}

#' Build forest plot of key indicators with 95% CIs
#'
#' Creates a horizontal point-and-CI plot (forest plot style) for all
#' key indicators, grouped by domain.
#'
#' @param key_indicators A data frame with domain, indicator, estimate, lower, upper.
#' @param country_name Country name for title.
#' @param survey_year Survey year for title.
#'
#' @return A ggplot2 object.
#' @export
build_forest_plot <- function(key_indicators, country_name, survey_year) {
  theme_steps <- theme_steps()
  steps_colors <- steps_colors()

  df <- key_indicators |>
    dplyr::mutate(
      indicator = vapply(indicator, function(x) paste(strwrap(x, width = 35), collapse = "\n"), character(1)),
      indicator = factor(indicator, levels = rev(indicator))
    )

  # Position labels beyond the CI upper bound to avoid overlap
  df$label_x <- pmax(df$estimate, df$upper) + 1.5
  x_max <- max(df$upper, na.rm = TRUE) + 12

  ggplot2::ggplot(df, ggplot2::aes(x = estimate, y = indicator, colour = domain)) +
    ggplot2::geom_vline(xintercept = c(10, 20, 30, 40, 50), linetype = "dotted",
                        colour = "grey80", linewidth = 0.3) +
    ggplot2::geom_point(size = 3.5) +
    ggplot2::geom_errorbar(ggplot2::aes(xmin = lower, xmax = upper),
                            height = 0.3, linewidth = 0.7, orientation = "y") +
    ggplot2::geom_text(ggplot2::aes(x = label_x, label = sprintf("%.1f%%", estimate)),
                       hjust = 0, size = 3, colour = "grey30") +
    ggplot2::scale_x_continuous(limits = c(0, x_max),
                                labels = function(x) paste0(x, "%"),
                                expand = ggplot2::expansion(mult = c(0, 0.02))) +
    ggplot2::scale_colour_brewer(palette = "Dark2") +
    ggplot2::labs(
      title    = glue::glue("NCD Risk Factor Prevalence \u2014 {country_name} {survey_year}"),
      subtitle = "Point estimates with 95% confidence intervals",
      x        = "Prevalence (%)",
      y        = NULL,
      colour   = "Domain",
      caption  = "Source: WHO STEPS Survey"
    ) +
    theme_steps +
    ggplot2::theme(
      panel.grid.major.y = ggplot2::element_blank(),
      legend.position = "bottom"
    )
}


#' Build radar / spider chart of NCD risk factor profile
#'
#' Creates a radar-style chart showing prevalence of key risk factors
#' on a polar coordinate system for quick visual comparison.
#'
#' @param key_indicators A data frame with domain, indicator, estimate.
#' @param country_name Country name for title.
#' @param survey_year Survey year for title.
#'
#' @return A ggplot2 object.
#' @export
build_radar_plot <- function(key_indicators, country_name, survey_year) {
  theme_steps <- theme_steps()
  steps_colors <- steps_colors()

  # Use short labels for radar
  short_labels <- c(
    "Current tobacco use" = "Tobacco",
    "Current alcohol use" = "Alcohol",
    "Heavy episodic drinking" = "HED",
    "Insufficient physical activity" = "Low PA",
    "Low fruit & vegetable intake (<5 servings/day)" = "Low F&V",
    "Overweight or obese (BMI >=25)" = "OW/Obese",
    "Obese (BMI >=30)" = "Obese",
    "Raised blood pressure" = "Raised BP",
    "Raised fasting blood glucose" = "Raised glucose",
    "Raised total cholesterol" = "Raised chol"
  )

  df <- key_indicators |>
    dplyr::mutate(
      short_label = dplyr::if_else(
        indicator %in% names(short_labels),
        short_labels[indicator],
        substr(indicator, 1, 15)
      )
    )

  # Separate extreme outliers (>60%) to prevent radar distortion
  outliers <- df |> dplyr::filter(estimate > 60)
  df_plot <- df |> dplyr::filter(estimate <= 60)

  if (nrow(df_plot) < 3) {
    # Not enough indicators for a meaningful radar; fall back to all
    df_plot <- df
    outliers <- df[0, ]
  }

  df_plot <- df_plot |>
    dplyr::mutate(short_label = factor(short_label, levels = short_label))

  # Dynamic y-axis: round up to nearest 10 above max
  y_max <- ceiling(max(df_plot$estimate, na.rm = TRUE) / 10) * 10
  y_breaks <- seq(0, y_max, by = ifelse(y_max > 30, 10, 5))

  p <- ggplot2::ggplot(df_plot, ggplot2::aes(x = short_label, y = estimate, fill = domain)) +
    ggplot2::geom_col(width = 0.8, alpha = 0.75, colour = "white", linewidth = 0.3) +
    ggplot2::geom_text(ggplot2::aes(label = sprintf("%.0f%%", estimate)),
                       size = 3, fontface = "bold",
                       position = ggplot2::position_stack(vjust = 0.5)) +
    ggplot2::coord_polar(start = 0) +
    ggplot2::scale_y_continuous(limits = c(0, y_max), breaks = y_breaks) +
    ggplot2::scale_fill_brewer(palette = "Set2") +
    ggplot2::labs(
      fill  = "Domain",
      x = NULL, y = NULL
    ) +
    theme_steps +
    ggplot2::theme(
      axis.text.y = ggplot2::element_blank(),
      axis.ticks.y = ggplot2::element_blank(),
      panel.grid.major.y = ggplot2::element_line(colour = "grey90"),
      legend.position = "bottom"
    )

  # Build title with outlier callout if needed
  if (nrow(outliers) > 0) {
    callout_parts <- vapply(seq_len(nrow(outliers)), function(i) {
      sprintf("%s: %.0f%%", outliers$short_label[i], outliers$estimate[i])
    }, character(1))
    callout <- paste(callout_parts, collapse = " | ")

    p <- p + ggplot2::labs(
      title    = glue::glue("{country_name} {survey_year} \u2014 Risk Factor Profile"),
      subtitle = callout
    )
  } else {
    p <- p + ggplot2::labs(
      title = glue::glue("{country_name} {survey_year} \u2014 Risk Factor Profile")
    )
  }

  p
}


#' Save STEPS plots to PNG files
#'
#' Exports all plots in a list to PNG files in the specified directory.
#'
#' @param plots A named list of ggplot2 objects (from [build_steps_plots()]).
#' @param output_dir Output directory path (default \code{tempdir()}).
#'
#' @return NULL (invisibly). Prints messages about saved files.
#'
#' @details
#' Files are named:
#'   - `01_overview_indicators.png` (12x8 in)
#'   - `02_by_sex_dashboard.png` (12x8 in)
#'   - `03_bp_by_age.png` (10x6 in)
#'   - `04_obesity_by_age.png` (10x6 in)
#'
#' All saved at 150 dpi with white background.
#'
#' @export
save_steps_plots <- function(plots, output_dir = tempdir()) {
  dir.create(output_dir, showWarnings = FALSE, recursive = TRUE)

  save_plot <- function(plot, name, width = 10, height = 6) {
    if (!is.null(plot)) {
      ggplot2::ggsave(
        file.path(output_dir, paste0(name, ".png")),
        plot = plot, width = width, height = height, dpi = 150, bg = "white"
      )
    }
  }

  save_plot(plots$overview,       "01_overview_indicators", width = 12, height = 8)
  save_plot(plots$sex_dashboard,  "02_by_sex_dashboard",    width = 12, height = 8)
  save_plot(plots$bp_by_age,      "03_bp_by_age")
  save_plot(plots$obesity_by_age, "04_obesity_by_age")

  n_saved <- sum(!sapply(plots, is.null))
  message(glue::glue("\u2713 Saved {n_saved} plots to {output_dir}/"))

  invisible(NULL)
}

test_that("build_forest_plot returns a ggplot", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)
  result <- compute_all_indicators(designs)

  p <- build_forest_plot(result$key_indicators, "Testland", 2024)
  expect_s3_class(p, "ggplot")
})

test_that("build_radar_plot returns a ggplot", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)
  result <- compute_all_indicators(designs)

  p <- build_radar_plot(result$key_indicators, "Testland", 2024)
  expect_s3_class(p, "ggplot")
})

test_that("build_steps_plots includes forest and radar", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)
  result <- compute_all_indicators(designs)

  plots <- build_steps_plots(
    result$results, result$key_indicators, "Testland", 2024
  )

  expect_type(plots, "list")
  expect_true("forest" %in% names(plots))
  expect_true("radar" %in% names(plots))
  expect_s3_class(plots$forest, "ggplot")
  expect_s3_class(plots$radar, "ggplot")
})

test_that("build_steps_plots produces overview plot", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)
  result <- compute_all_indicators(designs)

  plots <- build_steps_plots(
    result$results, result$key_indicators, "Testland", 2024
  )

  expect_true("overview" %in% names(plots))
  expect_s3_class(plots$overview, "ggplot")
})

test_that("theme_steps returns a ggplot theme", {
  th <- theme_steps()
  expect_s3_class(th, "theme")
})

test_that("steps_colors returns named color list", {
  pal <- steps_colors()
  expect_type(pal, "list")
  expect_true(all(c("blue", "male", "female") %in% names(pal)))
})

#' Generic Compute Engine for WHO STEPS Tables
#'
#' Takes a table specification from [steps_table_registry()] and a survey
#' design object, and produces the survey-weighted estimates needed to fill
#' the standard WHO STEPS data book table.
#'
#' @name compute_engine
#' @family compute
NULL


#' Compute a single table from a registry entry
#'
#' This is the main workhorse: given one registry entry and a survey design,
#' it dispatches to the appropriate method based on `entry$type` and returns
#' a standardised result list.
#'
#' @param entry A single list element from [steps_table_registry()].
#' @param design A survey design object (from [survey::svydesign()]).
#' @param data The cleaned data frame (used for variable availability checks).
#'
#' @return A list with:
#'   \describe{
#'     \item{id}{Table identifier.}
#'     \item{title}{Table title.}
#'     \item{type}{Table type.}
#'     \item{available}{Logical: TRUE if the required variable(s) exist.}
#'     \item{results}{A list of data frames:
#'       For proportion: total, by_sex, by_age (each with estimate, lower, upper).
#'       For mean: total, by_sex, by_age (each with estimate, lower, upper).
#'       For category: total, by_sex, by_age (each with level, estimate, lower, upper).
#'       For cascade: named list of proportion results.
#'     }
#'   }
#'
#' @export
compute_table <- function(entry, design, data = NULL) {
  if (is.null(data)) data <- design$variables

  result <- list(
    id        = entry$id,
    title     = entry$title,
    section   = entry$section,
    type      = entry$type,
    step      = entry$step,
    unit      = entry$unit,
    available = FALSE,
    results   = NULL
  )

  # Dispatch by type
  res <- tryCatch(
    switch(entry$type,
      "proportion" = .compute_proportion(entry, design, data),
      "mean"       = .compute_mean(entry, design, data),
      "category"   = .compute_category(entry, design, data),
      "cascade"    = .compute_cascade(entry, design, data),
      "combined"   = .compute_combined(entry, design, data),
      stop("Unknown table type: ", entry$type)
    ),
    error = function(e) {
      message(glue::glue("  \u26a0 {entry$id}: {e$message}"))
      NULL
    }
  )

  if (!is.null(res)) {
    result$available <- TRUE
    result$results   <- res
  }

  result
}


#' Compute all tables from the registry
#'
#' Iterates through the full [steps_table_registry()] and computes every
#' table that has available data. Returns a named list of results.
#'
#' @param designs A list of survey designs, with elements `step1`, `step2`,
#'   `step3` (as returned by [setup_survey_design()]).
#' @param data The cleaned data frame.
#'
#' @return A named list of table results (from [compute_table()]).
#'   Only entries with `available == TRUE` are included.
#'
#' @export
compute_all_tables <- function(designs, data = NULL) {
  registry <- steps_table_registry()
  results  <- list()

  n <- length(registry)
  message(glue::glue("Computing {n} tables from WHO STEPS registry..."))

  for (i in seq_along(registry)) {
    entry <- registry[[i]]

    # Select the appropriate design for this step
    design <- switch(as.character(entry$step),
      "1" = designs$step1,
      "2" = designs$step2,
      "3" = designs$step3,
      designs$step1  # fallback
    )

    if (is.null(design)) {
      message(glue::glue("  \u26a0 {entry$id}: No Step {entry$step} design available, skipping."))
      next
    }

    res <- compute_table(entry, design, data)
    results[[entry$id]] <- res

    status <- if (res$available) "\u2713" else "\u26a0 (no data)"
    message(glue::glue("  [{i}/{n}] {status} {entry$id}: {entry$title}"))
  }

  # Summary
  n_ok <- sum(vapply(results, function(x) x$available, logical(1)))
  message(glue::glue("\u2713 Computed {n_ok}/{n} tables ({n - n_ok} skipped due to missing data)."))

  results
}


# ============================================================================
# Internal compute methods
# ============================================================================

#' Check if variable(s) exist in data
#' @keywords internal
.vars_available <- function(vars, data) {
  all(vars %in% names(data))
}


#' Optionally subset a design by a denominator variable
#' @keywords internal
.apply_denominator <- function(design, denominator, data) {
  if (is.null(denominator)) return(design)
  if (!denominator %in% names(data)) return(NULL)
  subset(design, data[[denominator]] == TRUE)
}


#' Compute a proportion table (type = "proportion")
#'
#' Produces total, by_sex, by_age data frames with estimate, lower, upper.
#' Values are in percentage (0-100).
#'
#' @keywords internal
.compute_proportion <- function(entry, design, data) {
  var <- entry$variable
  if (length(var) > 1) var <- var[1]  # single variable for proportion
  if (!.vars_available(var, data)) return(NULL)

  # Apply denominator filter if specified
  dsn <- .apply_denominator(design, entry$denominator, data)
  if (is.null(dsn)) return(NULL)

  f <- stats::as.formula(paste0("~as.numeric(", var, ")"))

  list(
    total      = svyprop(f, dsn),
    by_sex     = svyprop(f, dsn, by = ~sex),
    by_age     = svyprop(f, dsn, by = ~age_group),
    by_sex_age = svyprop(f, dsn, by = ~sex + age_group)
  )
}


#' Compute a mean table (type = "mean")
#'
#' For single variable: total, by_sex, by_age.
#' For multiple variables: a named list, each with total, by_sex, by_age.
#'
#' @keywords internal
.compute_mean <- function(entry, design, data) {
  vars <- entry$variable
  dsn  <- .apply_denominator(design, entry$denominator, data)
  if (is.null(dsn)) return(NULL)

  # Filter to variables that actually exist
  vars <- vars[vars %in% names(data)]
  if (length(vars) == 0) return(NULL)

  if (length(vars) == 1) {
    f <- stats::as.formula(paste0("~", vars))
    return(list(
      total      = svymn(f, dsn),
      by_sex     = svymn(f, dsn, by = ~sex),
      by_age     = svymn(f, dsn, by = ~age_group),
      by_sex_age = svymn(f, dsn, by = ~sex + age_group)
    ))
  }

  # Multiple variables: return named list
  res <- list()
  for (v in vars) {
    f <- stats::as.formula(paste0("~", v))
    res[[v]] <- list(
      total      = svymn(f, dsn),
      by_sex     = svymn(f, dsn, by = ~sex),
      by_age     = svymn(f, dsn, by = ~age_group),
      by_sex_age = svymn(f, dsn, by = ~sex + age_group)
    )
  }
  res
}


#' Compute a category table (type = "category")
#'
#' The variable is a factor; compute the proportion in each level.
#' Returns a named list (one element per level), each with total, by_sex, by_age.
#'
#' @keywords internal
.compute_category <- function(entry, design, data) {
  var <- entry$variable
  if (length(var) > 1) var <- var[1]
  if (!.vars_available(var, data)) return(NULL)

  dsn <- .apply_denominator(design, entry$denominator, data)
  if (is.null(dsn)) return(NULL)

  # If levels are specified as named vector mapping label -> binary variable
  if (!is.null(entry$levels)) {
    res <- list()
    for (lbl in names(entry$levels)) {
      bvar <- entry$levels[[lbl]]
      if (bvar %in% names(data)) {
        f <- stats::as.formula(paste0("~as.numeric(", bvar, ")"))
        res[[lbl]] <- list(
          total      = svyprop(f, dsn),
          by_sex     = svyprop(f, dsn, by = ~sex),
          by_age     = svyprop(f, dsn, by = ~age_group),
          by_sex_age = svyprop(f, dsn, by = ~sex + age_group)
        )
      }
    }
    if (length(res) == 0) return(NULL)
    return(res)
  }

  # Otherwise, treat the variable as a factor and compute proportions per level
  if (!is.factor(data[[var]])) {
    data[[var]] <- as.factor(data[[var]])
    dsn$variables[[var]] <- as.factor(dsn$variables[[var]])
  }
  lvls <- levels(data[[var]])

  res <- list()
  for (lv in lvls) {
    bvar_name <- paste0(".cat_", var, "_", make.names(lv))
    dsn$variables[[bvar_name]] <- as.numeric(dsn$variables[[var]] == lv)
    f <- stats::as.formula(paste0("~", bvar_name))
    res[[lv]] <- list(
      total      = svyprop(f, dsn),
      by_sex     = svyprop(f, dsn, by = ~sex),
      by_age     = svyprop(f, dsn, by = ~age_group),
      by_sex_age = svyprop(f, dsn, by = ~sex + age_group)
    )
  }
  if (length(res) == 0) return(NULL)
  res
}


#' Compute a cascade table (type = "cascade")
#'
#' A cascade is a series of proportions, each possibly with a different
#' denominator (nested). Returns named list, each with total, by_sex, by_age.
#'
#' @keywords internal
.compute_cascade <- function(entry, design, data) {
  items <- entry$variable  # named list: label -> variable_name
  if (!is.list(items)) return(NULL)

  dsn <- .apply_denominator(design, entry$denominator, data)
  if (is.null(dsn)) return(NULL)

  res <- list()
  for (lbl in names(items)) {
    var <- items[[lbl]]
    if (var %in% names(data)) {
      f <- stats::as.formula(paste0("~as.numeric(", var, ")"))
      res[[lbl]] <- list(
        total      = svyprop(f, dsn),
        by_sex     = svyprop(f, dsn, by = ~sex),
        by_age     = svyprop(f, dsn, by = ~age_group),
        by_sex_age = svyprop(f, dsn, by = ~sex + age_group)
      )
    }
  }
  if (length(res) == 0) return(NULL)
  res
}


#' Compute combined risk factors (type = "combined")
#'
#' Counts how many of 5 key risk factors each respondent has, then computes
#' the proportion with 0, 1-2, and 3-5 risk factors.
#'
#' @keywords internal
.compute_combined <- function(entry, design, data) {
  items <- entry$variable  # named list of risk factor variables
  if (!is.list(items)) return(NULL)

  vars <- unlist(items)
  avail <- vars[vars %in% names(data)]
  if (length(avail) < 3) return(NULL)  # need at least 3 of 5 to be meaningful

  # Count risk factors per person
  rf_matrix <- as.data.frame(lapply(avail, function(v) {
    x <- as.numeric(data[[v]])
    dplyr::if_else(is.na(x), NA_real_, x)
  }))
  design$variables$.rf_count <- rowSums(rf_matrix, na.rm = FALSE)

  # Create category variables
  design$variables$.rf_0   <- as.numeric(design$variables$.rf_count == 0)
  design$variables$.rf_1_2 <- as.numeric(design$variables$.rf_count >= 1 & design$variables$.rf_count <= 2)
  design$variables$.rf_3_5 <- as.numeric(design$variables$.rf_count >= 3)

  list(
    "0 risk factors" = list(
      total      = svyprop(~.rf_0, design),
      by_sex     = svyprop(~.rf_0, design, by = ~sex),
      by_age     = svyprop(~.rf_0, design, by = ~age_group),
      by_sex_age = svyprop(~.rf_0, design, by = ~sex + age_group)
    ),
    "1-2 risk factors" = list(
      total      = svyprop(~.rf_1_2, design),
      by_sex     = svyprop(~.rf_1_2, design, by = ~sex),
      by_age     = svyprop(~.rf_1_2, design, by = ~age_group),
      by_sex_age = svyprop(~.rf_1_2, design, by = ~sex + age_group)
    ),
    "3-5 risk factors" = list(
      total      = svyprop(~.rf_3_5, design),
      by_sex     = svyprop(~.rf_3_5, design, by = ~sex),
      by_age     = svyprop(~.rf_3_5, design, by = ~age_group),
      by_sex_age = svyprop(~.rf_3_5, design, by = ~sex + age_group)
    )
  )
}

#' WHO STEPS Data Book Table Registry
#'
#' Defines all standard tables from the WHO STEPS Epi Info report template.
#' Each entry specifies the table metadata; generic compute and formatting
#' functions use this registry to produce the full data book automatically.
#'
#' @section Table types:
#' \describe{
#'   \item{proportion}{Single binary indicator: % (95% CI) by age × sex.
#'     Most common type. Example: "Current smokers among all respondents."}
#'   \item{mean}{Continuous variable: mean (95% CI) by age × sex.
#'     Example: "Mean BMI (kg/m²)."}
#'   \item{category}{Multi-level factor: % per level (95% CI) by age × sex.
#'     Example: "BMI classifications (Underweight / Normal / Overweight / Obese)."}
#'   \item{cascade}{Diagnosis → treatment → control chain: multiple proportions
#'     with nested denominators. Example: "Raised BP diagnosis, treatment and control."}
#'   \item{combined}{Summary of combined risk factors: 0, 1-2, 3-5 risk factors.}
#' }
#'
#' @section Registry fields:
#' \describe{
#'   \item{id}{Unique short identifier (e.g., "T_smoking_current").}
#'   \item{section}{Data book section (e.g., "Tobacco Use", "Blood Pressure").}
#'   \item{step}{STEPS step number (1, 2, or 3).}
#'   \item{title}{Table title as shown in the data book.}
#'   \item{description}{One-line description from the WHO template.}
#'   \item{type}{One of: "proportion", "mean", "category", "cascade", "combined".}
#'   \item{variable}{Column name(s) in the cleaned data frame to analyse.
#'     For proportion: single logical variable.
#'     For mean: single numeric variable.
#'     For category: single factor variable.
#'     For cascade: named list of logical variables.}
#'   \item{denominator}{NULL (= all respondents) or column name for subsetting
#'     (e.g., "current_alcohol" to restrict to drinkers).}
#'   \item{levels}{For category type: named character vector of level labels.}
#'   \item{epi_info}{Epi Info program name(s) for reference.}
#'   \item{unit}{Display unit (e.g., "%", "mmHg", "cm", "kg/m²", "mmol/L").}
#'   \item{questions}{STEPS instrument question codes used.}
#'   \item{sex_panels}{Logical. TRUE = 3 panels (Men/Women/Both);
#'     FALSE = 2 panels (Men/Women only, e.g., height/weight means).
#'     Default TRUE.}
#' }
#'
#' @return A list of table specification lists.
#' @export
steps_table_registry <- function() {
  list(

    # =========================================================================
    # SECTION: TOBACCO USE (Step 1)
    # =========================================================================

    list(
      id          = "T_smoking_current",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Current smokers",
      description = "Current smokers among all respondents.",
      type        = "proportion",
      variable    = "current_smoker",
      denominator = NULL,
      epi_info    = "TsmokestatusWT",
      unit        = "%",
      questions   = "T1, T2, T8"
    ),

    list(
      id          = "T_smoking_status",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Smoking status",
      description = "Smoking status of all respondents.",
      type        = "category",
      variable    = "smoking_status",
      denominator = NULL,
      levels      = c("Daily" = "daily_smoker",
                       "Non-daily" = "nondaily_smoker",
                       "Former smoker" = "former_smoker",
                       "Never smoker" = "never_smoker"),
      epi_info    = "TsmokestatusWT",
      unit        = "%",
      questions   = "T1, T2, T8"
    ),

    list(
      id          = "T_smoking_daily_among_smokers",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Current daily smokers among smokers",
      description = "Percentage of current daily smokers among smokers.",
      type        = "proportion",
      variable    = "daily_tobacco",
      denominator = "current_tobacco",
      epi_info    = "TsmokestatusWT",
      unit        = "%",
      questions   = "T1, T2"
    ),

    list(
      id          = "T_smoking_age_duration",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Mean age of initiation and duration of smoking",
      description = "Mean age started smoking and mean duration among daily smokers.",
      type        = "mean",
      variable    = c("smoking_start_age", "smoking_duration"),
      denominator = "daily_tobacco",
      epi_info    = "TagedurationWT",
      unit        = "years",
      questions   = "T3, T4",
      sex_panels  = FALSE
    ),

    list(
      id          = "T_smoking_manufactured_cig",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Type of tobacco smoked",
      description = "Percentage of smokers who use manufactured cigarettes.",
      type        = "proportion",
      variable    = "smokes_manufactured_cig",
      denominator = "current_smoker",
      epi_info    = "TtypeWT",
      unit        = "%",
      questions   = "T5a-T5f"
    ),

    list(
      id          = "T_smoking_mean_amount",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Mean amount of tobacco used by daily smokers by type",
      description = "Mean amount of tobacco used by daily smokers per day, by type.",
      type        = "mean",
      variable    = c("cig_per_day_manufactured", "cig_per_day_handrolled"),
      denominator = "daily_tobacco",
      epi_info    = "TmeanamountWT",
      unit        = "per day",
      questions   = "T5a-T5f"
    ),

    list(
      id          = "T_smoking_quantity_cat",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Daily cigarette smoking quantities",
      description = "Percentage of daily cigarette smokers smoking given quantities.",
      type        = "category",
      variable    = "cigarettes_per_day_cat",
      denominator = "daily_tobacco",
      levels      = c("<5" = "cig_lt5", "5-9" = "cig_5_9",
                       "10-14" = "cig_10_14", "15-24" = "cig_15_24",
                       "25+" = "cig_25plus"),
      epi_info    = "TamountWT",
      unit        = "%",
      questions   = "T5a, T5b"
    ),

    list(
      id          = "T_smoking_former",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Former daily smokers",
      description = "Percentage of former daily smokers among all respondents and among ever daily smokers.",
      type        = "proportion",
      variable    = "former_daily_smoker",
      denominator = NULL,
      epi_info    = "TformerWT",
      unit        = "%",
      questions   = "T8, T9, T10, T11"
    ),

    list(
      id          = "T_smoking_quit_attempt",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Quit attempt",
      description = "Percentage of current smokers who tried to stop smoking during past 12 months.",
      type        = "proportion",
      variable    = "quit_attempt",
      denominator = "current_smoker",
      epi_info    = "TquittingWT",
      unit        = "%",
      questions   = "T6"
    ),

    list(
      id          = "T_smoking_advice",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Advice to quit from doctor/health worker",
      description = "Percentage of current smokers advised to quit by doctor or health worker.",
      type        = "proportion",
      variable    = "quit_advice",
      denominator = "current_smoker",
      epi_info    = "TadviceWT",
      unit        = "%",
      questions   = "T7"
    ),

    list(
      id          = "T_smokeless_current",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Current smokeless tobacco users",
      description = "Current users of smokeless tobacco among all respondents.",
      type        = "proportion",
      variable    = "current_smokeless",
      denominator = NULL,
      epi_info    = "TsmokelessstatusWT",
      unit        = "%",
      questions   = "T12"
    ),

    list(
      id          = "T_tobacco_current_any",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Current tobacco users (smoking and/or smokeless)",
      description = "Percentage of daily and current tobacco users (smoking and/or smokeless).",
      type        = "proportion",
      variable    = "current_tobacco_any",
      denominator = NULL,
      epi_info    = "TanytobaccoWT",
      unit        = "%",
      questions   = "T1, T2, T12"
    ),

    list(
      id          = "T_secondhand_home",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Exposure to second-hand smoke at home",
      description = "Percentage of respondents exposed to second-hand smoke in the home.",
      type        = "proportion",
      variable    = "secondhand_home",
      denominator = NULL,
      epi_info    = "TsecondhandWT",
      unit        = "%",
      questions   = "T17"
    ),

    list(
      id          = "T_secondhand_work",
      section     = "Tobacco Use",
      step        = 1,
      title       = "Exposure to second-hand smoke at workplace",
      description = "Percentage of respondents exposed to second-hand smoke at workplace.",
      type        = "proportion",
      variable    = "secondhand_work",
      denominator = NULL,
      epi_info    = "TsecondhandWT",
      unit        = "%",
      questions   = "T18"
    ),

    # =========================================================================
    # SECTION: ALCOHOL CONSUMPTION (Step 1)
    # =========================================================================

    list(
      id          = "A_consumption_status",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Alcohol consumption status",
      description = "Alcohol consumption status of all respondents.",
      type        = "category",
      variable    = "alcohol_status",
      denominator = NULL,
      levels      = c("Lifetime abstainer" = "lifetime_abstainer",
                       "Past 12 months abstainer" = "past_12m_abstainer",
                       "Current (past 30 days)" = "current_alcohol",
                       "Drank past 12m not current" = "past_12m_not_current"),
      epi_info    = "AstatusWT",
      unit        = "%",
      questions   = "A1, A2, A5"
    ),

    list(
      id          = "A_frequency_12m",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Frequency of alcohol consumption in past 12 months",
      description = "Frequency of alcohol consumption in past 12 months among current drinkers.",
      type        = "category",
      variable    = "alcohol_freq_12m",
      denominator = "alcohol_12m",
      levels      = c("Daily" = "daily_drinker",
                       "5-6 per week" = "freq_5_6",
                       "3-4 per week" = "freq_3_4",
                       "1-2 per week" = "freq_1_2",
                       "1-3 per month" = "freq_1_3m",
                       "Less than monthly" = "freq_lt_monthly"),
      epi_info    = "Afrequency12WT",
      unit        = "%",
      questions   = "A4"
    ),

    list(
      id          = "A_mean_occasions_30d",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Mean drinking occasions in past 30 days",
      description = "Mean number of occasions with at least one drink in past 30 days among current drinkers.",
      type        = "mean",
      variable    = "drinking_occasions_30d",
      denominator = "current_alcohol",
      epi_info    = "AmeanoccasionsWT",
      unit        = "occasions",
      questions   = "A6"
    ),

    list(
      id          = "A_mean_drinks_occasion",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Mean drinks per drinking occasion",
      description = "Mean number of standard drinks consumed on a drinking occasion among current drinkers.",
      type        = "mean",
      variable    = "drinks_per_occasion",
      denominator = "current_alcohol",
      epi_info    = "AmeandrinkWT",
      unit        = "drinks",
      questions   = "A7"
    ),

    list(
      id          = "A_drinking_level",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Drinking levels among all respondents",
      description = "Percentage of respondents with different drinking levels.",
      type        = "category",
      variable    = "drinking_level",
      denominator = NULL,
      levels      = c("High-end" = "drinking_high",
                       "Intermediate" = "drinking_intermediate",
                       "Lower-end" = "drinking_lower",
                       "Non-drinker" = "non_drinker_30d"),
      epi_info    = "AdrinkinglevelWT",
      unit        = "%",
      questions   = "A5, A7"
    ),

    list(
      id          = "A_heavy_episodic",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Heavy episodic drinking (6+ drinks)",
      description = "Percentage of respondents who had 6+ drinks on any occasion in past 30 days.",
      type        = "proportion",
      variable    = "heavy_episodic",
      denominator = NULL,
      epi_info    = "AheavyepisodicWT",
      unit        = "%",
      questions   = "A9"
    ),

    list(
      id          = "A_7day_frequency",
      section     = "Alcohol Consumption",
      step        = 1,
      title       = "Frequency of alcohol consumption in past 7 days",
      description = "Frequency of alcohol consumption in past 7 days by current drinkers.",
      type        = "category",
      variable    = "alcohol_7day_freq_cat",
      denominator = "current_alcohol",
      levels      = c("0 days" = "drink_0d", "1-2 days" = "drink_1_2d",
                       "3-4 days" = "drink_3_4d", "5-6 days" = "drink_5_6d",
                       "Daily" = "drink_7d"),
      epi_info    = "A7dayfreqWT",
      unit        = "%",
      questions   = "A10a-A10g"
    ),

    # =========================================================================
    # SECTION: DIET (Step 1)
    # =========================================================================

    list(
      id          = "D_mean_fruit_veg_days",
      section     = "Diet",
      step        = 1,
      title       = "Mean days of fruit and vegetable consumption",
      description = "Mean number of days fruit and vegetables consumed in a typical week.",
      type        = "mean",
      variable    = c("fruit_days", "veg_days"),
      denominator = NULL,
      epi_info    = "DmeanfruitvegWT",
      unit        = "days",
      questions   = "D1, D3"
    ),

    list(
      id          = "D_mean_servings",
      section     = "Diet",
      step        = 1,
      title       = "Mean servings of fruit and vegetables per day",
      description = "Mean number of fruit, vegetable, and combined servings per day.",
      type        = "mean",
      variable    = c("avg_fruit_servings", "avg_veg_servings", "avg_fruit_veg_servings"),
      denominator = NULL,
      epi_info    = "DmeanservingsWT",
      unit        = "servings/day",
      questions   = "D1-D4"
    ),

    list(
      id          = "D_low_fruit_veg",
      section     = "Diet",
      step        = 1,
      title       = "Low fruit and vegetable consumption",
      description = "Percentage eating less than 5 servings of fruit and/or vegetables per day.",
      type        = "proportion",
      variable    = "low_fruit_veg",
      denominator = NULL,
      epi_info    = "DfruitvegWT",
      unit        = "%",
      questions   = "D1-D4"
    ),

    list(
      id          = "D_salt_add",
      section     = "Diet",
      step        = 1,
      title       = "Salt added to food",
      description = "Percentage who always or often add salt or salty sauce to food.",
      type        = "proportion",
      variable    = "always_add_salt",
      denominator = NULL,
      epi_info    = "DsaltWT",
      unit        = "%",
      questions   = "D5"
    ),

    list(
      id          = "D_processed_food",
      section     = "Diet",
      step        = 1,
      title       = "Processed food high in salt",
      description = "Percentage who always or often eat processed food high in salt.",
      type        = "proportion",
      variable    = "always_processed_salt",
      denominator = NULL,
      epi_info    = "DsaltWT",
      unit        = "%",
      questions   = "D7"
    ),

    list(
      id          = "D_salt_perception",
      section     = "Diet",
      step        = 1,
      title       = "Salt intake perception",
      description = "Percentage who think they consume far too much or too much salt.",
      type        = "proportion",
      variable    = "perceived_excess_salt",
      denominator = NULL,
      epi_info    = "DsaltperceptionWT",
      unit        = "%",
      questions   = "D8"
    ),

    # =========================================================================
    # SECTION: PHYSICAL ACTIVITY (Step 1)
    # =========================================================================

    list(
      id          = "P_insufficient_pa",
      section     = "Physical Activity",
      step        = 1,
      title       = "Not meeting WHO recommendations on physical activity",
      description = "Percentage not meeting WHO recommendations on physical activity for health.",
      type        = "proportion",
      variable    = "insufficient_pa",
      denominator = NULL,
      epi_info    = "PrecommendationsWT",
      unit        = "%",
      questions   = "P1-P15b"
    ),

    list(
      id          = "P_activity_level",
      section     = "Physical Activity",
      step        = 1,
      title       = "Level of total physical activity",
      description = "Percentage classified as Low, Moderate, or High physical activity.",
      type        = "category",
      variable    = "pa_category",
      denominator = NULL,
      levels      = c("Low" = "pa_low", "Moderate" = "pa_moderate", "High" = "pa_high"),
      epi_info    = "PlevelWT",
      unit        = "%",
      questions   = "P1-P15b"
    ),

    list(
      id          = "P_mean_minutes",
      section     = "Physical Activity",
      step        = 1,
      title       = "Mean minutes of total physical activity per day",
      description = "Mean minutes of total physical activity on average per day.",
      type        = "mean",
      variable    = "pa_minutes_per_day",
      denominator = NULL,
      epi_info    = "PtotalminutesWT",
      unit        = "minutes/day",
      questions   = "P1-P15b"
    ),

    list(
      id          = "P_mean_by_domain",
      section     = "Physical Activity",
      step        = 1,
      title       = "Mean minutes of physical activity by domain",
      description = "Mean minutes spent in work-, transport-, and recreation-related PA per day.",
      type        = "mean",
      variable    = c("pa_work_min_day", "pa_transport_min_day", "pa_recreation_min_day"),
      denominator = NULL,
      epi_info    = "PsetspecificWT",
      unit        = "minutes/day",
      questions   = "P1-P15b"
    ),

    list(
      id          = "P_sedentary",
      section     = "Physical Activity",
      step        = 1,
      title       = "Sedentary behaviour",
      description = "Mean minutes spent in sedentary activities on a typical day.",
      type        = "mean",
      variable    = "sedentary_min_day",
      denominator = NULL,
      epi_info    = "PsedentaryWT",
      unit        = "minutes/day",
      questions   = "P16a, P16b"
    ),

    list(
      id          = "P_no_vigorous",
      section     = "Physical Activity",
      step        = 1,
      title       = "No vigorous physical activity",
      description = "Percentage of respondents not engaging in vigorous physical activity.",
      type        = "proportion",
      variable    = "no_vigorous_pa",
      denominator = NULL,
      epi_info    = "PnovigorousWT",
      unit        = "%",
      questions   = "P1, P10"
    ),

    # =========================================================================
    # SECTION: HISTORY OF RAISED BP / DIABETES / CHOLESTEROL / CVD (Step 1)
    # =========================================================================

    list(
      id          = "H_bp_measurement",
      section     = "History of Raised Blood Pressure",
      step        = 1,
      title       = "Blood pressure measurement and diagnosis",
      description = "BP measurement and diagnosis among all respondents.",
      type        = "cascade",
      variable    = list(
        "Ever measured"       = "bp_ever_measured",
        "Diagnosed"           = "bp_diagnosed",
        "Diagnosed past 12m"  = "bp_diagnosed_12m"
      ),
      denominator = NULL,
      epi_info    = "HbloodpressureWT",
      unit        = "%",
      questions   = "H1, H2a, H2b"
    ),

    list(
      id          = "H_bp_treatment",
      section     = "History of Raised Blood Pressure",
      step        = 1,
      title       = "Raised BP treatment among those diagnosed",
      description = "Treatment results among those previously diagnosed with raised BP.",
      type        = "proportion",
      variable    = "on_bp_meds",
      denominator = "bp_diagnosed",
      epi_info    = "HbloodpressureWT",
      unit        = "%",
      questions   = "H2a, H3"
    ),

    list(
      id          = "H_dm_measurement",
      section     = "History of Diabetes",
      step        = 1,
      title       = "Blood sugar measurement and diagnosis",
      description = "Blood sugar measurement and diagnosis among all respondents.",
      type        = "cascade",
      variable    = list(
        "Ever measured"       = "glucose_ever_measured",
        "Diagnosed"           = "dm_diagnosed",
        "Diagnosed past 12m"  = "dm_diagnosed_12m"
      ),
      denominator = NULL,
      epi_info    = "HdiabetesWT",
      unit        = "%",
      questions   = "H6, H7a, H7b"
    ),

    list(
      id          = "H_dm_treatment",
      section     = "History of Diabetes",
      step        = 1,
      title       = "Diabetes treatment among those diagnosed",
      description = "Treatment results among those previously diagnosed with diabetes.",
      type        = "cascade",
      variable    = list(
        "On medication" = "on_dm_meds",
        "On insulin"    = "on_dm_insulin"
      ),
      denominator = "dm_diagnosed",
      epi_info    = "HdiabetesWT",
      unit        = "%",
      questions   = "H7a, H8, H9"
    ),

    list(
      id          = "H_chol_measurement",
      section     = "History of Raised Total Cholesterol",
      step        = 1,
      title       = "Total cholesterol measurement and diagnosis",
      description = "Total cholesterol measurement and diagnosis among all respondents.",
      type        = "cascade",
      variable    = list(
        "Ever measured"       = "chol_ever_measured",
        "Diagnosed"           = "chol_diagnosed",
        "Diagnosed past 12m"  = "chol_diagnosed_12m"
      ),
      denominator = NULL,
      epi_info    = "HcholWT",
      unit        = "%",
      questions   = "H12, H13a, H13b"
    ),

    list(
      id          = "H_chol_treatment",
      section     = "History of Raised Total Cholesterol",
      step        = 1,
      title       = "Cholesterol treatment among those diagnosed",
      description = "Treatment results among those previously diagnosed with raised cholesterol.",
      type        = "proportion",
      variable    = "on_chol_meds",
      denominator = "chol_diagnosed",
      epi_info    = "HcholWT",
      unit        = "%",
      questions   = "H13a, H14"
    ),

    list(
      id          = "H_cvd_history",
      section     = "History of Cardiovascular Diseases",
      step        = 1,
      title       = "History of heart attack, angina, or stroke",
      description = "Percentage who have ever had a heart attack, chest pain from heart disease, or stroke.",
      type        = "proportion",
      variable    = "cvd_history",
      denominator = NULL,
      epi_info    = "HcvdWT",
      unit        = "%",
      questions   = "H17"
    ),

    list(
      id          = "H_cvd_meds",
      section     = "History of Cardiovascular Diseases",
      step        = 1,
      title       = "CVD medication (aspirin and/or statins)",
      description = "Currently taking aspirin or statins regularly among those with CVD history.",
      type        = "cascade",
      variable    = list(
        "Taking aspirin" = "on_aspirin",
        "Taking statins" = "on_statins"
      ),
      denominator = "cvd_history",
      epi_info    = "HcvdmedsWT",
      unit        = "%",
      questions   = "H17, H18, H19"
    ),

    list(
      id          = "H_lifestyle_advice",
      section     = "History of Cardiovascular Diseases",
      step        = 1,
      title       = "Lifestyle advice from doctor or health worker",
      description = "Percentage advised to quit tobacco, reduce salt, eat fruit/veg, reduce fat, be active, maintain healthy weight.",
      type        = "cascade",
      variable    = list(
        "Quit tobacco"         = "advised_quit_tobacco",
        "Reduce salt"          = "advised_reduce_salt",
        "Eat fruit/veg"        = "advised_fruit_veg",
        "Reduce fat"           = "advised_reduce_fat",
        "Be more active"       = "advised_more_pa",
        "Maintain healthy wt"  = "advised_healthy_weight"
      ),
      denominator = NULL,
      epi_info    = "HlifestyleWT",
      unit        = "%",
      questions   = "H20a-H20f"
    ),

    list(
      id          = "H_cervical_screening",
      section     = "Cervical Cancer Screening",
      step        = 1,
      title       = "Cervical cancer screening",
      description = "Percentage of female respondents who have ever had cervical cancer screening.",
      type        = "proportion",
      variable    = "cervical_screened",
      denominator = NULL,
      sex_filter  = "female",
      epi_info    = "HcervcancerWT",
      unit        = "%",
      questions   = "CX1"
    ),

    # =========================================================================
    # SECTION: PHYSICAL MEASUREMENTS — BLOOD PRESSURE (Step 2)
    # =========================================================================

    list(
      id          = "M_bp_mean",
      section     = "Blood Pressure",
      step        = 2,
      title       = "Mean blood pressure",
      description = "Mean systolic and diastolic blood pressure including those on medication.",
      type        = "mean",
      variable    = c("mean_sbp", "mean_dbp"),
      denominator = NULL,
      epi_info    = "MbloodpressureWT",
      unit        = "mmHg",
      questions   = "M4a-M6b"
    ),

    list(
      id          = "M_bp_raised",
      section     = "Blood Pressure",
      step        = 2,
      title       = "Raised blood pressure",
      description = "Percentage with raised blood pressure (SBP>=140 and/or DBP>=90 or on medication).",
      type        = "proportion",
      variable    = "raised_bp",
      denominator = NULL,
      epi_info    = "MraisedbpWT",
      unit        = "%",
      questions   = "M4a-M6b, M7"
    ),

    list(
      id          = "M_bp_cascade",
      section     = "Blood Pressure",
      step        = 2,
      title       = "Raised BP diagnosis, treatment and control",
      description = "Diagnosis, treatment, and control among those with raised BP or on medication.",
      type        = "cascade",
      variable    = list(
        "Previously diagnosed"           = "bp_diagnosed",
        "On medication (among diagnosed)" = "on_bp_meds",
        "BP controlled (SBP<140 & DBP<90 among treated)" = "bp_controlled"
      ),
      denominator = "raised_bp",
      epi_info    = "MraisedbpWT",
      unit        = "%",
      questions   = "H1, H2a, H3, M4a-M6b, M7"
    ),

    list(
      id          = "M_heart_rate",
      section     = "Blood Pressure",
      step        = 2,
      title       = "Mean heart rate",
      description = "Mean heart rate (beats per minute).",
      type        = "mean",
      variable    = "mean_heart_rate",
      denominator = NULL,
      epi_info    = "MheartrateWT",
      unit        = "bpm",
      questions   = "M16a-M16c"
    ),

    # =========================================================================
    # SECTION: PHYSICAL MEASUREMENTS — ANTHROPOMETRY (Step 2)
    # =========================================================================

    list(
      id          = "M_height_weight_bmi",
      section     = "Anthropometry",
      step        = 2,
      title       = "Mean height, weight, and BMI",
      description = "Mean height, weight, and BMI excluding pregnant women.",
      type        = "mean",
      variable    = c("height_cm", "weight_kg", "bmi"),
      denominator = NULL,
      epi_info    = "MbmiWT",
      unit        = c("cm", "kg", "kg/m\u00B2"),
      questions   = "M8, M11, M12"
    ),

    list(
      id          = "M_bmi_categories",
      section     = "Anthropometry",
      step        = 2,
      title       = "BMI classifications",
      description = "Percentage in each BMI category excluding pregnant women.",
      type        = "category",
      variable    = "bmi_category",
      denominator = NULL,
      levels      = c("Underweight (<18.5)" = "underweight",
                       "Normal (18.5-24.9)"  = "normal_weight",
                       "Overweight (25-29.9)" = "overweight_only",
                       "Obese (>=30)"         = "obese"),
      epi_info    = "MbmiclassWT",
      unit        = "%",
      questions   = "M8, M11, M12"
    ),

    list(
      id          = "M_bmi_overweight",
      section     = "Anthropometry",
      step        = 2,
      title       = "Overweight (BMI >= 25)",
      description = "Percentage classified as overweight (BMI >= 25) excluding pregnant women.",
      type        = "proportion",
      variable    = "overweight_obese",
      denominator = NULL,
      epi_info    = "MbmiclassWT",
      unit        = "%",
      questions   = "M8, M11, M12"
    ),

    list(
      id          = "M_waist",
      section     = "Anthropometry",
      step        = 2,
      title       = "Mean waist circumference",
      description = "Mean waist circumference excluding pregnant women.",
      type        = "mean",
      variable    = "waist_cm",
      denominator = NULL,
      epi_info    = "MwaistWT",
      unit        = "cm",
      questions   = "M14"
    ),

    list(
      id          = "M_hip",
      section     = "Anthropometry",
      step        = 2,
      title       = "Mean hip circumference",
      description = "Mean hip circumference excluding pregnant women.",
      type        = "mean",
      variable    = "hip_cm",
      denominator = NULL,
      epi_info    = "MhipWT",
      unit        = "cm",
      questions   = "M15"
    ),

    list(
      id          = "M_waist_hip_ratio",
      section     = "Anthropometry",
      step        = 2,
      title       = "Mean waist-to-hip ratio",
      description = "Mean waist-to-hip ratio excluding pregnant women.",
      type        = "mean",
      variable    = "waist_hip_ratio",
      denominator = NULL,
      epi_info    = "MwaisthipratioWT",
      unit        = "",
      questions   = "M14, M15"
    ),

    # =========================================================================
    # SECTION: BIOCHEMICAL MEASUREMENTS (Step 3)
    # =========================================================================

    list(
      id          = "B_glucose_mean",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Mean fasting blood glucose",
      description = "Mean fasting blood glucose including those on medication.",
      type        = "mean",
      variable    = "fasting_glucose",
      denominator = NULL,
      epi_info    = "BglucoseWT",
      unit        = "mmol/L",
      questions   = "B5, B6"
    ),

    list(
      id          = "B_glucose_raised",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Raised blood glucose",
      description = "Categorization of blood glucose levels and percentage on medication.",
      type        = "cascade",
      variable    = list(
        "Impaired (6.1-6.9 mmol/L)"       = "impaired_glucose",
        "Raised (>=7.0 mmol/L) or on meds" = "raised_glucose",
        "On medication"                     = "on_dm_meds"
      ),
      denominator = NULL,
      epi_info    = "BglucoseWT",
      unit        = "%",
      questions   = "B5, B6, H8, H9"
    ),

    list(
      id          = "B_glucose_cascade",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Raised blood glucose diagnosis and treatment",
      description = "Raised blood glucose diagnosis and treatment among all respondents.",
      type        = "cascade",
      variable    = list(
        "Raised glucose or on meds"          = "raised_glucose",
        "Previously diagnosed"               = "dm_diagnosed",
        "Diagnosed & on medication"          = "on_dm_meds",
        "Diagnosed & on insulin"             = "on_dm_insulin",
        "Not previously diagnosed (new)"     = "dm_new"
      ),
      denominator = NULL,
      epi_info    = "BglucoseWT",
      unit        = "%",
      questions   = "B5, B6, H6-H9"
    ),

    list(
      id          = "B_chol_mean",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Mean total cholesterol",
      description = "Mean total cholesterol including those on medication.",
      type        = "mean",
      variable    = "total_chol",
      denominator = NULL,
      epi_info    = "BtotallipidsWT",
      unit        = "mmol/L",
      questions   = "B8"
    ),

    list(
      id          = "B_chol_raised",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Raised total cholesterol",
      description = "Percentage with raised total cholesterol at two thresholds.",
      type        = "cascade",
      variable    = list(
        ">= 5.0 mmol/L (190 mg/dL)"            = "raised_chol",
        ">= 6.2 mmol/L (240 mg/dL)"            = "raised_chol_high",
        ">= 5.0 or on medication"               = "raised_chol_or_meds",
        ">= 6.2 or on medication"               = "raised_chol_high_or_meds"
      ),
      denominator = NULL,
      epi_info    = "BraisedcholWT",
      unit        = "%",
      questions   = "B8, B9, H14"
    ),

    list(
      id          = "B_hdl",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Mean HDL cholesterol",
      description = "Mean HDL cholesterol and percentage with low HDL.",
      type        = "mean",
      variable    = "hdl_chol",
      denominator = NULL,
      epi_info    = "BhdlipidsWT",
      unit        = "mmol/L",
      questions   = "B17"
    ),

    list(
      id          = "B_triglycerides",
      section     = "Biochemical Measurements",
      step        = 3,
      title       = "Mean fasting triglycerides",
      description = "Mean fasting triglycerides and percentage with raised triglycerides.",
      type        = "mean",
      variable    = "triglycerides",
      denominator = NULL,
      epi_info    = "BtriglycerideWT",
      unit        = "mmol/L",
      questions   = "B16"
    ),

    # =========================================================================
    # SECTION: CARDIOVASCULAR DISEASE RISK (Steps 1-3)
    # =========================================================================

    list(
      id          = "R_cvd_risk_10yr",
      section     = "Cardiovascular Disease Risk",
      step        = 3,
      title       = "10-year cardiovascular disease risk",
      description = "10-year CVD risk distribution among respondents aged 40-69.",
      type        = "category",
      variable    = "cvd_risk_category",
      denominator = NULL,
      levels      = c("<10%" = "cvd_low", "10-<20%" = "cvd_medium",
                       "20-<30%" = "cvd_high", ">=30%" = "cvd_very_high"),
      epi_info    = "RcvdriskWT",
      unit        = "%",
      questions   = "Combined"
    ),

    # =========================================================================
    # SECTION: COMBINED RISK FACTORS (Steps 1+2)
    # =========================================================================

    list(
      id          = "RF_combined",
      section     = "Summary of Combined Risk Factors",
      step        = 2,
      title       = "Summary of combined risk factors",
      description = "Percentage with 0, 1-2, or 3-5 of 5 key risk factors.",
      type        = "combined",
      variable    = list(
        "Current daily smoking"        = "daily_tobacco",
        "Less than 5 servings fruit/veg" = "low_fruit_veg",
        "Insufficient PA"              = "insufficient_pa",
        "Overweight/obese (BMI>=25)"   = "overweight_obese",
        "Raised BP"                    = "raised_bp"
      ),
      denominator = NULL,
      epi_info    = "RaisedriskWT",
      unit        = "%",
      questions   = "T1, T2, D1-D4, P1-P15b, M4a-M6b, M7, M8, M11, M12"
    )
  )
}


#' Get table registry entries by section
#'
#' @param section Section name (e.g., "Tobacco Use", "Blood Pressure").
#'   If NULL, returns all entries.
#' @return A filtered list of registry entries.
#' @export
get_registry_by_section <- function(section = NULL) {
  reg <- steps_table_registry()
  if (is.null(section)) return(reg)
  Filter(function(x) x$section == section, reg)
}


#' Get table registry entries by step
#'
#' @param step STEPS step number (1, 2, or 3).
#' @return A filtered list of registry entries.
#' @export
get_registry_by_step <- function(step) {
  reg <- steps_table_registry()
  Filter(function(x) x$step == step, reg)
}


#' List all available sections in the registry
#'
#' @return Character vector of unique section names.
#' @export
list_registry_sections <- function() {
  unique(vapply(steps_table_registry(), function(x) x$section, character(1)))
}

#' Compute Alcohol Use Indicators
#'
#' Calculates prevalence of alcohol use from a survey design object.
#' Computes proportions of current alcohol use and heavy episodic drinking,
#' stratified by sex and age group where available.
#'
#' @param design A survey design object from [setup_survey_design()].
#'
#' @return A named list of survey estimates. Each element contains proportion
#'   estimates (as tibble with columns: estimate, lower, upper, etc.) for:
#'   - `current_alcohol_total`: current alcohol use, overall
#'   - `current_alcohol_by_sex`: current alcohol use, by sex
#'   - `current_alcohol_by_age`: current alcohol use, by age group
#'   - `heavy_episodic_total`: heavy episodic drinking, overall
#'   - `heavy_episodic_by_sex`: heavy episodic drinking, by sex
#'   - `heavy_episodic_by_age`: heavy episodic drinking, by age group
#'   (if the corresponding variables exist in design)
#'
#' @seealso [compute_all_indicators()]
#'
#' @export
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   alcohol_results <- compute_alcohol_indicators(design)
#' }
compute_alcohol_indicators <- function(design) {
  message("  Computing alcohol indicators...")
  results <- list()

  vars <- c("current_alcohol", "heavy_episodic")
  vars <- vars[vars %in% names(design$variables)]

  for (v in vars) {
    f <- as.formula(paste0("~as.numeric(", v, ")"))
    results[[paste0(v, "_total")]]  <- svyprop(f, design)
    results[[paste0(v, "_by_sex")]] <- svyprop(f, design, by = ~sex)
    results[[paste0(v, "_by_age")]] <- svyprop(f, design, by = ~age_group)
  }

  return(results)
}

#' Import raw STEPS survey data
#'
#' Reads a raw STEPS data file (CSV, Excel, Stata, or SPSS) and
#' standardises column names to lowercase with underscores.
#'
#' @param path Character. Path to the data file.
#' @return A data frame with cleaned column names.
#' @export
#' @examples
#' \dontrun{
#' raw <- import_steps_data("data/raw/steps_data.csv")
#' }
import_steps_data <- function(path) {
  if (!file.exists(path)) {
    stop(glue::glue("Data file not found: {path}"), call. = FALSE)
  }

  ext <- tolower(tools::file_ext(path))

  data <- switch(ext,
    "csv"  = readr::read_csv(path, show_col_types = FALSE),
    "xlsx" = readxl::read_excel(path),
    "xls"  = readxl::read_excel(path),
    "dta"  = tryCatch(
      haven::read_dta(path),
      error = function(e) {
        # Some STEPS .dta files have encoding issues — retry with latin1
        if (grepl("encoding|convert", e$message, ignore.case = TRUE)) {
          message("  \u26a0 Encoding issue detected, retrying with latin1 encoding...")
          haven::read_dta(path, encoding = "latin1")
        } else {
          stop(e)
        }
      }
    ),
    "sav"  = haven::read_spss(path),
    stop(glue::glue("Unsupported file type: .{ext}. Use CSV, Excel, Stata, or SPSS."),
         call. = FALSE)
  )

  data <- data |> janitor::clean_names()
  message(glue::glue("\u2713 Loaded {nrow(data)} rows \u00d7 {ncol(data)} columns from {basename(path)}"))
  return(data)
}

#' Detect a STEPS column by alias
#'
#' Tries to find a column in the data matching one of several candidate
#' names (case-insensitive).
#'
#' @param data A data frame.
#' @param candidates Character vector of possible column names.
#' @param label Optional label for progress messages.
#' @return The matched column name (character) or `NULL`.
#' @export
detect_col <- function(data, candidates, label = NULL) {
  # Build a lookup from lowercase data-name -> original data-name
  data_names_lc <- tolower(names(data))
  # Walk candidates in PRIORITY ORDER and return the first that exists in data
  for (cand in tolower(candidates)) {
    idx <- which(data_names_lc == cand)
    if (length(idx) > 0) {
      match <- names(data)[idx[1]]
      if (!is.null(label)) message(glue::glue("  \u2713 {label}: '{match}'"))
      return(match)
    }
  }
  if (!is.null(label)) message(glue::glue("  \u26a0 Could not auto-detect column for: {label}"))
  return(NULL)
}


#' Get the SPSS/haven label for a column (if any)
#' @param col A vector (typically from a data frame column).
#' @return Character string of the label, or "" if none.
#' @keywords internal
.col_label <- function(col) {
  lbl <- attr(col, "label")
  if (is.null(lbl)) return("")
  tolower(as.character(lbl))
}


#' Strip haven-labelled classes from all columns in a data frame
#'
#' Converts `haven_labelled` vectors to their base R types.
#' This prevents downstream errors (e.g., `dim<-.haven_labelled()
#' not supported`) when passing the data to DT, dplyr, or survey.
#'
#' @param data A data frame (typically from [import_steps_data()]).
#' @return The same data frame with all haven labels removed.
#' @keywords internal
.strip_haven_labels <- function(data) {
  for (j in seq_along(data)) {
    col <- data[[j]]
    if (inherits(col, "haven_labelled")) {
      data[[j]] <- as.vector(col)       # drops class + labels, keeps type
    }
    # Also remove residual "label" attributes that can confuse some packages
    attr(data[[j]], "label")  <- NULL
    attr(data[[j]], "format.spss") <- NULL
  }
  data
}


#' Label-aware column detection for ambiguous STEPS codes
#'
#' Some STEPS column codes (e.g., A1, A5) mean different things across
#' instrument versions.
#' This function first tries unambiguous candidate names, then checks
#' ambiguous codes only if their column label matches expected keywords.
#'
#' @param data A data frame.
#' @param safe_candidates Character vector of unambiguous column names.
#' @param ambiguous Named list: each element is a character vector of
#'   keywords that must appear in the column label for the code to match.
#'   Names are the ambiguous column codes.
#'   Example: `list(a1 = c("current", "30 day"), a5 = c("current", "30 day"))`
#' @param label Optional label for progress messages.
#' @return The matched column name or NULL.
#' @keywords internal
.detect_col_with_label <- function(data, safe_candidates, ambiguous = list(), label = NULL) {
  # First try safe (unambiguous) candidates via standard detect_col (no message yet)
  data_names_lc <- tolower(names(data))
  for (cand in tolower(safe_candidates)) {
    idx <- which(data_names_lc == cand)
    if (length(idx) > 0) {
      match <- names(data)[idx[1]]
      if (!is.null(label)) message(glue::glue("  \u2713 {label}: '{match}'"))
      return(match)
    }
  }

  # Then try ambiguous codes with label validation
  for (code in names(ambiguous)) {
    idx <- which(data_names_lc == tolower(code))
    if (length(idx) > 0) {
      col_name <- names(data)[idx[1]]
      col_lbl  <- .col_label(data[[col_name]])
      keywords <- ambiguous[[code]]
      # Check if ANY keyword appears in the column label
      if (any(vapply(keywords, function(kw) grepl(kw, col_lbl, ignore.case = TRUE), logical(1)))) {
        if (!is.null(label)) message(glue::glue("  \u2713 {label}: '{col_name}' (label: '{col_lbl}')"))
        return(col_name)
      } else {
        if (!is.null(label)) message(glue::glue("  \u2192 Skipped '{col_name}' for {label} (label '{col_lbl}' does not match)"))
      }
    }
  }

  if (!is.null(label)) message(glue::glue("  \u26a0 Could not auto-detect column for: {label}"))
  return(NULL)
}

#' Auto-detect all standard STEPS columns
#'
#' Scans a data frame for standard WHO STEPS variable names across
#' versions 3.1 and 3.2.  Aliases are listed in priority order: the
#' first match wins, so put the most specific / unambiguous name first.
#'
#' @param data A data frame (typically from [import_steps_data()]).
#' @return A named list of detected column names (or `NULL` for missing).
#'
#' @details
#' WHO STEPS reorganised variable codes between v3.1 and v3.2:
#'
#' **v3.1 / Epi Info codes** (still common in many country datasets):
#' B1-B6 = blood-pressure readings, B7 = BP meds,
#' C1 = fasting glucose, C5 = DM meds, C6 = total cholesterol, C10 = chol meds,
#' M1 = height, M2 = weight, M3 = waist.
#'
#' **v3.2 instrument codes**:
#' M4a/M5a/M6a = SBP readings, M4b/M5b/M6b = DBP readings, M7 = BP meds,
#' M11 = height, M12 = weight, M14 = waist, M15 = hip,
#' B5 = fasting glucose, B6 = DM meds, B8 = total cholesterol, B9 = chol meds,
#' B16 = triglycerides, B17 = HDL cholesterol,
#' C1 = sex, C3 = age.
#'
#' The function includes aliases for both versions so datasets from
#' either instrument version are detected automatically.
#'
#' @export
detect_steps_columns <- function(data) {
  message("  Detecting STEPS columns...")

  cols <- list(
    # -- Demographics -----------------------------------------------------------
    # v3.1: age/c1   v3.2: C3 = age, C1 = sex
    age       = detect_col(data, c("age", "age_years", "c3"), "Age"),
    # v3.1: sex/c2   v3.2: C1 = sex
    sex       = detect_col(data, c("sex", "gender", "c1", "c2"), "Sex"),
    # Pre-computed age range groups (standard in WHO-processed datasets)
    agerange  = detect_col(data, c("agerange", "age_range", "agegrp",
                                    "age_group"), "Age range group"),
    # Validity flag (WHO scripts filter on valid==1; not always present)
    valid     = detect_col(data, c("valid", "consent", "completed"), "Valid/consent flag"),
    # Urban/rural setting (common stratifier)
    urban_rural = detect_col(data, c("urbanrural", "urban_rural", "urban",
                                      "residence", "setting"), "Urban/rural"),
    # -- Survey design variables ------------------------------------------------
    # WHO STEPS toolkit standard: WStep1, WStep2, WStep3 (one weight per Step)
    # See Part 4, Section 1 of WHO STEPS Manual.
    # Also detect common country-specific variants (W1Dist, finalwt, etc.)
    weight_step1 = detect_col(data, c("wstep1", "w1dist", "w1_dist",
                                       "wt_final", "wt", "finalwt",
                                       "sampleweight", "samplingweight",
                                       "weight"), "Weight Step 1 (behavioural)"),
    weight_step2 = detect_col(data, c("wstep2", "w2dist", "w2_dist"), "Weight Step 2 (physical)"),
    weight_step3 = detect_col(data, c("wstep3", "w3dist", "w3_dist"), "Weight Step 3 (biochemical)"),
    strata    = detect_col(data, c("stratum", "strata", "stratum_id", "district",
                                    "region", "province"), "Stratum"),
    psu       = detect_col(data, c("psu", "cluster", "psu_id", "clusterid",
                                    "i1", "cluster_id", "ea_id"), "PSU"),

    # -- Step 1: Tobacco -------------------------------------------------------
    # T1 = currently smoke? (same in v3.1 & v3.2)
    tobacco_current = detect_col(data, c("t1", "smk_current", "current_smoker",
                                         "tobacco_current", "current_smoke"), "Current tobacco use"),
    # T2 = smoke daily? (same in v3.1 & v3.2)
    tobacco_daily   = detect_col(data, c("t2", "smk_daily", "daily_smoker",
                                         "tobacco_daily"), "Daily tobacco use"),
    # T3 = age started smoking (daily smokers)
    tobacco_start_age = detect_col(data, c("t3", "smoking_start_age",
                                            "age_start_smoking"), "Age started smoking"),
    # T4 = duration of smoking
    tobacco_duration = detect_col(data, c("t4", "smoking_duration", "t4type",
                                           "smoke_years"), "Smoking duration"),
    # T5a-f = tobacco products used (daily amounts)
    tobacco_cig_day  = detect_col(data, c("t5a", "manufactured_cigarettes_day",
                                           "cig_per_day"), "Manufactured cigarettes/day"),
    tobacco_cig_week = detect_col(data, c("t5aw", "manufactured_cigarettes_week"), "Manufactured cigarettes/week"),
    tobacco_hand_day = detect_col(data, c("t5b", "handrolled_day"), "Hand-rolled cigarettes/day"),
    tobacco_pipe_day = detect_col(data, c("t5c", "pipes_day"), "Pipes/day"),
    tobacco_cigar_day = detect_col(data, c("t5d", "cigars_day"), "Cigars/day"),
    tobacco_shisha_day = detect_col(data, c("t5e", "shisha_day", "waterpipe_day"), "Shisha/day"),
    # T6 = quit attempt in past 12 months
    tobacco_quit_attempt = detect_col(data, c("t6", "quit_attempt", "tried_quit"), "Quit attempt past 12m"),
    # T7 = advised to quit by doctor
    tobacco_quit_advice = detect_col(data, c("t7", "quit_advice", "advised_quit"), "Advised to quit by HCP"),
    # T8 = past smoking, T9 = past daily smoking
    tobacco_past     = detect_col(data, c("t8", "past_smoking", "ever_smoked"), "Past smoking"),
    tobacco_past_daily = detect_col(data, c("t9", "past_daily_smoking"), "Past daily smoking"),
    # T10/T11 = quitting details
    tobacco_quit_age = detect_col(data, c("t10", "quitting_age"), "Quitting age"),
    tobacco_quit_duration = detect_col(data, c("t11", "duration_since_quitting"), "Duration since quitting"),
    # T12/T13 = smokeless tobacco
    smokeless_current = detect_col(data, c("t12", "t15", "smokeless_current",
                                           "current_smokeless", "slt_current"), "Current smokeless tobacco"),
    smokeless_daily  = detect_col(data, c("t13", "smokeless_daily",
                                           "daily_smokeless"), "Daily smokeless tobacco"),
    # T14a-e = smokeless products
    slt_snuff_mouth  = detect_col(data, c("t14a", "snuff_daily_mouth"), "Snuff mouth/day"),
    slt_chewing      = detect_col(data, c("t14c", "chewing_tobacco_daily"), "Chewing tobacco/day"),
    slt_betel        = detect_col(data, c("t14d", "betel_daily", "betel_quid"), "Betel quid/day"),
    # T15/T16 = past smokeless
    smokeless_past   = detect_col(data, c("t15", "past_smokeless"), "Past smokeless tobacco"),
    smokeless_past_daily = detect_col(data, c("t16", "past_daily_smokeless"), "Past daily smokeless"),
    # T17/T18 = second-hand smoke
    secondhand_home  = detect_col(data, c("t17", "passive_home", "secondhand_home",
                                           "shs_home"), "Second-hand smoke at home"),
    secondhand_work  = detect_col(data, c("t18", "passive_workplace", "secondhand_work",
                                           "shs_work"), "Second-hand smoke at workplace"),
    # Derived: any tobacco product
    any_tobacco      = detect_col(data, c("tobacco_product_any",
                                           "daily_tobacco_product_any"), "Any tobacco product"),

    # -- Step 1: Alcohol -------------------------------------------------------
    # A1 = ever consumed alcohol (v3.2) -or- current drinker (v3.1) — AMBIGUOUS
    alcohol_ever    = .detect_col_with_label(data,
      safe_candidates = c("alc_ever", "ever_drinker", "alcohol_ever"),
      ambiguous = list(a1 = c("ever", "lifetime")),
      label = "Ever consumed alcohol"),

    # A2/A4 = consumed past 12 months (v3.2 CORE)
    alcohol_12m     = detect_col(data, c("a2", "a4", "alc_12m",
                                         "alcohol_12months"), "Alcohol past 12 months"),

    # Current alcohol use (past 30 days):
    # Unambiguous names tried first, then a1/a5 with label validation.
    # a1 = current drinker in v3.1 (label contains "current")
    # a5 = past 30 days in v3.2 (label contains "30 day" or "current")
    # a13 is safe (always AUDIT-C, not current use in v3.2) — removed
    alcohol_current = .detect_col_with_label(data,
      safe_candidates = c("alc_current", "current_drinker", "alcohol_current",
                          "alcohol_30days"),
      ambiguous = list(
        a5 = c("30 day", "current"),
        a1 = c("current")
      ),
      label = "Current alcohol use (past 30 days)"),

    # Heavy episodic drinking:
    # a9 = count of times consumed 6+ drinks on a single occasion (v3.2)
    heavy_episode   = .detect_col_with_label(data,
      safe_candidates = c("alcohol_6drin_occasions", "heavy_drinking", "binge",
                          "heavy_episodic", "hed", "hed_total"),
      ambiguous = list(
        a9 = c("6 or more", "6+", "heavy", "episodic", "binge")
      ),
      label = "Heavy episodic drinking"),

    # A3 = stopped drinking?
    alcohol_stopped = detect_col(data, c("a3", "alcohol_stopped", "stopped_drinking"), "Stopped drinking"),
    # A4 = frequency past 12 months
    alcohol_freq_12m = detect_col(data, c("a4", "alcohol_frequency_12m",
                                           "alcohol_frequency_past_12_months"), "Alcohol frequency (past 12m)"),
    # A6 = number of drinking occasions past 30 days
    alcohol_occasions = detect_col(data, c("a6", "drinking_occasions",
                                            "alcohol_number_of_drinking_occasions"), "Drinking occasions (past 30d)"),
    # A7 = number of drinks per occasion
    alcohol_drinks_occasion = detect_col(data, c("a7", "drinks_per_occasion",
                                                  "alcohol_number_of_drinks_on_one_occasion"), "Drinks per occasion"),
    # A8 = largest number of drinks
    alcohol_largest_drinks = detect_col(data, c("a8", "largest_drinks",
                                                 "alcohol_largest_number_of_drinks_on_one_occasion"), "Largest drinks on one occasion"),
    # A9 = number of times with 6+ drinks (count variable)
    alcohol_6plus_count = detect_col(data, c("a9", "alcohol_6_or_more_drinks"), "Times with 6+ drinks"),
    # A10a-g = drinking past 7 days by day of week
    alcohol_7day_freq = detect_col(data, c("alcohol_7_day_frequency",
                                            "alcohol_7day_freq"), "Alcohol 7-day frequency"),
    # A11 = homebrew consumption
    alcohol_homebrew = detect_col(data, c("a11", "homebrew"), "Homebrew consumption"),
    # Derived: drinking level (High_End_Drinking)
    drinking_level   = detect_col(data, c("high_end_drinking", "drinking_level",
                                           "alcohol_level"), "Drinking level category"),

    # -- Step 1: Physical activity (GPAQ) --------------------------------------
    # P1-P16 GPAQ items (v3.2 CORE); met_total is computed
    met_total       = detect_col(data, c("met_total", "total_met", "pa_met",
                                         "gpaq_met", "totalmet"), "Total MET minutes"),
    pa_work_vig     = detect_col(data, c("p1", "pa_work_vigorous"), "Vigorous work activity"),
    pa_work_mod     = detect_col(data, c("p4", "pa_work_moderate"), "Moderate work activity"),
    pa_transport    = detect_col(data, c("p7", "pa_transport", "walk_cycle"), "Walk/bicycle transport"),
    pa_rec_vig      = detect_col(data, c("p10", "pa_rec_vigorous"), "Vigorous recreational activity"),
    pa_rec_mod      = detect_col(data, c("p13", "pa_rec_moderate"), "Moderate recreational activity"),
    pa_sedentary    = detect_col(data, c("p16", "sedentary", "sitting_time"), "Sedentary behaviour"),

    # -- Step 1: Diet ----------------------------------------------------------
    # D1-D4 same in v3.1 & v3.2
    fruit_days      = detect_col(data, c("d1", "fruit_days", "days_fruit"), "Fruit days/week"),
    fruit_servings  = detect_col(data, c("d2", "fruit_serv", "servings_fruit"), "Fruit servings/day"),
    veg_days        = detect_col(data, c("d3", "veg_days", "days_veg"), "Vegetable days/week"),
    veg_servings    = detect_col(data, c("d4", "veg_serv", "servings_veg"), "Vegetable servings/day"),
    # D5 = salt added to food at table
    salt_table      = detect_col(data, c("d5", "d5_binary", "salt_addition_frequency",
                                          "salt_table"), "Salt added at table"),
    # D6 = salt in cooking (v3.2 EXPANDED)
    salt_cooking    = detect_col(data, c("d6", "salt_cooking", "salt_added"), "Salt added in cooking"),
    # D7 = processed food high in salt
    processed_salt  = detect_col(data, c("d7", "d7_binary", "processed_food_consumption_frequency",
                                          "processed_salt"), "Processed food high in salt"),
    # D8 = perceived salt intake
    salt_perception = detect_col(data, c("d8", "salt_perception",
                                          "perceived_salt"), "Perceived salt intake"),
    # D9 = importance of lowering salt
    salt_importance = detect_col(data, c("d9", "salt_importance"), "Importance of lowering salt"),
    # D10 = knowledge of salt health effects
    salt_knowledge  = detect_col(data, c("d10", "salt_knowledge"), "Knowledge of salt effects"),
    # D11a-f = actions to control salt
    salt_limit_processed = detect_col(data, c("d11a", "limit_processed_foods"), "Limit processed food for salt"),
    salt_check_labels    = detect_col(data, c("d11b", "look_at_salt_content"), "Check salt labels"),
    salt_buy_low         = detect_col(data, c("d11c", "buy_low_salt"), "Buy low-salt alternatives"),
    salt_use_spices      = detect_col(data, c("d11d", "use_spices"), "Use spices instead of salt"),
    salt_avoid_outside   = detect_col(data, c("d11e", "avoid_eating_outside"), "Avoid food outside home"),
    salt_other_action    = detect_col(data, c("d11f", "other_salt_action"), "Other salt control action"),
    # D12 = type of oil/fat, D13 = meals outside home
    oil_type         = detect_col(data, c("d12", "oil_type", "cooking_oil"), "Type of cooking oil/fat"),
    meals_outside    = detect_col(data, c("d13", "meals_outside"), "Meals outside home per week"),

    # -- Step 1: Physical activity GPAQ individual items ----------------------
    # P1-P16 individual items for domain-specific calculations
    pa_work_vig_days  = detect_col(data, c("p2", "pa_work_vig_days"), "Vigorous work days/week"),
    pa_work_vig_hrs   = detect_col(data, c("p3a", "pa_work_vig_hours"), "Vigorous work hours"),
    pa_work_vig_min   = detect_col(data, c("p3b", "pa_work_vig_minutes"), "Vigorous work minutes"),
    pa_work_mod_days  = detect_col(data, c("p5", "pa_work_mod_days"), "Moderate work days/week"),
    pa_work_mod_hrs   = detect_col(data, c("p6a", "pa_work_mod_hours"), "Moderate work hours"),
    pa_work_mod_min   = detect_col(data, c("p6b", "pa_work_mod_minutes"), "Moderate work minutes"),
    pa_transport_days = detect_col(data, c("p8", "pa_transport_days"), "Transport days/week"),
    pa_transport_hrs  = detect_col(data, c("p9a", "pa_transport_hours"), "Transport hours"),
    pa_transport_min  = detect_col(data, c("p9b", "pa_transport_minutes"), "Transport minutes"),
    pa_rec_vig_days   = detect_col(data, c("p11", "pa_rec_vig_days"), "Vigorous recreation days/week"),
    pa_rec_vig_hrs    = detect_col(data, c("p12a", "pa_rec_vig_hours"), "Vigorous recreation hours"),
    pa_rec_vig_min    = detect_col(data, c("p12b", "pa_rec_vig_minutes"), "Vigorous recreation minutes"),
    pa_rec_mod_days   = detect_col(data, c("p14", "pa_rec_mod_days"), "Moderate recreation days/week"),
    pa_rec_mod_hrs    = detect_col(data, c("p15a", "pa_rec_mod_hours"), "Moderate recreation hours"),
    pa_rec_mod_min    = detect_col(data, c("p15b", "pa_rec_mod_minutes"), "Moderate recreation minutes"),
    pa_sedentary_hrs  = detect_col(data, c("p16a", "sedentary_hours"), "Sedentary hours"),
    pa_sedentary_min  = detect_col(data, c("p16b", "sedentary_minutes"), "Sedentary minutes"),

    # -- Step 1: History of NCD diagnosis & treatment (H-codes) ---------------
    # Blood pressure history
    bp_ever_measured = detect_col(data, c("h1", "bp_ever_measured",
                                           "bp_measured"), "BP ever measured"),
    bp_diagnosed    = detect_col(data, c("h2a", "h2", "bp_diagnosed",
                                         "ever_told_bp", "diagnosed_htn"), "Ever diagnosed raised BP"),
    bp_diagnosed_12m = detect_col(data, c("h2b", "bp_diagnosed_12m"), "BP diagnosed past 12m"),
    bp_trad_healer  = detect_col(data, c("h4", "bp_traditional_healer"), "Traditional healer for BP"),
    bp_herbal       = detect_col(data, c("h5", "bp_herbal_remedy"), "Herbal remedy for BP"),

    # Diabetes history
    glucose_ever_measured = detect_col(data, c("h6", "glucose_ever_measured",
                                                "blood_sugar_measured"), "Blood sugar ever measured"),
    dm_diagnosed    = detect_col(data, c("h7a", "h7", "dm_diagnosed",
                                         "ever_told_dm", "diagnosed_dm"), "Ever diagnosed diabetes"),
    dm_diagnosed_12m = detect_col(data, c("h7b", "dm_diagnosed_12m"), "DM diagnosed past 12m"),
    dm_insulin      = detect_col(data, c("h9", "dm_insulin", "insulin_current"), "Currently on insulin"),
    dm_trad_healer  = detect_col(data, c("h10", "dm_traditional_healer"), "Traditional healer for DM"),
    dm_herbal       = detect_col(data, c("h11", "dm_herbal_remedy"), "Herbal remedy for DM"),

    # Cholesterol history
    chol_ever_measured = detect_col(data, c("h12", "chol_ever_measured",
                                             "cholesterol_measured"), "Cholesterol ever measured"),
    chol_diagnosed  = detect_col(data, c("h13a", "h13", "chol_diagnosed",
                                         "ever_told_chol"), "Ever diagnosed raised cholesterol"),
    chol_diagnosed_12m = detect_col(data, c("h13b", "chol_diagnosed_12m"), "Chol diagnosed past 12m"),
    chol_trad_healer = detect_col(data, c("h15", "chol_traditional_healer"), "Traditional healer for chol"),
    chol_herbal     = detect_col(data, c("h16", "chol_herbal_remedy"), "Herbal remedy for chol"),

    # CVD history
    cvd_history     = detect_col(data, c("h17", "cvd_history", "heart_attack_stroke",
                                          "ever_heart_attack"), "CVD history (heart attack/stroke)"),
    cvd_aspirin     = detect_col(data, c("h18", "aspirin_regular", "on_aspirin"), "Currently taking aspirin"),
    cvd_statins     = detect_col(data, c("h19", "statins_regular", "on_statins"), "Currently taking statins"),

    # Lifestyle advice (H20a-f)
    adv_quit_tobacco = detect_col(data, c("h20a", "advised_quit_tobacco"), "Advised: quit tobacco"),
    adv_reduce_salt  = detect_col(data, c("h20b", "advised_reduce_salt"), "Advised: reduce salt"),
    adv_fruit_veg    = detect_col(data, c("h20c", "advised_fruit_veg"), "Advised: eat fruit/veg"),
    adv_reduce_fat   = detect_col(data, c("h20d", "advised_reduce_fat"), "Advised: reduce fat"),
    adv_more_pa      = detect_col(data, c("h20e", "advised_more_pa"), "Advised: more PA"),
    adv_healthy_wt   = detect_col(data, c("h20f", "advised_healthy_weight"), "Advised: healthy weight"),

    # Cervical cancer screening
    cervical_screen  = detect_col(data, c("cx1", "cervical_screening",
                                           "cervical_cancer_screening"), "Cervical cancer screening"),

    # -- Demographics (extended) -----------------------------------------------
    education_years  = detect_col(data, c("c4", "education_years", "edu_years"), "Education years"),
    education_level  = detect_col(data, c("c5", "education_group", "highest_education",
                                           "education_level"), "Highest education level"),
    ethnicity        = detect_col(data, c("c6", "caste_group", "ethnicity", "race"), "Ethnicity/Caste"),
    marital_status   = detect_col(data, c("c7", "marital_status", "marital"), "Marital status"),
    employment       = detect_col(data, c("c8", "employment_status", "employment",
                                           "work_status"), "Employment status"),
    income           = detect_col(data, c("c11", "income", "household_income",
                                           "estimate_hh_income_groups"), "Household income"),

    # -- Step 2: Anthropometry -------------------------------------------------
    # v3.1: M1=height, M2=weight, M3=waist
    # v3.2: M11=height, M12=weight, M14=waist, M15=hip
    height   = detect_col(data, c("m11", "m1", "height", "height_cm"), "Height (cm)"),
    weight   = detect_col(data, c("m12", "m2", "weight_kg", "bodyweight"), "Weight (kg)"),
    waist    = detect_col(data, c("m14", "m3", "waist", "waist_cm",
                                  "waist_circumference"), "Waist circumference (cm)"),
    hip      = detect_col(data, c("m15", "hip", "hip_cm",
                                  "hip_circumference"), "Hip circumference (cm)"),

    # -- Step 2: Blood pressure ------------------------------------------------
    # v3.1: B1/B3/B5 = SBP, B2/B4/B6 = DBP, B7 = meds
    # v3.2: M4a/M5a/M6a = SBP, M4b/M5b/M6b = DBP, M7 = meds
    sbp1 = detect_col(data, c("m4a", "b1", "sbp1", "sys1", "systolic1",
                               "sbp_1", "m4_a"), "SBP reading 1"),
    sbp2 = detect_col(data, c("m5a", "b3", "sbp2", "sys2", "systolic2",
                               "sbp_2", "m5_a"), "SBP reading 2"),
    sbp3 = detect_col(data, c("m6a", "b5", "sbp3", "sys3", "systolic3",
                               "sbp_3", "m6_a"), "SBP reading 3"),
    dbp1 = detect_col(data, c("m4b", "b2", "dbp1", "dia1", "diastolic1",
                               "dbp_1", "m4_b"), "DBP reading 1"),
    dbp2 = detect_col(data, c("m5b", "b4", "dbp2", "dia2", "diastolic2",
                               "dbp_2", "m5_b"), "DBP reading 2"),
    dbp3 = detect_col(data, c("m6b", "b6", "dbp3", "dia3", "diastolic3",
                               "dbp_3", "m6_b"), "DBP reading 3"),
    bp_meds = detect_col(data, c("m7", "h3", "b7", "bp_meds",
                                  "antihypertensive", "htn_meds",
                                  "bp_medication"), "BP medications"),
    # M8 = pregnancy status (for excluding pregnant women from BMI)
    pregnant     = detect_col(data, c("m8", "pregnant", "pregnancy"), "Pregnancy status"),
    # M16a-c = heart rate readings
    heart_rate1  = detect_col(data, c("m16a", "hr1", "heart_rate_1"), "Heart rate reading 1"),
    heart_rate2  = detect_col(data, c("m16b", "hr2", "heart_rate_2"), "Heart rate reading 2"),
    heart_rate3  = detect_col(data, c("m16c", "hr3", "heart_rate_3"), "Heart rate reading 3"),
    # Derived mean heart rate (some datasets pre-compute this)
    mean_hr      = detect_col(data, c("m_hr", "mean_hr", "mean_heart_rate"), "Mean heart rate"),

    # -- Step 3: Biochemical ---------------------------------------------------
    # v3.1: C1=glucose, C5=DM meds, C6=chol, C10=chol meds
    # v3.2: B5=glucose, B6=DM meds, B8=chol, B9=chol meds, B16=trig, B17=HDL
    fasting_glucose = detect_col(data, c("b5", "b5_mmol", "c1_mmol",
                                          "fasting_glucose", "glucose_fasting",
                                          "fbg", "fpg"), "Fasting blood glucose"),
    random_glucose  = detect_col(data, c("c1_rand", "glucose_random", "rbg",
                                          "random_glucose"), "Random blood glucose"),
    fasting_status  = detect_col(data, c("b1", "fasting", "fasting_status",
                                          "c1_fast"), "Fasting status"),
    dm_meds         = detect_col(data, c("b6", "h8", "c5", "dm_meds",
                                          "diabetes_meds", "insulin",
                                          "dm_medication"), "Diabetes medications"),
    total_chol      = detect_col(data, c("b8", "b8_mmol", "c6", "cholesterol",
                                          "total_chol", "tc_mmol",
                                          "total_cholesterol"), "Total cholesterol"),
    hdl_chol        = detect_col(data, c("b17", "b17_mmol", "hdl", "hdl_chol",
                                          "hdl_cholesterol"), "HDL cholesterol"),
    triglycerides   = detect_col(data, c("b16", "b16_mmol", "trig",
                                          "triglycerides", "tg_mmol"), "Triglycerides"),
    chol_meds       = detect_col(data, c("b9", "h14", "c10", "chol_meds",
                                          "statin", "chol_medication"), "Cholesterol medications")
  )

  n_found <- sum(!sapply(cols, is.null))
  message(glue::glue("  \u2192 {n_found}/{length(cols)} columns detected automatically"))
  return(cols)
}

#' Read a column mapping file
#'
#' Reads a filled-in column mapping template (Excel or CSV) and returns a
#' named list suitable for passing to [clean_steps_data()].  The mapping file
#' should have at least two columns: one with the standard variable name
#' (column A) and one with the user's column name (column C in the template,
#' or the third column).
#'
#' This function is the manual alternative to [detect_steps_columns()].
#' Use it when your dataset has non-standard variable names that
#' auto-detection cannot resolve.
#'
#' @param path Path to the filled mapping file (.xlsx or .csv).
#' @param data Optional data frame. If provided, the function validates that
#'   every mapped column actually exists in the data.
#'
#' @return A named list where names are standard variable identifiers
#'   (e.g. \code{"age"}, \code{"sbp1"}) and values are the corresponding
#'   column names in the user's dataset.
#'   Unmapped variables are set to \code{NULL}.
#'
#' @details
#' A blank template can be obtained from
#' \code{system.file("templates", "column_mapping_template.xlsx",
#'                    package = "stepssurvey")}
#' or downloaded from the Shiny app.
#'
#' The function ignores domain-header rows (rows where column A is all-caps
#' with no entry in column C) and skips any row where the user's column name
#' is blank.
#'
#' @examples
#' \dontrun{
#'   cols <- read_column_mapping("my_mapping.xlsx")
#'   raw  <- import_steps_data("survey.dta")
#'   clean <- clean_steps_data(raw, cols)
#' }
#'
#' @export
read_column_mapping <- function(path, data = NULL) {
  if (!file.exists(path)) {
    stop(glue::glue("Mapping file not found: {path}"), call. = FALSE)
  }

  ext <- tolower(tools::file_ext(path))

  mapping_df <- switch(ext,
    "xlsx" = readxl::read_excel(path, sheet = "Column Mapping",
                                col_types = "text"),
    "xls"  = readxl::read_excel(path, sheet = "Column Mapping",
                                col_types = "text"),
    "csv"  = utils::read.csv(path, stringsAsFactors = FALSE,
                              colClasses = "character"),
    stop(glue::glue("Unsupported mapping file format: .{ext}"), call. = FALSE)
  )

  # Expect columns: Standard Variable (1), Description (2), Your Column Name (3)
  if (ncol(mapping_df) < 3) {
    stop("Mapping file must have at least 3 columns: ",
         "Standard Variable, Description, Your Column Name", call. = FALSE)
  }

  # Use positional columns (robust to header renaming)
  std_col  <- mapping_df[[1]]
  user_col <- mapping_df[[3]]

  # Build the cols list
  cols <- list()
  n_mapped <- 0
  n_missing <- 0
  missing_vars <- character(0)

  for (i in seq_along(std_col)) {
    std_name <- trimws(as.character(std_col[i]))
    usr_name <- trimws(as.character(user_col[i]))

    # Skip blank rows, domain headers (all-caps with no mapping), NA values
    if (is.na(std_name) || nchar(std_name) == 0) next
    if (std_name == toupper(std_name) && grepl("[A-Z]", std_name)) next
    if (is.na(usr_name) || nchar(usr_name) == 0) {
      cols[[std_name]] <- NULL
      next
    }

    # Validate against actual data if provided
    if (!is.null(data) && !(usr_name %in% names(data))) {
      missing_vars <- c(missing_vars, usr_name)
      n_missing <- n_missing + 1
    }

    cols[[std_name]] <- usr_name
    n_mapped <- n_mapped + 1
  }

  message(glue::glue("  \u2713 Column mapping loaded: {n_mapped} variables mapped"))


  if (n_missing > 0) {
    warning(glue::glue(
      "{n_missing} mapped column(s) not found in data: ",
      "{paste(missing_vars, collapse = ', ')}"
    ), call. = FALSE)
  }

  # Ensure all standard names that exist in detect_steps_columns are present
  # (as NULL if not mapped), so downstream code can safely check cols$xyz
  all_standard <- c(
    "age", "sex", "agerange", "valid", "urban_rural",
    "weight_step1", "weight_step2", "weight_step3", "strata", "psu",
    "tobacco_current", "tobacco_daily", "tobacco_start_age", "tobacco_duration",
    "tobacco_cig_day", "tobacco_cig_week", "tobacco_hand_day",
    "tobacco_pipe_day", "tobacco_cigar_day", "tobacco_shisha_day",
    "tobacco_quit_attempt", "tobacco_quit_advice", "tobacco_past",
    "tobacco_past_daily", "tobacco_quit_age", "tobacco_quit_duration",
    "smokeless_current", "smokeless_daily", "slt_snuff_mouth",
    "slt_chewing", "slt_betel", "smokeless_past", "smokeless_past_daily",
    "secondhand_home", "secondhand_work", "any_tobacco",
    "alcohol_ever", "alcohol_12m", "alcohol_current", "heavy_episode",
    "alcohol_stopped", "alcohol_freq_12m", "alcohol_occasions",
    "alcohol_drinks_occasion", "alcohol_largest_drinks",
    "alcohol_6plus_count", "alcohol_7day_freq", "alcohol_homebrew",
    "drinking_level",
    "met_total", "pa_work_vig", "pa_work_mod", "pa_transport",
    "pa_rec_vig", "pa_rec_mod", "pa_sedentary",
    "fruit_days", "fruit_servings", "veg_days", "veg_servings",
    "salt_table", "salt_cooking", "processed_salt", "salt_perception",
    "salt_importance", "salt_knowledge",
    "salt_limit_processed", "salt_check_labels", "salt_buy_low",
    "salt_use_spices", "salt_avoid_outside", "salt_other_action",
    "oil_type", "meals_outside",
    "pa_work_vig_days", "pa_work_vig_hrs", "pa_work_vig_min",
    "pa_work_mod_days", "pa_work_mod_hrs", "pa_work_mod_min",
    "pa_transport_days", "pa_transport_hrs", "pa_transport_min",
    "pa_rec_vig_days", "pa_rec_vig_hrs", "pa_rec_vig_min",
    "pa_rec_mod_days", "pa_rec_mod_hrs", "pa_rec_mod_min",
    "pa_sedentary_hrs", "pa_sedentary_min",
    "bp_ever_measured", "bp_diagnosed", "bp_diagnosed_12m",
    "bp_trad_healer", "bp_herbal",
    "glucose_ever_measured", "dm_diagnosed", "dm_diagnosed_12m",
    "dm_insulin", "dm_trad_healer", "dm_herbal",
    "chol_ever_measured", "chol_diagnosed", "chol_diagnosed_12m",
    "chol_trad_healer", "chol_herbal",
    "cvd_history", "cvd_aspirin", "cvd_statins",
    "adv_quit_tobacco", "adv_reduce_salt", "adv_fruit_veg",
    "adv_reduce_fat", "adv_more_pa", "adv_healthy_wt",
    "cervical_screen",
    "education_years", "education_level", "ethnicity",
    "marital_status", "employment", "income",
    "height", "weight", "waist", "hip",
    "sbp1", "sbp2", "sbp3", "dbp1", "dbp2", "dbp3",
    "bp_meds", "pregnant", "heart_rate1", "heart_rate2", "heart_rate3",
    "mean_hr",
    "fasting_glucose", "random_glucose", "fasting_status",
    "dm_meds", "total_chol", "hdl_chol", "triglycerides", "chol_meds"
  )

  for (nm in all_standard) {
    if (!(nm %in% names(cols))) {
      cols[[nm]] <- NULL
    }
  }

  return(cols)
}

test_that("detect_col finds columns case-insensitively", {
  df <- data.frame(Age = 1:5, SEX = c("M", "F", "M", "F", "M"))
  expect_equal(detect_col(df, c("age", "c1"), "Age"), "Age")
  expect_equal(detect_col(df, c("sex", "gender"), "Sex"), "SEX")
  expect_null(detect_col(df, c("missing_col"), "Missing"))
})

test_that("detect_steps_columns returns named list", {
  df <- data.frame(
    age = 18:22, sex = c(1, 2, 1, 2, 1),
    wt_final = rep(1, 5), stratum = rep("S1", 5), psu = rep("P1", 5),
    t1 = c(1, 0, 1, 0, 1), a1 = c(0, 1, 0, 1, 0)
  )
  cols <- detect_steps_columns(df)
  expect_type(cols, "list")
  expect_equal(cols$age, "age")
  expect_equal(cols$sex, "sex")
  expect_equal(cols$tobacco_current, "t1")
})

test_that("generate_test_data produces valid data frame", {
  td <- generate_test_data(n = 100, seed = 1)
  expect_s3_class(td, "data.frame")
  expect_equal(nrow(td), 100)
  expect_true(all(c("age", "sex", "wt_final", "stratum", "psu") %in% names(td)))
  expect_true(all(td$age >= 18 & td$age <= 69))
  expect_true(all(td$sex %in% c(1, 2)))
})

test_that("read_column_mapping reads CSV mapping correctly", {
  # Create a minimal CSV mapping
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp))

  mapping <- data.frame(
    Standard.Variable = c("DEMOGRAPHICS", "age", "sex", "sbp1", "height"),
    Description = c(NA, "Age", "Sex", "SBP 1", "Height cm"),
    Your.Column.Name = c(NA, "my_age", "my_sex", "bp_systolic_1", ""),
    stringsAsFactors = FALSE
  )
  write.csv(mapping, tmp, row.names = FALSE)

  cols <- read_column_mapping(tmp)

  # Mapped variables should have user's column names

  expect_equal(cols$age, "my_age")
  expect_equal(cols$sex, "my_sex")
  expect_equal(cols$sbp1, "bp_systolic_1")

  # Unmapped variable (blank "Your Column Name") should be NULL
  expect_null(cols$height)

  # Domain header row should be skipped (not appear as a key)
  expect_false("DEMOGRAPHICS" %in% names(cols))
})

test_that("read_column_mapping validates against data", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp))

  mapping <- data.frame(
    Standard.Variable = c("age", "sex"),
    Description = c("Age", "Sex"),
    Your.Column.Name = c("my_age", "nonexistent_col"),
    stringsAsFactors = FALSE
  )
  write.csv(mapping, tmp, row.names = FALSE)

  fake_data <- data.frame(my_age = 1:5, my_sex = c(1, 2, 1, 2, 1))

  # Should warn about nonexistent_col

  expect_warning(
    cols <- read_column_mapping(tmp, data = fake_data),
    "not found in data"
  )

  # But still map it (the user might fix the data later)
  expect_equal(cols$sex, "nonexistent_col")
})

test_that("read_column_mapping maps provided variables correctly", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp))

  mapping <- data.frame(
    Standard.Variable = c("age", "sbp1", "fasting_glucose"),
    Description = c("Age", "SBP 1", "Glucose"),
    Your.Column.Name = c("my_age", "my_sbp", "my_glucose"),
    stringsAsFactors = FALSE
  )
  write.csv(mapping, tmp, row.names = FALSE)

  cols <- read_column_mapping(tmp)

  # Mapped variables should be present
  expect_equal(cols$age, "my_age")
  expect_equal(cols$sbp1, "my_sbp")
  expect_equal(cols$fasting_glucose, "my_glucose")

  # Unmapped variables should be NULL (absent from list in R)
  expect_null(cols$height)
  expect_null(cols$weight)
})

test_that("read_column_mapping errors on missing file", {
  expect_error(read_column_mapping("nonexistent.xlsx"), "not found")
})

test_that("read_column_mapping errors on unsupported format", {
  tmp <- tempfile(fileext = ".json")
  on.exit(unlink(tmp))
  writeLines("{}", tmp)
  expect_error(read_column_mapping(tmp), "Unsupported")
})

test_that("read_column_mapping integrates with clean_steps_data", {
  # Create mapping that maps demo data columns to themselves
  td <- generate_test_data(n = 100, seed = 42)

  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp))

  # Map all key columns using their actual names from demo data
  auto_cols <- detect_steps_columns(td)
  mapping <- data.frame(
    Standard.Variable = c("age", "sex", "weight_step1", "strata", "psu",
                           "tobacco_current", "sbp1", "sbp2", "sbp3",
                           "dbp1", "dbp2", "dbp3", "height", "weight"),
    Description = rep("", 14),
    Your.Column.Name = c(auto_cols$age, auto_cols$sex, auto_cols$weight_step1,
                          auto_cols$strata, auto_cols$psu,
                          auto_cols$tobacco_current,
                          auto_cols$sbp1, auto_cols$sbp2, auto_cols$sbp3,
                          auto_cols$dbp1, auto_cols$dbp2, auto_cols$dbp3,
                          auto_cols$height, auto_cols$weight),
    stringsAsFactors = FALSE
  )
  write.csv(mapping, tmp, row.names = FALSE)

  cols <- read_column_mapping(tmp, data = td)
  clean <- clean_steps_data(td, cols)

  expect_s3_class(clean, "data.frame")
  expect_true(nrow(clean) > 0)
  expect_true("bmi" %in% names(clean))
})

#' Data Quality Module — UI
#'
#' @param id Module namespace id.
#' @keywords internal
mod_quality_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      title = "Data Quality",
      width = 280,
      shiny::actionButton(
        ns("btn_run"), "Run diagnostics",
        class = "btn-primary btn-lg", width = "100%",
        icon = shiny::icon("stethoscope")
      ),
      shiny::hr(),
      shiny::h6("Digit preference"),
      shiny::selectInput(
        ns("dp_measure"), "Measurement",
        choices = c("SBP", "DBP", "Height", "Weight", "Waist"),
        selected = "SBP"
      ),
      shiny::hr(),
      shiny::h6("Weight diagnostics"),
      shiny::selectInput(
        ns("wt_step"), "Sampling weight",
        choices = c(
          "Step 1" = "weight_step1",
          "Step 2" = "weight_step2",
          "Step 3" = "weight_step3"
        ),
        selected = "weight_step1"
      )
    ),

    # ---- Summary cards ----
    bslib::layout_column_wrap(
      width = 1 / 4,
      bslib::value_box(
        "Heaping severity",
        shiny::textOutput(ns("vb_heaping")),
        theme = "primary",
        showcase = shiny::icon("hashtag")
      ),
      bslib::value_box(
        "Overall completeness",
        shiny::textOutput(ns("vb_completeness")),
        theme = "success",
        showcase = shiny::icon("check-circle")
      ),
      bslib::value_box(
        "Implausible values",
        shiny::textOutput(ns("vb_implausible")),
        theme = "warning",
        showcase = shiny::icon("exclamation-triangle")
      ),
      bslib::value_box(
        "Weight CV",
        shiny::textOutput(ns("vb_weight_cv")),
        theme = "info",
        showcase = shiny::icon("balance-scale")
      )
    ),

    # ---- Tabbed panels ----
    bslib::navset_card_tab(
      title = "Diagnostics",

      bslib::nav_panel(
        "Digit preference",
        shiny::plotOutput(ns("plot_dp"), height = "420px"),
        shiny::br(),
        DT::DTOutput(ns("tbl_dp_summary"))
      ),

      bslib::nav_panel(
        "Completeness",
        shiny::plotOutput(ns("plot_completeness"), height = "520px"),
        shiny::br(),
        DT::DTOutput(ns("tbl_completeness"))
      ),

      bslib::nav_panel(
        "Plausibility",
        DT::DTOutput(ns("tbl_plausibility"))
      ),

      bslib::nav_panel(
        "Sampling weights",
        shiny::plotOutput(ns("plot_weights"), height = "400px"),
        shiny::br(),
        DT::DTOutput(ns("tbl_weights"))
      )
    )
  )
}


#' Data Quality Module — Server
#'
#' @param id Module namespace id.
#' @param upload_out Reactive list returned by [mod_upload_server()], containing
#'   `raw` (data frame), `cols` (column mapping), and `config` (survey config).
#' @param clean_out Reactive returning the cleaned data frame from
#'   [mod_clean_server()].
#'
#' @return A reactive returning the `steps_quality` object (or NULL).
#' @keywords internal
mod_quality_server <- function(id, upload_out, clean_out) {
  shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns

    # Run diagnostics on button click
    dq <- shiny::eventReactive(input$btn_run, {
      shiny::req(upload_out$raw(), upload_out$cols())

      # Use cleaned data if available, otherwise pass raw twice
      cleaned <- tryCatch(clean_out(), error = function(e) NULL)
      if (is.null(cleaned)) cleaned <- upload_out$raw()

      shiny::withProgress(message = "Running data quality diagnostics...", {
        steps_data_quality(
          raw     = upload_out$raw(),
          cleaned = cleaned,
          cols    = upload_out$cols()
        )
      })
    })

    # ---- Value boxes ----

    output$vb_heaping <- shiny::renderText({
      shiny::req(dq())
      dp <- dq()$digit_preference
      if (length(dp) == 0) return("N/A")
      # Worst severity across all measures
      sevs <- vapply(dp, function(x) x$severity, character(1))
      sev_order <- c("None", "Mild", "Moderate", "Severe")
      worst <- sev_order[max(match(sevs, sev_order))]
      worst
    })

    output$vb_completeness <- shiny::renderText({
      shiny::req(dq())
      comp <- dq()$completeness
      all_rows <- do.call(rbind, comp)
      # Exclude "(not detected)" columns — they are expected missing
      detected <- all_rows[all_rows$column != "(not detected)", ]
      if (nrow(detected) == 0) return("N/A")
      overall_pct <- round(100 - mean(detected$pct_missing), 1)
      paste0(overall_pct, "%")
    })

    output$vb_implausible <- shiny::renderText({
      shiny::req(dq())
      pl <- dq()$plausibility
      if (nrow(pl) == 0) return("0")
      format(sum(pl$n_outlier), big.mark = ",")
    })

    output$vb_weight_cv <- shiny::renderText({
      shiny::req(dq())
      wt <- dq()$weights
      if (length(wt) == 0) return("N/A")
      # Show CV of step 1 weight by default
      first_wt <- wt[[1]]
      sprintf("%.3f", first_wt$cv)
    })

    # ---- Digit preference tab ----

    output$plot_dp <- shiny::renderPlot({
      shiny::req(dq(), input$dp_measure)
      dp <- dq()$digit_preference
      if (!input$dp_measure %in% names(dp)) {
        return(NULL)
      }
      plot_digit_preference(dq(), input$dp_measure)
    })

    output$tbl_dp_summary <- DT::renderDT({
      shiny::req(dq())
      dp <- dq()$digit_preference
      if (length(dp) == 0) return(NULL)

      summary_df <- data.frame(
        Measure      = names(dp),
        N            = vapply(dp, function(x) x$n, integer(1)),
        `Heaping (0)` = vapply(dp, function(x) x$heaping_0, numeric(1)),
        `Heaping (5)` = vapply(dp, function(x) x$heaping_5, numeric(1)),
        `Heaping (0+5)` = vapply(dp, function(x) x$heaping_0_5, numeric(1)),
        Severity     = vapply(dp, function(x) x$severity, character(1)),
        stringsAsFactors = FALSE,
        check.names = FALSE
      )

      DT::datatable(
        summary_df,
        options = list(dom = "t", pageLength = 10),
        rownames = FALSE
      )
    })

    # ---- Completeness tab ----

    output$plot_completeness <- shiny::renderPlot({
      shiny::req(dq())
      plot_completeness(dq())
    })

    output$tbl_completeness <- DT::renderDT({
      shiny::req(dq())
      comp <- dq()$completeness
      all_rows <- do.call(rbind, lapply(names(comp), function(domain) {
        df <- comp[[domain]]
        df$domain <- domain
        df
      }))
      all_rows <- all_rows[, c("domain", "variable", "column",
                                "n_total", "n_missing", "pct_missing")]

      DT::datatable(
        all_rows,
        options = list(
          pageLength = 30,
          scrollY = "400px",
          dom = "ft"
        ),
        rownames = FALSE,
        colnames = c("Domain", "Variable", "Column",
                     "Total", "Missing", "Missing %")
      ) |>
        DT::formatStyle(
          "pct_missing",
          backgroundColor = DT::styleInterval(
            c(5, 20, 50),
            c("#d4edda", "#fff3cd", "#f8d7da", "#e24b4a")
          )
        )
    })

    # ---- Plausibility tab ----

    output$tbl_plausibility <- DT::renderDT({
      shiny::req(dq())
      pl <- dq()$plausibility
      if (nrow(pl) == 0) return(NULL)

      pl$range <- sprintf("[%g, %g]", pl$range_low, pl$range_high)
      pl_display <- pl[, c("variable", "n_valid", "n_below", "n_above",
                            "n_outlier", "pct_outlier", "range")]

      DT::datatable(
        pl_display,
        options = list(dom = "t", pageLength = 20),
        rownames = FALSE,
        colnames = c("Variable", "Valid obs", "Below range",
                     "Above range", "Total outliers", "% Outlier",
                     "Plausible range")
      ) |>
        DT::formatStyle(
          "pct_outlier",
          backgroundColor = DT::styleInterval(
            c(1, 5),
            c("#d4edda", "#fff3cd", "#f8d7da")
          )
        )
    })

    # ---- Weights tab ----

    output$plot_weights <- shiny::renderPlot({
      shiny::req(dq(), input$wt_step)
      wt <- dq()$weights
      if (!input$wt_step %in% names(wt)) return(NULL)
      plot_weights(dq(), input$wt_step)
    })

    output$tbl_weights <- DT::renderDT({
      shiny::req(dq())
      wt <- dq()$weights
      if (length(wt) == 0) return(NULL)

      wt_df <- do.call(rbind, lapply(names(wt), function(nm) {
        w <- wt[[nm]]
        data.frame(
          Weight     = w$column,
          N_valid    = w$n_valid,
          N_NA       = w$n_na,
          N_zero     = w$n_zero,
          Min        = w$min,
          Median     = w$median,
          Mean       = w$mean,
          Max        = w$max,
          CV         = w$cv,
          `Max/Min`  = w$ratio_max_min,
          Severity   = w$cv_severity,
          stringsAsFactors = FALSE,
          check.names = FALSE
        )
      }))

      DT::datatable(
        wt_df,
        options = list(dom = "t", pageLength = 5),
        rownames = FALSE
      )
    })

    # Return dq object
    dq
  })
}

#' Launch the stepssurvey Shiny Application
#'
#' Starts the interactive STEPS survey analysis app in the user's browser.
#' The app provides a guided workflow: upload data, clean, set survey design,
#' compute indicators, visualise results, and generate Word reports.
#'
#' @param ... Additional arguments passed to [shiny::shinyApp()].
#'
#' @return A Shiny app object (invisibly). Called for its side effect of
#'   launching the application.
#'
#' @export
#'
#' @examples
#' \dontrun{
#'   run_app()
#' }
run_app <- function(...) {
  # Make inst/www assets (logo) available to the app
  shiny::addResourcePath(
    "www",
    system.file("www", package = "stepssurvey")
  )

  shiny::shinyApp(
    ui     = app_ui(),
    server = app_server,
    ...
  )
}

#' Data Quality Diagnostics for WHO STEPS Data
#'
#' Produces a comprehensive data quality report covering digit preference,
#' completeness, plausibility, and sampling weight diagnostics.
#'
#' @param raw  The raw (pre-cleaning) data frame, typically from
#'   [import_steps_data()].
#' @param cleaned The cleaned data frame from [clean_steps_data()].
#' @param cols Column mapping list from [detect_steps_columns()].
#'
#' @return A list of class `"steps_quality"` with elements:
#'   \describe{
#'     \item{digit_preference}{Terminal-digit tables and heaping indices for
#'       physical measurements (SBP, DBP, height, weight, waist).}
#'     \item{completeness}{Per-variable missingness counts and percentages,
#'       grouped by STEPS domain.}
#'     \item{plausibility}{Summary of values outside plausible ranges.}
#'     \item{weights}{Sampling weight distribution statistics.}
#'   }
#'
#' @details
#' **Digit preference / heaping** is assessed using the Whipple-style
#' heaping index: the ratio of observed frequency at a digit (0 or 5)
#' to the expected frequency under uniform distribution.
#' An index of 1.0 = no preference; >1.5 = moderate heaping; >2.0 = severe.
#'
#' **Completeness** reports missing values for key STEPS variables grouped
#' by Step (behavioural, physical, biochemical).
#'
#' **Plausibility** counts values outside WHO-recommended ranges
#' (e.g. height 100--250 cm, weight 20--300 kg, SBP 60--300 mmHg).
#'
#' **Weight diagnostics** summarise the distribution of sampling weights
#' and flag potential issues (high CV, zero/NA weights).
#'
#' @export
steps_data_quality <- function(raw, cleaned, cols) {

  message("Running data quality diagnostics...")

  dq <- list(
    digit_preference = .digit_preference(raw, cols),
    completeness     = .completeness(raw, cols),
    plausibility     = .plausibility(raw, cols),
    weights          = .weight_diagnostics(raw, cols)
  )

  class(dq) <- c("steps_quality", "list")
  message("Data quality diagnostics complete.")
  dq
}


# ---------------------------------------------------------------------------
# Digit preference / heaping
# ---------------------------------------------------------------------------

#' @keywords internal
.digit_preference <- function(data, cols) {

  # Physical measurement variables to check
  measure_map <- list(
    SBP    = c("sbp1", "sbp2", "sbp3"),
    DBP    = c("dbp1", "dbp2", "dbp3"),
    Height = "height",
    Weight = "weight",
    Waist  = "waist"
  )

  results <- list()

  for (measure_name in names(measure_map)) {
    col_keys <- measure_map[[measure_name]]
    # Collect all values across the relevant columns
    vals <- numeric(0)
    for (ck in col_keys) {
      if (!is.null(cols[[ck]]) && cols[[ck]] %in% names(data)) {
        v <- as.numeric(data[[cols[[ck]]]])
        vals <- c(vals, v[!is.na(v)])
      }
    }

    if (length(vals) < 30) next

    # Terminal digit frequency
    terminal <- vals %% 10
    digit_counts <- table(factor(terminal, levels = 0:9))
    digit_pct    <- as.numeric(digit_counts) / sum(digit_counts) * 100
    names(digit_pct) <- 0:9

    # Heaping indices
    expected_pct <- 10  # uniform expectation
    hi_0 <- digit_pct[["0"]] / expected_pct
    hi_5 <- digit_pct[["5"]] / expected_pct
    hi_0_5 <- (digit_pct[["0"]] + digit_pct[["5"]]) / (2 * expected_pct)

    # Severity assessment
    severity <- dplyr::case_when(
      hi_0_5 > 2.0 ~ "Severe",
      hi_0_5 > 1.5 ~ "Moderate",
      hi_0_5 > 1.2 ~ "Mild",
      TRUE          ~ "None"
    )

    results[[measure_name]] <- list(
      n             = length(vals),
      digit_counts  = as.numeric(digit_counts),
      digit_pct     = digit_pct,
      heaping_0     = round(hi_0, 2),
      heaping_5     = round(hi_5, 2),
      heaping_0_5   = round(hi_0_5, 2),
      severity      = severity
    )
  }

  message(sprintf("  Digit preference: checked %d measures", length(results)))
  results
}


# ---------------------------------------------------------------------------
# Completeness / missingness
# ---------------------------------------------------------------------------

#' @keywords internal
.completeness <- function(data, cols) {

  # Group variables by STEPS domain
  domain_map <- list(
    "Demographics" = c("age", "sex"),
    "Survey design" = c("strata", "psu", "weight_step1", "weight_step2", "weight_step3"),
    "Tobacco (Step 1)" = c("tobacco_current", "tobacco_daily", "smokeless_current",
                            "secondhand_home", "secondhand_work"),
    "Alcohol (Step 1)" = c("alcohol_current", "heavy_episode"),
    "Diet (Step 1)" = c("fruit_days", "fruit_servings", "veg_days", "veg_servings"),
    "Physical activity (Step 1)" = c("pa_work_vig", "pa_work_days", "pa_transport",
                                      "pa_rec_vig", "pa_rec_days"),
    "Physical measurements (Step 2)" = c("height", "weight", "waist",
                                          "sbp1", "sbp2", "sbp3",
                                          "dbp1", "dbp2", "dbp3"),
    "Biochemical (Step 3)" = c("fasting_glucose", "total_cholesterol")
  )

  results <- list()
  n_total <- nrow(data)

  for (domain in names(domain_map)) {
    var_keys <- domain_map[[domain]]
    domain_results <- data.frame(
      variable   = character(0),
      column     = character(0),
      n_total    = integer(0),
      n_missing  = integer(0),
      pct_missing = numeric(0),
      stringsAsFactors = FALSE
    )

    for (vk in var_keys) {
      col_name <- cols[[vk]]
      if (is.null(col_name) || !col_name %in% names(data)) {
        domain_results <- rbind(domain_results, data.frame(
          variable = vk, column = "(not detected)", n_total = n_total,
          n_missing = n_total, pct_missing = 100.0,
          stringsAsFactors = FALSE
        ))
      } else {
        n_miss <- sum(is.na(data[[col_name]]))
        domain_results <- rbind(domain_results, data.frame(
          variable = vk, column = col_name, n_total = n_total,
          n_missing = n_miss, pct_missing = round(n_miss / n_total * 100, 1),
          stringsAsFactors = FALSE
        ))
      }
    }

    results[[domain]] <- domain_results
  }

  message(sprintf("  Completeness: checked %d domains",
                  length(results)))
  results
}


# ---------------------------------------------------------------------------
# Plausibility checks
# ---------------------------------------------------------------------------

#' @keywords internal
.plausibility <- function(data, cols) {

  # WHO-recommended plausible ranges
  range_checks <- list(
    list(key = "height",  label = "Height (cm)",         lo = 100, hi = 250),
    list(key = "weight",  label = "Weight (kg)",         lo = 20,  hi = 300),
    list(key = "waist",   label = "Waist (cm)",          lo = 40,  hi = 200),
    list(key = "sbp1",    label = "SBP reading 1 (mmHg)",lo = 60,  hi = 300),
    list(key = "sbp2",    label = "SBP reading 2 (mmHg)",lo = 60,  hi = 300),
    list(key = "sbp3",    label = "SBP reading 3 (mmHg)",lo = 60,  hi = 300),
    list(key = "dbp1",    label = "DBP reading 1 (mmHg)",lo = 30,  hi = 200),
    list(key = "dbp2",    label = "DBP reading 2 (mmHg)",lo = 30,  hi = 200),
    list(key = "dbp3",    label = "DBP reading 3 (mmHg)",lo = 30,  hi = 200),
    list(key = "fasting_glucose", label = "Fasting glucose (mmol/L)", lo = 1, hi = 40),
    list(key = "total_cholesterol", label = "Total cholesterol (mmol/L)", lo = 1, hi = 20)
  )

  results <- data.frame(
    variable   = character(0),
    n_valid    = integer(0),
    n_below    = integer(0),
    n_above    = integer(0),
    n_outlier  = integer(0),
    pct_outlier = numeric(0),
    range_low  = numeric(0),
    range_high = numeric(0),
    stringsAsFactors = FALSE
  )

  for (rc in range_checks) {
    col_name <- cols[[rc$key]]
    if (is.null(col_name) || !col_name %in% names(data)) next

    vals <- as.numeric(data[[col_name]])
    valid <- vals[!is.na(vals)]
    if (length(valid) == 0) next

    n_below <- sum(valid < rc$lo)
    n_above <- sum(valid > rc$hi)
    n_out   <- n_below + n_above

    results <- rbind(results, data.frame(
      variable    = rc$label,
      n_valid     = length(valid),
      n_below     = n_below,
      n_above     = n_above,
      n_outlier   = n_out,
      pct_outlier = round(n_out / length(valid) * 100, 2),
      range_low   = rc$lo,
      range_high  = rc$hi,
      stringsAsFactors = FALSE
    ))
  }

  message(sprintf("  Plausibility: checked %d variables", nrow(results)))
  results
}


# ---------------------------------------------------------------------------
# Sampling weight diagnostics
# ---------------------------------------------------------------------------

#' @keywords internal
.weight_diagnostics <- function(data, cols) {

  wt_keys <- c("weight_step1", "weight_step2", "weight_step3")
  results <- list()

  for (wk in wt_keys) {
    col_name <- cols[[wk]]
    if (is.null(col_name) || !col_name %in% names(data)) next

    w <- as.numeric(data[[col_name]])
    n_total <- length(w)
    n_na    <- sum(is.na(w))
    n_zero  <- sum(w == 0, na.rm = TRUE)
    w_valid <- w[!is.na(w) & w > 0]

    if (length(w_valid) == 0) next

    cv <- stats::sd(w_valid) / mean(w_valid)

    # Severity assessment for CV
    cv_severity <- dplyr::case_when(
      cv > 1.5 ~ "High (>1.5) -- extreme weight variability",
      cv > 1.0 ~ "Moderate (>1.0)",
      cv > 0.5 ~ "Acceptable (0.5-1.0)",
      TRUE     ~ "Low (<0.5)"
    )

    results[[wk]] <- list(
      column    = col_name,
      n_total   = n_total,
      n_na      = n_na,
      n_zero    = n_zero,
      n_valid   = length(w_valid),
      min       = round(min(w_valid), 4),
      q1        = round(stats::quantile(w_valid, 0.25), 4),
      median    = round(stats::median(w_valid), 4),
      mean      = round(mean(w_valid), 4),
      q3        = round(stats::quantile(w_valid, 0.75), 4),
      max       = round(max(w_valid), 4),
      sd        = round(stats::sd(w_valid), 4),
      cv        = round(cv, 3),
      cv_severity = cv_severity,
      ratio_max_min = round(max(w_valid) / min(w_valid), 1)
    )
  }

  message(sprintf("  Weight diagnostics: checked %d weight columns", length(results)))
  results
}


# ---------------------------------------------------------------------------
# Plotting helpers
# ---------------------------------------------------------------------------

#' Plot digit preference histogram for a physical measurement
#'
#' Creates a bar chart of terminal-digit frequencies with the expected
#' uniform line at 10 %.
#'
#' @param dq A `steps_quality` object from [steps_data_quality()].
#' @param measure Character: one of "SBP", "DBP", "Height", "Weight", "Waist".
#'
#' @return A ggplot object.
#' @export
plot_digit_preference <- function(dq, measure) {
  if (!measure %in% names(dq$digit_preference)) {
    stop(sprintf("Measure '%s' not found. Available: %s",
                 measure, paste(names(dq$digit_preference), collapse = ", ")))
  }

  dp <- dq$digit_preference[[measure]]

  df <- data.frame(
    digit = factor(0:9),
    pct   = dp$digit_pct
  )

  ggplot2::ggplot(df, ggplot2::aes(x = digit, y = pct)) +
    ggplot2::geom_col(
      fill = dplyr::case_when(
        dp$severity == "Severe"   ~ "#E24B4A",
        dp$severity == "Moderate" ~ "#EF9F27",
        dp$severity == "Mild"     ~ "#639922",
        TRUE                      ~ "#378ADD"
      ),
      alpha = 0.85
    ) +
    ggplot2::geom_hline(yintercept = 10, linetype = "dashed",
                        colour = "grey40", linewidth = 0.6) +
    ggplot2::annotate("text", x = 9.5, y = 11, label = "Expected (10%)",
                      size = 3, hjust = 1, colour = "grey40") +
    ggplot2::labs(
      title    = sprintf("%s: terminal digit distribution (n = %s)",
                         measure, format(dp$n, big.mark = ",")),
      subtitle = sprintf("Heaping index (0+5): %.2f -- %s", dp$heaping_0_5, dp$severity),
      x = "Terminal digit",
      y = "Frequency (%)"
    ) +
    ggplot2::scale_y_continuous(limits = c(0, NA), expand = ggplot2::expansion(mult = c(0, 0.05))) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      plot.title    = ggplot2::element_text(face = "bold", size = 14),
      plot.subtitle = ggplot2::element_text(size = 11, colour = "grey40"),
      panel.grid.major.x = ggplot2::element_blank()
    )
}


#' Plot completeness heatmap across STEPS domains
#'
#' Creates a tile heatmap showing missingness percentage by variable,
#' grouped by STEPS domain.
#'
#' @param dq A `steps_quality` object from [steps_data_quality()].
#'
#' @return A ggplot object.
#' @export
plot_completeness <- function(dq) {
  # Combine all domains into one data frame
  all_comp <- do.call(rbind, lapply(names(dq$completeness), function(domain) {
    df <- dq$completeness[[domain]]
    df$domain <- domain
    df
  }))

  # Order by domain appearance, then variable within domain
  all_comp$variable <- factor(all_comp$variable,
                               levels = rev(all_comp$variable))

  ggplot2::ggplot(all_comp, ggplot2::aes(x = domain, y = variable, fill = pct_missing)) +
    ggplot2::geom_tile(colour = "white", linewidth = 0.5) +
    ggplot2::geom_text(ggplot2::aes(label = sprintf("%.0f%%", pct_missing)),
                       size = 3, colour = ifelse(all_comp$pct_missing > 50, "white", "grey20")) +
    ggplot2::scale_fill_gradient2(
      low = "#1D9E75", mid = "#EF9F27", high = "#E24B4A",
      midpoint = 25, limits = c(0, 100),
      name = "Missing %"
    ) +
    ggplot2::labs(
      title = "Variable completeness by STEPS domain",
      x = NULL, y = NULL
    ) +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 35, hjust = 1, size = 10),
      plot.title   = ggplot2::element_text(face = "bold", size = 14),
      panel.grid   = ggplot2::element_blank()
    )
}


#' Plot sampling weight distribution
#'
#' Creates a histogram of sampling weights with summary statistics.
#'
#' @param dq A `steps_quality` object from [steps_data_quality()].
#' @param step Character: which weight to plot ("weight_step1", "weight_step2",
#'   or "weight_step3"). Defaults to "weight_step1".
#'
#' @return A ggplot object.
#' @export
plot_weights <- function(dq, step = "weight_step1") {
  if (!step %in% names(dq$weights)) {
    stop(sprintf("Weight '%s' not found. Available: %s",
                 step, paste(names(dq$weights), collapse = ", ")))
  }

  wt_info <- dq$weights[[step]]

  # We need the original data to plot the histogram, but we only have

  # summary stats. Return a stat summary plot instead.
  df <- data.frame(
    stat = c("Min", "Q1", "Median", "Mean", "Q3", "Max"),
    value = c(wt_info$min, wt_info$q1, wt_info$median,
              wt_info$mean, wt_info$q3, wt_info$max)
  )
  df$stat <- factor(df$stat, levels = df$stat)

  subtitle <- sprintf(
    "n = %s | NA: %d | Zero: %d | CV: %.3f (%s) | Max/Min: %.1f",
    format(wt_info$n_valid, big.mark = ","),
    wt_info$n_na, wt_info$n_zero,
    wt_info$cv, wt_info$cv_severity,
    wt_info$ratio_max_min
  )

  ggplot2::ggplot(df, ggplot2::aes(x = stat, y = value)) +
    ggplot2::geom_col(fill = "#378ADD", alpha = 0.8) +
    ggplot2::geom_text(ggplot2::aes(label = round(value, 3)),
                       vjust = -0.5, size = 3.5) +
    ggplot2::labs(
      title    = sprintf("Sampling weight distribution: %s", wt_info$column),
      subtitle = subtitle,
      x = NULL, y = "Weight value"
    ) +
    ggplot2::scale_y_continuous(expand = ggplot2::expansion(mult = c(0, 0.15))) +
    ggplot2::theme_minimal(base_size = 13) +
    ggplot2::theme(
      plot.title    = ggplot2::element_text(face = "bold", size = 14),
      plot.subtitle = ggplot2::element_text(size = 10, colour = "grey40"),
      panel.grid.major.x = ggplot2::element_blank()
    )
}

test_that("steps_data_quality returns expected structure", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  dq <- steps_data_quality(td, clean, cols)

  expect_s3_class(dq, "steps_quality")
  expect_true(all(c("digit_preference", "completeness",
                     "plausibility", "weights") %in% names(dq)))
})

test_that("digit_preference returns data for SBP", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  dq <- steps_data_quality(td, clean, cols)
  dp <- dq$digit_preference

  # Should have SBP entry
  expect_true("SBP" %in% names(dp))

  # SBP entry should have digit_pct and heaping_0_5
  sbp_dp <- dp$SBP
  expect_true("digit_pct" %in% names(sbp_dp))
  expect_true("heaping_0_5" %in% names(sbp_dp))
  expect_true(is.numeric(sbp_dp$heaping_0_5))
})

test_that("completeness returns a named list of data frames", {
  td <- generate_test_data(n = 200, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  dq <- steps_data_quality(td, clean, cols)
  comp <- dq$completeness

  # completeness is a named list of data frames (one per domain)
  expect_type(comp, "list")
  expect_true(length(comp) > 0)

  # Each domain entry should be a data frame with pct_missing
  first_domain <- comp[[1]]
  expect_s3_class(first_domain, "data.frame")
  expect_true(nrow(first_domain) > 0)
  expect_true("pct_missing" %in% names(first_domain))
})

test_that("weight diagnostics returns weight stats", {
  td <- generate_test_data(n = 200, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)

  dq <- steps_data_quality(td, clean, cols)
  wt <- dq$weights

  expect_type(wt, "list")
  # Should have at least Step 1 weight info
  expect_true(length(wt) > 0)
})

test_that("plot_digit_preference produces a ggplot", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  dq <- steps_data_quality(td, clean, cols)

  # plot_digit_preference requires a measure argument
  p <- plot_digit_preference(dq, "SBP")
  expect_s3_class(p, "ggplot")
})

test_that("plot_completeness produces a ggplot", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  dq <- steps_data_quality(td, clean, cols)

  p <- plot_completeness(dq)
  expect_s3_class(p, "ggplot")
})

test_that("plot_weights produces a ggplot", {
  td <- generate_test_data(n = 300, seed = 42)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  dq <- steps_data_quality(td, clean, cols)

  p <- plot_weights(dq)
  expect_s3_class(p, "ggplot")
})

#' Compute All STEPS Indicators
#'
#' Runs all indicator modules (tobacco, alcohol, diet & physical activity,
#' anthropometry, blood pressure, and biochemical), using the appropriate
#' step-specific survey design for each domain per WHO STEPS methodology:
#' - Step 1 (behavioural): tobacco, alcohol, diet & physical activity
#' - Step 2 (physical):    anthropometry, blood pressure
#' - Step 3 (biochemical): biochemical measures
#'
#' @param design A `steps_designs` list from [setup_survey_design()]
#'   (with elements `$step1`, `$step2`, `$step3`), or a single
#'   [survey::svydesign] object for backward compatibility.
#'
#' @return A list with two elements:
#'   - `results`: a named list containing indicator results grouped by domain
#'     (tobacco, alcohol, diet_pa, anthropometry, blood_pressure, biochemical)
#'   - `key_indicators`: a tibble with columns domain, indicator, estimate,
#'     lower, and upper, summarising headline estimates across all domains
#'
#' @export
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   all_indicators <- compute_all_indicators(design)
#'   names(all_indicators$results)
#' }
compute_all_indicators <- function(design) {
  # Support both old (single design) and new (steps_designs list) format

  if (inherits(design, "steps_designs")) {
    d1 <- design$step1
    d2 <- design$step2
    d3 <- design$step3
  } else {
    # Backward compatibility: single design for all steps
    d1 <- design
    d2 <- design
    d3 <- design
  }

  # Step 1 — Behavioural indicators
  message("Computing tobacco indicators...")
  tobacco <- compute_tobacco_indicators(d1)
  message("Computing alcohol indicators...")
  alcohol <- compute_alcohol_indicators(d1)
  message("Computing diet & physical activity indicators...")
  diet_pa <- compute_diet_pa_indicators(d1)

  # Step 2 — Physical measurement indicators
  message("Computing anthropometry indicators...")
  anthropometry <- compute_anthropometry_indicators(d2)
  message("Computing blood pressure indicators...")
  blood_pressure <- compute_bp_indicators(d2)

  # Step 3 — Biochemical indicators
  message("Computing biochemical indicators...")
  biochemical <- compute_biochemical_indicators(d3)

  indicators <- list(
    tobacco        = tobacco,
    alcohol        = alcohol,
    diet_pa        = diet_pa,
    anthropometry  = anthropometry,
    blood_pressure = blood_pressure,
    biochemical    = biochemical
  )

  # Prefer current_tobacco_any (smoking + smokeless) as headline; fall back

  tob_key <- if (!is.null(indicators$tobacco[["current_tobacco_any_total"]])) {
    "current_tobacco_any_total"
  } else {
    "current_tobacco_total"
  }

  key_indicators <- dplyr::bind_rows(
    extract_key(indicators$tobacco,       "Tobacco",       tob_key,                    "Current tobacco use"),
    extract_key(indicators$alcohol,       "Alcohol",       "current_alcohol_total",    "Current alcohol use"),
    extract_key(indicators$alcohol,       "Alcohol",       "heavy_episodic_total",     "Heavy episodic drinking"),
    extract_key(indicators$diet_pa,       "Physical Activity", "insufficient_pa_total", "Insufficient physical activity"),
    extract_key(indicators$diet_pa,       "Diet",          "low_fruit_veg_total",      "Low fruit & vegetable intake (<5 servings/day)"),
    extract_key(indicators$anthropometry, "Obesity",       "overweight_obese_total",   "Overweight or obese (BMI >=25)"),
    extract_key(indicators$anthropometry, "Obesity",       "obese_total",              "Obese (BMI >=30)"),
    extract_key(indicators$blood_pressure,"Blood Pressure","raised_bp_total",          "Raised blood pressure"),
    extract_key(indicators$biochemical,   "Diabetes",      "raised_glucose_total",     "Raised fasting blood glucose"),
    extract_key(indicators$biochemical,   "Cholesterol",   "raised_chol_total",        "Raised total cholesterol")
  )

  message(glue::glue("\u2713 Computed {nrow(key_indicators)} key indicators across all domains."))

  return(list(results = indicators, key_indicators = key_indicators))
}


#' Extract Key Indicator for Fact Sheet
#'
#' Internal helper function to extract a single key indicator from an
#' indicator list and format it as a tibble row with domain and label.
#'
#' @param ind_list A list of indicator results (e.g., from compute_tobacco_indicators)
#' @param domain The domain name (e.g., "Tobacco", "Alcohol")
#' @param key The result name to extract (e.g., "current_tobacco_total")
#' @param label The display label for the indicator
#'
#' @return A tibble row with columns: domain, indicator, estimate, lower, upper,
#'   or NULL if the key does not exist in ind_list
#'
#' @keywords internal
extract_key <- function(ind_list, domain, key, label) {
  if (!is.null(ind_list[[key]])) {
    row <- ind_list[[key]] |>
      head(1) |>
      dplyr::mutate(domain = domain, indicator = label) |>
      dplyr::select(domain, indicator, estimate, lower, upper)
    return(row)
  }
  return(NULL)
}

#' Create a single survey design for a given weight column
#'
#' Internal helper that builds one [survey::svydesign] object.
#'
#' @param data A data frame (typically from [clean_steps_data()]).
#' @param wt_col Character name of the weight column to use.
#' @param label Label for messages (e.g. "Step 1").
#'
#' @return A [survey::svydesign] object.
#' @keywords internal
.make_design <- function(data, wt_col, label = "") {

  has_weight  <- wt_col %in% names(data) && !all(data[[wt_col]] == 1)
  has_strata  <- "stratum"  %in% names(data)
  has_cluster <- "psu"      %in% names(data)

  # Temporarily rename wt_col to a fixed name for formula interface
  data$.wt <- data[[wt_col]]

  # Drop rows with NA or zero weights (survey::svydesign fails on NA weights)
  if (has_weight) {
    n_na_wt <- sum(is.na(data$.wt) | data$.wt == 0)
    if (n_na_wt > 0) {
      data <- data[!is.na(data$.wt) & data$.wt > 0, ]
      if (nchar(label)) message(glue::glue("  - Dropped {n_na_wt} rows with NA/zero weights"))
    }
  }

  # -- Case 1: Full complex design (weights + strata + clusters) -------------
  if (has_weight & has_strata & has_cluster) {
    if (nchar(label)) message(glue::glue("  {label}: Stratified cluster sampling with weights"))
    design <- survey::svydesign(
      ids     = ~psu,
      strata  = ~stratum,
      weights = ~.wt,
      data    = data,
      nest    = TRUE
    )

  # -- Case 2: Weights + clusters, no strata ---------------------------------
  } else if (has_weight & has_cluster & !has_strata) {
    if (nchar(label)) message(glue::glue("  {label}: Cluster sampling with weights (no strata)"))
    design <- survey::svydesign(
      ids     = ~psu,
      weights = ~.wt,
      data    = data
    )

  # -- Case 3: Weights + strata, no clusters ---------------------------------
  } else if (has_weight & has_strata & !has_cluster) {
    if (nchar(label)) message(glue::glue("  {label}: Stratified sampling with weights (no clusters)"))
    design <- survey::svydesign(
      ids     = ~1,
      strata  = ~stratum,
      weights = ~.wt,
      data    = data
    )

  # -- Case 4: Weights only ---------------------------------------------------
  } else if (has_weight & !has_strata & !has_cluster) {
    if (nchar(label)) message(glue::glue("  {label}: \u26a0 Weights only (no strata or cluster info) - SE may be underestimated"))
    design <- survey::svydesign(
      ids     = ~1,
      weights = ~.wt,
      data    = data
    )

  # -- Case 5: Unweighted ----------------------------------------------------
  } else {
    if (nchar(label)) message(glue::glue("  {label}: \u26a0 Unweighted (simple random sampling assumed)"))
    design <- survey::svydesign(ids = ~1, data = data)
  }

  # NOTE: WHO official STEPS scripts do NOT trim weights.
  # Previous versions of this package applied trimWeights(median/10, median*10),
  # but this caused systematic ~2.5pp discrepancies in tobacco indicators.
  # Removed to align with WHO methodology.

  return(design)
}


#' Set up survey designs for STEPS data (one per Step)
#'
#' Creates up to three survey design objects — one per WHO STEPS Step —
#' each using the appropriate step-specific weight column
#' (`wt_step1`, `wt_step2`, `wt_step3`).
#'
#' The returned object is a list of class `"steps_designs"` with elements
#' `$step1`, `$step2`, `$step3`.  For backward compatibility it can also
#' be used directly as a single design (it delegates to `$step1`).
#'
#' @param data A data frame (typically from [clean_steps_data()]).
#'
#' @return A list of class `"steps_designs"` with three
#'   [survey::svydesign] objects (step1, step2, step3).
#'
#' @details
#' The function handles five design cases per step:
#' 1. Full complex design: weights + strata + clusters
#' 2. Weights + clusters, no strata
#' 3. Weights + strata, no clusters
#' 4. Weights only
#' 5. Unweighted (simple random sampling)
#'
#' Weights are used as-is without trimming, consistent with the
#' WHO official STEPS analysis scripts.
#'
#' @export
setup_survey_design <- function(data) {

  message("  Setting up survey designs (per WHO STEPS Step)...")

  # Match WHO STEPS official scripts: handle lone PSUs gracefully
  old_psu <- getOption("survey.lonely.psu")
  old_domain <- getOption("survey.adjust.domain.lonely")
  on.exit({
    options(survey.lonely.psu = old_psu)
    options(survey.adjust.domain.lonely = old_domain)
  }, add = TRUE)
  options(survey.lonely.psu = "adjust")
  options(survey.adjust.domain.lonely = TRUE)

  # Check whether we actually have 3 distinct weight columns
  has_s1 <- "wt_step1" %in% names(data)
  has_s2 <- "wt_step2" %in% names(data)
  has_s3 <- "wt_step3" %in% names(data)

  # Determine if the step weights actually differ
  steps_differ <- FALSE
  if (has_s1 && has_s2 && !identical(data$wt_step1, data$wt_step2)) steps_differ <- TRUE
  if (has_s1 && has_s3 && !identical(data$wt_step1, data$wt_step3)) steps_differ <- TRUE

  if (steps_differ) {
    message("  \u2713 Distinct step-specific weights detected (WStep1/WStep2/WStep3)")
    d1 <- .make_design(data, "wt_step1", "Step 1 (behavioural)")
    d2 <- .make_design(data, "wt_step2", "Step 2 (physical)")
    d3 <- .make_design(data, "wt_step3", "Step 3 (biochemical)")
  } else {
    # All weights are the same — create one and reuse
    wt_col <- if (has_s1) "wt_step1" else "wt_final"
    d1 <- .make_design(data, wt_col, "Design")
    d2 <- d1
    d3 <- d1
  }

  n    <- nrow(data)
  wn   <- round(sum(stats::weights(d1)))
  message(glue::glue("  \u2192 Unweighted n = {format(n, big.mark=',')}"))
  message(glue::glue("  \u2192 Weighted  N (Step 1) = {format(wn, big.mark=',')}"))
  if (steps_differ) {
    wn2 <- round(sum(stats::weights(d2)))
    wn3 <- round(sum(stats::weights(d3)))
    message(glue::glue("  \u2192 Weighted  N (Step 2) = {format(wn2, big.mark=',')}"))
    message(glue::glue("  \u2192 Weighted  N (Step 3) = {format(wn3, big.mark=',')}"))
  }
  message("  Survey design created")

  designs <- list(step1 = d1, step2 = d2, step3 = d3)
  class(designs) <- c("steps_designs", "list")
  return(designs)
}

#' Weighted proportion estimation with 95% CI
#'
#' Calculates weighted proportions (as percentages) with 95% confidence
#' intervals for a yes/no variable, optionally stratified by a grouping
#' variable.
#'
#' @param formula A formula (e.g., `~variable` or using binary variables).
#' @param design A survey design object (from [setup_survey_design()]).
#' @param by Optional formula for stratification (e.g., `~sex`).
#' @param na.rm Logical; if TRUE (default), omit NA values.
#'
#' @return A data frame with columns:
#' - `estimate`: estimated proportion (%)
#' - `lower`: 95% CI lower bound (%)
#' - `upper`: 95% CI upper bound (%)
#' - `se`: standard error (%)
#' - If `by` is specified: grouping column(s) prepended
#'
#' @export
svyprop <- function(formula, design, by = NULL, na.rm = TRUE) {

  if (is.null(by)) {
    est <- survey::svymean(formula, design, na.rm = na.rm)
    ci  <- confint(est)
    data.frame(
      estimate = as.numeric(est) * 100,
      lower    = ci[, 1] * 100,
      upper    = ci[, 2] * 100,
      se       = sqrt(diag(attr(est, "var"))) * 100
    )
  } else {
    result <- survey::svyby(formula, by, design, survey::svymean, na.rm = na.rm, vartype = "ci")
    # Dynamically detect CI column names (may vary by survey package version)
    ci_l_col <- grep("^ci_l", names(result), value = TRUE)[1]
    ci_u_col <- grep("^ci_u", names(result), value = TRUE)[1]
    if (is.na(ci_l_col) || is.na(ci_u_col)) {
      stop("Could not detect CI columns in svyby result. Columns: ",
           paste(names(result), collapse = ", "))
    }
    # Rename formula-derived estimate column to 'estimate' for consistency
    est_col <- setdiff(names(result), c(all.vars(by), ci_l_col, ci_u_col, "se"))
    result |>
      dplyr::rename(estimate = dplyr::all_of(est_col),
                     lower = dplyr::all_of(ci_l_col),
                     upper = dplyr::all_of(ci_u_col)) |>
      dplyr::mutate(dplyr::across(dplyr::where(is.numeric), ~ . * 100))
  }
}

#' Weighted mean estimation with 95% CI
#'
#' Calculates weighted means with 95% confidence intervals for a
#' continuous variable, optionally stratified by a grouping variable.
#'
#' @param formula A formula (e.g., `~age`).
#' @param design A survey design object (from [setup_survey_design()]).
#' @param by Optional formula for stratification (e.g., `~sex`).
#' @param na.rm Logical; if TRUE (default), omit NA values.
#'
#' @return A data frame with columns:
#' - `estimate`: estimated mean
#' - `lower`: 95% CI lower bound
#' - `upper`: 95% CI upper bound
#' - `se`: standard error
#' - If `by` is specified: grouping column(s) prepended
#'
#' @export
svymn <- function(formula, design, by = NULL, na.rm = TRUE) {
  if (is.null(by)) {
    est <- survey::svymean(formula, design, na.rm = na.rm)
    ci  <- confint(est)
    data.frame(
      estimate = as.numeric(est),
      lower    = ci[, 1],
      upper    = ci[, 2],
      se       = sqrt(diag(attr(est, "var")))
    )
  } else {
    result <- survey::svyby(formula, by, design, survey::svymean, na.rm = na.rm, vartype = "ci")
    # Dynamically detect CI column names (may vary by survey package version)
    ci_l_col <- grep("^ci_l", names(result), value = TRUE)[1]
    ci_u_col <- grep("^ci_u", names(result), value = TRUE)[1]
    if (is.na(ci_l_col) || is.na(ci_u_col)) {
      stop("Could not detect CI columns in svyby result. Columns: ",
           paste(names(result), collapse = ", "))
    }
    est_col <- setdiff(names(result), c(all.vars(by), ci_l_col, ci_u_col, "se"))
    result |>
      dplyr::rename(estimate = dplyr::all_of(est_col),
                     lower = dplyr::all_of(ci_l_col),
                     upper = dplyr::all_of(ci_u_col))
  }
}

test_that("setup_survey_design creates valid design object", {
  td <- generate_test_data(n = 200, seed = 1)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)

  expect_s3_class(designs, "steps_designs")
  expect_s3_class(designs$step1, "survey.design")
  expect_s3_class(designs$step2, "survey.design")
  expect_s3_class(designs$step3, "survey.design")
})

test_that("svyprop returns consistent columns", {
  td <- generate_test_data(n = 200, seed = 1)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)

  # Without by (tobacco is Step 1)
  res_total <- svyprop(~as.numeric(current_tobacco), designs$step1)
  expect_true(all(c("estimate", "lower", "upper") %in% names(res_total)))

  # With by
  res_sex <- svyprop(~as.numeric(current_tobacco), designs$step1, by = ~sex)
  expect_true(all(c("estimate", "lower", "upper") %in% names(res_sex)))
})

test_that("svymn returns consistent columns", {
  td <- generate_test_data(n = 200, seed = 1)
  cols <- detect_steps_columns(td)
  clean <- clean_steps_data(td, cols)
  designs <- setup_survey_design(clean)

  # BMI is Step 2
  res <- svymn(~bmi, designs$step2)
  expect_true(all(c("estimate", "lower", "upper") %in% names(res)))

  res_by <- svymn(~bmi, designs$step2, by = ~sex)
  expect_true(all(c("estimate", "lower", "upper") %in% names(res_by)))
})

#' Reports Module — UI
#'
#' @param id Module namespace id.
#' @keywords internal
mod_reports_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      title = "Report Generation",
      width = 280,
      shiny::actionButton(
        ns("btn_generate"),
        "Generate report",
        class = "btn-primary btn-lg",
        width = "100%",
        icon = shiny::icon("file-word")
      ),
      shiny::hr(),
      shiny::downloadButton(
        ns("dl_report"),
        "Download Summary Report",
        class = "btn-success btn-sm",
        style = "width:100%;",
        icon = shiny::icon("file-word")
      ),
      shiny::downloadButton(
        ns("dl_data_book"),
        "Download Data Book",
        class = "btn-success btn-sm",
        style = "width:100%;margin-top:5px;",
        icon = shiny::icon("book")
      ),
      shiny::hr(),
      shiny::h6("Fact Sheet"),
      shiny::downloadButton(
        ns("dl_fact_sheet_html"),
        "Fact Sheet (HTML)",
        class = "btn-info btn-sm",
        style = "width:100%;",
        icon = shiny::icon("file-lines")
      ),
      shiny::downloadButton(
        ns("dl_fact_sheet_word"),
        "Fact Sheet (Word)",
        class = "btn-info btn-sm",
        style = "width:100%;margin-top:5px;",
        icon = shiny::icon("file-word")
      ),
      shiny::hr(),
      shiny::downloadButton(
        ns("dl_tables_plots"),
        "Download tables & plots",
        class = "btn-outline-secondary btn-sm",
        style = "width:100%;margin-top:5px;",
        icon = shiny::icon("download")
      )
    ),

    # Progress and status
    bslib::card(
      bslib::card_header("Generation Status"),
      shiny::div(
        id = ns("status_container"),
        shiny::textOutput(ns("status_text")),
        shiny::br(),
        shiny::uiOutput(ns("progress_bar"))
      ),
      class = "mt-3"
    ),

    # Summary information
    bslib::card(
      bslib::card_header("Report Summary"),
      bslib::layout_column_wrap(
        width = 1 / 2,
        bslib::value_box(
          "Report Status",
          shiny::textOutput(ns("report_status")),
          theme = "primary",
          showcase = shiny::icon("check-circle")
        ),
        bslib::value_box(
          "Output Files",
          shiny::textOutput(ns("n_files")),
          theme = "info",
          showcase = shiny::icon("folder")
        )
      ),
      class = "mt-3"
    )
  )
}


#' Reports Module — Server
#'
#' @param id Module namespace id.
#' @param upload_out Reactive list returned by [mod_upload_server()], containing
#'   `raw` (data frame), `cols` (column mapping), and `config` (survey configuration).
#'
#' @return A list with reactive elements:
#'   - `report_path`: Path to generated Word document
#'   - `output_dir`: Directory containing all outputs
#'   - `generation_status`: Current status string
#'
#' @keywords internal
#'
#' @details
#' The full pipeline is executed internally:
#' 1. Cleans data using [clean_steps_data()]
#' 2. Sets up survey design with [setup_survey_design()]
#' 3. Computes all indicators with [compute_all_indicators()]
#' 4. Builds tables with [build_steps_tables()]
#' 5. Builds plots with [build_steps_plots()]
#' 6. Saves all outputs to temp directory
#' 7. Renders report using [render_country_report()]
#'
#' Progress is shown via status text and progress bar during generation.
#' Errors are caught and displayed to the user via notifications.
#'
mod_reports_server <- function(id, upload_out) {
  shiny::moduleServer(id, function(input, output, session) {

    ns <- session$ns

    # Reactive values for tracking state
    report_state <- shiny::reactiveValues(
      status = "Ready",
      is_generating = FALSE,
      report_path = NULL,
      data_book_path = NULL,
      fact_sheet_html = NULL,
      fact_sheet_word = NULL,
      output_dir = NULL,
      output_files = list(),
      error_message = NULL
    )

    # Main report generation pipeline
    report_result <- shiny::eventReactive(input$btn_generate, {
      shiny::req(upload_out$raw(), upload_out$cols(), upload_out$config())

      report_state$is_generating <- TRUE
      report_state$status <- "Initializing..."
      report_state$error_message <- NULL

      tryCatch({
        # Step 0: Extract inputs
        report_state$status <- "Extracting data..."
        raw <- upload_out$raw()
        cols <- upload_out$cols()
        cfg <- upload_out$config()

        # Step 1: Clean data
        report_state$status <- "Cleaning data..."
        clean_data <- clean_steps_data(
          raw, cols,
          age_min = cfg$age_min,
          age_max = cfg$age_max,
          bp_sbp_threshold = cfg$bp_sbp_threshold,
          bp_dbp_threshold = cfg$bp_dbp_threshold,
          bmi_overweight = cfg$bmi_overweight,
          bmi_obese = cfg$bmi_obese,
          glucose_threshold = cfg$glucose_threshold,
          glucose_impaired_threshold = cfg$glucose_impaired_threshold,
          chol_threshold = cfg$chol_threshold
        )
        message(sprintf("  Cleaned data: %d rows, %d cols", nrow(clean_data), ncol(clean_data)))

        # Step 2: Setup survey design
        report_state$status <- "Setting up survey design..."
        design <- setup_survey_design(clean_data)
        message("  Survey design created")

        # Step 3: Compute all indicators
        report_state$status <- "Computing indicators..."
        indicators <- compute_all_indicators(design)
        message("  Indicators computed")

        # Step 4: Build summary tables (for country report)
        report_state$status <- "Building tables..."
        tables <- build_steps_tables(indicators$results)
        message("  Tables built")

        # Step 4b: Build detailed 3-panel tables (for data book)
        report_state$status <- "Building detailed tables..."
        computed_results <- tryCatch(
          compute_all_tables(design),
          error = function(e) {
            message(glue::glue("  Warning: Could not compute detailed tables: {e$message}"))
            NULL
          }
        )
        detailed_tables <- if (!is.null(computed_results)) {
          tryCatch(
            build_all_tables(computed_results),
            error = function(e) {
              message(glue::glue("  Warning: Could not build detailed tables: {e$message}"))
              NULL
            }
          )
        } else {
          NULL
        }
        message("  Detailed tables built")

        # Step 5: Build plots
        report_state$status <- "Building plots..."
        plots <- build_steps_plots(
          indicators$results,
          indicators$key_indicators,
          cfg$country_name,
          cfg$survey_year
        )
        message("  Plots built")

        # Step 6: Create temp output directory
        report_state$status <- "Saving outputs..."
        output_dir <- tempfile(pattern = "steps_report_")
        dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
        data_dir <- file.path(output_dir, "data")
        dir.create(data_dir, showWarnings = FALSE)

        # Save indicators, tables, and plots as RDS
        saveRDS(indicators$results, file.path(data_dir, "indicators.rds"))
        saveRDS(indicators$key_indicators, file.path(data_dir, "key_indicators.rds"))
        saveRDS(tables, file.path(data_dir, "tables.rds"))
        saveRDS(plots, file.path(data_dir, "plots.rds"))

        # Save detailed 3-panel tables and computed results for data book
        if (!is.null(detailed_tables)) {
          saveRDS(detailed_tables, file.path(data_dir, "detailed_tables.rds"))
        }
        if (!is.null(computed_results)) {
          saveRDS(computed_results, file.path(data_dir, "computed_results.rds"))
        }

        # Save clean data for reference
        saveRDS(clean_data, file.path(data_dir, "clean_data.rds"))
        readr::write_csv(clean_data, file.path(data_dir, "clean_data.csv"))

        message(sprintf("  Outputs saved to %s", output_dir))

        # Step 7: Render country report (summary)
        report_state$status <- "Rendering summary report..."

        # Prepare config object for rendering
        # If cfg is not a steps_config object, ensure it has required fields
        if (!all(c("country_name", "survey_year", "age_min", "age_max") %in% names(cfg))) {
          cfg <- list(
            country_name = if (!is.null(cfg$country_name)) cfg$country_name else "Country Name",
            survey_year  = if (!is.null(cfg$survey_year))  cfg$survey_year  else 2024,
            age_min      = if (!is.null(cfg$age_min))      cfg$age_min      else 18,
            age_max      = if (!is.null(cfg$age_max))      cfg$age_max      else 69
          )
        }

        # Call render_country_report (summary)
        report_path <- render_country_report(cfg, output_dir)

        # If render_country_report doesn't return a path, construct it
        if (is.null(report_path)) {
          report_path <- file.path(output_dir, "country_report.docx")
        }

        # Render data book (detailed tables) if detailed tables were built
        data_book_path <- NULL
        if (!is.null(detailed_tables)) {
          report_state$status <- "Rendering data book..."
          data_book_path <- tryCatch({
            render_data_book(cfg, output_dir)
          }, error = function(e) {
            message(glue::glue("  Warning: Could not render data book: {e$message}"))
            NULL
          })
        }

        # Render fact sheets (HTML + Word)
        report_state$status <- "Rendering fact sheets..."
        fact_sheet_html <- tryCatch({
          render_fact_sheet(cfg, output_dir, format = "html")
        }, error = function(e) {
          message(glue::glue("  Warning: Could not render HTML fact sheet: {e$message}"))
          NULL
        })
        fact_sheet_word <- tryCatch({
          render_fact_sheet(cfg, output_dir, format = "word")
        }, error = function(e) {
          message(glue::glue("  Warning: Could not render Word fact sheet: {e$message}"))
          NULL
        })

        report_state$status <- "Complete"
        report_state$report_path <- report_path
        report_state$data_book_path <- data_book_path
        report_state$fact_sheet_html <- fact_sheet_html
        report_state$fact_sheet_word <- fact_sheet_word
        report_state$output_dir <- output_dir

        # List all output files
        all_files <- list.files(output_dir, recursive = TRUE, full.names = TRUE)
        report_state$output_files <- all_files

        message(sprintf("Report generation complete: %s", report_path))

        list(
          success = TRUE,
          report_path = report_path,
          output_dir = output_dir,
          output_files = all_files
        )
      },
      error = function(e) {
        error_msg <- as.character(e$message)
        report_state$status <- "Error"
        report_state$error_message <- error_msg
        report_state$is_generating <- FALSE

        shiny::showNotification(
          sprintf("Report generation failed: %s", error_msg),
          type = "error",
          duration = 10
        )

        list(
          success = FALSE,
          error = error_msg
        )
      })
    })

    # Update is_generating flag when generation completes
    shiny::observe({
      if (!is.null(report_result())) {
        report_state$is_generating <- FALSE
      }
    })

    # Status text output
    output$status_text <- shiny::renderText({
      result <- report_result()
      if (is.null(result)) {
        return(report_state$status)
      }
      if (result$success) {
        "Report generated successfully!"
      } else {
        sprintf("Error: %s", result$error)
      }
    })

    # Progress bar
    output$progress_bar <- shiny::renderUI({
      if (report_state$is_generating) {
        shiny::div(
          class = "progress",
          style = "height: 20px;",
          shiny::div(
            class = "progress-bar progress-bar-striped progress-bar-animated",
            style = "width: 100%;",
            "Generating..."
          )
        )
      } else {
        NULL
      }
    })

    # Report status output
    output$report_status <- shiny::renderText({
      if (!is.null(report_state$report_path) &&
          file.exists(report_state$report_path)) {
        "Generated"
      } else if (!is.null(report_state$error_message)) {
        "Failed"
      } else {
        "Not generated"
      }
    })

    # Number of files output
    output$n_files <- shiny::renderText({
      if (length(report_state$output_files) > 0) {
        as.character(length(report_state$output_files))
      } else {
        "0"
      }
    })

    # Download handler for Word report
    output$dl_report <- shiny::downloadHandler(
      filename = function() {
        cfg <- upload_out$config()
        country_clean <- gsub("[^a-zA-Z0-9]", "_", cfg$country_name)
        sprintf("steps_country_report_%s_%d.docx",
                country_clean, cfg$survey_year)
      },
      content = function(file) {
        shiny::req(report_state$report_path)
        if (!file.exists(report_state$report_path)) {
          shiny::showNotification(
            "Report file not found. Please generate the report first.",
            type = "error"
          )
          return(NULL)
        }
        file.copy(report_state$report_path, file, overwrite = TRUE)
      }
    )

    # Download handler for Data Book
    output$dl_data_book <- shiny::downloadHandler(
      filename = function() {
        cfg <- upload_out$config()
        country_clean <- gsub("[^a-zA-Z0-9]", "_", cfg$country_name)
        sprintf("steps_data_book_%s_%d.docx",
                country_clean, cfg$survey_year)
      },
      content = function(file) {
        shiny::req(report_state$data_book_path)
        if (!file.exists(report_state$data_book_path)) {
          shiny::showNotification(
            "Data book not found. Please generate the report first.",
            type = "error"
          )
          return(NULL)
        }
        file.copy(report_state$data_book_path, file, overwrite = TRUE)
      }
    )

    # Download handler for tables and plots zip
    output$dl_tables_plots <- shiny::downloadHandler(
      filename = function() {
        cfg <- upload_out$config()
        country_clean <- gsub("[^a-zA-Z0-9]", "_", cfg$country_name)
        sprintf("steps_tables_plots_%s_%d.zip",
                country_clean, cfg$survey_year)
      },
      content = function(file) {
        shiny::req(report_state$output_dir)
        if (length(report_state$output_files) == 0) {
          shiny::showNotification(
            "No output files found. Please generate the report first.",
            type = "error"
          )
          return(NULL)
        }

        # Create temporary directory for zip contents
        temp_zip_dir <- tempfile()
        dir.create(temp_zip_dir, recursive = TRUE, showWarnings = FALSE)

        # Copy only data files (not including final report itself if we want)
        data_dir <- file.path(report_state$output_dir, "data")
        if (dir.exists(data_dir)) {
          file.copy(
            file.path(data_dir, c("tables.rds", "plots.rds", "clean_data.csv")),
            file.path(temp_zip_dir, c("tables.rds", "plots.rds", "clean_data.csv")),
            overwrite = TRUE
          )
        }

        # Create zip file
        zip(
          zipfile = file,
          files = list.files(temp_zip_dir, full.names = TRUE),
          flags = "-j"
        )

        # Clean up
        unlink(temp_zip_dir, recursive = TRUE)
      }
    )

    # Download handler for HTML fact sheet
    output$dl_fact_sheet_html <- shiny::downloadHandler(
      filename = function() {
        cfg <- upload_out$config()
        country_clean <- gsub("[^a-zA-Z0-9]", "_", cfg$country_name)
        sprintf("steps_fact_sheet_%s_%d.html",
                country_clean, cfg$survey_year)
      },
      content = function(file) {
        shiny::req(report_state$fact_sheet_html)
        if (!file.exists(report_state$fact_sheet_html)) {
          shiny::showNotification(
            "Fact sheet not found. Please generate the report first.",
            type = "error"
          )
          return(NULL)
        }
        file.copy(report_state$fact_sheet_html, file, overwrite = TRUE)
      }
    )

    # Download handler for Word fact sheet
    output$dl_fact_sheet_word <- shiny::downloadHandler(
      filename = function() {
        cfg <- upload_out$config()
        country_clean <- gsub("[^a-zA-Z0-9]", "_", cfg$country_name)
        sprintf("steps_fact_sheet_%s_%d.docx",
                country_clean, cfg$survey_year)
      },
      content = function(file) {
        shiny::req(report_state$fact_sheet_word)
        if (!file.exists(report_state$fact_sheet_word)) {
          shiny::showNotification(
            "Fact sheet not found. Please generate the report first.",
            type = "error"
          )
          return(NULL)
        }
        file.copy(report_state$fact_sheet_word, file, overwrite = TRUE)
      }
    )

    # Return reactive values for downstream use
    list(
      report_path = shiny::reactive({ report_state$report_path }),
      output_dir = shiny::reactive({ report_state$output_dir }),
      generation_status = shiny::reactive({ report_state$status }),
      output_files = shiny::reactive({ report_state$output_files })
    )
  })
}

#' Main Shiny Server
#'
#' Wires up all module servers and passes reactive outputs between them.
#'
#' @param input,output,session Standard Shiny server arguments.
#' @keywords internal
app_server <- function(input, output, session) {


  # -- Module chain ----------------------------------------------------------
  # Each module returns reactive(s) consumed by the next.

  # 1. Upload: returns list(raw, cols, config)
  upload_out <- mod_upload_server("upload")

  # 2. Clean: returns reactive(clean_data)
  clean_out <- mod_clean_server("clean", upload_out)

  # 3. Quality: returns reactive(steps_quality)
  mod_quality_server("quality", upload_out, clean_out)

  # 4. Design: returns reactive(design)
  design_out <- mod_design_server("design", clean_out)

  # 5. Indicators: returns list(results, key_indicators)
  results_out <- mod_indicators_server("indicators", design_out)

  # 6. Visualise: consumes indicators + config
  mod_visualise_server("visualise", results_out, upload_out)

  # 7. Reports: consumes config (runs full pipeline internally)
  mod_reports_server("reports", upload_out)

  # About page
  mod_about_server("about")
}

#' Create STEPS analysis configuration
#'
#' Builds a configuration list that specifies data paths, design variables,
#' and report parameters for the STEPS pipeline.
#'
#' @param data_path Path to raw STEPS data file (CSV or Excel).
#' @param country_name Country name for reports (default "Country Name").
#' @param survey_year Survey year (default 2024).
#' @param age_min Minimum age (default 18).
#' @param age_max Maximum age (default 69).
#' @param weight_var Weight variable name (default "wt_final", set NULL if none).
#' @param strata_var Strata variable name (default "stratum", set NULL if none).
#' @param cluster_var Cluster variable name (default "psu", set NULL if none).
#' @param bp_sbp_threshold SBP threshold for raised BP (default 140).
#' @param bp_dbp_threshold DBP threshold for raised BP (default 90).
#' @param bmi_overweight BMI threshold for overweight (default 25.0).
#' @param bmi_obese BMI threshold for obesity (default 30.0).
#' @param glucose_threshold Fasting glucose threshold in mmol/L (default 7.0).
#' @param glucose_impaired_threshold Impaired fasting glucose threshold in mmol/L (default 6.1).
#' @param chol_threshold Total cholesterol threshold in mmol/L (default 5.0).
#'
#' @return A list with elements:
#'   - `data_path`: Input file path
#'   - `country_name`: Country name
#'   - `survey_year`: Survey year
#'   - `age_min`, `age_max`: Age range
#'   - `weight_var`, `strata_var`, `cluster_var`: Design variable names
#'   - Threshold parameters for BP, BMI, glucose, cholesterol
#'
#' @examples
#' \dontrun{
#'   cfg <- steps_config("data/steps_2023.csv", "Senegal", 2023)
#'   cfg <- steps_config("data/steps.csv", "Mongolia", 2019,
#'                       bp_sbp_threshold = 130, bp_dbp_threshold = 80)
#' }
#'
#' @export
steps_config <- function(data_path, country_name = "Country Name", survey_year = 2024,
                         age_min = 18, age_max = 69,
                         weight_var = "wt_final",
                         strata_var = "stratum",
                         cluster_var = "psu",
                         bp_sbp_threshold = 140, bp_dbp_threshold = 90,
                         bmi_overweight = 25.0, bmi_obese = 30.0,
                         glucose_threshold = 7.0,
                         glucose_impaired_threshold = 6.1,
                         chol_threshold = 5.0) {
  list(
    data_path                  = data_path,
    country_name               = country_name,
    survey_year                = survey_year,
    age_min                    = age_min,
    age_max                    = age_max,
    weight_var                 = weight_var,
    strata_var                 = strata_var,
    cluster_var                = cluster_var,
    bp_sbp_threshold           = bp_sbp_threshold,
    bp_dbp_threshold           = bp_dbp_threshold,
    bmi_overweight             = bmi_overweight,
    bmi_obese                  = bmi_obese,
    glucose_threshold          = glucose_threshold,
    glucose_impaired_threshold = glucose_impaired_threshold,
    chol_threshold             = chol_threshold
  )
}

#' Render STEPS Fact Sheet report
#'
#' Generates a Word document with an overview of key NCD risk factor
#' prevalence, including summary table and sex-stratified charts.
#'
#' @param config A list from [steps_config()] with survey metadata and paths.
#'   Expected to have `country_name`, `survey_year`, `age_min`, `age_max`.
#' @param output_dir Directory for output reports (default \code{tempdir()}).
#' @param format Output format: `"html"` for self-contained HTML (default)
#'   or `"word"` for Word (.docx).
#'
#' @return Path to generated output file (invisibly).
#'   Prints message with output location.
#'
#' @details
#' The fact sheet template uses pre-computed indicators, key_indicators,
#' and plots (via .rds files in data/processed/).
#' Requires rmarkdown, flextable, ggplot2, glue, patchwork packages.
#'
#' @export
render_fact_sheet <- function(config, output_dir = tempdir(),
                              format = c("html", "word")) {
  format <- match.arg(format)
  dir.create(output_dir, showWarnings = FALSE, recursive = TRUE)

  template_path <- system.file("rmd", "fact_sheet.Rmd", package = "stepssurvey")

  if (!file.exists(template_path)) {
    stop("Fact sheet template not found. Install stepssurvey package correctly.")
  }

  ext <- if (format == "html") "html" else "docx"
  output_file <- file.path(output_dir, paste0("fact_sheet.", ext))
  output_format <- if (format == "html") "html_document" else "word_document"

  tryCatch({
    rmarkdown::render(
      template_path,
      output_file = output_file,
      output_format = output_format,
      params = list(config = config, output_dir = output_dir),
      quiet = TRUE
    )
    message(glue::glue("\u2713 Fact sheet rendered: {output_file}"))
  },
  error = function(e) {
    message(glue::glue("\u2717 Failed to render fact sheet: {e$message}"))
  })

  invisible(output_file)
}

#' Render STEPS Data Book report
#'
#' Generates a Word document with detailed age-stratified prevalence tables
#' for all available indicators, organized by STEPS step.
#'
#' @param config A list from [steps_config()] with survey metadata.
#'   Expected to have `country_name`, `survey_year`, `age_min`, `age_max`.
#' @param output_dir Directory for output reports (default \code{tempdir()}).
#'
#' @return Path to generated Word document (invisibly).
#'   Prints message with output location.
#'
#' @details
#' Sections correspond to STEPS steps:
#'   - Step 1: Behavioural Risk Factors (tobacco, alcohol, diet, physical activity)
#'   - Step 2: Physical Measurements (overweight/obesity, blood pressure)
#'   - Step 3: Biochemical (glucose, cholesterol)
#'
#' Requires pre-computed tables and plots in data/processed/.
#'
#' @export
render_data_book <- function(config, output_dir = tempdir()) {
  dir.create(output_dir, showWarnings = FALSE, recursive = TRUE)

  template_path <- system.file("rmd", "data_book.Rmd", package = "stepssurvey")

  if (!file.exists(template_path)) {
    stop("Data book template not found. Install stepssurvey package correctly.")
  }

  output_file <- file.path(output_dir, "data_book.docx")

  # Ensure config has data_dir pointing to where RDS files live
  if (is.null(config$data_dir)) {
    config$data_dir <- file.path(output_dir, "data")
  }

  tryCatch({
    rmarkdown::render(
      template_path,
      output_file = output_file,
      params = list(config = config),
      quiet = TRUE
    )
    message(glue::glue("\u2713 Data book rendered: {output_file}"))
  },
  error = function(e) {
    message(glue::glue("\u2717 Failed to render data book: {e$message}"))
  })

  invisible(output_file)
}

#' Render STEPS Country Report
#'
#' Generates a comprehensive Word document with executive summary,
#' indicator-by-indicator analysis, and recommendations for public health action.
#'
#' @param config A list from [steps_config()] with survey metadata.
#'   Expected to have `country_name`, `survey_year`, `age_min`, `age_max`.
#' @param output_dir Directory for output reports (default \code{tempdir()}).
#'
#' @return Path to generated Word document (invisibly).
#'   Prints message with output location.
#'
#' @details
#' Sections include:
#'   - Executive summary with key findings
#'   - Tobacco use
#'   - Physical activity
#'   - Overweight and obesity
#'   - Blood pressure
#'   - Blood glucose and cholesterol
#'   - Recommendations for public health action
#'   - Methodology
#'
#' Requires pre-computed indicators, tables, and plots in data/processed/.
#'
#' @export
render_country_report <- function(config, output_dir = tempdir()) {
  dir.create(output_dir, showWarnings = FALSE, recursive = TRUE)

  template_path <- system.file("rmd", "country_report.Rmd", package = "stepssurvey")

  if (!file.exists(template_path)) {
    stop("Country report template not found. Install stepssurvey package correctly.")
  }

  output_file <- file.path(output_dir, "country_report.docx")

  # Ensure config has data_dir pointing to where RDS files live
  if (is.null(config$data_dir)) {
    # Default: look for a "data" subdirectory in output_dir
    config$data_dir <- file.path(output_dir, "data")
  }

  tryCatch({
    rmarkdown::render(
      template_path,
      output_file = output_file,
      params = list(config = config),
      quiet = TRUE
    )
    message(glue::glue("\u2713 Country report rendered: {output_file}"))
  },
  error = function(e) {
    message(glue::glue("\u2717 Failed to render country report: {e$message}"))
  })

  invisible(output_file)
}

#' WHO STEPS colour palette
#'
#' A named list of colours used in WHO STEPS reports and visualisations.
#'
#' @return A named list of hex colour codes.
#' @export
#' @examples
#' steps_colors()$blue
steps_colors <- function() {
  list(
    blue       = "#009ADE",
    dark_blue  = "#00427A",
    green      = "#7AC143",
    orange     = "#F26522",
    red        = "#ED1C24",
    grey       = "#6D6E71",
    light_grey = "#D1D3D4",
    male       = "#009ADE",
    female     = "#F26522"
  )
}

#' WHO STEPS ggplot2 theme
#'
#' A clean, minimal ggplot2 theme styled with WHO STEPS colours.
#'
#' @param base_size Base font size (default 11).
#' @return A [ggplot2::theme] object.
#' @export
#' @examples
#' library(ggplot2)
#' ggplot(mtcars, aes(wt, mpg)) + geom_point() + theme_steps()
theme_steps <- function(base_size = 11) {
  colors <- steps_colors()
  ggplot2::theme_minimal(base_size = base_size) %+replace%
    ggplot2::theme(
      plot.title       = ggplot2::element_text(color = colors$dark_blue, face = "bold", size = base_size + 2),
      plot.subtitle    = ggplot2::element_text(color = colors$grey, size = base_size),
      axis.title       = ggplot2::element_text(color = colors$grey, size = base_size - 1),
      axis.text        = ggplot2::element_text(color = colors$grey),
      panel.grid.major = ggplot2::element_line(color = colors$light_grey, linewidth = 0.3),
      panel.grid.minor = ggplot2::element_blank(),
      legend.position  = "bottom",
      legend.title     = ggplot2::element_blank(),
      strip.text       = ggplot2::element_text(face = "bold", color = colors$dark_blue)
    )
}

#' Upload Module — UI
#' @param id Module namespace id.
#' @keywords internal
mod_upload_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      title = "Data & Settings",
      width = 320,

      shiny::fileInput(ns("file"), "Upload STEPS data",
                       accept = c(".csv", ".xlsx", ".xls", ".dta", ".sav")),

      shiny::actionButton(ns("use_demo"), "Use demo data",
                          class = "btn-outline-primary btn-sm mb-3",
                          width = "100%"),

      shiny::hr(),

      shiny::tags$details(
        shiny::tags$summary(shiny::tags$strong("Column mapping (optional)")),
        shiny::helpText(
          "Upload a filled mapping template if your variables have ",
          "non-standard names. Leave empty to use auto-detection."
        ),
        shiny::fileInput(ns("mapping_file"), NULL,
                         accept = c(".xlsx", ".xls", ".csv")),
        shiny::downloadButton(ns("dl_mapping_template"),
                              "Download blank template",
                              class = "btn-outline-secondary btn-sm mb-2",
                              style = "width:100%")
      ),

      shiny::hr(),

      shiny::textInput(ns("country"), "Country name", value = "Country"),
      shiny::numericInput(ns("year"), "Survey year", value = 2024,
                          min = 1990, max = 2030),
      shiny::sliderInput(ns("age_range"), "Age range",
                         min = 15, max = 100, value = c(18, 69)),

      shiny::hr(),
      shiny::h6("Indicator thresholds"),
      shiny::helpText("Customise thresholds for indicator definitions. WHO defaults shown."),

      # BP thresholds
      shiny::tags$details(
        shiny::tags$summary(shiny::tags$strong("Blood pressure")),
        shiny::helpText("Raised BP definition (default: 140/90 mmHg)."),
        shiny::fluidRow(
          shiny::column(6,
            shiny::numericInput(ns("bp_sbp"), "SBP >=", value = 140,
                                min = 100, max = 180, step = 5)
          ),
          shiny::column(6,
            shiny::numericInput(ns("bp_dbp"), "DBP >=", value = 90,
                                min = 60, max = 120, step = 5)
          )
        )
      ),

      # BMI thresholds
      shiny::tags$details(
        shiny::tags$summary(shiny::tags$strong("BMI")),
        shiny::helpText("WHO standard: overweight >=25, obese >=30."),
        shiny::fluidRow(
          shiny::column(6,
            shiny::numericInput(ns("bmi_ow"), "Overweight >=", value = 25.0,
                                min = 20, max = 30, step = 0.5)
          ),
          shiny::column(6,
            shiny::numericInput(ns("bmi_ob"), "Obese >=", value = 30.0,
                                min = 25, max = 40, step = 0.5)
          )
        )
      ),

      # Glucose thresholds
      shiny::tags$details(
        shiny::tags$summary(shiny::tags$strong("Fasting glucose")),
        shiny::helpText("mmol/L. WHO: raised >=7.0, impaired >=6.1."),
        shiny::fluidRow(
          shiny::column(6,
            shiny::numericInput(ns("gluc_raised"), "Raised >=", value = 7.0,
                                min = 5, max = 15, step = 0.1)
          ),
          shiny::column(6,
            shiny::numericInput(ns("gluc_impaired"), "Impaired >=", value = 6.1,
                                min = 4, max = 10, step = 0.1)
          )
        )
      ),

      # Cholesterol threshold
      shiny::tags$details(
        shiny::tags$summary(shiny::tags$strong("Total cholesterol")),
        shiny::helpText("mmol/L. WHO: raised >=5.0."),
        shiny::numericInput(ns("chol_raised"), "Raised >=", value = 5.0,
                            min = 3, max = 8, step = 0.1)
      ),

      shiny::hr(),
      shiny::h6("Column overrides"),
      shiny::helpText("Leave blank to use auto-detected columns."),
      shiny::uiOutput(ns("col_overrides"))
    ),

    # Main panel
    bslib::card(
      bslib::card_header("Raw data preview"),
      DT::DTOutput(ns("raw_table"))
    ),
    bslib::card(
      bslib::card_header("Detected STEPS columns"),
      DT::DTOutput(ns("detected_table"))
    )
  )
}


#' Upload Module — Server
#'
#' @param id Module namespace id.
#' @return A [shiny::reactiveValues] with elements `raw`, `cols`, `config`.
#' @keywords internal
mod_upload_server <- function(id) {
  shiny::moduleServer(id, function(input, output, session) {
    ns <- session$ns

    rv <- shiny::reactiveValues(raw = NULL, cols = NULL)

    # -- Import from file upload ------------------------------------------------
    shiny::observeEvent(input$file, {
      req(input$file)
      imported <- import_steps_data(input$file$datapath)

      # Use mapping file if provided, otherwise auto-detect
      if (!is.null(input$mapping_file)) {
        rv$cols <- read_column_mapping(input$mapping_file$datapath, data = imported)
      } else {
        rv$cols <- detect_steps_columns(imported)
      }

      # Strip haven labels after detection (labels are used for
      # disambiguation in detect_steps_columns but cause issues
      # with DT::datatable and dim<- operations downstream)
      rv$raw   <- .strip_haven_labels(imported)
    })

    # -- Re-apply mapping if uploaded after data ---------------------------------
    shiny::observeEvent(input$mapping_file, {
      req(input$mapping_file, rv$raw)
      rv$cols <- read_column_mapping(input$mapping_file$datapath, data = rv$raw)
    })

    # -- Download blank mapping template -----------------------------------------
    output$dl_mapping_template <- shiny::downloadHandler(
      filename = function() "column_mapping_template.xlsx",
      content = function(file) {
        template_path <- system.file("templates",
                                      "column_mapping_template.xlsx",
                                      package = "stepssurvey")
        file.copy(template_path, file)
      }
    )

    # -- Import demo data -------------------------------------------------------
    shiny::observeEvent(input$use_demo, {
      rv$raw  <- generate_test_data(n = 3000, seed = 42)
      rv$cols <- detect_steps_columns(rv$raw)
    })

    # -- Column override dropdowns ----------------------------------------------
    output$col_overrides <- shiny::renderUI({
      shiny::req(rv$raw)
      choices <- c("(auto)" = "", sort(names(rv$raw)))

      key_cols <- c("age", "sex", "weight_step1", "weight_step2", "weight_step3",
                     "strata", "psu",
                     "tobacco_current", "alcohol_current",
                     "height", "weight", "sbp1", "fasting_glucose")

      purrr::map(key_cols, function(col_name) {
        detected <- rv$cols[[col_name]]
        label <- paste0(col_name, if (!is.null(detected)) paste0(" [", detected, "]") else " [not found]")
        shiny::selectInput(ns(paste0("ov_", col_name)), label,
                           choices = choices, selected = "",
                           width = "100%")
      })
    })

    # -- Apply overrides to cols ------------------------------------------------
    cols_final <- shiny::reactive({
      shiny::req(rv$cols)
      cols <- rv$cols
      key_cols <- c("age", "sex", "weight_step1", "weight_step2", "weight_step3",
                     "strata", "psu",
                     "tobacco_current", "alcohol_current",
                     "height", "weight", "sbp1", "fasting_glucose")
      for (col_name in key_cols) {
        ov <- input[[paste0("ov_", col_name)]]
        if (!is.null(ov) && nchar(ov) > 0) {
          cols[[col_name]] <- ov
        }
      }
      cols
    })

    # -- Raw data table ---------------------------------------------------------
    output$raw_table <- DT::renderDT({
      shiny::req(rv$raw)
      DT::datatable(utils::head(rv$raw, 100),
                    options = list(scrollX = TRUE, pageLength = 10),
                    rownames = FALSE)
    })

    # -- Detected columns table -------------------------------------------------
    output$detected_table <- DT::renderDT({
      shiny::req(rv$cols)
      detected <- rv$cols
      df <- data.frame(
        Variable   = names(detected),
        Detected   = sapply(detected, function(x) if (is.null(x)) "-- not found --" else x),
        stringsAsFactors = FALSE
      )
      DT::datatable(df, options = list(pageLength = 50, dom = "t"),
                    rownames = FALSE)
    })

    # -- Return reactive config -------------------------------------------------
    config <- shiny::reactive({
      list(
        country_name              = input$country,
        survey_year               = input$year,
        age_min                   = input$age_range[1],
        age_max                   = input$age_range[2],
        bp_sbp_threshold          = input$bp_sbp,
        bp_dbp_threshold          = input$bp_dbp,
        bmi_overweight            = input$bmi_ow,
        bmi_obese                 = input$bmi_ob,
        glucose_threshold         = input$gluc_raised,
        glucose_impaired_threshold = input$gluc_impaired,
        chol_threshold            = input$chol_raised
      )
    })

    # Return values for downstream modules
    list(
      raw    = shiny::reactive(rv$raw),
      cols   = cols_final,
      config = config
    )
  })
}

#' Compute Anthropometry Indicators
#'
#' Calculates prevalence of overweight, obesity, and central obesity, plus
#' mean BMI and waist circumference, from a survey design object.
#'
#' @param design A survey design object from [setup_survey_design()].
#'
#' @return A named list of survey estimates. Each element contains estimates
#'   (as tibble with columns: estimate, lower, upper, etc.) for:
#'   - `overweight_obese_total`: overweight or obese (BMI >=25), overall
#'   - `overweight_obese_by_sex`: overweight or obese, by sex
#'   - `overweight_obese_by_age`: overweight or obese, by age group
#'   - `obese_total`: obese (BMI >=30), overall
#'   - `obese_by_sex`: obese, by sex
#'   - `obese_by_age`: obese, by age group
#'   - `central_obesity_total`: central obesity, overall
#'   - `central_obesity_by_sex`: central obesity, by sex
#'   - `central_obesity_by_age`: central obesity, by age group
#'   - `bmi_mean_total`: mean BMI, overall
#'   - `bmi_mean_by_sex`: mean BMI, by sex
#'   - `waist_cm_mean_total`: mean waist circumference, overall
#'   - `waist_cm_mean_by_sex`: mean waist circumference, by sex
#'   (if the corresponding variables exist in design)
#'
#' @seealso [compute_all_indicators()]
#'
#' @export
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   anthropometry_results <- compute_anthropometry_indicators(design)
#' }
compute_anthropometry_indicators <- function(design) {
  message("  Computing anthropometry indicators...")
  results <- list()

  bin_vars <- c("overweight_obese", "obese", "central_obesity")
  bin_vars <- bin_vars[bin_vars %in% names(design$variables)]
  for (v in bin_vars) {
    f <- as.formula(paste0("~as.numeric(", v, ")"))
    results[[paste0(v, "_total")]]  <- svyprop(f, design)
    results[[paste0(v, "_by_sex")]] <- svyprop(f, design, by = ~sex)
    results[[paste0(v, "_by_age")]] <- svyprop(f, design, by = ~age_group)
  }

  cont_vars <- c("bmi", "waist_cm")
  cont_vars <- cont_vars[cont_vars %in% names(design$variables)]
  for (v in cont_vars) {
    f <- as.formula(paste0("~", v))
    results[[paste0(v, "_mean_total")]]  <- svymn(f, design)
    results[[paste0(v, "_mean_by_sex")]] <- svymn(f, design, by = ~sex)
  }

  return(results)
}

#' Main Shiny UI
#'
#' Builds the top-level UI using bslib sidebar page.
#'
#' @return A Shiny UI definition.
#' @keywords internal
app_ui <- function() {

  bslib::page_navbar(
    id       = "main_nav",
    title    = "stepssurvey",
    theme    = bslib::bs_theme(
      version   = 5,
      bootswatch = "flatly",
      primary   = "#008DC9",
      "navbar-bg" = "#00427A"
    ),
    fillable = TRUE,

    # -- Tab 1: Upload & Detect ---
    bslib::nav_panel(
      title = "1. Upload",
      icon  = shiny::icon("upload"),
      mod_upload_ui("upload")
    ),

    # -- Tab 2: Clean ---
    bslib::nav_panel(
      title = "2. Clean",
      icon  = shiny::icon("broom"),
      mod_clean_ui("clean")
    ),

    # -- Tab 3: Data Quality ---
    bslib::nav_panel(
      title = "3. Quality",
      icon  = shiny::icon("stethoscope"),
      mod_quality_ui("quality")
    ),

    # -- Tab 4: Survey Design ---
    bslib::nav_panel(
      title = "4. Design",
      icon  = shiny::icon("scale-balanced"),
      mod_design_ui("design")
    ),

    # -- Tab 5: Indicators ---
    bslib::nav_panel(
      title = "5. Indicators",
      icon  = shiny::icon("chart-bar"),
      mod_indicators_ui("indicators")
    ),

    # -- Tab 6: Visualisations ---
    bslib::nav_panel(
      title = "6. Visualise",
      icon  = shiny::icon("chart-line"),
      mod_visualise_ui("visualise")
    ),

    # -- Tab 7: Reports ---
    bslib::nav_panel(
      title = "7. Reports",
      icon  = shiny::icon("file-word"),
      mod_reports_ui("reports")
    ),

    # -- About ---
    bslib::nav_spacer(),
    bslib::nav_panel(
      title = "About",
      icon  = shiny::icon("info-circle"),
      mod_about_ui("about")
    )
  )
}

#' Indicators Module — UI
#'
#' @param id Module namespace id.
#'
#' @return UI definition for indicators computation and display.
#'
#' @keywords internal
mod_indicators_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      title = "Indicators",
      width = 280,
      shiny::actionButton(ns("btn_compute"), "Compute indicators",
                          class = "btn-primary btn-lg", width = "100%",
                          icon = shiny::icon("calculator")),
      shiny::hr(),
      shiny::downloadButton(ns("dl_key_indicators"), "Download key indicators CSV",
                            class = "btn-outline-secondary btn-sm",
                            style = "width:100%;")
    ),

    # Summary value boxes
    bslib::layout_column_wrap(
      width = 1 / 2,
      bslib::value_box("Domains computed", shiny::textOutput(ns("n_domains")),
                       theme = "primary", showcase = shiny::icon("layer-group")),
      bslib::value_box("Key indicators",  shiny::textOutput(ns("n_key_indicators")),
                       theme = "success",  showcase = shiny::icon("chart-line"))
    ),

    # Key indicators summary table
    bslib::navset_card_tab(
      title = "Results",
      bslib::nav_panel(
        "Key indicators",
        DT::DTOutput(ns("tbl_key_indicators"))
      ),
      bslib::nav_panel(
        "Tobacco",
        DT::DTOutput(ns("tbl_tobacco"))
      ),
      bslib::nav_panel(
        "Alcohol",
        DT::DTOutput(ns("tbl_alcohol"))
      ),
      bslib::nav_panel(
        "Diet & PA",
        DT::DTOutput(ns("tbl_diet_pa"))
      ),
      bslib::nav_panel(
        "Anthropometry",
        DT::DTOutput(ns("tbl_anthropometry"))
      ),
      bslib::nav_panel(
        "Blood Pressure",
        DT::DTOutput(ns("tbl_blood_pressure"))
      ),
      bslib::nav_panel(
        "Biochemical",
        DT::DTOutput(ns("tbl_biochemical"))
      )
    )
  )
}


#' Indicators Module — Server
#'
#' @param id Module namespace id.
#' @param design_out Reactive returning a survey::svydesign object.
#'
#' @return A list with two reactive elements:
#'   - `results`: reactive list of indicator results by domain
#'   - `key_indicators`: reactive tibble of headline key indicators
#'
#' @keywords internal
mod_indicators_server <- function(id, design_out) {
  shiny::moduleServer(id, function(input, output, session) {

    # -- Reactive computation of indicators ------------------------------------
    indicators_data <- shiny::eventReactive(input$btn_compute, {
      shiny::req(design_out())

      shiny::withProgress(message = "Computing indicators...", value = 0.1, {
        result <- tryCatch(
          {
            shiny::incProgress(0.2, detail = "Running indicator modules")
            res <- compute_all_indicators(design_out())
            shiny::incProgress(0.6, detail = "Done!")
            res
          },
          error = function(e) {
            shiny::showNotification(
              paste("Error computing indicators:", e$message),
              type = "error", duration = 15
            )
            NULL
          }
        )
      })

      shiny::req(result)
      result
    })

    # Extract individual reactives for downstream modules
    results <- shiny::reactive({
      shiny::req(indicators_data())
      indicators_data()$results
    })

    key_indicators <- shiny::reactive({
      shiny::req(indicators_data())
      indicators_data()$key_indicators
    })

    # -- Summary value boxes ----------------------------------------------------
    output$n_domains <- shiny::renderText({
      shiny::req(results())
      length(results())
    })

    output$n_key_indicators <- shiny::renderText({
      shiny::req(key_indicators())
      nrow(key_indicators())
    })

    # -- Key indicators summary table -------------------------------------------
    output$tbl_key_indicators <- DT::renderDT({
      shiny::req(key_indicators())
      tbl <- key_indicators() |>
        dplyr::mutate(
          estimate = sprintf("%.1f%%", estimate),
          lower = sprintf("%.1f%%", lower),
          upper = sprintf("%.1f%%", upper)
        ) |>
        dplyr::rename(
          Domain = domain,
          Indicator = indicator,
          Estimate = estimate,
          "95% CI Lower" = lower,
          "95% CI Upper" = upper
        )
      DT::datatable(tbl,
                    options = list(dom = "t", pageLength = 15),
                    rownames = FALSE)
    })

    # -- Domain-specific result tables ------------------------------------------
    output$tbl_tobacco <- DT::renderDT({
      shiny::req(results())
      tbl_domain("Tobacco", results()$tobacco)
    })

    output$tbl_alcohol <- DT::renderDT({
      shiny::req(results())
      tbl_domain("Alcohol", results()$alcohol)
    })

    output$tbl_diet_pa <- DT::renderDT({
      shiny::req(results())
      tbl_domain("Diet & PA", results()$diet_pa)
    })

    output$tbl_anthropometry <- DT::renderDT({
      shiny::req(results())
      tbl_domain("Anthropometry", results()$anthropometry)
    })

    output$tbl_blood_pressure <- DT::renderDT({
      shiny::req(results())
      tbl_domain("Blood Pressure", results()$blood_pressure)
    })

    output$tbl_biochemical <- DT::renderDT({
      shiny::req(results())
      tbl_domain("Biochemical", results()$biochemical)
    })

    # -- Download handler -------------------------------------------------------
    output$dl_key_indicators <- shiny::downloadHandler(
      filename = function() paste0("steps_key_indicators_", Sys.Date(), ".csv"),
      content  = function(file) {
        readr::write_csv(key_indicators(), file)
      }
    )

    # -- Return list of reactives for downstream modules ----------------------
    list(
      results = results,
      key_indicators = key_indicators
    )
  })
}


#' Format indicator results tibble for display
#'
#' Internal helper to convert a domain's indicator results list into a
#' formatted data frame for DT table display.
#'
#' @param domain_name Character name of the domain for display.
#' @param domain_results List of indicator results for the domain.
#'
#' @return A formatted data frame ready for DT::datatable, or NULL if empty.
#'
#' @keywords internal
tbl_domain <- function(domain_name, domain_results) {
  if (is.null(domain_results) || length(domain_results) == 0) {
    return(data.frame())
  }

  # Convert list of indicator tibbles into a single long-format table
  result_list <- list()

  for (indicator_name in names(domain_results)) {
    ind_tbl <- domain_results[[indicator_name]]

    # Skip if not a data frame/tibble
    if (!is.data.frame(ind_tbl)) {
      next
    }

    # Ensure required columns exist
    if (!all(c("estimate", "lower", "upper") %in% names(ind_tbl))) {
      next
    }

    # Add indicator name column and format
    ind_tbl <- ind_tbl |>
      dplyr::mutate(
        indicator_name = indicator_name,
        .before = 1
      ) |>
      dplyr::select(indicator_name, dplyr::everything()) |>
      dplyr::mutate(
        estimate = sprintf("%.2f", estimate),
        lower = sprintf("%.2f", lower),
        upper = sprintf("%.2f", upper)
      )

    result_list[[indicator_name]] <- ind_tbl
  }

  if (length(result_list) == 0) {
    return(data.frame())
  }

  # Bind all indicator results
  tbl <- dplyr::bind_rows(result_list)

  # Rename columns for display
  if (nrow(tbl) > 0) {
    names(tbl) <- gsub("_", " ", tools::toTitleCase(names(tbl)))
  }

  DT::datatable(tbl,
                options = list(dom = "t", pageLength = 20, scrollX = TRUE),
                rownames = FALSE)
}

#' stepssurvey: Analyse WHO STEPS Survey Data
#'
#' @description
#' A complete analysis pipeline for the WHO STEPwise Approach to NCD Risk
#' Factor Surveillance (STEPS).
#'
#' @docType package
#' @name stepssurvey-package
"_PACKAGE"

# Silence R CMD check notes about tidyverse NSE variables
utils::globalVariables(c(
  # Core demographics & survey design
  "age", "sex", "age_group", "estimate", "lower", "upper", "se",
  "ci_l", "ci_u", "domain", "indicator",
  "wt_final", "wt_step1", "wt_step2", "wt_step3", ".wt",
  "stratum", "psu",

  # Tobacco
  "tobacco_current_raw", "current_tobacco", "tobacco_daily_raw",
  "daily_tobacco", "smokeless_current_raw", "current_smokeless",
  "current_smoker", "current_tobacco_any", "smoking_status", "daily_smoker", "nondaily_smoker",
  "former_smoker", "never_smoker", "former_daily_smoker",
  "smoking_start_age", "smoking_duration",
  "smokes_manufactured_cig", "cig_per_day_manufactured", "cig_per_day_handrolled",
  "cigarettes_per_day_cat", "quit_attempt", "quit_advice",
  "secondhand_home", "secondhand_work",

  # Alcohol
  "alcohol_current_raw", "current_alcohol", "alcohol_12m",
  "heavy_episode_raw", "heavy_episode_raw_n", "heavy_episodic",
  "alcohol_status", "lifetime_abstainer", "past_12m_abstainer",
  "past_12m_not_current", "alcohol_freq_12m",
  "drinking_occasions_30d", "drinks_per_occasion", "drinking_level",
  "drinking_high", "drinking_intermediate", "drinking_lower", "non_drinker_30d",
  "alcohol_7day_freq_cat",

  # Diet
  "fruit_days", "fruit_serv", "veg_days", "veg_serv",
  "avg_fruit_servings", "avg_veg_servings", "avg_fruit_veg_servings",
  "low_fruit_veg",
  "always_add_salt", "always_processed_salt", "perceived_excess_salt",

  # Physical activity
  "met_total", "insufficient_pa", "low_pa", "pa_category",
  "pa_minutes_per_day", "pa_work_min_day", "pa_transport_min_day",
  "pa_recreation_min_day", "sedentary_min_day", "no_vigorous_pa",

  # Anthropometry
  "height_cm", "weight_kg", "bmi", "bmi_category", "overweight_obese",
  "obese", "underweight", "normal_weight", "overweight_only",
  "waist_cm", "central_obesity", "hip_cm", "waist_hip_ratio",

  # Blood pressure
  "mean_sbp", "mean_dbp", "mean_heart_rate",
  "sbp1", "sbp2", "sbp3", "dbp1", "dbp2", "dbp3",
  "bp_meds_raw", "on_bp_meds", "raised_bp", "bp_stage", "bp_controlled",
  "bp_ever_measured", "bp_diagnosed", "bp_diagnosed_12m",

  # Biochemical
  "fasting_glucose", "dm_meds_raw", "on_dm_meds", "on_dm_insulin",
  "raised_glucose", "diabetes", "impaired_glucose",
  "glucose_ever_measured", "dm_diagnosed", "dm_diagnosed_12m", "dm_new",
  "total_chol", "raised_chol", "raised_chol_high",
  "raised_chol_or_meds", "raised_chol_high_or_meds", "on_chol_meds",
  "chol_ever_measured", "chol_diagnosed", "chol_diagnosed_12m",
  "hdl_chol", "low_hdl", "triglycerides", "raised_trig",

  # CVD history & lifestyle
  "cvd_history", "on_aspirin", "on_statins",
  "advised_quit_tobacco", "advised_reduce_salt", "advised_fruit_veg",
  "advised_reduce_fat", "advised_more_pa", "advised_healthy_weight",
  "cervical_screened",

  # Combined risk & CVD risk
  "cvd_risk_category", ".rf_count", ".rf_0", ".rf_1_2", ".rf_3_5",

  # Table formatting
  "both", "men", "women", "Prevalence",

  # Cleaning internals
  "smokeless_daily_raw", "tobacco_past_raw", "tobacco_past_daily_raw",
  "daily_smokeless", "alcohol_ever", ".data",

  # Diagnostics / plotting
  "digit", "pct", "pct_missing", "stat", "value", "variable",
  "short_label", "label_x", "label_y",

  # Shiny modules
  "zip", "req"
))

#' @importFrom stats as.formula confint rbinom rnorm runif
#' @importFrom utils head
#' @importFrom ggplot2 %+replace%
NULL

#' Visualise Module — UI
#'
#' Creates the user interface for the visualisation module, including a "Generate plots"
#' action button and tabbed layout for displaying STEPS survey visualizations.
#'
#' @param id Module namespace id.
#'
#' @return A Shiny UI definition.
#'
#' @keywords internal
mod_visualise_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      title = "Visualisation",
      width = 280,
      shiny::actionButton(
        ns("btn_generate"),
        "Generate plots",
        class = "btn-primary btn-lg",
        width = "100%",
        icon = shiny::icon("chart-bar")
      ),
      shiny::hr(),
      shiny::downloadButton(
        ns("dl_plots"),
        "Download all plots (ZIP)",
        class = "btn-outline-secondary btn-sm",
        style = "width:100%;"
      )
    ),

    # Tabbed plot display
    bslib::navset_card_tab(
      title = "STEPS Visualizations",
      id = ns("plot_tabs"),
      bslib::nav_panel("Overview", shiny::plotOutput(ns("plot_overview"), height = "600px")),
      bslib::nav_panel("Forest plot", shiny::plotOutput(ns("plot_forest"), height = "600px")),
      bslib::nav_panel("Risk profile", shiny::plotOutput(ns("plot_radar"), height = "600px")),
      bslib::nav_panel("Tobacco by Sex", shiny::plotOutput(ns("plot_tobacco_sex"), height = "600px")),
      bslib::nav_panel("BP by Sex", shiny::plotOutput(ns("plot_bp_sex"), height = "600px")),
      bslib::nav_panel("Obesity by Sex", shiny::plotOutput(ns("plot_obesity_sex"), height = "600px")),
      bslib::nav_panel("Glucose by Sex", shiny::plotOutput(ns("plot_glucose_sex"), height = "600px")),
      bslib::nav_panel("BP by Age", shiny::plotOutput(ns("plot_bp_age"), height = "600px")),
      bslib::nav_panel("Obesity by Age", shiny::plotOutput(ns("plot_obesity_age"), height = "600px")),
      bslib::nav_panel("Sex Dashboard", shiny::plotOutput(ns("plot_sex_dashboard"), height = "700px"))
    )
  )
}


#' Visualise Module — Server
#'
#' Server logic for the visualisation module. Generates plots on button click,
#' renders them in tabs, and provides ZIP download functionality.
#'
#' @param id Module namespace id.
#' @param results_out Reactive list returned by [mod_indicators_server()], containing
#'   `results` (list of indicator results) and `key_indicators` (tibble of key indicators).
#' @param upload_out Reactive list returned by [mod_upload_server()], containing
#'   `config` (list with country_name and survey_year).
#'
#' @return NULL (invisible). This is a terminal module for visualisation.
#'
#' @details
#' The module calls [build_steps_plots()] with the reactive inputs to generate
#' a named list of ggplot2 objects. These are then rendered in individual tabs.
#' The download handler creates a ZIP archive of all plots as PNG files.
#'
#' @keywords internal
mod_visualise_server <- function(id, results_out, upload_out) {
  shiny::moduleServer(id, function(input, output, session) {

    # Generate plots on button click
    plots <- shiny::eventReactive(input$btn_generate, {
      shiny::req(results_out$results(), results_out$key_indicators(), upload_out$config())

      cfg <- upload_out$config()

      build_steps_plots(
        results_out$results(),
        results_out$key_indicators(),
        cfg$country_name,
        cfg$survey_year
      )
    })

    # -- Helper: safe plot renderer that catches margin errors -----------------
    safe_render_plot <- function(plot_name, label) {
      shiny::renderPlot({
        shiny::req(plots())
        p <- plots()[[plot_name]]
        shiny::validate(shiny::need(!is.null(p), paste(label, "not available")))
        p
      }, res = 96)
    }

    # -- Render individual plots ------------------------------------------------
    output$plot_overview      <- safe_render_plot("overview", "Overview plot")
    output$plot_forest        <- safe_render_plot("forest", "Forest plot")
    output$plot_radar         <- safe_render_plot("radar", "Risk profile radar")
    output$plot_tobacco_sex   <- safe_render_plot("tobacco_by_sex", "Tobacco by sex plot")
    output$plot_bp_sex        <- safe_render_plot("bp_by_sex", "BP by sex plot")
    output$plot_obesity_sex   <- safe_render_plot("obesity_by_sex", "Obesity by sex plot")
    output$plot_glucose_sex   <- safe_render_plot("glucose_by_sex", "Glucose by sex plot")
    output$plot_bp_age        <- safe_render_plot("bp_by_age", "BP by age plot")
    output$plot_obesity_age   <- safe_render_plot("obesity_by_age", "Obesity by age plot")
    output$plot_sex_dashboard <- safe_render_plot("sex_dashboard", "Sex dashboard")

    # -- Download handler -------------------------------------------------------
    output$dl_plots <- shiny::downloadHandler(
      filename = function() {
        cfg <- upload_out$config()
        glue::glue("STEPS_plots_{cfg$country_name}_{cfg$survey_year}_{Sys.Date()}.zip")
      },
      content = function(file) {
        shiny::req(plots())

        plot_list <- plots()
        cfg <- upload_out$config()

        # Create temporary directory for PNGs
        temp_dir <- tempdir()
        png_files <- character(0)

        # Save each plot as PNG
        plot_names <- c(
          "overview", "forest", "radar",
          "tobacco_by_sex", "bp_by_sex", "obesity_by_sex",
          "glucose_by_sex", "bp_by_age", "obesity_by_age", "sex_dashboard"
        )

        for (plot_name in plot_names) {
          if (!is.null(plot_list[[plot_name]])) {
            png_path <- file.path(temp_dir, glue::glue("{plot_name}.png"))
            ggplot2::ggsave(
              filename = png_path,
              plot = plot_list[[plot_name]],
              width = 10,
              height = 6,
              dpi = 300
            )
            png_files <- c(png_files, png_path)
          }
        }

        # Create ZIP archive
        zip(
          zipfile = file,
          files = png_files,
          flags = "-j"
        )

        # Clean up temporary files
        unlink(png_files)
      }
    )

    # Return NULL (terminal module)
    invisible(NULL)
  })
}

#' Clean Module — UI
#' @param id Module namespace id.
#' @keywords internal
mod_clean_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      title = "Cleaning",
      width = 280,
      shiny::actionButton(ns("btn_clean"), "Clean data",
                          class = "btn-primary btn-lg", width = "100%",
                          icon = shiny::icon("broom")),
      shiny::hr(),
      shiny::downloadButton(ns("dl_clean"), "Download cleaned CSV",
                            class = "btn-outline-secondary btn-sm",
                            style = "width:100%;")
    ),

    # Summary cards
    bslib::layout_column_wrap(
      width = 1 / 4,
      bslib::value_box("Rows (raw)",    shiny::textOutput(ns("n_raw")),
                       theme = "primary", showcase = shiny::icon("database")),
      bslib::value_box("Rows (clean)",  shiny::textOutput(ns("n_clean")),
                       theme = "success", showcase = shiny::icon("check")),
      bslib::value_box("Rows dropped",  shiny::textOutput(ns("n_dropped")),
                       theme = "warning", showcase = shiny::icon("filter")),
      bslib::value_box("Variables",     shiny::textOutput(ns("n_vars")),
                       theme = "info",    showcase = shiny::icon("columns"))
    ),

    # Distribution checks
    bslib::navset_card_tab(
      title = "Derived variable distributions",
      bslib::nav_panel("Age & Sex",   DT::DTOutput(ns("tbl_demo"))),
      bslib::nav_panel("BMI",         DT::DTOutput(ns("tbl_bmi"))),
      bslib::nav_panel("BP staging",  DT::DTOutput(ns("tbl_bp"))),
      bslib::nav_panel("PA levels",   DT::DTOutput(ns("tbl_pa")))
    )
  )
}


#' Clean Module — Server
#'
#' @param id Module namespace id.
#' @param upload_out Reactive list returned by [mod_upload_server()].
#' @return A [shiny::reactive] returning the cleaned data frame.
#' @keywords internal
mod_clean_server <- function(id, upload_out) {
  shiny::moduleServer(id, function(input, output, session) {

    clean_data <- shiny::eventReactive(input$btn_clean, {
      shiny::req(upload_out$raw(), upload_out$cols())
      cfg <- upload_out$config()
      clean_steps_data(upload_out$raw(), upload_out$cols(),
                       age_min = cfg$age_min, age_max = cfg$age_max,
                       bp_sbp_threshold = cfg$bp_sbp_threshold,
                       bp_dbp_threshold = cfg$bp_dbp_threshold,
                       bmi_overweight = cfg$bmi_overweight,
                       bmi_obese = cfg$bmi_obese,
                       glucose_threshold = cfg$glucose_threshold,
                       glucose_impaired_threshold = cfg$glucose_impaired_threshold,
                       chol_threshold = cfg$chol_threshold)
    })

    # -- Summary value boxes ----------------------------------------------------
    output$n_raw     <- shiny::renderText({
      shiny::req(upload_out$raw()); format(nrow(upload_out$raw()), big.mark = ",")
    })
    output$n_clean   <- shiny::renderText({
      shiny::req(clean_data()); format(nrow(clean_data()), big.mark = ",")
    })
    output$n_dropped <- shiny::renderText({
      shiny::req(upload_out$raw(), clean_data())
      format(nrow(upload_out$raw()) - nrow(clean_data()), big.mark = ",")
    })
    output$n_vars    <- shiny::renderText({
      shiny::req(clean_data()); ncol(clean_data())
    })

    # -- Distribution tables ----------------------------------------------------
    output$tbl_demo <- DT::renderDT({
      shiny::req(clean_data())
      tbl <- as.data.frame(table(clean_data()$age_group, clean_data()$sex))
      names(tbl) <- c("Age group", "Sex", "n")
      DT::datatable(tbl, options = list(dom = "t", pageLength = 20),
                    rownames = FALSE)
    })

    output$tbl_bmi <- DT::renderDT({
      shiny::req(clean_data(), "bmi_category" %in% names(clean_data()))
      tbl <- as.data.frame(table(clean_data()$bmi_category, clean_data()$sex))
      names(tbl) <- c("BMI category", "Sex", "n")
      DT::datatable(tbl, options = list(dom = "t"), rownames = FALSE)
    })

    output$tbl_bp <- DT::renderDT({
      shiny::req(clean_data(), "bp_stage" %in% names(clean_data()))
      tbl <- as.data.frame(table(clean_data()$bp_stage, clean_data()$sex))
      names(tbl) <- c("BP stage", "Sex", "n")
      DT::datatable(tbl, options = list(dom = "t"), rownames = FALSE)
    })

    output$tbl_pa <- DT::renderDT({
      shiny::req(clean_data(), "pa_category" %in% names(clean_data()))
      tbl <- as.data.frame(table(clean_data()$pa_category, clean_data()$sex))
      names(tbl) <- c("PA level", "Sex", "n")
      DT::datatable(tbl, options = list(dom = "t"), rownames = FALSE)
    })

    # -- Download handler -------------------------------------------------------
    output$dl_clean <- shiny::downloadHandler(
      filename = function() paste0("steps_clean_", Sys.Date(), ".csv"),
      content  = function(file) {
        readr::write_csv(clean_data(), file)
      }
    )

    # Return
    clean_data
  })
}

#' Generic Table Builder for WHO STEPS Data Book
#'
#' Takes computed results from [compute_table()] or [compute_all_tables()]
#' and produces formatted flextable objects in the standard WHO STEPS
#' 3-panel format (Men / Women / Both Sexes).
#'
#' @name table_builder
#' @family tables
NULL


#' Build a formatted table from a computed result
#'
#' Dispatches to the appropriate formatting method based on table type.
#'
#' @param result A result list from [compute_table()].
#' @return A flextable object, or NULL if the table is not available.
#'
#' @export
build_table <- function(result) {
  if (!result$available) return(NULL)

  tryCatch(
    switch(result$type,
      "proportion" = .build_proportion_table(result),
      "mean"       = .build_mean_table(result),
      "category"   = .build_category_table(result),
      "cascade"    = .build_cascade_table(result),
      "combined"   = .build_cascade_table(result),   # same format
      {
        message(glue::glue("Unknown table type: {result$type}"))
        NULL
      }
    ),
    error = function(e) {
      message(glue::glue("  \u26a0 Could not build table {result$id}: {e$message}"))
      NULL
    }
  )
}


#' Build all tables from computed results
#'
#' @param results A named list of results from [compute_all_tables()].
#' @return A named list of flextable objects. NULL entries excluded.
#'
#' @export
build_all_tables <- function(results) {
  tables <- list()
  for (id in names(results)) {
    tbl <- build_table(results[[id]])
    if (!is.null(tbl)) tables[[id]] <- tbl
  }
  n <- length(tables)
  message(glue::glue("\u2713 Built {n} formatted tables."))
  tables
}


# ============================================================================
# WHO STEPS standard table format
# ============================================================================
# Every table follows this layout:
#
#  [Title]
#  [Description]
#
#  Age Group | n | Men % (95% CI) | n | Women % (95% CI) | n | Both % (95% CI)
#  ----------|---|----------------|---|------------------|---|----------------
#  18-29     |   |                |   |                  |   |
#  30-44     |   |                |   |                  |   |
#  45-59     |   |                |   |                  |   |
#  60-69     |   |                |   |                  |   |
#  18-69     |   |                |   |                  |   |
# ============================================================================

#' Format a single estimate with CI
#' @param est Numeric estimate.
#' @param lower Lower CI bound.
#' @param upper Upper CI bound.
#' @param digits Decimal places.
#' @param is_pct If TRUE, values are already in percent; add "%" suffix.
#' @return Formatted string.
#' @keywords internal
.fmt_ci <- function(est, lower, upper, digits = 1, is_pct = TRUE) {
  suffix <- if (is_pct) "%" else ""
  mapply(function(e, l, u) {
    if (is.na(e)) return("-")
    paste0(round(e, digits), suffix,
           " (", round(l, digits), "\u2013", round(u, digits), ")")
  }, est, lower, upper, USE.NAMES = FALSE)
}


#' Merge by_sex and by_age results into a single wide data frame
#'
#' Produces the standard 3-panel layout: Age Group | Men | Women | Both Sexes.
#'
#' @param total_df Total estimate (1-row data frame).
#' @param by_sex_df By-sex estimates (with `sex` column).
#' @param by_age_df By-age estimates (with `age_group` column).
#' @param is_pct Whether values are percentages.
#'
#' @return A data frame with columns: Age Group, Men, Women, Both Sexes.
#' @keywords internal
.merge_panels <- function(total_df, by_sex_df, by_age_df, is_pct = TRUE,
                          by_sex_age_df = NULL) {

  # Helper: identify male / female rows from a sex column
  .is_male   <- function(x) x %in% c("1", "Male", "male", "M", "m")
  .is_female <- function(x) x %in% c("2", "Female", "female", "F", "f")

  # --- By-age panel (Both Sexes) ---
  age_both <- by_age_df |>
    dplyr::mutate(both = .fmt_ci(estimate, lower, upper, is_pct = is_pct)) |>
    dplyr::select(age_group, both)

  # --- By-sex × age panel (Men / Women columns per age row) ---
  if (!is.null(by_sex_age_df) && nrow(by_sex_age_df) > 0) {
    sex_col <- by_sex_age_df$sex
    if (is.factor(sex_col)) sex_col <- as.character(sex_col)

    men_age <- by_sex_age_df[.is_male(sex_col), , drop = FALSE] |>
      dplyr::mutate(men = .fmt_ci(estimate, lower, upper, is_pct = is_pct)) |>
      dplyr::select(age_group, men)

    women_age <- by_sex_age_df[.is_female(sex_col), , drop = FALSE] |>
      dplyr::mutate(women = .fmt_ci(estimate, lower, upper, is_pct = is_pct)) |>
      dplyr::select(age_group, women)

    age_rows <- age_both |>
      dplyr::left_join(men_age, by = "age_group") |>
      dplyr::left_join(women_age, by = "age_group") |>
      dplyr::mutate(
        men   = dplyr::if_else(is.na(men), "-", men),
        women = dplyr::if_else(is.na(women), "-", women)
      ) |>
      dplyr::select(age_group, men, women, both)
  } else {
    # Fallback: no sex × age data
    age_rows <- age_both |>
      dplyr::mutate(men = "-", women = "-") |>
      dplyr::select(age_group, men, women, both)
  }

  # --- Total row (Men / Women from by_sex, Both from total) ---
  men_est   <- "-"
  women_est <- "-"
  if (!is.null(by_sex_df) && nrow(by_sex_df) >= 2) {
    sex_col <- by_sex_df$sex
    if (is.factor(sex_col)) sex_col <- as.character(sex_col)
    male_row   <- which(.is_male(sex_col))
    female_row <- which(.is_female(sex_col))
    if (length(male_row) > 0) {
      r <- by_sex_df[male_row[1], ]
      men_est <- .fmt_ci(r$estimate, r$lower, r$upper, is_pct = is_pct)
    }
    if (length(female_row) > 0) {
      r <- by_sex_df[female_row[1], ]
      women_est <- .fmt_ci(r$estimate, r$lower, r$upper, is_pct = is_pct)
    }
  }

  total_both <- .fmt_ci(total_df$estimate[1], total_df$lower[1], total_df$upper[1],
                         is_pct = is_pct)

  total_row <- data.frame(
    age_group = "18-69 (total)",
    men       = men_est,
    women     = women_est,
    both      = total_both,
    stringsAsFactors = FALSE
  )

  dplyr::bind_rows(age_rows, total_row)
}


#' Apply WHO STEPS styling to a flextable
#' @param ft A flextable object.
#' @param title Table title.
#' @param description Optional description text.
#' @return Styled flextable.
#' @keywords internal
.style_steps_table <- function(ft, title, description = NULL) {
  ft <- ft |>
    flextable::set_caption(title) |>
    flextable::theme_vanilla() |>
    flextable::bold(part = "header") |>
    flextable::bg(part = "header", bg = "#00427A") |>
    flextable::color(part = "header", color = "white") |>
    flextable::align(align = "center", part = "all") |>
    flextable::align(j = 1, align = "left", part = "body") |>
    flextable::autofit()

  if (!is.null(description)) {
    ft <- flextable::add_footer_lines(ft, values = description)
    ft <- flextable::italic(ft, part = "footer")
    ft <- flextable::fontsize(ft, size = 8, part = "footer")
  }

  ft
}


# ============================================================================
# Type-specific table builders
# ============================================================================

#' Build a proportion table
#' @keywords internal
.build_proportion_table <- function(result) {
  res <- result$results

  wide <- .merge_panels(
    total_df       = res$total,
    by_sex_df      = res$by_sex,
    by_age_df      = res$by_age,
    is_pct         = TRUE,
    by_sex_age_df  = res$by_sex_age
  )

  ft <- wide |>
    dplyr::rename(
      `Age Group`  = age_group,
      `Men`        = men,
      `Women`      = women,
      `Both Sexes` = both
    ) |>
    flextable::flextable()

  .style_steps_table(ft, result$title, result$results$description)
}


#' Build a mean table
#' @keywords internal
.build_mean_table <- function(result) {
  res <- result$results

  # Single variable
  if ("total" %in% names(res)) {
    unit_label <- if (length(result$unit) == 1) result$unit else ""
    title <- paste0(result$title, if (unit_label != "") paste0(" (", unit_label, ")") else "")

    wide <- .merge_panels(
      total_df       = res$total,
      by_sex_df      = res$by_sex,
      by_age_df      = res$by_age,
      is_pct         = FALSE,
      by_sex_age_df  = res$by_sex_age
    )

    ft <- wide |>
      dplyr::rename(
        `Age Group`  = age_group,
        `Men`        = men,
        `Women`      = women,
        `Both Sexes` = both
      ) |>
      flextable::flextable()

    return(.style_steps_table(ft, title))
  }

  # Multiple variables: build a stacked table
  units <- result$unit
  if (length(units) == 1) units <- rep(units, length(res))

  all_wide <- list()
  var_names <- names(res)
  for (i in seq_along(var_names)) {
    v <- var_names[i]
    sub <- res[[v]]
    u <- if (i <= length(units)) units[i] else ""
    subtitle <- paste0(gsub("_", " ", v), if (u != "") paste0(" (", u, ")") else "")

    wide <- .merge_panels(sub$total, sub$by_sex, sub$by_age, is_pct = FALSE,
                           by_sex_age_df = sub$by_sex_age)
    # Add a header row for this sub-variable
    header_row <- data.frame(
      age_group = subtitle, men = "", women = "", both = "",
      stringsAsFactors = FALSE
    )
    all_wide[[v]] <- dplyr::bind_rows(header_row, wide)
  }

  stacked <- dplyr::bind_rows(all_wide)
  ft <- stacked |>
    dplyr::rename(
      `Age Group`  = age_group,
      `Men`        = men,
      `Women`      = women,
      `Both Sexes` = both
    ) |>
    flextable::flextable()

  .style_steps_table(ft, result$title)
}


#' Build a category table
#' @keywords internal
.build_category_table <- function(result) {
  res <- result$results
  # res is a named list: level_label -> {total, by_sex, by_age}

  all_panels <- list()
  for (lbl in names(res)) {
    sub <- res[[lbl]]
    wide <- .merge_panels(sub$total, sub$by_sex, sub$by_age, is_pct = TRUE,
                           by_sex_age_df = sub$by_sex_age)
    # Add category header row
    header_row <- data.frame(
      age_group = paste0("  ", lbl), men = "", women = "", both = "",
      stringsAsFactors = FALSE
    )
    all_panels[[lbl]] <- dplyr::bind_rows(header_row, wide)
  }

  stacked <- dplyr::bind_rows(all_panels)
  ft <- stacked |>
    dplyr::rename(
      `Age Group`  = age_group,
      `Men`        = men,
      `Women`      = women,
      `Both Sexes` = both
    ) |>
    flextable::flextable()

  .style_steps_table(ft, result$title)
}


#' Build a cascade table (also used for combined risk factors)
#' @keywords internal
.build_cascade_table <- function(result) {
  res <- result$results
  # Same structure as category: named list of {total, by_sex, by_age}

  all_rows <- list()
  for (lbl in names(res)) {
    sub <- res[[lbl]]
    wide <- .merge_panels(sub$total, sub$by_sex, sub$by_age, is_pct = TRUE,
                           by_sex_age_df = sub$by_sex_age)
    # Add cascade step header
    header_row <- data.frame(
      age_group = lbl, men = "", women = "", both = "",
      stringsAsFactors = FALSE
    )
    all_rows[[lbl]] <- dplyr::bind_rows(header_row, wide)
  }

  stacked <- dplyr::bind_rows(all_rows)
  ft <- stacked |>
    dplyr::rename(
      `Age Group`  = age_group,
      `Men`        = men,
      `Women`      = women,
      `Both Sexes` = both
    ) |>
    flextable::flextable()

  .style_steps_table(ft, result$title)
}

#' Design Module — UI
#' @param id Module namespace id.
#' @keywords internal
mod_design_ui <- function(id) {
  ns <- shiny::NS(id)

  bslib::layout_sidebar(
    sidebar = bslib::sidebar(
      shiny::actionButton(
        ns("btn_design"),
        "Set up design",
        class = "w-100",
        icon = shiny::icon("cogs")
      ),
      title = "Survey Design"
    ),

    bslib::card(
      bslib::card_header("Design Summary"),
      shiny::fluidRow(
        shiny::column(
          3,
          bslib::value_box(
            title = "Design Type",
            value = shiny::textOutput(ns("design_type")),
            theme = "primary"
          )
        ),
        shiny::column(
          3,
          bslib::value_box(
            title = "Unweighted N",
            value = shiny::textOutput(ns("unweighted_n")),
            theme = "info"
          )
        ),
        shiny::column(
          3,
          bslib::value_box(
            title = "Weighted N",
            value = shiny::textOutput(ns("weighted_n")),
            theme = "success"
          )
        ),
        shiny::column(
          3,
          bslib::value_box(
            title = "Strata",
            value = shiny::textOutput(ns("n_strata")),
            theme = "warning"
          )
        )
      ),
      shiny::fluidRow(
        shiny::column(
          3,
          bslib::value_box(
            title = "PSUs",
            value = shiny::textOutput(ns("n_psus")),
            theme = "danger"
          )
        )
      ),
      class = "mt-3"
    ),

    bslib::card(
      bslib::card_header("Design Elements"),
      DT::DTOutput(ns("design_table")),
      class = "mt-3"
    )
  )
}

#' Design Module — Server
#' @param id Module namespace id.
#' @param clean_out Reactive returning the cleaned data frame.
#' @return A [shiny::reactive] returning the survey design object.
#' @keywords internal
mod_design_server <- function(id, clean_out) {
  shiny::moduleServer(id, function(input, output, session) {

    # Create design when button clicked
    design <- shiny::eventReactive(
      input$btn_design,
      {
        shiny::req(clean_out())
        setup_survey_design(clean_out())
      },
      ignoreNULL = TRUE
    )

    # Extract design information
    design_info <- shiny::reactive({
      shiny::req(design())
      d <- design()

      # setup_survey_design now returns a steps_designs list
      d1 <- if (inherits(d, "steps_designs")) d$step1 else d

      # Get the variables data frame
      data <- d1$variables

      # Check for design elements
      has_weight <- "wt_step1" %in% names(data) || "wt_final" %in% names(data)
      has_strata <- "stratum" %in% names(data)
      has_cluster <- "psu" %in% names(data)

      # Determine design type
      if (has_strata && has_cluster && has_weight) {
        design_type <- "Stratified Clustered (Weighted)"
      } else if (has_strata && has_cluster) {
        design_type <- "Stratified Clustered"
      } else if (has_strata && has_weight) {
        design_type <- "Stratified (Weighted)"
      } else if (has_cluster && has_weight) {
        design_type <- "Clustered (Weighted)"
      } else if (has_strata) {
        design_type <- "Stratified"
      } else if (has_cluster) {
        design_type <- "Clustered"
      } else if (has_weight) {
        design_type <- "Weighted"
      } else {
        design_type <- "Simple"
      }

      # Calculate counts
      unweighted_n <- nrow(data)

      wt_col <- if ("wt_step1" %in% names(data)) "wt_step1" else if ("wt_final" %in% names(data)) "wt_final" else NULL
      weighted_n <- if (!is.null(wt_col)) {
        round(sum(data[[wt_col]], na.rm = TRUE), 0)
      } else {
        unweighted_n
      }

      n_strata <- if (has_strata) {
        length(unique(data$stratum[!is.na(data$stratum)]))
      } else {
        0
      }

      n_psus <- if (has_cluster) {
        length(unique(data$psu[!is.na(data$psu)]))
      } else {
        0
      }

      list(
        design_type = design_type,
        unweighted_n = unweighted_n,
        weighted_n = weighted_n,
        n_strata = n_strata,
        n_psus = n_psus,
        has_weight = has_weight,
        has_strata = has_strata,
        has_cluster = has_cluster
      )
    })

    # Render value boxes
    output$design_type <- shiny::renderText({
      shiny::req(design_info())
      design_info()$design_type
    })

    output$unweighted_n <- shiny::renderText({
      shiny::req(design_info())
      format(design_info()$unweighted_n, big.mark = ",")
    })

    output$weighted_n <- shiny::renderText({
      shiny::req(design_info())
      format(design_info()$weighted_n, big.mark = ",")
    })

    output$n_strata <- shiny::renderText({
      shiny::req(design_info())
      design_info()$n_strata
    })

    output$n_psus <- shiny::renderText({
      shiny::req(design_info())
      design_info()$n_psus
    })

    # Render design summary table
    output$design_table <- DT::renderDT({
      shiny::req(design_info())
      info <- design_info()

      summary_df <- data.frame(
        Element = c("Weight (Step 1)", "Weight (Step 2)", "Weight (Step 3)",
                     "Stratum Column", "PSU Column"),
        Detected = c(
          if (info$has_weight) "Yes (wt_step1)" else "No",
          if (info$has_weight) "Yes (wt_step2)" else "No",
          if (info$has_weight) "Yes (wt_step3)" else "No",
          if (info$has_strata) "Yes (stratum)" else "No",
          if (info$has_cluster) "Yes (psu)" else "No"
        ),
        Status = c(
          if (info$has_weight) "Active" else "Not used",
          if (info$has_weight) "Active" else "Not used",
          if (info$has_weight) "Active" else "Not used",
          if (info$has_strata) "Active" else "Not used",
          if (info$has_cluster) "Active" else "Not used"
        ),
        stringsAsFactors = FALSE
      )

      DT::datatable(
        summary_df,
        options = list(
          paging = FALSE,
          searching = FALSE,
          info = FALSE,
          ordering = FALSE,
          columnDefs = list(
            list(width = "150px", targets = 0)
          )
        ),
        rownames = FALSE
      )
    })

    # Return design reactive for downstream use
    design
  })
}

#' Build survey-weighted tables for STEPS indicators
#'
#' Generates formatted flextable objects for all available STEPS indicators,
#' with rows for age groups and columns for both sexes combined, males, and
#' females. Tables include 95% confidence intervals.
#'
#' @param indicators A list of indicator results from [compute_all_indicators()],
#'   containing elements like `tobacco`, `alcohol`, `diet_pa`, `anthropometry`,
#'   `blood_pressure`, `biochemical`, etc. Each indicator list should contain
#'   `*_total`, `*_by_sex`, and `*_by_age` elements.
#'
#' @return A named list of flextable objects, one per indicator.
#'   Names correspond to indicators (e.g., `current_tobacco`, `raised_bp`).
#'   NULL entries are excluded. Prints count of tables generated.
#'
#' @details
#' Each table has age groups as rows and prevalence (with 95% CI) as a column.
#' The last row shows the total (age-standardised) estimate.
#' Column header styling uses WHO STEPS branding (dark blue background).
#'
#' @examples
#' \donttest{
#'   test_data <- generate_test_data(n = 500, seed = 42)
#'   cols <- detect_steps_columns(test_data)
#'   clean <- clean_steps_data(test_data, cols)
#'   design <- setup_survey_design(clean)
#'   all_ind <- compute_all_indicators(design)
#'   tables <- build_steps_tables(all_ind$results)
#'   names(tables)
#' }
#'
#' @export
build_steps_tables <- function(indicators) {
  tables <- list()

  # Prefer current_tobacco_any (smoking + smokeless) as headline tobacco table
  tob_var <- if (!is.null(indicators$tobacco[["current_tobacco_any_total"]])) {
    "current_tobacco_any"
  } else {
    "current_tobacco"
  }
  tables$current_tobacco <- build_table_if_available(
    indicators$tobacco,
    paste0(tob_var, "_total"), paste0(tob_var, "_by_sex"), paste0(tob_var, "_by_age"),
    "Table 1. Current tobacco use (%)"
  )
  tables$current_alcohol <- build_table_if_available(
    indicators$alcohol, "current_alcohol_total", "current_alcohol_by_sex", "current_alcohol_by_age",
    "Table 2. Current alcohol use (%)"
  )
  tables$insufficient_pa <- build_table_if_available(
    indicators$diet_pa, "insufficient_pa_total", "insufficient_pa_by_sex", "insufficient_pa_by_age",
    "Table 3. Insufficient physical activity (%)"
  )
  tables$low_fruit_veg <- build_table_if_available(
    indicators$diet_pa, "low_fruit_veg_total", "low_fruit_veg_by_sex", "low_fruit_veg_by_age",
    "Table 4. Low fruit and vegetable intake (%)"
  )
  tables$overweight_obese <- build_table_if_available(
    indicators$anthropometry, "overweight_obese_total", "overweight_obese_by_sex", "overweight_obese_by_age",
    "Table 5. Overweight or obese (BMI >=25, %)"
  )
  tables$raised_bp <- build_table_if_available(
    indicators$blood_pressure, "raised_bp_total", "raised_bp_by_sex", "raised_bp_by_age",
    "Table 6. Raised blood pressure (%)"
  )
  tables$raised_glucose <- build_table_if_available(
    indicators$biochemical, "raised_glucose_total", "raised_glucose_by_sex", "raised_glucose_by_age",
    "Table 7. Raised fasting blood glucose (%)"
  )
  tables$raised_chol <- build_table_if_available(
    indicators$biochemical, "raised_chol_total", "raised_chol_by_sex", "raised_chol_by_age",
    "Table 8. Raised total cholesterol (%)"
  )

  # Remove NULL entries
  tables <- Filter(Negate(is.null), tables)

  n_tables <- length(tables)
  message(glue::glue("\u2713 Generated {n_tables} tables."))

  return(tables)
}

#' Format estimate with confidence interval
#'
#' @param est Estimated value.
#' @param lower Lower confidence bound.
#' @param upper Upper confidence bound.
#' @param digits Number of decimal places (default 1).
#' @param pct Logical; add percent sign? (default TRUE).
#'
#' @return Formatted string like "42.1% (39.2–45.0)".
#'
#' @keywords internal
fmt_est <- function(est, lower, upper, digits = 1, pct = TRUE) {
  suf <- if (pct) "%" else ""
  glue::glue("{round(est, digits)}{suf} ({round(lower, digits)}\u2013{round(upper, digits)})")
}

#' Build a single STEPS table
#'
#' Constructs a flextable from age-stratified and total estimates.
#'
#' @param total_df Data frame with total estimate, lower, upper.
#' @param by_sex_df Data frame with sex-stratified estimates (not used, kept for compatibility).
#' @param by_age_df Data frame with age-stratified estimates and `age_group` column.
#' @param label Table caption.
#' @param pct Logical; format as percent? (default TRUE).
#'
#' @return A flextable object.
#'
#' @keywords internal
make_steps_table <- function(total_df, by_sex_df, by_age_df, label, pct = TRUE) {

  # By age - combine sexes
  age_total <- by_age_df |>
    dplyr::mutate(dplyr::across(dplyr::where(is.numeric), ~ round(., 1))) |>
    dplyr::mutate(both = fmt_est(estimate, lower, upper, pct = pct)) |>
    dplyr::select(age_group, both)

  # Total row
  total_row <- total_df |>
    dplyr::mutate(both = fmt_est(estimate, lower, upper, pct = pct)) |>
    dplyr::mutate(age_group = "Total (age-standardised)") |>
    dplyr::select(age_group, both)

  dplyr::bind_rows(age_total, total_row) |>
    dplyr::rename(`Age group` = age_group, `% (95% CI)` = both) |>
    flextable::flextable() |>
    flextable::set_caption(label) |>
    flextable::theme_vanilla() |>
    flextable::bold(part = "header") |>
    flextable::bg(part = "header", bg = "#00427A") |>
    flextable::color(part = "header", color = "white") |>
    flextable::autofit()
}

#' Build a table if indicator is available
#'
#' Wrapper that safely builds a table only if the required indicator elements exist.
#' Catches and reports errors gracefully.
#'
#' @param ind_list Indicator list (e.g., `indicators$tobacco`).
#' @param total_key Name of total estimate element (e.g., "current_tobacco_total").
#' @param by_sex_key Name of by-sex element (e.g., "current_tobacco_by_sex").
#' @param by_age_key Name of by-age element (e.g., "current_tobacco_by_age").
#' @param label Table caption.
#' @param pct Logical; format as percent? (default TRUE).
#'
#' @return A flextable object or NULL if not available.
#'
#' @keywords internal
build_table_if_available <- function(ind_list, total_key, by_sex_key, by_age_key, label, pct = TRUE) {
  if (!is.null(ind_list[[total_key]])) {
    tryCatch(
      make_steps_table(
        ind_list[[total_key]],
        ind_list[[by_sex_key]],
        ind_list[[by_age_key]],
        label, pct
      ),
      error = function(e) {
        message(glue::glue("  \u26a0 Could not build table for {label}: {e$message}"))
        NULL
      }
    )
  } else NULL
}

#' Run the complete STEPS analysis pipeline
#'
#' Imports raw data, cleans it, sets up the survey design, computes
#' all indicators, generates publication-ready tables and plots, and
#' optionally renders Word reports.
#'
#' This is the main entry point for end-to-end STEPS analysis.
#'
#' @param data_path Path to raw STEPS data file (CSV, Excel, Stata, or SPSS).
#' @param country_name Country name for reports (default "Country Name").
#' @param survey_year Survey year (default 2024).
#' @param age_min Minimum age in years (default 18).
#' @param age_max Maximum age in years (default 69).
#' @param output_dir Directory for all outputs (default \code{tempdir()}).
#' @param render_reports Logical; render Word documents? (default TRUE).
#' @param mapping_file Optional path to a filled column mapping template
#'   (Excel or CSV). If provided, uses [read_column_mapping()] instead of
#'   auto-detection. See the template at
#'   \code{system.file("templates", "column_mapping_template.xlsx",
#'   package = "stepssurvey")}.
#'
#' @return A list with elements:
#'   \describe{
#'     \item{raw_data}{Original imported data frame}
#'     \item{clean_data}{Cleaned and recoded data}
#'     \item{cols}{Detected column mapping from [detect_steps_columns()]}
#'     \item{design}{[survey::svydesign] object}
#'     \item{indicators}{List of all computed indicator results by domain}
#'     \item{key_indicators}{Summary tibble of headline estimates}
#'     \item{tables}{List of [flextable::flextable] objects}
#'     \item{plots}{List of [ggplot2::ggplot] objects}
#'     \item{config}{Configuration list from [steps_config()]}
#'   }
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Auto-detect columns
#' result <- run_steps_pipeline("data/raw/steps_data.csv",
#'                              country_name = "Senegal",
#'                              survey_year = 2023)
#' result$key_indicators
#' result$plots$overview
#'
#' # Use a custom column mapping
#' result <- run_steps_pipeline("data/raw/steps_data.csv",
#'                              country_name = "Senegal",
#'                              survey_year = 2023,
#'                              mapping_file = "my_mapping.xlsx")
#' }
run_steps_pipeline <- function(data_path,
                               country_name = "Country Name",
                               survey_year = 2024,
                               age_min = 18,
                               age_max = 69,
                               output_dir = tempdir(),
                               render_reports = TRUE,
                               mapping_file = NULL) {

  message("\n========================================")
  message(" WHO STEPS Analysis Pipeline")
  message("========================================\n")

  # -- 1. Import data ----------------------------------------------------------
  message("-- Step 1: Importing data ------------------------------------------")

  raw_data <- import_steps_data(data_path)

  if (!is.null(mapping_file)) {
    message("  Using column mapping from: ", mapping_file)
    cols <- read_column_mapping(mapping_file, data = raw_data)
  } else {
    cols <- detect_steps_columns(raw_data)
  }

  # -- 2. Clean data ----------------------------------------------------------
  message("-- Step 2: Cleaning data -----------------------------------------")
  clean_data <- clean_steps_data(raw_data, cols,
                                 age_min = age_min,
                                 age_max = age_max)

  # -- 3. Survey design -------------------------------------------------------
  message("-- Step 3: Survey design -----------------------------------------")
  design <- setup_survey_design(clean_data)

  # -- 4. Compute indicators --------------------------------------------------
  message("-- Step 4: Computing indicators --------------------------------")
  all_ind <- compute_all_indicators(design)
  indicators <- all_ind$results
  key_indicators <- all_ind$key_indicators

  # -- 5. Tables --------------------------------------------------------------
  message("-- Step 5: Generating tables -----------------------------------")
  tables <- build_steps_tables(indicators)

  # -- 6. Visualisations ------------------------------------------------------
  message("-- Step 6: Generating visualisations --------------------------")
  plots <- build_steps_plots(indicators, key_indicators,
                             country_name, survey_year)

  fig_dir <- file.path(output_dir, "figures")
  dir.create(fig_dir, recursive = TRUE, showWarnings = FALSE)
  save_steps_plots(plots, fig_dir)

  # -- 7. Config for reports --------------------------------------------------
  config <- steps_config(
    data_path    = data_path,
    country_name = country_name,
    survey_year  = survey_year,
    age_min      = age_min,
    age_max      = age_max
  )

  # -- 8. Reports (optional) --------------------------------------------------
  if (render_reports) {
    message("-- Step 7: Rendering reports ----------------------------------")

    # Save intermediate results for Rmd templates
    proc_dir <- file.path(output_dir, "data", "processed")
    dir.create(proc_dir, recursive = TRUE, showWarnings = FALSE)
    saveRDS(indicators,     file.path(proc_dir, "indicators.rds"))
    saveRDS(key_indicators, file.path(proc_dir, "key_indicators.rds"))
    saveRDS(tables,         file.path(proc_dir, "tables.rds"))
    saveRDS(plots,          file.path(proc_dir, "plots.rds"))

    # Set data_dir so the Rmd template knows where to find the RDS files
    config$data_dir <- proc_dir

    tryCatch({
      render_fact_sheet(config, output_dir)
      render_data_book(config, output_dir)
      render_country_report(config, output_dir)
    }, error = function(e) {
      message(glue::glue("  \u26a0 Report rendering failed: {e$message}"))
    })
  }

  message(glue::glue("\n\u2713 Pipeline complete.\n"))

  list(
    raw_data       = raw_data,
    clean_data     = clean_data,
    cols           = cols,
    design         = design,
    indicators     = indicators,
    key_indicators = key_indicators,
    tables         = tables,
    plots          = plots,
    config         = config
  )
}

---
title: "Validation Against WHO Official Fact Sheets"
author: "Abhijit Pakhare"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Validation Against WHO Official Fact Sheets}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)
```

## Acknowledgements

This validation relies on several resources developed and maintained by
the World Health Organization (WHO):

- **WHO STEPS instrument and methodology**: The STEPwise Approach to NCD
  Risk Factor Surveillance (STEPS) was developed by the WHO Department
  of Noncommunicable Diseases. The instrument, survey manuals, and
  analysis guidelines (including the *STEPS Part 4: Guide to Physical
  Measurements* and *Part 5: Data Management and Analysis*) define the
  standard definitions and indicator computations used throughout this
  package.

- **WHO official STEPS analysis scripts**: The R scripts maintained by
  WHO for STEPS data processing and databook generation
  (available at <https://github.com/WorldHealthOrganization>)
  served as the authoritative reference for indicator derivation logic,
  survey design setup, weight handling, and data quality filters
  (`smk_cln`, `smkless_cln`). Key methodological decisions in
  **stepssurvey** --- such as not trimming sampling weights, setting
  skip-pattern NAs to zero for denominator alignment, and applying
  screening-question logic for GPAQ --- were directly informed by
  studying these scripts.

- **WHO STEPS country fact sheets**: Published fact sheet values for
  nine countries (see table below) were used as the gold-standard
  reference estimates. These fact sheets are publicly available from
  the WHO NCD Microdata Repository
  (<https://extranet.who.int/ncdsmicrodata>).

- **Licensed STEPS microdata**: The underlying survey microdata were
  accessed under license from the WHO NCD Microdata Repository. Use of
  these data is subject to the WHO STEPS data access agreement, and the
  microdata files are not redistributable.

We gratefully acknowledge the national STEPS survey teams and
coordinators in all nine validation countries for designing, conducting,
and managing these surveys and for making the data available for
research use through the WHO NCD Microdata Repository. The quality and
completeness of these datasets made systematic validation possible.

We also acknowledge the WHO NCD Surveillance team for developing and
maintaining the STEPS methodology, analysis tools, and data sharing
infrastructure that underpin this work.

## Overview

The **stepssurvey** package has been validated against WHO-published fact
sheet values for nine national STEPS surveys spanning different WHO
regions, time periods, and STEPS instrument versions:

| Country               | Year | WHO Region | Age range | Instrument |
|:----------------------|:----:|:----------:|:---------:|:----------:|
| Republic of Moldova   | 2021 | EUR        | 18--69    | v3.2       |
| Mongolia              | 2019 | WPR        | 15--69    | v3.2       |
| Georgia               | 2016 | EUR        | 18--69    | v3.1       |
| Afghanistan           | 2018 | EMR        | 18--69    | v3.2       |
| Algeria               | 2016 | AFR        | 18--69    | v3.1       |
| Ukraine               | 2019 | EUR        | 18--69    | v3.2       |
| Ecuador               | 2018 | AMR        | 18--69    | v3.2       |
| Cabo Verde            | 2020 | AFR        | 18--69    | v3.2       |
| Bahamas               | 2019 | AMR        | 18--69    | v3.2       |

For each country, the package pipeline (import -> detect -> clean -> survey
design -> indicator computation) was run end-to-end and results compared
with the "Both Sexes" estimates published in the corresponding WHO STEPS
country fact sheet.

## Validation criteria

Two complementary criteria were used to judge agreement:

1. **Point-estimate match** --- the package estimate falls within 1.0
   percentage point (pp) of the fact sheet value for proportions, or 1.0
   unit for continuous measures (BMI, SBP, cholesterol).
2. **Confidence-interval overlap** --- the 95 % confidence intervals from
   the package and the fact sheet share at least one common value.

The 1 pp tolerance accounts for rounding at different stages and minor
differences in the treatment of edge cases (e.g. "don't know" responses
coded as 77 or 88).

## Indicators compared

Up to 17 indicators were compared per country, covering all three STEPS
Steps:

**Step 1 --- Behavioural risk factors:**
current tobacco use, current smoking, second-hand smoke exposure (home
and workplace), current alcohol use (past 30 days), heavy episodic
drinking, insufficient fruit and vegetable consumption, insufficient
physical activity.

**Step 2 --- Physical measurements:**
mean BMI, overweight (BMI >= 25), obesity (BMI >= 30), mean systolic
blood pressure, raised blood pressure or on medication.

**Step 3 --- Biochemical measurements:**
mean total cholesterol, raised cholesterol or on medication, raised
fasting glucose or on medication, impaired fasting glucose.

Not all indicators were available in every country fact sheet. Some fact
sheets omit second-hand smoke, and the Bahamas fact sheet reports Step 3
biochemical measurements as unweighted estimates (response rate < 60 %),
so those were excluded from comparison.

## Results summary

```{r summary-table, eval=TRUE, echo=FALSE, results='asis'}
summary_data <- data.frame(
  Country = c("Moldova 2021", "Mongolia 2019", "Georgia 2016",
              "Afghanistan 2018", "Algeria 2016", "Ukraine 2019",
              "Ecuador 2018", "Cabo Verde 2020", "Bahamas 2019",
              "**TOTAL**"),
  Compared = c(17, 15, 15, 14, 14, 14, 14, 15, 10, 128),
  Within_1pp = c(16, 15, 15, 14, 14, 13, 13, 14, 9, 123),
  CI_Overlap = c(17, 15, 15, 14, 14, 14, 14, 15, 10, 128),
  Match_Rate = c("94%", "100%", "100%", "100%", "100%", "93%",
                 "93%", "93%", "90%", "**96%**")
)

knitr::kable(
  summary_data,
  col.names = c("Country", "Indicators compared", "Within 1 pp",
                "CI overlap", "Match rate"),
  align = c("l", "c", "c", "c", "c"),
  caption = "Validation summary: stepssurvey vs. WHO fact sheets (9 countries)"
)
```

Overall, **123 of 128 indicators (96 %) match within 1 pp**, and
**all 128 (100 %) have overlapping confidence intervals**. The five
remaining mismatches are small (1.0--2.2 pp) and all have overlapping
CIs, suggesting they reflect minor methodological differences rather
than errors in the package.

## Detailed results by country

### Moldova 2021

```{r moldova, eval=TRUE, echo=FALSE, results='asis'}
mda <- data.frame(
  Indicator = c(
    "Current tobacco use", "Current smoking", "Second-hand smoke (home)",
    "Second-hand smoke (work)", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(27.7, 27.6, 23.3, 26.4, 63.2, 13.8, 63.2, 9.1,
           63.9, 22.7, 35.0, 6.3, 28.4, 9.9, 26.9, 129.2, 4.4),
  WHO = c(29.9, 27.6, 23.2, 26.4, 63.2, 13.8, 63.4, 9.1,
          63.9, 22.7, 34.8, 6.3, 27.7, 9.9, 26.9, 129.2, 4.4),
  Diff = c(-2.2, 0.0, 0.1, 0.0, 0.0, 0.0, -0.2, 0.0,
           0.0, 0.0, 0.2, 0.0, 0.7, 0.0, 0.0, 0.0, 0.0),
  Match = c("No", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes"),
  CI_Overlap = c("Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes",
                 "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes", "Yes")
)

knitr::kable(mda, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Moldova 2021: 16/17 within 1 pp, 17/17 CI overlap")
```

The single mismatch is **current tobacco use** (--2.2 pp). The confidence
intervals overlap (ours: 25.6--29.8 vs. WHO: 27.7--32.1), suggesting the
difference may reflect rounding or a slightly different indicator
definition in the published fact sheet.

### Mongolia 2019

```{r mongolia, eval=TRUE, echo=FALSE, results='asis'}
mng <- data.frame(
  Indicator = c(
    "Current tobacco use", "Current smoking (daily)",
    "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds (130/80)",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(25.0, 21.6, 34.8, 20.2, 83.2, 22.5,
           49.3, 18.5, 44.3, 8.3, 27.8, 17.4, 25.6, 120.5, 4.4),
  WHO = c(24.2, 21.6, 34.8, 19.8, 83.4, 21.9,
          49.4, 18.5, 44.0, 8.3, 27.8, 17.4, 25.5, 120.5, 4.4),
  Diff = c(0.8, 0.0, 0.0, 0.4, -0.2, 0.6,
           -0.1, 0.0, 0.3, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0),
  Match = rep("Yes", 15),
  CI_Overlap = rep("Yes", 15)
)

knitr::kable(mng, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Mongolia 2019: 15/15 within 1 pp, 15/15 CI overlap")
```

Mongolia uses a raised blood-pressure threshold of **130/80 mmHg**
(rather than the standard 140/90), and the age range begins at 15
(rather than 18). Second-hand smoke indicators were not reported in the
Mongolia fact sheet.

Note: the Mongolia WHO fact sheet reports **daily smoking** under the
"current smoking" label. The package computes both any-current and
daily smoking; the daily-smoking variable is used for this comparison.

### Georgia 2016

```{r georgia, eval=TRUE, echo=FALSE, results='asis'}
geo <- data.frame(
  Indicator = c(
    "Current tobacco use", "Current smoking (daily)",
    "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(31.1, 28.0, 39.0, 18.7, 62.9, 18.2,
           64.6, 33.4, 37.7, 4.5, 27.7, 2.0, 28.2, 129.4, 4.3),
  WHO = c(31.0, 28.0, 39.1, 18.3, 63.0, 17.4,
          64.6, 33.2, 37.7, 4.5, 27.7, 2.0, 28.1, 129.4, 4.3),
  Diff = c(0.1, 0.0, -0.1, 0.4, -0.1, 0.8,
           0.0, 0.2, 0.0, 0.0, 0.0, 0.0, 0.1, 0.0, 0.0),
  Match = rep("Yes", 15),
  CI_Overlap = rep("Yes", 15)
)

knitr::kable(geo, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Georgia 2016: 15/15 within 1 pp, 15/15 CI overlap")
```

As with Mongolia, the Georgia WHO fact sheet reports **daily smoking**
under the "current smoking" label. Second-hand smoke indicators were
not reported in the Georgia fact sheet.

### Afghanistan 2018

```{r afghanistan, eval=TRUE, echo=FALSE, results='asis'}
afg <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(8.6, 0.2, 0.1, 97.3, 26.6,
           42.7, 17.2, 29.2, 9.2, 17.8, 4.9,
           25.2, 125.5, 3.8),
  WHO = c(8.6, 0.2, 0.1, 97.3, 26.5,
          42.7, 17.0, 29.2, 9.2, 18.1, 4.7,
          25.1, 125.5, 3.8),
  Diff = c(0.0, 0.0, 0.0, 0.0, 0.1,
           0.0, 0.2, 0.0, 0.0, -0.3, 0.2,
           0.1, 0.0, 0.0),
  Match = rep("Yes", 14),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(afg, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Afghanistan 2018: 14/14 within 1 pp, 14/14 CI overlap")
```

All 14 comparable indicators match within 1 pp. The Afghanistan fact
sheet reports cholesterol in mg/dl; the package auto-converts to mmol/L,
and the raised-cholesterol threshold was aligned to 4.914 mmol/L
(= 190 mg/dl) to match the WHO definition.

### Algeria 2016

```{r algeria, eval=TRUE, echo=FALSE, results='asis'}
dza <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(16.4, 2.1, 1.3, 85.2, 23.7,
           55.5, 21.9, 23.7, 8.8, 23.5, 8.6,
           26.4, 126.4, 4.2),
  WHO = c(16.5, 2.1, 1.3, 85.3, 23.7,
          55.6, 21.8, 23.6, 9.0, 24.0, 8.2,
          26.4, 126.3, 4.2),
  Diff = c(-0.1, 0.0, 0.0, -0.1, 0.0,
           -0.1, 0.1, 0.1, -0.2, -0.5, 0.4,
           0.0, 0.1, 0.0),
  Match = rep("Yes", 14),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(dza, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Algeria 2016: 14/14 within 1 pp, 14/14 CI overlap")
```

All 14 comparable indicators match within 1 pp.

### Ukraine 2019

```{r ukraine, eval=TRUE, echo=FALSE, results='asis'}
ukr <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(33.7, 55.5, 20.3, 66.1, 10.7,
           59.1, 24.9, 36.7, 7.1, 40.7, 9.0,
           26.9, 129.2, 4.7),
  WHO = c(33.9, 55.6, 19.7, 66.4, 10.0,
          59.0, 24.8, 34.8, 7.1, 40.7, 8.8,
          26.8, 129.1, 4.7),
  Diff = c(-0.2, -0.1, 0.6, -0.3, 0.7,
           0.1, 0.1, 1.9, 0.0, 0.0, 0.2,
           0.1, 0.1, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "No", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(ukr, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Ukraine 2019: 13/14 within 1 pp, 14/14 CI overlap")
```

The single mismatch is **raised BP or on meds** (+1.9 pp). The CIs
overlap (ours: 33.7--39.6 vs. WHO: 31.2--38.4), suggesting a minor
difference in medication question coding.

### Ecuador 2018

```{r ecuador, eval=TRUE, echo=FALSE, results='asis'}
ecu <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(13.7, 39.3, 24.1, 94.6, 17.8,
           63.6, 25.7, 20.5, 6.9, 33.7, 8.3,
           27.2, 119.7, 4.4),
  WHO = c(13.7, 39.3, 23.8, 94.6, 17.8,
          63.6, 25.7, 19.8, 7.1, 34.7, 7.8,
          27.2, 119.7, 4.4),
  Diff = c(0.0, 0.0, 0.3, 0.0, 0.0,
           0.0, 0.0, 0.7, -0.2, -1.0, 0.5,
           0.0, 0.0, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes", "Yes", "No", "Yes",
            "Yes", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 14)
)

knitr::kable(ecu, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Ecuador 2018: 13/14 within 1 pp, 14/14 CI overlap")
```

The single mismatch is **raised cholesterol or on meds** (--1.0 pp,
right at the threshold boundary). The CIs overlap (ours: 31.7--35.8
vs. WHO: 32.6--36.8).

### Cabo Verde 2020

```{r caboverde, eval=TRUE, echo=FALSE, results='asis'}
cpv <- data.frame(
  Indicator = c(
    "Current smoking", "Second-hand smoke (work)",
    "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Raised glucose or on meds", "Raised cholesterol or on meds",
    "Impaired fasting glucose",
    "Mean BMI", "Mean SBP", "Mean total cholesterol"
  ),
  Ours = c(9.6, 15.0, 45.0, 17.5, 77.3, 31.6,
           44.2, 14.3, 31.1, 3.6, 17.9, 2.4,
           25.1, 128.8, 4.0),
  WHO = c(9.6, 15.0, 45.0, 17.5, 79.0, 31.8,
          44.2, 14.3, 30.8, 3.7, 18.8, 2.3,
          25.1, 128.8, 4.0),
  Diff = c(0.0, 0.0, 0.0, 0.0, -1.7, -0.2,
           0.0, 0.0, 0.3, -0.1, -0.9, 0.1,
           0.0, 0.0, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "No", "Yes",
            "Yes", "Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 15)
)

knitr::kable(cpv, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Cabo Verde 2020: 14/15 within 1 pp, 15/15 CI overlap")
```

The single mismatch is **insufficient fruit/veg** (--1.7 pp). The CIs
overlap (ours: 75.3--79.4 vs. WHO: 77.1--80.9). The Cabo Verde fact
sheet is one of the few that also reports second-hand smoke at work,
which matched exactly.

### Bahamas 2019

```{r bahamas, eval=TRUE, echo=FALSE, results='asis'}
bhs <- data.frame(
  Indicator = c(
    "Current smoking", "Current alcohol (30 d)",
    "Heavy episodic drinking", "Insufficient fruit/veg",
    "Insufficient physical activity",
    "Overweight (BMI >= 25)", "Obese (BMI >= 30)",
    "Raised BP or on meds",
    "Mean BMI", "Mean SBP"
  ),
  Ours = c(17.4, 49.5, 18.1, 85.0, 30.1,
           71.7, 44.5, 35.3, 30.6, 125.4),
  WHO = c(17.4, 49.6, 17.6, 85.3, 30.2,
          71.6, 43.6, 36.7, 29.8, 125.4),
  Diff = c(0.0, -0.1, 0.5, -0.3, -0.1,
           0.1, 0.9, -1.4, 0.8, 0.0),
  Match = c("Yes", "Yes", "Yes", "Yes", "Yes",
            "Yes", "Yes", "No", "Yes", "Yes"),
  CI_Overlap = rep("Yes", 10)
)

knitr::kable(bhs, align = c("l", "r", "r", "r", "c", "c"),
             caption = "Bahamas 2019: 9/10 within 1 pp, 10/10 CI overlap")
```

The single mismatch is **raised BP or on meds** (--1.4 pp). Step 3
biochemical indicators were excluded from comparison because the Bahamas
fact sheet reports them as unweighted estimates (Step 3 response rate
was below 60 %).

## Analysis of mismatches

Across all nine countries, only five of 128 indicators exceed the
1 pp tolerance. The mismatches are:

**Current tobacco use** (Moldova --2.2 pp): the largest remaining
mismatch. The confidence intervals overlap, suggesting a slightly
different indicator definition in the published fact sheet (e.g.
whether smokeless tobacco is included).

**Raised blood pressure or on medication** (Ukraine +1.9, Bahamas
--1.4): the mismatches are in opposite directions, suggesting
country-specific differences in how the medication question is coded
rather than a systematic package issue.

**Raised cholesterol or on medication** (Ecuador --1.0 pp): right at
the 1 pp threshold boundary. The CIs overlap.

**Insufficient fruit/veg** (Cabo Verde --1.7 pp): the CIs overlap.

All five mismatches have overlapping 95 % confidence intervals,
confirming that the differences are not statistically significant at
conventional levels.

## Key methodological alignments

During validation, several methodological details were identified where
alignment with the WHO official analysis scripts was essential for
reproducibility:

**Survey design.** Sampling weights are used as-is without trimming,
matching the WHO official STEPS analysis scripts. Lone PSUs are handled
with `options(survey.lonely.psu = "adjust")`.

**Alcohol skip patterns.** Non-drinkers who skip the heavy-episodic
drinking question (A9) are coded as `FALSE` (not `NA`), ensuring the
denominator covers the total population rather than only current
drinkers.

**Physical activity screening questions.** GPAQ screening questions
(P1, P4, P7, P10, P13) where the respondent answers "No" set the
domain contribution to zero, rather than leaving it as `NA`. Special
codes 77 and 88 ("don't know" / "refused") are cleaned to `NA`.

**Diet zero-days handling.** When a respondent reports eating fruit or
vegetables on zero days per week, the servings-per-day variable (which
is skipped and therefore `NA`) is set to zero rather than excluded from
the denominator.

**Configurable blood-pressure thresholds.** The package supports custom
SBP/DBP thresholds via `clean_steps_data(bp_sbp_threshold, bp_dbp_threshold)`,
needed because some countries (e.g. Mongolia) use thresholds other than
the standard 140/90.

**Cholesterol unit conversion and threshold alignment.** Some country
datasets store cholesterol in mg/dl rather than mmol/L. The package
auto-detects units and converts to mmol/L. For validation, the
raised-cholesterol threshold was set to 4.914 mmol/L (= 190 mg/dl,
the WHO standard) rather than the default 5.0 mmol/L, aligning with
the WHO fact sheet definition.

**Physical activity data quality (P\_clean).** Following the WHO GPAQ
Analysis Guide, respondents with inconsistent physical activity data
are excluded from all PA computations. Specifically, if a respondent
answers "Yes" to a GPAQ screening question (e.g. "Do you do vigorous
work?") but has missing or invalid follow-up data for that domain, all
PA variables are set to `NA`. Additionally, if any single domain
reports more than 960 minutes (16 hours) per day, all PA data for that
respondent is invalidated. Without this cleaning, partial domain data
would be silently dropped by `rowSums(na.rm = TRUE)`, underestimating
total PA and systematically inflating the `insufficient_pa` indicator.

**Data quality filters.** The WHO `smk_cln` filter is applied: logically
inconsistent tobacco respondents (e.g. "I currently smoke" combined with
"I never smoked in the past") are excluded from the tobacco denominator.

**Tobacco indicator mapping.** WHO fact sheets vary in which tobacco
indicator they report under "current smoking": some report daily smoking
(Mongolia, Georgia), while others report any-frequency tobacco smoking
(Afghanistan, Algeria, Ukraine, Ecuador, Cabo Verde, Bahamas). The
package computes all variants and the appropriate one is used for each
country comparison.

**Column mapping.** Country datasets often use non-standard variable
names. The package's `read_column_mapping()` function and Excel mapping
template were used to handle datasets where auto-detection required
manual overrides.

## Running the validation yourself

The full validation script is included in the package at
`inst/validation/validate_all.R`. To run it, you need the licensed STEPS
microdata files placed in a `STEPS Licensed Datasets` directory alongside
the package:

```r
library(stepssurvey)
## Run from the package source directory
source(system.file("validation", "validate_all.R", package = "stepssurvey"))
```

Per-country results are saved as CSV files in `inst/validation/`.

## Reproducibility notes

The validation was performed with R `r paste0(R.version$major, ".", R.version$minor)`,
using survey package version `r packageVersion("survey")`, and
stepssurvey version `r packageVersion("stepssurvey")`.

The licensed STEPS microdata files are not redistributable and are
therefore not included in the package. Researchers with access to the
WHO STEPS microdata repository can request datasets for the nine
countries listed above and reproduce these results independently.

## References

1. World Health Organization. (2017). *WHO STEPS Surveillance Manual:
   The WHO STEPwise Approach to Noncommunicable Disease Risk Factor
   Surveillance.* Geneva: WHO.
   <https://www.who.int/teams/noncommunicable-diseases/surveillance/systems-tools/steps>

2. World Health Organization. *WHO NCD Microdata Repository: STEPS
   Survey Data.* <https://extranet.who.int/ncdsmicrodata>

3. World Health Organization. *STEPS data analysis scripts (R).*
   GitHub repository.
   <https://github.com/WorldHealthOrganization>

4. Republic of Moldova STEPS Survey 2021. *WHO NCD Country Fact
   Sheet.* Geneva: WHO, 2022.

5. Mongolia STEPS Survey 2019. *WHO NCD Country Fact Sheet.* Geneva:
   WHO, 2020.

6. Georgia STEPS Survey 2016. *WHO NCD Country Fact Sheet.* Geneva:
   WHO, 2017.

7. Afghanistan STEPS Survey 2018. *WHO NCD Country Fact Sheet.*
   Geneva: WHO, 2019.

8. Algeria STEPS Survey 2016. *WHO NCD Country Fact Sheet.* Geneva:
   WHO, 2017.

9. Ukraine STEPS Survey 2019. *WHO NCD Country Fact Sheet.* Geneva:
   WHO, 2020.

10. Ecuador STEPS Survey 2018. *WHO NCD Country Fact Sheet.* Geneva:
    WHO, 2019.

11. Cabo Verde STEPS Survey 2020. *WHO NCD Country Fact Sheet.*
    Geneva: WHO, 2021.

12. Bahamas STEPS Survey 2019. *WHO NCD Country Fact Sheet.* Geneva:
    WHO, 2020.

13. Lumley, T. (2004). Analysis of complex survey samples. *Journal of
    Statistical Software*, 9(8), 1--19. R package **survey** version
    `r packageVersion("survey")`.

---
title: "Analysing WHO STEPS Survey Data with stepssurvey"
author: "Abhijit Pakhare"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Analysing WHO STEPS Survey Data with stepssurvey}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 5,
  fig.align  = "center",
  eval = TRUE
)
```

## Introduction

The WHO STEPwise Approach to NCD Risk Factor Surveillance (STEPS) is the
standard method for collecting population-level data on chronic disease
risk factors.  Surveys measure behavioural risk factors (tobacco, alcohol,
diet, physical activity), physical measurements (anthropometry, blood
pressure), and biochemical markers (blood glucose, cholesterol).

The **stepssurvey** package provides a complete, end-to-end analysis
pipeline that takes raw STEPS data from any country and produces
publication-ready tables, visualisations, and Word reports -- all while
properly accounting for the complex survey design (stratification,
clustering, sampling weights).

### What this guide covers

1. Installing the package
2. Understanding the data pipeline
3. Importing and detecting STEPS variables (v3.1 and v3.2)
4. Column mapping for non-standard datasets
5. Cleaning and deriving WHO-standard indicators
6. Configurable indicator thresholds
7. Data quality diagnostics
8. Setting up the complex survey design
9. Computing weighted prevalence estimates
10. Building tables and visualisations (including forest plot and radar chart)
11. Generating reports (fact sheet, data book, country report)
12. Running the full pipeline in one call
13. Using the interactive Shiny app
14. Working with your own data


## Installation

Install the development version from GitHub:

```{r install, eval = FALSE}
# install.packages("pak")
pak::pak("drpakhare/stepssurvey")
```

Or with **devtools**:

```{r install-devtools, eval = FALSE}
# install.packages("devtools")
devtools::install_github("drpakhare/stepssurvey")
```

Load the package:

```{r library}
library(stepssurvey)
```


## The pipeline at a glance

The package follows a linear, modular pipeline.  You can use the
one-command `run_steps_pipeline()` shortcut or call each step yourself
for full control.

```
  Raw data (.csv / .xlsx / .dta / .sav)
       |
       v
  import_steps_data()          -- read any format
       |
       v
  detect_steps_columns()       -- auto-detect v3.1/v3.2 codes
  -- OR --
  read_column_mapping()        -- use Excel mapping template
       |
       v
  clean_steps_data()           -- derive WHO indicators
       |                          (configurable thresholds)
       v
  steps_data_quality()         -- digit preference, completeness,
       |                          plausibility, weight diagnostics
       v
  setup_survey_design()        -- step-specific weights
       |
       v
  compute_all_indicators()     -- weighted prevalences & means
       |
       +---> build_steps_tables()      -> summary flextables
       +---> build_steps_plots()       -> ggplot2 charts + forest + radar
       +---> render_fact_sheet()       -> Fact Sheet (HTML or Word)
       +---> render_country_report()   -> Summary Report (Word)
       |
       v
  compute_all_tables()         -- 60+ WHO registry tables
       |
       v
  build_all_tables()           -> 3-panel flextables (Men|Women|Both)
       |
       +---> render_data_book()        -> Detailed Data Book (Word)
```

Each function returns its output, so you can inspect, modify, or export
results at every stage.


## Step 1: Generate or import data

### Using the built-in test data generator

For learning and testing, the package includes a realistic data simulator:

```{r generate}
raw <- generate_test_data(n = 3000, seed = 42)
dim(raw)
names(raw)
```

The generated dataset mimics a real STEPS survey with 5 strata,
40 primary sampling units (PSUs), sampling weights, and realistic
correlations between risk factors (e.g. blood pressure increasing
with age, higher tobacco use in males).

### Importing real STEPS data

In practice you will have a data file exported from Epi Info, SPSS, or
Stata.  The `import_steps_data()` function reads all common formats and
standardises column names to lowercase with underscores:

```{r import, eval = FALSE}
# CSV
raw <- import_steps_data("data/raw/steps_survey_2024.csv")

# Excel
raw <- import_steps_data("data/raw/steps_survey_2024.xlsx")

# Stata (.dta) -- common for STEPS exports
raw <- import_steps_data("data/raw/steps_survey_2024.dta")

# SPSS (.sav)
raw <- import_steps_data("data/raw/steps_survey_2024.sav")
```

The function uses the file extension to pick the right reader
(`readr::read_csv`, `readxl::read_excel`, `haven::read_dta`, or
`haven::read_spss`), then passes column names through
`janitor::clean_names()` so that regardless of original casing you get
a consistent format like `wt_final`, `age`, `sex`.


## Step 2: Auto-detect STEPS variables

WHO STEPS datasets use standardised variable codes, but the codes changed
between instrument versions and many countries add their own prefixes.
The `detect_steps_columns()` function searches for each variable using a
prioritised alias list:

```{r detect}
cols <- detect_steps_columns(raw)
```

It returns a named list mapping each conceptual variable to the actual
column found in your data.  You can inspect the mapping:

```{r detect-inspect}
# Which column was matched for fasting glucose?
cols$fasting_glucose

# Which column for SBP reading 1?
cols$sbp1

# How many columns were detected?
sum(!sapply(cols, is.null))
```

### Version 3.1 vs 3.2 variable codes

A key feature of the package is transparent support for both WHO STEPS
instrument versions.  The variable codes changed substantially between
versions:

| Measurement        | v3.1 / Epi Info | v3.2 Instrument |
|--------------------|-----------------|-----------------|
| SBP readings       | B1, B3, B5      | M4a, M5a, M6a   |
| DBP readings       | B2, B4, B6      | M4b, M5b, M6b   |
| BP medications     | B7              | M7 / H3          |
| Height             | M1              | M11              |
| Weight             | M2              | M12              |
| Waist              | M3              | M14              |
| Fasting glucose    | C1 (c1_mmol)    | B5               |
| Diabetes meds      | C5              | B6 / H8          |
| Total cholesterol  | C6              | B8               |
| Cholesterol meds   | C10             | B9 / H14         |
| Sex                | --              | C1               |
| Age                | C1              | C3               |

The detection function includes aliases for both versions, so a dataset
using `b1` for SBP (v3.1) and one using `m4a` (v3.2) will both be
detected correctly.  The search is case-insensitive.

If a column is not found automatically, you can override the mapping
before cleaning:

```{r override, eval = FALSE}
cols$fasting_glucose <- "my_custom_glucose_variable"
```

### Column mapping for non-standard datasets

Many real-world STEPS datasets use country-specific variable names that
auto-detection cannot resolve.  The package includes an Excel mapping
template that lets you specify the correspondence between your column
names and the standard STEPS variables.

**Step 1:** Get the blank template:

```{r mapping-template, eval = FALSE}
# Copy the template to your working directory
file.copy(
  system.file("templates", "column_mapping_template.xlsx", package = "stepssurvey"),
  file.path(tempdir(), "my_mapping.xlsx")
)
```

The template has two sheets: **Instructions** with usage guidance, and
**Column Mapping** with 110 standard variables organised by domain
(Demographics, Tobacco, Alcohol, Diet, Physical Activity, Anthropometry,
Blood Pressure, Biochemical, History & Treatment).  Required variables
are highlighted in red; optional ones in yellow.

**Step 2:** Open the template in Excel, and for each variable in
column A, type your dataset's column name in column C ("Your Column
Name").  Leave blank any variables your dataset does not have.

**Step 3:** Read the filled template:

```{r mapping-read, eval = FALSE}
cols <- read_column_mapping("my_mapping.xlsx", data = raw)
```

The `data` argument is optional but recommended -- it validates that
every mapped column actually exists in your dataset and warns about
typos.  The returned `cols` list is identical in structure to what
`detect_steps_columns()` produces, so you can pass it directly to
`clean_steps_data()`.

The `run_steps_pipeline()` function also accepts a `mapping_file`
parameter:

```{r mapping-pipeline, eval = FALSE}
result <- run_steps_pipeline(
  "my_data.dta",
  country_name = "My Country",
  survey_year  = 2024,
  mapping_file = "my_mapping.xlsx"
)
```


## Step 3: Clean and derive indicators

The `clean_steps_data()` function performs all WHO-recommended data
processing in a single call:

```{r clean}
clean <- clean_steps_data(raw, cols, age_min = 18, age_max = 69)
dim(clean)
```

### What the cleaning step does

**Demographics:**

- Restricts age to the specified range (default 18--69)
- Creates WHO standard age groups: 18--24, 25--34, 35--44, 45--54,
  55--64, 65+
- Harmonises sex coding (1/2, "Male"/"Female", "M"/"F" all accepted)
- Ensures survey weight, stratum, and PSU columns are present

**Behavioural risk factors (Step 1):**

- Recodes tobacco and alcohol variables to logical TRUE/FALSE using
  `recode_yn()`, which understands 0/1, 1/2, "yes"/"no" patterns
- Computes average daily fruit and vegetable servings and flags
  `low_fruit_veg` (combined < 5 servings/day)
- Classifies physical activity into Low / Moderate / High based on
  MET-minutes/week thresholds (< 600, 600--2999, >= 3000)

**Physical measurements (Step 2):**

- Applies plausibility checks (e.g. height 100--250 cm, weight 20--300 kg,
  waist 40--200 cm) and sets implausible values to `NA`
- Computes BMI and classifies into Underweight / Normal / Overweight / Obese
- Flags central obesity using WHO waist circumference thresholds
  (>= 102 cm male, >= 88 cm female)
- Computes waist-to-hip ratio if both measurements are available
- Averages the last two of three BP readings (WHO protocol) to obtain
  mean SBP and mean DBP
- Creates the `raised_bp` indicator (SBP >= 140 or DBP >= 90 or on
  medication) and WHO blood pressure staging

**Biochemical measurements (Step 3):**

- Flags raised fasting glucose (>= 7.0 mmol/L or on diabetes medication)
- Flags impaired fasting glucose (6.1--6.9 mmol/L)
- Flags raised total cholesterol (>= 5.0 mmol/L)
- Flags low HDL cholesterol (sex-specific thresholds)
- Flags raised triglycerides (>= 1.7 mmol/L)

### Configurable indicator thresholds

All indicator thresholds can be customised.  This is essential when a
country uses non-standard definitions (e.g. Mongolia uses 130/80 mmHg
for raised blood pressure instead of the WHO default 140/90):

```{r thresholds, eval = FALSE}
clean <- clean_steps_data(raw, cols,
  bp_sbp_threshold          = 130,   # SBP threshold (default 140)
  bp_dbp_threshold          = 80,    # DBP threshold (default 90)
  bmi_overweight             = 25.0,  # BMI overweight (default 25)
  bmi_obese                  = 30.0,  # BMI obese (default 30)
  glucose_threshold          = 7.0,   # Raised glucose mmol/L (default 7.0)
  glucose_impaired_threshold = 6.1,   # Impaired glucose mmol/L (default 6.1)
  chol_threshold             = 5.0    # Raised cholesterol mmol/L (default 5.0)
)
```

The same thresholds are available in `steps_config()` and propagate
through `run_steps_pipeline()` and the Shiny app interface.

You can inspect the derived variables:

```{r clean-inspect}
# BMI categories
table(clean$bmi_category, clean$sex)

# Blood pressure staging
table(clean$bp_stage)

# Physical activity levels
table(clean$pa_category, clean$sex)
```


## Step 3b: Data quality diagnostics

Before proceeding with analysis, the package provides a comprehensive
data quality assessment.  The `steps_data_quality()` function checks
four dimensions:

```{r quality, eval = FALSE}
quality <- steps_data_quality(clean)
names(quality)
# [1] "digit_preference" "completeness" "plausibility" "weights"
```

**Digit preference** detects heaping on terminal digits 0 and 5 in
blood pressure and anthropometric measurements -- a common data
collection artefact:

```{r digit-plot, eval = FALSE}
plot_digit_preference(quality, measure = "sbp")
```

**Completeness** reports the percentage of non-missing values for every
key variable, helping identify modules that may have been skipped.

```{r completeness-plot, eval = FALSE}
plot_completeness(quality)
```

**Plausibility** flags values outside physiologically reasonable ranges
(e.g. systolic BP > 300 mmHg, height < 100 cm).

**Sampling weights** shows the distribution and coefficient of variation
of each step-specific weight, helping detect extreme weights that might
destabilise survey estimates:

```{r weights-plot, eval = FALSE}
plot_weights(quality)
```

In the Shiny app, the **Quality** tab presents all four diagnostics
interactively with summary value boxes.


## Step 4: Set up the survey design

STEPS surveys use complex sampling designs.  Ignoring the design leads to
biased estimates and incorrect confidence intervals.  The
`setup_survey_design()` function creates a `survey::svydesign` object
that accounts for weights, stratification, and clustering:

```{r design}
designs <- setup_survey_design(clean)
```

The returned object is a list with three elements (`step1`, `step2`,
`step3`), each a `survey::svydesign` object weighted appropriately
for that step of the survey.  Functions like `compute_all_indicators()`
accept this list directly, but for custom estimates you pick the
design matching the step of the variable you are analysing:

The function auto-detects the design complexity based on which columns
are present:

- **Full complex design**: weights + strata + clusters
- **Weights + clusters**: no stratification variable
- **Weights + strata**: no clustering (rare)
- **Weights only**: self-representing design
- **Unweighted**: simple random sample (weights set to 1)

Sampling weights are used as-is without trimming, consistent with
the WHO official STEPS analysis scripts.

The returned object can be used with any function from the **survey**
package if you need custom analyses beyond what the package provides.


## Step 5: Compute indicators

### All indicators at once

```{r indicators}
result <- compute_all_indicators(designs)
```

This returns a list with two elements:

- `result$results` -- a nested list of domain-specific estimates
  (total, by sex, by age group)
- `result$key_indicators` -- a tidy data frame of headline prevalences

```{r key}
result$key_indicators
```

### Domain-specific functions

For more control, call each domain function separately:

```{r domain, eval = FALSE}
tob  <- compute_tobacco_indicators(designs$step1)
alc  <- compute_alcohol_indicators(designs$step1)
diet <- compute_diet_pa_indicators(designs$step1)
anth <- compute_anthropometry_indicators(designs$step2)
bp   <- compute_bp_indicators(designs$step2)
bio  <- compute_biochemical_indicators(designs$step3)
```

Each returns a named list.  For example, the tobacco module returns:

```{r tob-example}
tob <- compute_tobacco_indicators(designs$step1)
names(tob)

# Overall prevalence of current tobacco use
tob$current_tobacco_total

# Prevalence by sex
tob$current_tobacco_by_sex
```

### Custom weighted estimates

The package exports two low-level helpers for any weighted estimate you
need:

```{r custom}
# Weighted proportion with 95% CI (raised_bp is a Step 2 variable)
svyprop(~raised_bp, designs$step2)

# Stratified by sex
svyprop(~raised_bp, designs$step2, by = ~sex)

# Weighted mean with 95% CI
svymn(~mean_sbp, designs$step2, by = ~sex)
```


## Step 6: Build publication-ready tables

The package provides **two table systems** for different purposes.

### Summary tables (Both Sexes only)

```{r tables}
tables <- build_steps_tables(result$results)
names(tables)
```

Each table is a **flextable** object styled with WHO STEPS branding
(dark blue headers, formatted confidence intervals).  These tables show
estimates by age group for Both Sexes combined -- ideal for summary
reports and quick reference.

```{r table-show, eval = FALSE}
# Display the raised blood pressure table
tables$raised_bp

# Export to Word
flextable::save_as_docx(tables$raised_bp, path = file.path(tempdir(), "bp_table.docx"))
```

### Detailed WHO 3-panel tables (Men | Women | Both Sexes)

For the full WHO STEPS data book format, use the detailed table engine.
This produces ~60 tables in the standard 3-panel layout (Age Group | Men |
Women | Both Sexes):

```{r detailed-tables, eval = FALSE}
# Step 1: Compute raw results from the table registry
computed <- compute_all_tables(designs)

# Step 2: Format into flextable objects with WHO styling
detailed <- build_all_tables(computed)
names(detailed)  # e.g. "T_current_smokers", "M_bp_mean", "B_glucose_raised"
```

The table IDs use prefixes matching WHO STEPS domains:

| Prefix | Domain                    |
|--------|---------------------------|
| T_     | Tobacco                   |
| A_     | Alcohol                   |
| D_     | Diet                      |
| P_     | Physical Activity         |
| H_     | Health History & Treatment|
| M_     | Physical Measurements     |
| B_     | Biochemical Measurements  |
| R_     | Cardiovascular Risk       |
| RF_    | Combined Risk Factors     |

You can access individual tables by ID or filter by section:

```{r registry, eval = FALSE}
# Browse the full table registry
registry <- steps_table_registry()

# Get all tables for a specific section
bp_entries <- get_registry_by_section("Blood Pressure")

# Get all Step 2 tables
step2_entries <- get_registry_by_step(2)

# List available sections
list_registry_sections()
```


## Step 7: Create visualisations

```{r plots, fig.width = 8, fig.height = 5}
plots <- build_steps_plots(
  indicators    = result$results,
  key_indicators = result$key_indicators,
  country_name  = "Exampleland",
  survey_year   = 2024
)
names(plots)
```

### Overview chart

The overview plot shows all key indicators as a horizontal bar chart with
95% confidence intervals, sorted by prevalence:

```{r overview, fig.width = 8, fig.height = 5}
plots$overview
```

### Sex-stratified dashboard

If multiple sex-stratified indicators are available, the package creates
a 2 x 2 dashboard using **patchwork**:

```{r dashboard, fig.width = 10, fig.height = 7, eval = FALSE}
plots$sex_dashboard
```

### Age-stratified trends

Age trend plots show how each risk factor varies across the WHO standard
age groups, with shaded confidence bands:

```{r age-trend, fig.width = 8, fig.height = 4.5}
plots$bp_by_age
```

### Forest plot

The forest plot shows all key indicators as horizontal point-and-CI
estimates, colour-coded by STEPS domain:

```{r forest, fig.width = 8, fig.height = 6, eval = FALSE}
plots$forest
# Or build standalone:
build_forest_plot(result$key_indicators, "Exampleland", 2024)
```

### Risk profile radar chart

The radar (spider) chart provides a visual fingerprint of the country's
NCD risk factor profile, making it easy to spot which domains are most
affected:

```{r radar, fig.width = 7, fig.height = 7, eval = FALSE}
plots$radar
# Or build standalone:
build_radar_plot(result$key_indicators, "Exampleland", 2024)
```

### Saving plots

```{r save-plots, eval = FALSE}
save_steps_plots(plots, output_dir = file.path(tempdir(), "figures"))
# Creates:
#   outputs/figures/01_overview_indicators.png
#   outputs/figures/02_by_sex_dashboard.png
#   outputs/figures/03_bp_by_age.png
#   outputs/figures/04_obesity_by_age.png
#   outputs/figures/05_forest_plot.png
#   outputs/figures/06_radar_plot.png
```

### WHO STEPS colour palette and theme

The package uses a consistent visual identity.  You can apply the same
styling to your own ggplot2 plots:

```{r theme, fig.width = 6, fig.height = 3.5}
pal <- steps_colors()
str(pal)

library(ggplot2)
ggplot(clean, aes(x = age_group, fill = sex)) +
  geom_bar(position = "dodge") +
  scale_fill_manual(values = c(Male = pal$male, Female = pal$female)) +
  theme_steps() +
  labs(title = "Sample distribution by age and sex")
```


## Step 8: Generate Word reports

The package produces three complementary reports:

| Report | Function | Format | Content |
|--------|----------|--------|---------|
| **Fact Sheet** | `render_fact_sheet()` | HTML or Word | One-page overview with radar chart, summary table, and key findings |
| **Summary Report** | `render_country_report()` | Word | Narrative with key findings, charts, and recommendations |
| **Detailed Data Book** | `render_data_book()` | Word | Complete WHO 3-panel tables (Men \| Women \| Both Sexes) across all domains |

```{r reports, eval = FALSE}
cfg <- steps_config(
  data_path    = "data/raw/steps_survey_2024.csv",
  country_name = "Exampleland",
  survey_year  = 2024,
  age_min      = 18,
  age_max      = 69
)

# Fact sheet -- one-page overview (HTML for sharing, Word for print)
render_fact_sheet(cfg, output_dir = "outputs", format = "html")
render_fact_sheet(cfg, output_dir = "outputs", format = "word")

# Summary report -- narrative with key findings, charts, recommendations
render_country_report(cfg, output_dir = "outputs")

# Data book -- detailed WHO 3-panel tables by domain
render_data_book(cfg, output_dir = "outputs")
```

Each function runs the entire pipeline internally (import, clean, analyse)
and renders an R Markdown template to a Word document.  The output files
are saved in the specified directory.

### What each report contains

The **Fact Sheet** is a single-page overview with a branded header,
summary table of key indicators (noting any non-default thresholds),
the radar chart, sex-stratified dashboard, and forest plot.  The HTML
version is self-contained and ideal for web sharing; the Word version
is print-ready.

The **Summary Report** includes an executive summary table, narrative
sections for each risk factor domain with inline prevalence figures,
embedded charts (overview indicators, by-sex breakdowns, age trends),
and WHO-aligned policy recommendations.

The **Data Book** contains the full set of ~60 WHO STEPS tables in the
standard 3-panel format.  Each table shows estimates by age group
separately for Men, Women, and Both Sexes.  Tables are organised by
STEPS step: Step 1 (Behavioural), Step 1.5 (Health History), Step 2
(Physical Measurements), Step 3 (Biochemical), and Combined Risk Factors.


## One-command pipeline

For the fastest path from raw data to results, `run_steps_pipeline()`
chains every step and returns all intermediate objects:

```{r pipeline, eval = FALSE}
out <- run_steps_pipeline(
  data_path      = "data/raw/steps_survey_2024.csv",
  country_name   = "Exampleland",
  survey_year    = 2024,
  age_min        = 18,
  age_max        = 69,
  output_dir     = "outputs",
  render_reports = TRUE
)

# Access any intermediate result
out$raw_data
out$clean_data
out$design
out$indicators
out$key_indicators
out$tables
out$plots
```

Setting `render_reports = FALSE` skips the Word documents (useful for
interactive exploration or when **rmarkdown** / Pandoc are not available).

If your dataset uses non-standard column names, pass a filled mapping
template:

```{r pipeline-mapping, eval = FALSE}
out <- run_steps_pipeline(
  "my_data.dta",
  country_name = "My Country",
  survey_year  = 2024,
  mapping_file = "my_mapping.xlsx"
)
```


## Working with real STEPS data

### Preparing your data file

The package accepts data in four formats:

| Format     | Extension | Typical source                   |
|------------|-----------|----------------------------------|
| CSV        | .csv      | Spreadsheet export               |
| Excel      | .xlsx     | Direct data entry                |
| Stata      | .dta      | WHO Epi Info / analysis template |
| SPSS       | .sav      | SPSS data export                 |

Before importing, ensure the file contains at minimum:

- **Age** and **sex** columns (required for all analyses)
- **Sampling weight** column (recommended; if absent, all weights are
  set to 1)
- At least some risk factor measurements from Step 1, 2, or 3

### Handling custom variable names

For datasets with a few non-standard names, override individual mappings
after auto-detection:

```{r custom-names, eval = FALSE}
raw  <- import_steps_data("my_steps_data.csv")
cols <- detect_steps_columns(raw)
cols$fasting_glucose <- "blood_sugar_fasting"
cols$sbp1 <- "systolic_bp_1"
clean <- clean_steps_data(raw, cols)
```

For datasets where many or most variables have non-standard names, use
the column mapping template instead (see the "Column mapping for
non-standard datasets" section above).  This is the recommended
approach for real-world STEPS microdata.

### Adjusting age range

Some STEPS surveys target populations outside the standard 18--69 range.
Adjust with the `age_min` and `age_max` parameters:

```{r age-range, eval = FALSE}
# Include up to age 79
clean <- clean_steps_data(raw, cols, age_min = 18, age_max = 79)
```

Note that changing the upper age limit adds a wider final age group
(e.g. 65--79 instead of 65+).


## Complete worked example

This section walks through a full analysis using simulated data, showing
every step from generation to output.

```{r worked, fig.width = 8, fig.height = 5}
library(stepssurvey)

# 1. Generate a realistic test dataset
raw <- generate_test_data(n = 3000, seed = 42)

# 2. Detect standard STEPS variable columns
cols <- detect_steps_columns(raw)

# 3. Clean data and derive all indicators
clean <- clean_steps_data(raw, cols, age_min = 18, age_max = 69)

# 4. Create the complex survey design
designs <- setup_survey_design(clean)

# 5. Compute all NCD risk factor indicators
result <- compute_all_indicators(designs)

# 6. View headline estimates
result$key_indicators

# 7. Build formatted tables
tables <- build_steps_tables(result$results)

# 8. Build visualisations
plots <- build_steps_plots(
  indicators     = result$results,
  key_indicators = result$key_indicators,
  country_name   = "Exampleland",
  survey_year    = 2024
)

# 9. Display the overview chart
plots$overview
```


## Interactive Shiny app

For users who prefer a point-and-click interface, the package includes
a full-featured Shiny application:

```{r shiny, eval = FALSE}
library(stepssurvey)
run_app()
```

The app guides you through the same pipeline in seven tabs:

1. **Upload** -- load data (or use built-in demo data), set country
   name, survey year, age range, indicator thresholds, and optionally
   upload a column mapping template
2. **Clean** -- run WHO-standard cleaning with summary statistics
3. **Quality** -- interactive data quality diagnostics (digit preference,
   completeness, plausibility, sampling weights)
4. **Design** -- set up the complex survey design with step-specific weights
5. **Indicators** -- compute all NCD risk factor indicators with
   tabulated results
6. **Visualise** -- interactive plots including overview, sex dashboard,
   age trends, forest plot, and radar chart
7. **Reports** -- one-click generation of fact sheet (HTML/Word), country
   report, and data book with download buttons

A deployed version is available at
<https://cfm-stepssurvey.share.connect.posit.cloud/>.


## WHO standard definitions used

The package implements the following WHO STEPS definitions for all
derived indicators:

| Indicator                     | Definition                                                                  |
|-------------------------------|-----------------------------------------------------------------------------|
| Current tobacco use           | Currently smokes any tobacco product (T1 = Yes)                             |
| Daily tobacco use             | Smokes tobacco daily (T2 = Yes)                                             |
| Current alcohol use           | Consumed alcohol in the past 30 days (A5 = Yes)                             |
| Heavy episodic drinking       | 6 or more standard drinks on a single occasion in past 30 days (A9)         |
| Insufficient physical activity| Total MET-minutes per week < 600                                            |
| Low fruit and vegetable intake| Combined < 5 servings per day                                               |
| Overweight or obese           | BMI >= 25 kg/m^2^ (configurable)                                            |
| Obese                         | BMI >= 30 kg/m^2^ (configurable)                                            |
| Central obesity               | Waist >= 102 cm (male) or >= 88 cm (female)                                |
| Raised blood pressure         | Mean SBP >= 140 or mean DBP >= 90 or on BP meds (configurable)              |
| Raised fasting glucose        | Fasting glucose >= 7.0 mmol/L or on diabetes meds (configurable)            |
| Impaired fasting glucose      | Fasting glucose 6.1--6.9 mmol/L (configurable)                              |
| Raised total cholesterol      | Total cholesterol >= 5.0 mmol/L (configurable)                              |
| Low HDL cholesterol           | HDL < 1.0 mmol/L (male) or < 1.3 mmol/L (female)                           |
| Raised triglycerides          | Triglycerides >= 1.7 mmol/L                                                 |

Blood pressure readings follow the WHO protocol of averaging the last two
of three measurements taken three minutes apart.


## FAQ

**Can I use this package with STEPS surveys from any country?**
Yes.
The variable detection system supports both v3.1 and v3.2 naming
conventions, plus common country-specific aliases.  Override any
undetected columns manually as shown above.

**What if my dataset is missing some risk factor modules?**
The package handles missing modules gracefully.  If, for example, no
biochemical columns are found, the glucose and cholesterol indicators
are simply skipped and the tables and plots adapt accordingly.

**Can I add my own indicators?**
Absolutely.  After the cleaning step you have a standard data frame with
all derived variables.  Use the `survey::svydesign` object with `svyprop()`
or `svymn()` (or any **survey** package function) for custom analyses.

**How do I cite this package?**
```{r cite, eval = FALSE}
citation("stepssurvey")
```


## Further resources

- [WHO STEPS Manual](https://www.who.int/teams/noncommunicable-diseases/surveillance/systems-tools/steps/manuals)
- [STEPS Instrument v3.2 (Q-by-Q Guide)](https://www.who.int/teams/noncommunicable-diseases/surveillance/systems-tools/steps/instrument)
- [STEPS Data Analysis Tools](https://www.who.int/teams/noncommunicable-diseases/surveillance/systems-tools/steps/data-analysis-reporting-tools)
- [Package source on GitHub](https://github.com/drpakhare/stepssurvey)


## Session info

```{r session}
sessionInfo()
```

---
title: "Preparing STEPS Data for Analysis"
author: "Abhijit Pakhare"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Preparing STEPS Data for Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)
```

## Introduction

This guide helps you prepare your WHO STEPS survey data file for use
with the `stepssurvey` package.  It covers the variables the package
expects, how auto-detection works, common data quality issues, and how
to resolve mismatches between your data and the package expectations.

The package is designed to work with STEPS data from any country,
regardless of instrument version (v3.1 or v3.2) or data management
system (Epi Info, SPSS, Stata, Excel).


## Supported file formats

| Format | Extension | Typical source | Reader used |
|--------|-----------|----------------|-------------|
| SPSS | `.sav` | WHO STEPS data entry / Epi Info export | `haven::read_spss()` |
| Stata | `.dta` | WHO analysis template | `haven::read_dta()` |
| Excel | `.xlsx` | Custom data entry | `readxl::read_excel()` |
| CSV | `.csv` | Any spreadsheet export | `readr::read_csv()` |

**Recommendation**: Use the `.sav` file directly as exported from your
data management system.  The package preserves SPSS variable labels
during column detection (to disambiguate codes like A1 that mean
different things across versions) and then strips them before analysis
to avoid compatibility issues.


## Minimum required variables

At a minimum, the package needs **age** and **sex** to produce any
output.  Beyond that, each additional variable you provide enables more
indicators and tables.

### Essential (required)

| Variable | STEPS codes | Description |
|----------|-------------|-------------|
| Age | `C3` (v3.2), `age`, `c1` (v3.1) | Respondent age in completed years |
| Sex | `C1` (v3.2), `sex`, `gender`, `c2` (v3.1) | Male/Female coding (1/2, M/F, or text) |

### Strongly recommended

| Variable | STEPS codes | Description |
|----------|-------------|-------------|
| Sampling weight (Step 1) | `WStep1`, `wt_final`, `sampleweight` | Probability weight for behavioural module |
| Sampling weight (Step 2) | `WStep2` | Weight for physical measurements |
| Sampling weight (Step 3) | `WStep3` | Weight for biochemical measurements |
| PSU / Cluster | `psu`, `cluster`, `I1`, `ea_id` | Primary sampling unit identifier |
| Stratum | `stratum`, `strata`, `district`, `region` | Stratification variable |

If only one weight column is present, it is used for all three steps.
If no weight is found, the package assumes equal weights (simple random
sample).

### Step 1: Behavioural risk factors

**Tobacco:**

| Variable | v3.1 code | v3.2 code | Description |
|----------|-----------|-----------|-------------|
| Current smoker | T1 | T1 | Currently smoke tobacco (yes/no) |
| Daily smoker | T2 | T2 | Smoke daily (yes/no) |
| Age started | T3 | T3 | Age of smoking initiation |
| Cigarettes/day | T5a | T5a | Manufactured cigarettes per day |
| Quit attempt | T6 | T6 | Tried to quit in past 12 months |
| Past smoker | T8 | T8 | Ever smoked in the past |
| Smokeless tobacco | T12/T15 | T12 | Current smokeless use |
| Second-hand (home) | T17 | T17 | Exposure to smoke at home |
| Second-hand (work) | T18 | T18 | Exposure to smoke at workplace |

**Alcohol:**

| Variable | v3.1 code | v3.2 code | Description |
|----------|-----------|-----------|-------------|
| Ever consumed | -- | A1 | Lifetime alcohol consumption |
| Past 12 months | A2/A4 | A2 | Consumed in past year |
| Current (30 days) | A1 | A5 | Consumed in past 30 days |
| Occasions (30 days) | A6 | A6 | Number of drinking occasions |
| Drinks per occasion | A7 | A7 | Typical number of drinks |
| Heavy episodic | -- | A9 | Times with 6+ drinks (30 days) |

**Note on A1/A5 ambiguity**: In v3.1, `A1` means "current drinker (past
30 days)".  In v3.2, `A1` means "ever consumed alcohol" and `A5` is
"past 30 days".  The package uses SPSS variable labels to disambiguate
when the column code alone is ambiguous.  This is one reason why `.sav`
files (which carry labels) work better than plain CSV.

**Diet:**

| Variable | v3.1 code | v3.2 code | Description |
|----------|-----------|-----------|-------------|
| Fruit days/week | D1 | D1 | Days eating fruit in typical week |
| Fruit servings/day | D2 | D2 | Servings of fruit on those days |
| Vegetable days/week | D3 | D3 | Days eating vegetables |
| Vegetable servings/day | D4 | D4 | Servings of vegetables on those days |
| Salt at table | D5 | D5 | Frequency of adding salt |
| Processed salt food | D7 | D7 | Frequency of processed salty food |

**Physical Activity (GPAQ):**

| Variable | v3.2 code | Description |
|----------|-----------|-------------|
| Vigorous work (y/n) | P1 | Does vigorous work activity |
| Vigorous work days | P2 | Days per week |
| Vigorous work hours | P3a | Hours per day |
| Vigorous work minutes | P3b | Minutes per day |
| Moderate work (y/n) | P4 | Does moderate work activity |
| Moderate work days | P5 | Days per week |
| Moderate work hours | P6a | Hours per day |
| Moderate work minutes | P6b | Minutes per day |
| Transport (y/n) | P7 | Walks or cycles for transport |
| Transport days | P8 | Days per week |
| Transport hours | P9a | Hours per day |
| Transport minutes | P9b | Minutes per day |
| Vigorous recreation (y/n) | P10 | Does vigorous recreational activity |
| Vigorous recreation days | P11 | Days per week |
| Vigorous recreation hours | P12a | Hours per day |
| Vigorous recreation minutes | P12b | Minutes per day |
| Moderate recreation (y/n) | P13 | Does moderate recreational activity |
| Moderate recreation days | P14 | Days per week |
| Moderate recreation hours | P15a | Hours per day |
| Moderate recreation minutes | P15b | Minutes per day |
| Sedentary hours | P16a | Sitting time, hours per day |
| Sedentary minutes | P16b | Sitting time, minutes per day |

The package computes MET-minutes/week from these raw items using WHO
MET multipliers: vigorous activities × 8 MET, moderate and transport
activities × 4 MET.  The `insufficient_pa` indicator (< 600
MET-minutes/week) is then derived automatically.

If your dataset already has a pre-computed `met_total` variable, the
package uses it directly instead of calculating from raw items.

### Step 2: Physical measurements

| Variable | v3.1 code | v3.2 code | Description |
|----------|-----------|-----------|-------------|
| Height (cm) | M1 | M11 | Standing height |
| Weight (kg) | M2 | M12 | Body weight |
| Waist (cm) | M3 | M14 | Waist circumference |
| Hip (cm) | -- | M15 | Hip circumference |
| SBP reading 1 | B1 | M4a | First systolic BP |
| SBP reading 2 | B3 | M5a | Second systolic BP |
| SBP reading 3 | B5 | M6a | Third systolic BP |
| DBP reading 1 | B2 | M4b | First diastolic BP |
| DBP reading 2 | B4 | M5b | Second diastolic BP |
| DBP reading 3 | B6 | M6b | Third diastolic BP |
| BP medication | B7/H3 | M7 | Currently on antihypertensives |
| Heart rate 1 | -- | M16a | First heart rate reading |
| Heart rate 2 | -- | M16b | Second heart rate reading |
| Heart rate 3 | -- | M16c | Third heart rate reading |

Blood pressure: The package averages the last two of three readings
(WHO protocol).  If only two readings are available, their average is
used.  If only one reading is available, it is used directly.

### Step 3: Biochemical measurements

| Variable | v3.1 code | v3.2 code | Description |
|----------|-----------|-----------|-------------|
| Fasting glucose | C1 | B5 | Fasting blood glucose (mmol/L) |
| Diabetes meds | C5 | B6/H8 | Currently on diabetes medication |
| Total cholesterol | C6 | B8 | Total cholesterol (mmol/L) |
| Cholesterol meds | C10 | B9/H14 | Currently on cholesterol medication |
| HDL cholesterol | -- | B17 | HDL cholesterol (mmol/L) |
| Triglycerides | -- | B16 | Fasting triglycerides (mmol/L) |

### Health history (H-codes)

| Variable | Code | Description |
|----------|------|-------------|
| BP ever measured | H1 | Ever had BP measured by health worker |
| BP diagnosed | H2a | Ever told by doctor that BP is raised |
| Glucose ever measured | H6 | Ever had blood sugar measured |
| DM diagnosed | H7a | Ever told by doctor that blood sugar is raised |
| Cholesterol ever measured | H12 | Ever had cholesterol measured |
| Cholesterol diagnosed | H13a | Ever told by doctor that cholesterol is raised |
| CVD history | H17 | History of heart attack, angina, or stroke |
| Aspirin use | H18 | Currently taking aspirin regularly |
| Statin use | H19 | Currently taking statins regularly |
| Advised: quit tobacco | H20a | Doctor/health worker advised to quit tobacco |
| Advised: reduce salt | H20b | Advised to reduce salt intake |
| Advised: eat fruit/veg | H20c | Advised to eat more fruit/vegetables |
| Advised: reduce fat | H20d | Advised to reduce dietary fat |
| Advised: more PA | H20e | Advised to increase physical activity |
| Advised: healthy weight | H20f | Advised to maintain healthy body weight |


## How auto-detection works

When you call `detect_steps_columns(data)` (or upload a file in the
Shiny app), the package searches for each expected variable using a
prioritised alias list.  For example, the fasting glucose variable is
searched for as:

```
b5, b5_mmol, c1_mmol, fasting_glucose, glucose_fasting, fbg, fpg
```

The search is case-insensitive and uses the column names after
`janitor::clean_names()` has standardised them.

For ambiguous codes (like A1 which means different things in v3.1 vs
v3.2), the package also checks the SPSS variable label to disambiguate.
This is why `.sav` files produce the most reliable auto-detection.

After detection, you can inspect the mapping:

```{r inspect}
raw  <- import_steps_data("my_data.sav")
cols <- detect_steps_columns(raw)

# See all detected columns
str(cols[!sapply(cols, is.null)])

# See what was NOT detected
names(cols[sapply(cols, is.null)])
```


## Common data issues and solutions

### Issue: Sex coded as numeric without labels

Some datasets code sex as 1/2 without clear labels.  The package
handles this automatically using the WHO STEPS convention:
1 = Male, 2 = Female.  If your data uses a different coding, recode
before analysis or override the column.

### Issue: Yes/No variables coded inconsistently

STEPS datasets use various codings for binary variables: 1/2 (yes/no),
0/1, "Yes"/"No", "Y"/"N".  The `recode_yn()` function handles all of
these automatically.  It treats 1 = Yes and 2 = No (the WHO convention),
as well as 0/1 where 1 = Yes.

### Issue: Glucose values in mg/dL instead of mmol/L

The package expects biochemical values in mmol/L (the WHO standard).  If
your data uses mg/dL, convert before analysis:

```{r convert}
# Glucose: mg/dL to mmol/L (divide by 18)
raw$b5 <- raw$b5 / 18

# Cholesterol: mg/dL to mmol/L (divide by 38.67)
raw$b8 <- raw$b8 / 38.67
```

### Issue: Multiple datasets for different STEPS steps

Some surveys store Step 1 (interview), Step 2 (measurements), and
Step 3 (blood tests) in separate files.  Merge them by respondent ID
before importing:

```{r merge}
step1 <- haven::read_spss("step1_interview.sav")
step2 <- haven::read_spss("step2_measurements.sav")
step3 <- haven::read_spss("step3_biochemistry.sav")

combined <- dplyr::left_join(step1, step2, by = "pid") |>
  dplyr::left_join(step3, by = "pid")

# Save combined file
haven::write_sav(combined, file.path(tempdir(), "steps_combined.sav"))

# Or import directly
raw <- combined |> janitor::clean_names()
```

### Issue: Missing sampling weights

If your dataset does not include sampling weights, the package will
proceed with equal weights (equivalent to assuming a simple random
sample).  This produces valid point estimates but confidence intervals
may not correctly reflect the true survey design.

For proper analysis, you need at minimum one weight variable.  The WHO
STEPS toolkit computes three step-specific weights (WStep1, WStep2,
WStep3) that account for non-response at each step.  Contact your
survey statistician if weights are not in the data file.

### Issue: Variables detected as wrong type

If a numeric variable is stored as character (common with Epi Info
exports), the cleaning step will attempt to convert it.  If conversion
fails, the variable is set to NA with a warning message.  Check the
console for messages like "NAs introduced by coercion".


## Pre-flight checklist

Before running the analysis, verify these items:

1. **File format**: Preferably `.sav` (preserves variable labels for
   better auto-detection).  CSV works but may require more manual
   column mapping.

2. **One row per respondent**: The data should be in wide format with
   one row per survey participant and columns for each variable.

3. **Age and sex present**: These are the only truly required variables.
   Verify they contain reasonable values (age should be numeric,
   sex should have exactly two levels).

4. **Weights present**: Check for columns named `WStep1`, `WStep2`,
   `WStep3`, or similar.  If using a single weight, ensure it maps to
   `weight_step1`.

5. **Biochemical units**: Glucose should be in mmol/L (typical values
   3--15), cholesterol in mmol/L (typical values 2--10).  Values in
   hundreds suggest mg/dL units that need conversion.

6. **Blood pressure readings**: Should be in mmHg.  Typical SBP range
   is 80--250, typical DBP range is 40--150.  Values outside this
   range are set to NA during cleaning.

7. **No duplicate respondent IDs**: If the same person appears multiple
   times, prevalence estimates will be biased.

8. **Consistent coding**: Binary variables should use the same coding
   scheme throughout (ideally 1 = Yes, 2 = No per WHO convention).


## Quick diagnostic script

Run this after importing to check data quality:

```{r diagnostic}
library(stepssurvey)

raw  <- import_steps_data("my_data.sav")
cols <- detect_steps_columns(raw)

# Summary
cat("Rows:", nrow(raw), "\n")
cat("Columns:", ncol(raw), "\n")
cat("Detected:", sum(!sapply(cols, is.null)), "/", length(cols), "\n")

# Check key variables
if (!is.null(cols$age)) {
  cat("\nAge range:", range(raw[[cols$age]], na.rm = TRUE), "\n")
  cat("Age NAs:", sum(is.na(raw[[cols$age]])), "\n")
}
if (!is.null(cols$sex)) {
  cat("\nSex distribution:\n")
  print(table(raw[[cols$sex]], useNA = "ifany"))
}
if (!is.null(cols$weight_step1)) {
  wt <- raw[[cols$weight_step1]]
  cat("\nWeight range:", round(range(wt, na.rm = TRUE), 3), "\n")
  cat("Weight NAs:", sum(is.na(wt)), "\n")
}

# List undetected variables
missing <- names(cols[sapply(cols, is.null)])
cat("\nUndetected variables (", length(missing), "):\n")
cat(paste(" ", missing, collapse = "\n"), "\n")
```


## Further reading

- `vignette("stepssurvey-guide")` -- full API documentation
- `vignette("shiny-walkthrough")` -- interactive Shiny app guide
- [WHO STEPS Manual, Part 4: Data Analysis](https://www.who.int/teams/noncommunicable-diseases/surveillance/systems-tools/steps/manuals)
- [WHO STEPS Instrument v3.2](https://www.who.int/teams/noncommunicable-diseases/surveillance/systems-tools/steps/instrument)

---
title: "NCD Risk Factor Survey Report"
subtitle: "`r params$config$country_name` | `r params$config$survey_year`"
output:
  word_document: default
  html_document:
    theme: flatly
    toc: true
params:
  config: NULL
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE, fig.width = 10)
library(flextable); library(ggplot2); library(glue); library(patchwork)
library(dplyr)

cfg <- params$config

# Read pre-computed data from the data directory passed via config
data_dir <- cfg$data_dir
indicators     <- readRDS(file.path(data_dir, "indicators.rds"))
key_indicators <- readRDS(file.path(data_dir, "key_indicators.rds"))
tables         <- readRDS(file.path(data_dir, "tables.rds"))
plots          <- readRDS(file.path(data_dir, "plots.rds"))

# Helper to pull a single estimate
get_est <- function(ind, domain, key) {
  val <- ind[[domain]][[key]]
  if (is.null(val)) return("N/A")
  paste0(round(val$estimate[1], 1), "%")
}
```

# Executive Summary

This report presents the findings of the WHO STEPwise Approach to NCD Risk Factor Surveillance (STEPS) survey conducted in **`r cfg$country_name`** in **`r cfg$survey_year`**. The survey collected data on key non-communicable disease (NCD) risk factors among adults aged `r cfg$age_min`–`r cfg$age_max` years.

## Key Findings

```{r key-findings}
key_indicators |>
  mutate(Result = paste0(round(estimate, 1), "% (95% CI: ", round(lower, 1), "–", round(upper, 1), "%)")) |>
  select(Domain = domain, Indicator = indicator, Result) |>
  flextable() |>
  theme_vanilla() |>
  bold(part = "header") |>
  bg(part = "header", bg = "#00427A") |>
  color(part = "header", color = "white") |>
  merge_v(j = "Domain") |>
  autofit()
```

---

# Tobacco Use

```{r tobacco-headline, include=FALSE}
# Prefer combined (smoking + smokeless) if available; fall back to smoking only
tob_est <- indicators$tobacco[["current_tobacco_any_total"]]
if (is.null(tob_est)) tob_est <- indicators$tobacco[["current_tobacco_total"]]
tob_label <- if (!is.null(tob_est)) paste0(round(tob_est$estimate[1], 1), "%") else "N/A"
```

Current tobacco use (smoking or smokeless) was estimated at **`r tob_label`** among the adult population. Tobacco use is a leading risk factor for cardiovascular disease, cancer, and chronic respiratory disease.

```{r tobacco-chart, fig.height=5}
if (!is.null(plots$tobacco_by_sex)) plots$tobacco_by_sex
```

---

# Physical Activity

```{r pa-headline, include=FALSE}
pa_est <- get_est(indicators, "diet_pa", "insufficient_pa_total")
pa_text <- if (pa_est == "N/A") {
  "Data on physical activity were not available for this survey."
} else {
  paste0("Insufficient physical activity (less than 600 MET-minutes/week) was found in **", pa_est, "** of the population.")
}
```

`r pa_text` Regular physical activity reduces the risk of cardiovascular disease, diabetes, and some cancers.

---

# Overweight and Obesity

The prevalence of overweight or obesity (BMI ≥25 kg/m²) was **`r get_est(indicators, "anthropometry", "overweight_obese_total")`**.

```{r obesity-age, fig.height=5}
if (!is.null(plots$obesity_by_age)) plots$obesity_by_age
```

---

# Blood Pressure

Raised blood pressure (≥140/90 mmHg or on antihypertensive medication) was found in **`r get_est(indicators, "blood_pressure", "raised_bp_total")`** of adults.

```{r bp-age, fig.height=5}
if (!is.null(plots$bp_by_age)) plots$bp_by_age
```

---

# Blood Glucose and Cholesterol

Raised fasting blood glucose (≥7.0 mmol/L or on diabetes medication) was found in **`r get_est(indicators, "biochemical", "raised_glucose_total")`** of adults. Raised total cholesterol (≥5.0 mmol/L) was found in **`r get_est(indicators, "biochemical", "raised_chol_total")`**.

---

# Recommendations

Based on the survey findings, the following public health priorities are recommended:

1. **Strengthen tobacco control** policies including taxation, plain packaging, and cessation support
2. **Promote physical activity** through community programs and urban planning
3. **Improve dietary quality** by increasing access to fruits and vegetables
4. **Scale up screening and treatment** for hypertension and diabetes
5. **Enhance surveillance capacity** with regular repeat surveys

---

# Methodology

This survey followed the WHO STEPwise approach to NCD risk factor surveillance. Data were collected using a multistage cluster sampling design. Survey-weighted estimates were produced using the `survey` package in R to account for the complex sampling design. Confidence intervals were calculated at the 95% level.

---

*Report generated: `r Sys.Date()`*
*Analysis: WHO STEPS R Pipeline*
*Country: `r cfg$country_name` | Survey year: `r cfg$survey_year`*

---
title: "WHO STEPS NCD Risk Factor Survey — Fact Sheet"
output:
  html_document:
    theme: flatly
    self_contained: true
    css: null
  word_document: default
params:
  config: NULL
  output_dir: NULL
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE,
                      fig.width = 10, fig.height = 6, dpi = 150)
library(flextable); library(ggplot2); library(glue); library(patchwork)
library(stepssurvey)

cfg <- params$config

# Load pre-computed data from output_dir
data_dir <- file.path(params$output_dir, "data")
indicators     <- readRDS(file.path(data_dir, "indicators.rds"))
key_indicators <- readRDS(file.path(data_dir, "key_indicators.rds"))
plots          <- readRDS(file.path(data_dir, "plots.rds"))
```

<div style="background:#00427A; color:white; padding:15px 20px; margin-bottom:20px; border-radius:4px;">
<h1 style="margin:0; color:white;">`r cfg$country_name`</h1>
<h3 style="margin:5px 0 0; color:#B0D4F1;">WHO STEPS NCD Risk Factor Survey `r cfg$survey_year`</h3>
<p style="margin:5px 0 0; font-size:0.9em;">Population: ages `r cfg$age_min`–`r cfg$age_max` years</p>
</div>

## Key Risk Factor Prevalence

```{r overview-chart, fig.height=7}
plots$overview
```

## Summary Table

```{r key-table}
# Format threshold info
threshold_notes <- c()
if (!is.null(cfg$bp_sbp_threshold) && cfg$bp_sbp_threshold != 140) {
  threshold_notes <- c(threshold_notes,
    glue("BP threshold: ≥{cfg$bp_sbp_threshold}/{cfg$bp_dbp_threshold} mmHg"))
}
if (!is.null(cfg$bmi_overweight) && cfg$bmi_overweight != 25) {
  threshold_notes <- c(threshold_notes,
    glue("BMI overweight: ≥{cfg$bmi_overweight}"))
}
if (!is.null(cfg$glucose_threshold) && cfg$glucose_threshold != 7.0) {
  threshold_notes <- c(threshold_notes,
    glue("Glucose threshold: ≥{cfg$glucose_threshold} mmol/L"))
}
if (!is.null(cfg$chol_threshold) && cfg$chol_threshold != 5.0) {
  threshold_notes <- c(threshold_notes,
    glue("Cholesterol threshold: ≥{cfg$chol_threshold} mmol/L"))
}

key_indicators |>
  dplyr::mutate(
    Prevalence = paste0(round(estimate, 1), "% (",
                        round(lower, 1), "–", round(upper, 1), ")")
  ) |>
  dplyr::select(Domain = domain,
                `Risk Factor` = indicator,
                `Prevalence (95% CI)` = Prevalence) |>
  flextable() |>
  theme_vanilla() |>
  bold(part = "header") |>
  bg(part = "header", bg = "#00427A") |>
  color(part = "header", color = "white") |>
  merge_v(j = "Domain") |>
  valign(j = "Domain", valign = "top") |>
  autofit() |>
  set_caption(if (length(threshold_notes) > 0) {
    paste("Note: Non-default thresholds used:", paste(threshold_notes, collapse = "; "))
  } else {
    "WHO standard thresholds applied"
  })
```

## Risk Factor Profile

```{r radar-chart, fig.height=6}
if (!is.null(plots$radar)) {
  plots$radar
} else {
  build_radar_plot(key_indicators, cfg$country_name, cfg$survey_year)
}
```

## Prevalence by Sex

```{r sex-dashboard, fig.height=8}
if (!is.null(plots$sex_dashboard)) plots$sex_dashboard
```

## Forest Plot — Estimates with 95% Confidence Intervals

```{r forest-plot, fig.height=6}
if (!is.null(plots$forest)) {
  plots$forest
} else {
  build_forest_plot(key_indicators, cfg$country_name, cfg$survey_year)
}
```

---

<div style="font-size:0.85em; color:#666; border-top:1px solid #ccc; padding-top:10px;">
**Data source:** WHO STEPS NCD Risk Factor Survey, `r cfg$country_name` `r cfg$survey_year`<br>
**Age group:** `r cfg$age_min`–`r cfg$age_max` years<br>
**Analysis:** Generated by the `stepssurvey` R package using WHO-aligned methodology<br>
**Generated:** `r format(Sys.Date(), "%d %B %Y")`
</div>

---
title: "STEPS Survey Data Book"
subtitle: "`r params$config$country_name` | `r params$config$survey_year`"
output:
  word_document: default
  html_document:
    theme: flatly
    toc: true
    toc_float: true
params:
  config: NULL
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = FALSE, warning = FALSE, message = FALSE,
                      ft.keepnext = FALSE, ft.split = TRUE)
library(flextable); library(ggplot2); library(glue)

cfg      <- params$config
data_dir <- cfg$data_dir

# Read pre-computed data (with safe fallbacks)
detailed_tables <- if (file.exists(file.path(data_dir, "detailed_tables.rds"))) {
  readRDS(file.path(data_dir, "detailed_tables.rds"))
} else {
  list()
}
computed_results <- if (file.exists(file.path(data_dir, "computed_results.rds"))) {
  readRDS(file.path(data_dir, "computed_results.rds"))
} else {
  list()
}
plots <- if (file.exists(file.path(data_dir, "plots.rds"))) {
  readRDS(file.path(data_dir, "plots.rds"))
} else {
  list()
}

# Helper: render all tables whose registry ID matches a set of prefixes
show_tables <- function(id_prefixes) {
  found <- FALSE
  for (id in names(detailed_tables)) {
    if (any(startsWith(id, id_prefixes))) {
      found <- TRUE
      cat("\n\n")
      flextable::flextable_to_rmd(detailed_tables[[id]])
      cat("\n\n")
    }
  }
  if (!found) {
    cat("\n\n*No data available for this section.*\n\n")
  }
}
```

# `r cfg$country_name` -- WHO STEPS NCD Risk Factor Survey `r cfg$survey_year`

**Age range:** `r cfg$age_min`--`r cfg$age_max` years
**Survey instrument:** WHO STEPwise Approach to NCD Risk Factor Surveillance (STEPS)

All estimates are survey-weighted with 95% confidence intervals. Tables follow the standard WHO STEPS 3-panel format: **Men | Women | Both Sexes**, stratified by age group.

---

# Step 1: Behavioural Risk Factors

## Tobacco Use

```{r tobacco, results='asis'}
show_tables(c("T_smoking", "T_smokeless", "T_tobacco", "T_secondhand"))
```

```{r tobacco-chart, fig.width=8, fig.height=5}
if (!is.null(plots$tobacco_by_sex)) plots$tobacco_by_sex
```

## Alcohol Consumption

```{r alcohol, results='asis'}
show_tables("A_")
```

## Diet

```{r diet, results='asis'}
show_tables("D_")
```

## Physical Activity

```{r pa, results='asis'}
show_tables("P_")
```

---

# Step 1.5: Health History

```{r health, results='asis'}
show_tables("H_")
```

---

# Step 2: Physical Measurements

## Blood Pressure

```{r bp, results='asis'}
show_tables(c("M_bp", "M_heart"))
```

```{r bp-chart, fig.width=10, fig.height=5}
if (!is.null(plots$bp_by_age)) plots$bp_by_age
```

## Overweight and Obesity

```{r anthro, results='asis'}
show_tables(c("M_height", "M_bmi", "M_waist", "M_hip"))
```

```{r obesity-chart, fig.width=10, fig.height=5}
if (!is.null(plots$obesity_by_age)) plots$obesity_by_age
```

---

# Step 3: Biochemical Measurements

## Blood Glucose

```{r glucose, results='asis'}
show_tables(c("B_glucose"))
```

## Cholesterol and Lipids

```{r chol, results='asis'}
show_tables(c("B_chol", "B_hdl", "B_triglycerides"))
```

---

# Cardiovascular Risk

```{r cvd-risk, results='asis'}
show_tables("R_")
```

---

# Combined Risk Factors

```{r combined, results='asis'}
show_tables("RF_")
```

---

*Data book generated: `r Sys.Date()`*
*Analysis: WHO STEPS R Pipeline (stepssurvey package)*
*Country: `r cfg$country_name` | Survey year: `r cfg$survey_year`*

---
title: "Interactive Analysis with the Shiny App"
author: "Abhijit Pakhare"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Interactive Analysis with the Shiny App}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  eval     = FALSE
)
```

## Overview

The `stepssurvey` package includes a Shiny web application that provides
a guided, point-and-click interface for analysing WHO STEPS survey data.
It calls the same functions available in the R API, so results are
identical whether you use the app or write scripts.

The app is designed for public health professionals who may not write R
code but need to produce standardised STEPS outputs.

### Launching the app

```{r launch}
library(stepssurvey)
run_app()
```

This opens a browser window with a six-tab workflow.  Each tab
corresponds to one stage of the pipeline: **Data & Settings**,
**Clean**, **Design**, **Indicators**, **Visualise**, and **Reports**.

Work through the tabs from left to right.  Each tab activates once the
previous step has completed successfully.


## Tab 1: Data & Settings

This is where you load your STEPS data.

### Uploading a data file

Click **Browse** and select your file.  Supported formats are CSV
(`.csv`), Excel (`.xlsx`), Stata (`.dta`), and SPSS (`.sav`).  SPSS is
the most common format for STEPS data exported from Epi Info.

After upload, you will see:

- **Raw data preview** -- a scrollable table showing the first 100 rows
  so you can verify the file loaded correctly.
- **Detected columns** -- a table showing which STEPS variables were
  auto-detected and which column in your data they map to.

### Using demo data

If you do not have a STEPS data file, click **Use demo data** to load a
realistic simulated dataset with 3,000 observations.  This is useful for
exploring the app and understanding the outputs before running your own
data.

### Column overrides

The auto-detection system recognises standard WHO STEPS variable codes
from both instrument v3.1 and v3.2.  However, some country datasets use
non-standard names.  If a variable is marked "not found", use the
dropdown menus under **Column overrides** to manually select the correct
column from your data.

Key variables to check:

| Variable | What it is | Why it matters |
|----------|-----------|----------------|
| `age` | Respondent age in years | Required for age group classification |
| `sex` | Respondent sex | Required for sex-stratified tables |
| `weight_step1` | Sampling weight (Step 1) | Needed for correct weighted estimates |
| `psu` | Primary sampling unit / cluster | Needed for correct standard errors |
| `tobacco_current` | Current tobacco smoking | Core Step 1 indicator |
| `sbp1` | First SBP reading | Core Step 2 indicator |
| `fasting_glucose` | Fasting blood glucose | Core Step 3 indicator |

### Survey settings

Set the country name, survey year, and target age range (default 18--69
years) in the settings panel.  These appear in the report headers and
determine which observations are included in the analysis.


## Tab 2: Clean

Click the **Clean** button to process the raw data.  The cleaning step:

- Restricts the sample to the specified age range
- Creates WHO standard age groups (18--24, 25--34, ... 65+)
- Harmonises sex coding to Male/Female
- Derives all binary indicators (raised BP, overweight, etc.)
- Computes BMI, mean blood pressure, MET-minutes/week, and other
  continuous measures

After cleaning, you will see:

- **Row counts** -- how many observations were retained versus excluded
- **Clean data preview** -- a table of the processed data with all
  derived variables

If cleaning fails, check the console output (or R Studio console) for
messages about which variables were missing or had unexpected values.


## Tab 3: Design

This tab creates the complex survey design object that ensures all
estimates account for the sampling weights, stratification, and
clustering used in STEPS surveys.

The app automatically detects the survey design complexity:

- **Full complex design** if weight, stratum, and PSU columns are present
- **Weights only** if no stratification or clustering information is found
- **Unweighted** if no weight column is present (not recommended)

You will see a summary confirming which design was created.  Extreme
weights are automatically trimmed to prevent individual observations from
dominating the estimates.


## Tab 4: Indicators

Click **Compute indicators** to calculate all weighted prevalence
estimates and means across six domains:

- Tobacco use (current, daily, smokeless, second-hand exposure)
- Alcohol consumption (current, heavy episodic, mean drinks)
- Diet and Physical Activity (fruit/vegetable intake, MET-minutes)
- Anthropometry (BMI, waist circumference, overweight/obesity)
- Blood Pressure (mean SBP/DBP, raised BP, treatment cascade)
- Biochemical (glucose, cholesterol, HDL, triglycerides)

After computation, you will see:

- **Value boxes** showing the number of indicators computed and key
  headline figures
- **Key indicators table** -- a summary table of all headline
  prevalences with 95% confidence intervals, downloadable as CSV


## Tab 5: Visualise

This tab displays pre-built charts.  All plots use the WHO STEPS colour
palette and `theme_steps()` styling.

### Available charts

- **Overview** -- horizontal bar chart of all key indicators with 95% CIs,
  sorted by prevalence
- **By sex** -- grouped bar charts comparing Men vs Women for tobacco,
  blood pressure, obesity, and glucose
- **By age** -- line charts with confidence bands showing how blood
  pressure and obesity vary across age groups
- **Sex dashboard** -- a combined 2×2 panel of the most important
  sex-stratified indicators

If a particular chart is not available (because the underlying data
was missing), a "not available" message is shown instead of an error.


## Tab 6: Reports

This is the final step.  Click **Generate report** to produce both
reports.

### What happens when you click Generate

The app runs a pipeline that:

1. Re-cleans the data and rebuilds the survey design
2. Computes all indicators and builds summary tables
3. Computes the full WHO table registry (~60 detailed 3-panel tables)
4. Generates plots
5. Renders both Word documents

A status message keeps you informed of progress.  The full process
typically takes 30--90 seconds depending on sample size.

### Two reports

| Report | Button | Contents |
|--------|--------|----------|
| **Summary Report** | Download Summary Report | Executive summary, narrative by domain, embedded charts, recommendations, methodology |
| **Detailed Data Book** | Download Data Book | Complete WHO 3-panel tables (Men \| Women \| Both Sexes) by age group, organised by STEPS step |

### Summary Report

The country report includes:

- **Key Findings table** listing all headline indicators with 95% CIs
- **Domain sections** for Tobacco, Physical Activity, Obesity, Blood
  Pressure, Blood Glucose, and Cholesterol -- each with an inline
  prevalence estimate and an embedded chart
- **Recommendations** aligned with WHO best-buy interventions
- **Methodology** section describing the survey design and analysis

### Detailed Data Book

The data book follows the WHO STEPS standard layout with tables organised
by survey step:

- **Step 1: Behavioural** -- Tobacco (smoking status, smokeless, quit
  attempts, second-hand exposure), Alcohol (drinking patterns, heavy
  episodic), Diet (fruit/vegetable, salt), Physical Activity (total
  minutes, domain breakdown, insufficient PA)
- **Step 1.5: Health History** -- BP/glucose/cholesterol screening and
  diagnosis cascades, CVD history, lifestyle advice
- **Step 2: Physical Measurements** -- Mean BP, raised BP, treatment
  cascade, BMI classifications, waist/hip measurements
- **Step 3: Biochemical** -- Fasting glucose (impaired + raised),
  treatment cascade, total cholesterol, HDL, triglycerides
- **Combined Risk Factors** -- summary of 0, 1--2, and 3--5 concurrent
  risk factors

Every table uses the 3-panel format:

```
Age Group | Men          | Women        | Both Sexes
18-24     | 4.3% (2-6.5)| 0% (0-0)     | 1.8% (0.8-2.8)
25-34     | ...          | ...          | ...
...
18-69     | ...          | ...          | ...
```

### Additional downloads

- **Download tables & plots** -- a ZIP file containing all pre-computed
  RDS files (indicators, tables, plots) for further analysis in R


## Tips and troubleshooting

### The app shows an error on upload

If you see `dim<-.haven_labelled() not supported`, this was a known issue
with SPSS files that has been fixed.  Make sure you are running the latest
version of the package (`devtools::load_all(".")` or reinstall from
GitHub).

### Charts don't display or show "figure margins too large"

This happens when the RStudio Viewer pane is too narrow.  Open the app
in a full browser window instead: after `run_app()`, click "Open in
Browser" in the Viewer toolbar.

### Physical Activity shows "not available"

The Summary Report derives the insufficient PA indicator from
MET-minutes/week.  If your dataset has raw GPAQ items (P1--P16) but no
pre-computed `met_total`, the package now computes MET-minutes/week
automatically using WHO MET multipliers (8 for vigorous, 4 for
moderate/transport activities).  Ensure your GPAQ columns are correctly
mapped in Tab 1.

### Some tables are empty in the Data Book

Tables are only produced when the required variables are present in the
data.  For example, CVD risk tables require both clinical measurements
and a risk scoring algorithm.  Missing tables show "No data available
for this section."

### Can I customise the report templates?

Yes.  The R Markdown templates are located in `inst/rmd/` within the
package source:

- `country_report.Rmd` -- Summary Report
- `data_book.Rmd` -- Detailed Data Book
- `fact_sheet.Rmd` -- Fact Sheet

Copy the template you want to modify, edit it, and pass the path via the
config object.


## Comparison: Shiny app vs R scripting

| Feature | Shiny App | R Scripts |
|---------|-----------|-----------|
| Audience | Non-coders, quick analysis | R users, reproducible workflows |
| Column mapping | Interactive dropdowns | `detect_steps_columns()` + manual overrides |
| Customisation | Limited to built-in options | Full control over every step |
| Batch processing | One dataset at a time | Loop over multiple datasets |
| Reproducibility | Manual (same clicks each time) | Full script = full reproducibility |
| Output | Word reports via browser | Word, HTML, PDF, or custom formats |

For routine country-level STEPS analysis, the Shiny app is the fastest
path.  For multi-country comparisons, methodological research, or custom
indicators, use the R API documented in `vignette("stepssurvey-guide")`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{.build_cascade_table}
\alias{.build_cascade_table}
\title{Build a cascade table (also used for combined risk factors)}
\usage{
.build_cascade_table(result)
}
\description{
Build a cascade table (also used for combined risk factors)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_upload.R
\name{mod_upload_server}
\alias{mod_upload_server}
\title{Upload Module — Server}
\usage{
mod_upload_server(id)
}
\arguments{
\item{id}{Module namespace id.}
}
\value{
A \link[shiny:reactiveValues]{shiny::reactiveValues} with elements \code{raw}, \code{cols}, \code{config}.
}
\description{
Upload Module — Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{make_age_chart}
\alias{make_age_chart}
\title{Create age-stratified trend chart}
\usage{
make_age_chart(by_age_df, title, steps_colors, theme_steps)
}
\arguments{
\item{by_age_df}{Data frame with \code{age_group}, \code{estimate}, \code{lower}, \code{upper} columns.}

\item{title}{Plot title.}

\item{steps_colors}{Colour list from \code{\link[=steps_colors]{steps_colors()}}.}

\item{theme_steps}{Theme from \code{\link[=theme_steps]{theme_steps()}}.}
}
\value{
A ggplot2 object or NULL if input is NULL.
}
\description{
Create age-stratified trend chart
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators.R
\name{extract_key}
\alias{extract_key}
\title{Extract Key Indicator for Fact Sheet}
\usage{
extract_key(ind_list, domain, key, label)
}
\arguments{
\item{ind_list}{A list of indicator results (e.g., from compute_tobacco_indicators)}

\item{domain}{The domain name (e.g., "Tobacco", "Alcohol")}

\item{key}{The result name to extract (e.g., "current_tobacco_total")}

\item{label}{The display label for the indicator}
}
\value{
A tibble row with columns: domain, indicator, estimate, lower, upper,
or NULL if the key does not exist in ind_list
}
\description{
Internal helper function to extract a single key indicator from an
indicator list and format it as a tibble row with domain and label.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_quality.R
\name{mod_quality_server}
\alias{mod_quality_server}
\title{Data Quality Module — Server}
\usage{
mod_quality_server(id, upload_out, clean_out)
}
\arguments{
\item{id}{Module namespace id.}

\item{upload_out}{Reactive list returned by \code{\link[=mod_upload_server]{mod_upload_server()}}, containing
\code{raw} (data frame), \code{cols} (column mapping), and \code{config} (survey config).}

\item{clean_out}{Reactive returning the cleaned data frame from
\code{\link[=mod_clean_server]{mod_clean_server()}}.}
}
\value{
A reactive returning the \code{steps_quality} object (or NULL).
}
\description{
Data Quality Module — Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{table_builder}
\alias{table_builder}
\title{Generic Table Builder for WHO STEPS Data Book}
\description{
Takes computed results from \code{\link[=compute_table]{compute_table()}} or \code{\link[=compute_all_tables]{compute_all_tables()}}
and produces formatted flextable objects in the standard WHO STEPS
3-panel format (Men / Women / Both Sexes).
}
\concept{tables}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators_bp.R
\name{compute_bp_indicators}
\alias{compute_bp_indicators}
\title{Compute Blood Pressure Indicators}
\usage{
compute_bp_indicators(design)
}
\arguments{
\item{design}{A survey design object from \code{\link[=setup_survey_design]{setup_survey_design()}}.}
}
\value{
A named list of survey estimates. Each element contains estimates
(as tibble with columns: estimate, lower, upper, etc.) for:
\itemize{
\item \code{raised_bp_total}: raised blood pressure, overall
\item \code{raised_bp_by_sex}: raised blood pressure, by sex
\item \code{raised_bp_by_age}: raised blood pressure, by age group
\item \code{mean_sbp_mean_total}: mean systolic BP, overall
\item \code{mean_sbp_mean_by_sex}: mean systolic BP, by sex
\item \code{mean_sbp_mean_by_age}: mean systolic BP, by age group
\item \code{mean_dbp_mean_total}: mean diastolic BP, overall
\item \code{mean_dbp_mean_by_sex}: mean diastolic BP, by sex
\item \code{mean_dbp_mean_by_age}: mean diastolic BP, by age group
(if the corresponding variables exist in design)
}
}
\description{
Calculates prevalence of raised blood pressure and mean systolic and
diastolic blood pressure from a survey design object.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  bp_results <- compute_bp_indicators(design)
}
}
\seealso{
\code{\link[=compute_all_indicators]{compute_all_indicators()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators_diet_pa.R
\name{compute_diet_pa_indicators}
\alias{compute_diet_pa_indicators}
\title{Compute Diet and Physical Activity Indicators}
\usage{
compute_diet_pa_indicators(design)
}
\arguments{
\item{design}{A survey design object from \code{\link[=setup_survey_design]{setup_survey_design()}}.}
}
\value{
A named list of survey estimates. Each element contains estimates
(as tibble with columns: estimate, lower, upper, etc.) for:
\itemize{
\item \code{insufficient_pa_total}: insufficient physical activity, overall
\item \code{insufficient_pa_by_sex}: insufficient physical activity, by sex
\item \code{insufficient_pa_by_age}: insufficient physical activity, by age group
\item \code{low_fruit_veg_total}: low fruit & vegetable intake, overall
\item \code{low_fruit_veg_by_sex}: low fruit & vegetable intake, by sex
\item \code{low_fruit_veg_by_age}: low fruit & vegetable intake, by age group
\item \code{met_mean_total}: mean MET (if available)
\item \code{met_mean_by_sex}: mean MET by sex (if available)
(if the corresponding variables exist in design)
}
}
\description{
Calculates prevalence of insufficient physical activity and low fruit &
vegetable intake, plus mean metabolic equivalent (MET) values, from a
survey design object.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  diet_pa_results <- compute_diet_pa_indicators(design)
}
}
\seealso{
\code{\link[=compute_all_indicators]{compute_all_indicators()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey_design.R
\name{setup_survey_design}
\alias{setup_survey_design}
\title{Set up survey designs for STEPS data (one per Step)}
\usage{
setup_survey_design(data)
}
\arguments{
\item{data}{A data frame (typically from \code{\link[=clean_steps_data]{clean_steps_data()}}).}
}
\value{
A list of class \code{"steps_designs"} with three
\link[survey:svydesign]{survey::svydesign} objects (step1, step2, step3).
}
\description{
Creates up to three survey design objects — one per WHO STEPS Step —
each using the appropriate step-specific weight column
(\code{wt_step1}, \code{wt_step2}, \code{wt_step3}).
}
\details{
The returned object is a list of class \code{"steps_designs"} with elements
\verb{$step1}, \verb{$step2}, \verb{$step3}.  For backward compatibility it can also
be used directly as a single design (it delegates to \verb{$step1}).

The function handles five design cases per step:
\enumerate{
\item Full complex design: weights + strata + clusters
\item Weights + clusters, no strata
\item Weights + strata, no clusters
\item Weights only
\item Unweighted (simple random sampling)
}

Weights are used as-is without trimming, consistent with the
WHO official STEPS analysis scripts.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.compute_category}
\alias{.compute_category}
\title{Compute a category table (type = "category")}
\usage{
.compute_category(entry, design, data)
}
\description{
The variable is a factor; compute the proportion in each level.
Returns a named list (one element per level), each with total, by_sex, by_age.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{build_forest_plot}
\alias{build_forest_plot}
\title{Build forest plot of key indicators with 95\% CIs}
\usage{
build_forest_plot(key_indicators, country_name, survey_year)
}
\arguments{
\item{key_indicators}{A data frame with domain, indicator, estimate, lower, upper.}

\item{country_name}{Country name for title.}

\item{survey_year}{Survey year for title.}
}
\value{
A ggplot2 object.
}
\description{
Creates a horizontal point-and-CI plot (forest plot style) for all
key indicators, grouped by domain.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_visualise.R
\name{mod_visualise_ui}
\alias{mod_visualise_ui}
\title{Visualise Module — UI}
\usage{
mod_visualise_ui(id)
}
\arguments{
\item{id}{Module namespace id.}
}
\value{
A Shiny UI definition.
}
\description{
Creates the user interface for the visualisation module, including a "Generate plots"
action button and tabbed layout for displaying STEPS survey visualizations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{detect_col}
\alias{detect_col}
\title{Detect a STEPS column by alias}
\usage{
detect_col(data, candidates, label = NULL)
}
\arguments{
\item{data}{A data frame.}

\item{candidates}{Character vector of possible column names.}

\item{label}{Optional label for progress messages.}
}
\value{
The matched column name (character) or \code{NULL}.
}
\description{
Tries to find a column in the data matching one of several candidate
names (case-insensitive).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_indicators.R
\name{tbl_domain}
\alias{tbl_domain}
\title{Format indicator results tibble for display}
\usage{
tbl_domain(domain_name, domain_results)
}
\arguments{
\item{domain_name}{Character name of the domain for display.}

\item{domain_results}{List of indicator results for the domain.}
}
\value{
A formatted data frame ready for DT::datatable, or NULL if empty.
}
\description{
Internal helper to convert a domain's indicator results list into a
formatted data frame for DT table display.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{steps_data_quality}
\alias{steps_data_quality}
\title{Data Quality Diagnostics for WHO STEPS Data}
\usage{
steps_data_quality(raw, cleaned, cols)
}
\arguments{
\item{raw}{The raw (pre-cleaning) data frame, typically from
\code{\link[=import_steps_data]{import_steps_data()}}.}

\item{cleaned}{The cleaned data frame from \code{\link[=clean_steps_data]{clean_steps_data()}}.}

\item{cols}{Column mapping list from \code{\link[=detect_steps_columns]{detect_steps_columns()}}.}
}
\value{
A list of class \code{"steps_quality"} with elements:
\describe{
\item{digit_preference}{Terminal-digit tables and heaping indices for
physical measurements (SBP, DBP, height, weight, waist).}
\item{completeness}{Per-variable missingness counts and percentages,
grouped by STEPS domain.}
\item{plausibility}{Summary of values outside plausible ranges.}
\item{weights}{Sampling weight distribution statistics.}
}
}
\description{
Produces a comprehensive data quality report covering digit preference,
completeness, plausibility, and sampling weight diagnostics.
}
\details{
\strong{Digit preference / heaping} is assessed using the Whipple-style
heaping index: the ratio of observed frequency at a digit (0 or 5)
to the expected frequency under uniform distribution.
An index of 1.0 = no preference; >1.5 = moderate heaping; >2.0 = severe.

\strong{Completeness} reports missing values for key STEPS variables grouped
by Step (behavioural, physical, biochemical).

\strong{Plausibility} counts values outside WHO-recommended ranges
(e.g. height 100--250 cm, weight 20--300 kg, SBP 60--300 mmHg).

\strong{Weight diagnostics} summarise the distribution of sampling weights
and flag potential issues (high CV, zero/NA weights).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{.col_label}
\alias{.col_label}
\title{Get the SPSS/haven label for a column (if any)}
\usage{
.col_label(col)
}
\arguments{
\item{col}{A vector (typically from a data frame column).}
}
\value{
Character string of the label, or "" if none.
}
\description{
Get the SPSS/haven label for a column (if any)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reports.R
\name{render_fact_sheet}
\alias{render_fact_sheet}
\title{Render STEPS Fact Sheet report}
\usage{
render_fact_sheet(config, output_dir = tempdir(), format = c("html", "word"))
}
\arguments{
\item{config}{A list from \code{\link[=steps_config]{steps_config()}} with survey metadata and paths.
Expected to have \code{country_name}, \code{survey_year}, \code{age_min}, \code{age_max}.}

\item{output_dir}{Directory for output reports (default \code{tempdir()}).}

\item{format}{Output format: \code{"html"} for self-contained HTML (default)
or \code{"word"} for Word (.docx).}
}
\value{
Path to generated output file (invisibly).
Prints message with output location.
}
\description{
Generates a Word document with an overview of key NCD risk factor
prevalence, including summary table and sex-stratified charts.
}
\details{
The fact sheet template uses pre-computed indicators, key_indicators,
and plots (via .rds files in data/processed/).
Requires rmarkdown, flextable, ggplot2, glue, patchwork packages.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tables.R
\name{build_table_if_available}
\alias{build_table_if_available}
\title{Build a table if indicator is available}
\usage{
build_table_if_available(
  ind_list,
  total_key,
  by_sex_key,
  by_age_key,
  label,
  pct = TRUE
)
}
\arguments{
\item{ind_list}{Indicator list (e.g., \code{indicators$tobacco}).}

\item{total_key}{Name of total estimate element (e.g., "current_tobacco_total").}

\item{by_sex_key}{Name of by-sex element (e.g., "current_tobacco_by_sex").}

\item{by_age_key}{Name of by-age element (e.g., "current_tobacco_by_age").}

\item{label}{Table caption.}

\item{pct}{Logical; format as percent? (default TRUE).}
}
\value{
A flextable object or NULL if not available.
}
\description{
Wrapper that safely builds a table only if the required indicator elements exist.
Catches and reports errors gracefully.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.apply_denominator}
\alias{.apply_denominator}
\title{Optionally subset a design by a denominator variable}
\usage{
.apply_denominator(design, denominator, data)
}
\description{
Optionally subset a design by a denominator variable
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{.fmt_ci}
\alias{.fmt_ci}
\title{Format a single estimate with CI}
\usage{
.fmt_ci(est, lower, upper, digits = 1, is_pct = TRUE)
}
\arguments{
\item{est}{Numeric estimate.}

\item{lower}{Lower CI bound.}

\item{upper}{Upper CI bound.}

\item{digits}{Decimal places.}

\item{is_pct}{If TRUE, values are already in percent; add "\%" suffix.}
}
\value{
Formatted string.
}
\description{
Format a single estimate with CI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reports.R
\name{render_country_report}
\alias{render_country_report}
\title{Render STEPS Country Report}
\usage{
render_country_report(config, output_dir = tempdir())
}
\arguments{
\item{config}{A list from \code{\link[=steps_config]{steps_config()}} with survey metadata.
Expected to have \code{country_name}, \code{survey_year}, \code{age_min}, \code{age_max}.}

\item{output_dir}{Directory for output reports (default \code{tempdir()}).}
}
\value{
Path to generated Word document (invisibly).
Prints message with output location.
}
\description{
Generates a comprehensive Word document with executive summary,
indicator-by-indicator analysis, and recommendations for public health action.
}
\details{
Sections include:
\itemize{
\item Executive summary with key findings
\item Tobacco use
\item Physical activity
\item Overweight and obesity
\item Blood pressure
\item Blood glucose and cholesterol
\item Recommendations for public health action
\item Methodology
}

Requires pre-computed indicators, tables, and plots in data/processed/.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{build_table}
\alias{build_table}
\title{Build a formatted table from a computed result}
\usage{
build_table(result)
}
\arguments{
\item{result}{A result list from \code{\link[=compute_table]{compute_table()}}.}
}
\value{
A flextable object, or NULL if the table is not available.
}
\description{
Dispatches to the appropriate formatting method based on table type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{compute_all_tables}
\alias{compute_all_tables}
\title{Compute all tables from the registry}
\usage{
compute_all_tables(designs, data = NULL)
}
\arguments{
\item{designs}{A list of survey designs, with elements \code{step1}, \code{step2},
\code{step3} (as returned by \code{\link[=setup_survey_design]{setup_survey_design()}}).}

\item{data}{The cleaned data frame.}
}
\value{
A named list of table results (from \code{\link[=compute_table]{compute_table()}}).
Only entries with \code{available == TRUE} are included.
}
\description{
Iterates through the full \code{\link[=steps_table_registry]{steps_table_registry()}} and computes every
table that has available data. Returns a named list of results.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{.detect_col_with_label}
\alias{.detect_col_with_label}
\title{Label-aware column detection for ambiguous STEPS codes}
\usage{
.detect_col_with_label(data, safe_candidates, ambiguous = list(), label = NULL)
}
\arguments{
\item{data}{A data frame.}

\item{safe_candidates}{Character vector of unambiguous column names.}

\item{ambiguous}{Named list: each element is a character vector of
keywords that must appear in the column label for the code to match.
Names are the ambiguous column codes.
Example: \code{list(a1 = c("current", "30 day"), a5 = c("current", "30 day"))}}

\item{label}{Optional label for progress messages.}
}
\value{
The matched column name or NULL.
}
\description{
Some STEPS column codes (e.g., A1, A5) mean different things across
instrument versions.
This function first tries unambiguous candidate names, then checks
ambiguous codes only if their column label matches expected keywords.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read_column_mapping}
\alias{read_column_mapping}
\title{Read a column mapping file}
\usage{
read_column_mapping(path, data = NULL)
}
\arguments{
\item{path}{Path to the filled mapping file (.xlsx or .csv).}

\item{data}{Optional data frame. If provided, the function validates that
every mapped column actually exists in the data.}
}
\value{
A named list where names are standard variable identifiers
(e.g. \code{"age"}, \code{"sbp1"}) and values are the corresponding
column names in the user's dataset.
Unmapped variables are set to \code{NULL}.
}
\description{
Reads a filled-in column mapping template (Excel or CSV) and returns a
named list suitable for passing to \code{\link[=clean_steps_data]{clean_steps_data()}}.  The mapping file
should have at least two columns: one with the standard variable name
(column A) and one with the user's column name (column C in the template,
or the third column).
}
\details{
This function is the manual alternative to \code{\link[=detect_steps_columns]{detect_steps_columns()}}.
Use it when your dataset has non-standard variable names that
auto-detection cannot resolve.

A blank template can be obtained from
\code{system.file("templates", "column_mapping_template.xlsx",
                   package = "stepssurvey")}
or downloaded from the Shiny app.

The function ignores domain-header rows (rows where column A is all-caps
with no entry in column C) and skips any row where the user's column name
is blank.
}
\examples{
\dontrun{
  cols <- read_column_mapping("my_mapping.xlsx")
  raw  <- import_steps_data("survey.dta")
  clean <- clean_steps_data(raw, cols)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{.merge_panels}
\alias{.merge_panels}
\title{Merge by_sex and by_age results into a single wide data frame}
\usage{
.merge_panels(
  total_df,
  by_sex_df,
  by_age_df,
  is_pct = TRUE,
  by_sex_age_df = NULL
)
}
\arguments{
\item{total_df}{Total estimate (1-row data frame).}

\item{by_sex_df}{By-sex estimates (with \code{sex} column).}

\item{by_age_df}{By-age estimates (with \code{age_group} column).}

\item{is_pct}{Whether values are percentages.}
}
\value{
A data frame with columns: Age Group, Men, Women, Both Sexes.
}
\description{
Produces the standard 3-panel layout: Age Group | Men | Women | Both Sexes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators_anthropometry.R
\name{compute_anthropometry_indicators}
\alias{compute_anthropometry_indicators}
\title{Compute Anthropometry Indicators}
\usage{
compute_anthropometry_indicators(design)
}
\arguments{
\item{design}{A survey design object from \code{\link[=setup_survey_design]{setup_survey_design()}}.}
}
\value{
A named list of survey estimates. Each element contains estimates
(as tibble with columns: estimate, lower, upper, etc.) for:
\itemize{
\item \code{overweight_obese_total}: overweight or obese (BMI >=25), overall
\item \code{overweight_obese_by_sex}: overweight or obese, by sex
\item \code{overweight_obese_by_age}: overweight or obese, by age group
\item \code{obese_total}: obese (BMI >=30), overall
\item \code{obese_by_sex}: obese, by sex
\item \code{obese_by_age}: obese, by age group
\item \code{central_obesity_total}: central obesity, overall
\item \code{central_obesity_by_sex}: central obesity, by sex
\item \code{central_obesity_by_age}: central obesity, by age group
\item \code{bmi_mean_total}: mean BMI, overall
\item \code{bmi_mean_by_sex}: mean BMI, by sex
\item \code{waist_cm_mean_total}: mean waist circumference, overall
\item \code{waist_cm_mean_by_sex}: mean waist circumference, by sex
(if the corresponding variables exist in design)
}
}
\description{
Calculates prevalence of overweight, obesity, and central obesity, plus
mean BMI and waist circumference, from a survey design object.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  anthropometry_results <- compute_anthropometry_indicators(design)
}
}
\seealso{
\code{\link[=compute_all_indicators]{compute_all_indicators()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_upload.R
\name{mod_upload_ui}
\alias{mod_upload_ui}
\title{Upload Module — UI}
\usage{
mod_upload_ui(id)
}
\arguments{
\item{id}{Module namespace id.}
}
\description{
Upload Module — UI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/steps_registry.R
\name{get_registry_by_step}
\alias{get_registry_by_step}
\title{Get table registry entries by step}
\usage{
get_registry_by_step(step)
}
\arguments{
\item{step}{STEPS step number (1, 2, or 3).}
}
\value{
A filtered list of registry entries.
}
\description{
Get table registry entries by step
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{.strip_haven_labels}
\alias{.strip_haven_labels}
\title{Strip haven-labelled classes from all columns in a data frame}
\usage{
.strip_haven_labels(data)
}
\arguments{
\item{data}{A data frame (typically from \code{\link[=import_steps_data]{import_steps_data()}}).}
}
\value{
The same data frame with all haven labels removed.
}
\description{
Converts \code{haven_labelled} vectors to their base R types.
This prevents downstream errors (e.g., \verb{dim<-.haven_labelled() not supported}) when passing the data to DT, dplyr, or survey.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.vars_available}
\alias{.vars_available}
\title{Check if variable(s) exist in data}
\usage{
.vars_available(vars, data)
}
\description{
Check if variable(s) exist in data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/steps_registry.R
\name{list_registry_sections}
\alias{list_registry_sections}
\title{List all available sections in the registry}
\usage{
list_registry_sections()
}
\value{
Character vector of unique section names.
}
\description{
List all available sections in the registry
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app_server.R
\name{app_server}
\alias{app_server}
\title{Main Shiny Server}
\usage{
app_server(input, output, session)
}
\arguments{
\item{input, output, session}{Standard Shiny server arguments.}
}
\description{
Wires up all module servers and passes reactive outputs between them.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.compute_mean}
\alias{.compute_mean}
\title{Compute a mean table (type = "mean")}
\usage{
.compute_mean(entry, design, data)
}
\description{
For single variable: total, by_sex, by_age.
For multiple variables: a named list, each with total, by_sex, by_age.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{.build_proportion_table}
\alias{.build_proportion_table}
\title{Build a proportion table}
\usage{
.build_proportion_table(result)
}
\description{
Build a proportion table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_design.R
\name{mod_design_server}
\alias{mod_design_server}
\title{Design Module — Server}
\usage{
mod_design_server(id, clean_out)
}
\arguments{
\item{id}{Module namespace id.}

\item{clean_out}{Reactive returning the cleaned data frame.}
}
\value{
A \link[shiny:reactive]{shiny::reactive} returning the survey design object.
}
\description{
Design Module — Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators_alcohol.R
\name{compute_alcohol_indicators}
\alias{compute_alcohol_indicators}
\title{Compute Alcohol Use Indicators}
\usage{
compute_alcohol_indicators(design)
}
\arguments{
\item{design}{A survey design object from \code{\link[=setup_survey_design]{setup_survey_design()}}.}
}
\value{
A named list of survey estimates. Each element contains proportion
estimates (as tibble with columns: estimate, lower, upper, etc.) for:
\itemize{
\item \code{current_alcohol_total}: current alcohol use, overall
\item \code{current_alcohol_by_sex}: current alcohol use, by sex
\item \code{current_alcohol_by_age}: current alcohol use, by age group
\item \code{heavy_episodic_total}: heavy episodic drinking, overall
\item \code{heavy_episodic_by_sex}: heavy episodic drinking, by sex
\item \code{heavy_episodic_by_age}: heavy episodic drinking, by age group
(if the corresponding variables exist in design)
}
}
\description{
Calculates prevalence of alcohol use from a survey design object.
Computes proportions of current alcohol use and heavy episodic drinking,
stratified by sex and age group where available.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  alcohol_results <- compute_alcohol_indicators(design)
}
}
\seealso{
\code{\link[=compute_all_indicators]{compute_all_indicators()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{build_steps_plots}
\alias{build_steps_plots}
\title{Build publication-ready STEPS visualizations}
\usage{
build_steps_plots(indicators, key_indicators, country_name, survey_year)
}
\arguments{
\item{indicators}{A list of indicator results from \code{\link[=compute_all_indicators]{compute_all_indicators()}}.}

\item{key_indicators}{A data frame with key indicators (domain, indicator, estimate, lower, upper).}

\item{country_name}{Country name for plot titles.}

\item{survey_year}{Survey year for plot titles.}
}
\value{
A named list of ggplot2 objects:
\itemize{
\item \code{overview}: Horizontal bar chart of key indicators
\item \code{tobacco_by_sex}: Sex-stratified tobacco use
\item \code{bp_by_sex}: Sex-stratified blood pressure
\item \code{obesity_by_sex}: Sex-stratified overweight/obesity
\item \code{glucose_by_sex}: Sex-stratified blood glucose
\item \code{bp_by_age}: Age-stratified blood pressure with ribbon CI
\item \code{obesity_by_age}: Age-stratified overweight/obesity with ribbon CI
\item \code{sex_dashboard}: Combined 2x2 dashboard of sex-stratified charts (if >=2 sex plots available)
NULL entries are preserved in the list.
}
}
\description{
Generates a list of ggplot2 plots showing key NCD risk factor prevalence
with 95\% confidence intervals, stratified by sex and age group.
}
\details{
All plots use the WHO STEPS colour scheme and professional styling.
Error bars represent 95\% confidence intervals.
Prevalence values are displayed on bars/points with light background text.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  all_ind <- compute_all_indicators(design)
  plots <- build_steps_plots(all_ind$results, all_ind$key_indicators, "Test", 2023)
  names(plots)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean_steps_data}
\alias{clean_steps_data}
\title{Clean and recode WHO STEPS data}
\usage{
clean_steps_data(
  data,
  cols,
  age_min = 18,
  age_max = 69,
  bp_sbp_threshold = 140,
  bp_dbp_threshold = 90,
  bmi_overweight = 25,
  bmi_obese = 30,
  glucose_threshold = 7,
  glucose_impaired_threshold = 6.1,
  chol_threshold = 5
)
}
\arguments{
\item{data}{A data frame (typically from \code{\link[=import_steps_data]{import_steps_data()}}).}

\item{cols}{A named list of column names, as returned by \code{\link[=detect_steps_columns]{detect_steps_columns()}}.}

\item{age_min}{Minimum age for inclusion (default 18).}

\item{age_max}{Maximum age for inclusion (default 69).}

\item{bp_sbp_threshold}{SBP threshold for raised BP (default 140; Mongolia uses 130).}

\item{bp_dbp_threshold}{DBP threshold for raised BP (default 90; Mongolia uses 80).}

\item{bmi_overweight}{BMI threshold for overweight (default 25.0).}

\item{bmi_obese}{BMI threshold for obesity (default 30.0).}

\item{glucose_threshold}{Fasting glucose threshold for raised glucose / diabetes
in mmol/L (default 7.0).}

\item{glucose_impaired_threshold}{Fasting glucose threshold for impaired fasting
glucose in mmol/L (default 6.1).}

\item{chol_threshold}{Total cholesterol threshold for raised cholesterol
in mmol/L (default 5.0).}
}
\value{
A data frame with standardised and derived variables, ready for
survey design setup.
}
\description{
Processes raw STEPS survey data: renames columns, coerces types,
derives standard indicators, handles missing values, and applies
plausibility checks.
}
\details{
The function performs the following transformations:
\itemize{
\item Renames columns to standard names (age, sex, wt_final, etc.)
\item Converts numeric strings to appropriate types
\item Restricts age to \code{[age_min, age_max]}
\item Creates WHO standard age groups (18-24, 25-34, etc.)
\item Harmonises sex coding to Male/Female
\item Derives body mass index (BMI) and categories
\item Averages blood pressure readings (last 2 of 3)
\item Recodes yes/no variables to logical
\item Creates derived risk indicators (raised BP, diabetes, etc.)
\item Applies plausibility checks to measurements
\item Drops records with missing age or sex
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.compute_cascade}
\alias{.compute_cascade}
\title{Compute a cascade table (type = "cascade")}
\usage{
.compute_cascade(entry, design, data)
}
\description{
A cascade is a series of proportions, each possibly with a different
denominator (nested). Returns named list, each with total, by_sex, by_age.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/steps_registry.R
\name{get_registry_by_section}
\alias{get_registry_by_section}
\title{Get table registry entries by section}
\usage{
get_registry_by_section(section = NULL)
}
\arguments{
\item{section}{Section name (e.g., "Tobacco Use", "Blood Pressure").
If NULL, returns all entries.}
}
\value{
A filtered list of registry entries.
}
\description{
Get table registry entries by section
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tables.R
\name{fmt_est}
\alias{fmt_est}
\title{Format estimate with confidence interval}
\usage{
fmt_est(est, lower, upper, digits = 1, pct = TRUE)
}
\arguments{
\item{est}{Estimated value.}

\item{lower}{Lower confidence bound.}

\item{upper}{Upper confidence bound.}

\item{digits}{Number of decimal places (default 1).}

\item{pct}{Logical; add percent sign? (default TRUE).}
}
\value{
Formatted string like "42.1\% (39.2–45.0)".
}
\description{
Format estimate with confidence interval
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_weights}
\alias{plot_weights}
\title{Plot sampling weight distribution}
\usage{
plot_weights(dq, step = "weight_step1")
}
\arguments{
\item{dq}{A \code{steps_quality} object from \code{\link[=steps_data_quality]{steps_data_quality()}}.}

\item{step}{Character: which weight to plot ("weight_step1", "weight_step2",
or "weight_step3"). Defaults to "weight_step1".}
}
\value{
A ggplot object.
}
\description{
Creates a histogram of sampling weights with summary statistics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_visualise.R
\name{mod_visualise_server}
\alias{mod_visualise_server}
\title{Visualise Module — Server}
\usage{
mod_visualise_server(id, results_out, upload_out)
}
\arguments{
\item{id}{Module namespace id.}

\item{results_out}{Reactive list returned by \code{\link[=mod_indicators_server]{mod_indicators_server()}}, containing
\code{results} (list of indicator results) and \code{key_indicators} (tibble of key indicators).}

\item{upload_out}{Reactive list returned by \code{\link[=mod_upload_server]{mod_upload_server()}}, containing
\code{config} (list with country_name and survey_year).}
}
\value{
NULL (invisible). This is a terminal module for visualisation.
}
\description{
Server logic for the visualisation module. Generates plots on button click,
renders them in tabs, and provides ZIP download functionality.
}
\details{
The module calls \code{\link[=build_steps_plots]{build_steps_plots()}} with the reactive inputs to generate
a named list of ggplot2 objects. These are then rendered in individual tabs.
The download handler creates a ZIP archive of all plots as PNG files.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{compute_engine}
\alias{compute_engine}
\title{Generic Compute Engine for WHO STEPS Tables}
\description{
Takes a table specification from \code{\link[=steps_table_registry]{steps_table_registry()}} and a survey
design object, and produces the survey-weighted estimates needed to fill
the standard WHO STEPS data book table.
}
\concept{compute}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reports.R
\name{render_data_book}
\alias{render_data_book}
\title{Render STEPS Data Book report}
\usage{
render_data_book(config, output_dir = tempdir())
}
\arguments{
\item{config}{A list from \code{\link[=steps_config]{steps_config()}} with survey metadata.
Expected to have \code{country_name}, \code{survey_year}, \code{age_min}, \code{age_max}.}

\item{output_dir}{Directory for output reports (default \code{tempdir()}).}
}
\value{
Path to generated Word document (invisibly).
Prints message with output location.
}
\description{
Generates a Word document with detailed age-stratified prevalence tables
for all available indicators, organized by STEPS step.
}
\details{
Sections correspond to STEPS steps:
\itemize{
\item Step 1: Behavioural Risk Factors (tobacco, alcohol, diet, physical activity)
\item Step 2: Physical Measurements (overweight/obesity, blood pressure)
\item Step 3: Biochemical (glucose, cholesterol)
}

Requires pre-computed tables and plots in data/processed/.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{compute_table}
\alias{compute_table}
\title{Compute a single table from a registry entry}
\usage{
compute_table(entry, design, data = NULL)
}
\arguments{
\item{entry}{A single list element from \code{\link[=steps_table_registry]{steps_table_registry()}}.}

\item{design}{A survey design object (from \code{\link[survey:svydesign]{survey::svydesign()}}).}

\item{data}{The cleaned data frame (used for variable availability checks).}
}
\value{
A list with:
\describe{
\item{id}{Table identifier.}
\item{title}{Table title.}
\item{type}{Table type.}
\item{available}{Logical: TRUE if the required variable(s) exist.}
\item{results}{A list of data frames:
For proportion: total, by_sex, by_age (each with estimate, lower, upper).
For mean: total, by_sex, by_age (each with estimate, lower, upper).
For category: total, by_sex, by_age (each with level, estimate, lower, upper).
For cascade: named list of proportion results.
}
}
}
\description{
This is the main workhorse: given one registry entry and a survey design,
it dispatches to the appropriate method based on \code{entry$type} and returns
a standardised result list.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{save_steps_plots}
\alias{save_steps_plots}
\title{Save STEPS plots to PNG files}
\usage{
save_steps_plots(plots, output_dir = tempdir())
}
\arguments{
\item{plots}{A named list of ggplot2 objects (from \code{\link[=build_steps_plots]{build_steps_plots()}}).}

\item{output_dir}{Output directory path (default \code{tempdir()}).}
}
\value{
NULL (invisibly). Prints messages about saved files.
}
\description{
Exports all plots in a list to PNG files in the specified directory.
}
\details{
Files are named:
\itemize{
\item \verb{01_overview_indicators.png} (12x8 in)
\item \verb{02_by_sex_dashboard.png} (12x8 in)
\item \verb{03_bp_by_age.png} (10x6 in)
\item \verb{04_obesity_by_age.png} (10x6 in)
}

All saved at 150 dpi with white background.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{import_steps_data}
\alias{import_steps_data}
\title{Import raw STEPS survey data}
\usage{
import_steps_data(path)
}
\arguments{
\item{path}{Character. Path to the data file.}
}
\value{
A data frame with cleaned column names.
}
\description{
Reads a raw STEPS data file (CSV, Excel, Stata, or SPSS) and
standardises column names to lowercase with underscores.
}
\examples{
\dontrun{
raw <- import_steps_data("data/raw/steps_data.csv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_app.R
\name{run_app}
\alias{run_app}
\title{Launch the stepssurvey Shiny Application}
\usage{
run_app(...)
}
\arguments{
\item{...}{Additional arguments passed to \code{\link[shiny:shinyApp]{shiny::shinyApp()}}.}
}
\value{
A Shiny app object (invisibly). Called for its side effect of
launching the application.
}
\description{
Starts the interactive STEPS survey analysis app in the user's browser.
The app provides a guided workflow: upload data, clean, set survey design,
compute indicators, visualise results, and generate Word reports.
}
\examples{
\dontrun{
  run_app()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_indicators.R
\name{mod_indicators_ui}
\alias{mod_indicators_ui}
\title{Indicators Module — UI}
\usage{
mod_indicators_ui(id)
}
\arguments{
\item{id}{Module namespace id.}
}
\value{
UI definition for indicators computation and display.
}
\description{
Indicators Module — UI
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_about.R
\name{mod_about_server}
\alias{mod_about_server}
\title{About Page Module Server}
\usage{
mod_about_server(id)
}
\arguments{
\item{id}{Module namespace ID.}
}
\description{
About Page Module Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_digit_preference}
\alias{plot_digit_preference}
\title{Plot digit preference histogram for a physical measurement}
\usage{
plot_digit_preference(dq, measure)
}
\arguments{
\item{dq}{A \code{steps_quality} object from \code{\link[=steps_data_quality]{steps_data_quality()}}.}

\item{measure}{Character: one of "SBP", "DBP", "Height", "Weight", "Waist".}
}
\value{
A ggplot object.
}
\description{
Creates a bar chart of terminal-digit frequencies with the expected
uniform line at 10 \%.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{recode_yn}
\alias{recode_yn}
\title{Recode a yes/no variable to logical}
\usage{
recode_yn(x)
}
\arguments{
\item{x}{A vector of yes/no values (numeric or character).}
}
\value{
A logical vector, with NA for unrecognized values.
}
\description{
Converts various representations of yes/no (numeric, text, case-insensitive)
to logical TRUE/FALSE values.
}
\examples{
recode_yn(c(1, 2, "yes", "no", NA))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tables.R
\name{make_steps_table}
\alias{make_steps_table}
\title{Build a single STEPS table}
\usage{
make_steps_table(total_df, by_sex_df, by_age_df, label, pct = TRUE)
}
\arguments{
\item{total_df}{Data frame with total estimate, lower, upper.}

\item{by_sex_df}{Data frame with sex-stratified estimates (not used, kept for compatibility).}

\item{by_age_df}{Data frame with age-stratified estimates and \code{age_group} column.}

\item{label}{Table caption.}

\item{pct}{Logical; format as percent? (default TRUE).}
}
\value{
A flextable object.
}
\description{
Constructs a flextable from age-stratified and total estimates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tables.R
\name{build_steps_tables}
\alias{build_steps_tables}
\title{Build survey-weighted tables for STEPS indicators}
\usage{
build_steps_tables(indicators)
}
\arguments{
\item{indicators}{A list of indicator results from \code{\link[=compute_all_indicators]{compute_all_indicators()}},
containing elements like \code{tobacco}, \code{alcohol}, \code{diet_pa}, \code{anthropometry},
\code{blood_pressure}, \code{biochemical}, etc. Each indicator list should contain
\verb{*_total}, \verb{*_by_sex}, and \verb{*_by_age} elements.}
}
\value{
A named list of flextable objects, one per indicator.
Names correspond to indicators (e.g., \code{current_tobacco}, \code{raised_bp}).
NULL entries are excluded. Prints count of tables generated.
}
\description{
Generates formatted flextable objects for all available STEPS indicators,
with rows for age groups and columns for both sexes combined, males, and
females. Tables include 95\% confidence intervals.
}
\details{
Each table has age groups as rows and prevalence (with 95\% CI) as a column.
The last row shows the total (age-standardised) estimate.
Column header styling uses WHO STEPS branding (dark blue background).
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  all_ind <- compute_all_indicators(design)
  tables <- build_steps_tables(all_ind$results)
  names(tables)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_about.R
\name{mod_about_ui}
\alias{mod_about_ui}
\title{About Page Module UI}
\usage{
mod_about_ui(id)
}
\arguments{
\item{id}{Module namespace ID.}
}
\value{
A Shiny UI element.
}
\description{
Displays information about the stepssurvey package, authors,
and institutional affiliations.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipeline.R
\name{run_steps_pipeline}
\alias{run_steps_pipeline}
\title{Run the complete STEPS analysis pipeline}
\usage{
run_steps_pipeline(
  data_path,
  country_name = "Country Name",
  survey_year = 2024,
  age_min = 18,
  age_max = 69,
  output_dir = tempdir(),
  render_reports = TRUE,
  mapping_file = NULL
)
}
\arguments{
\item{data_path}{Path to raw STEPS data file (CSV, Excel, Stata, or SPSS).}

\item{country_name}{Country name for reports (default "Country Name").}

\item{survey_year}{Survey year (default 2024).}

\item{age_min}{Minimum age in years (default 18).}

\item{age_max}{Maximum age in years (default 69).}

\item{output_dir}{Directory for all outputs (default \code{tempdir()}).}

\item{render_reports}{Logical; render Word documents? (default TRUE).}

\item{mapping_file}{Optional path to a filled column mapping template
(Excel or CSV). If provided, uses \code{\link[=read_column_mapping]{read_column_mapping()}} instead of
auto-detection. See the template at
\code{system.file("templates", "column_mapping_template.xlsx",
  package = "stepssurvey")}.}
}
\value{
A list with elements:
\describe{
\item{raw_data}{Original imported data frame}
\item{clean_data}{Cleaned and recoded data}
\item{cols}{Detected column mapping from \code{\link[=detect_steps_columns]{detect_steps_columns()}}}
\item{design}{\link[survey:svydesign]{survey::svydesign} object}
\item{indicators}{List of all computed indicator results by domain}
\item{key_indicators}{Summary tibble of headline estimates}
\item{tables}{List of \link[flextable:flextable]{flextable::flextable} objects}
\item{plots}{List of \link[ggplot2:ggplot]{ggplot2::ggplot} objects}
\item{config}{Configuration list from \code{\link[=steps_config]{steps_config()}}}
}
}
\description{
Imports raw data, cleans it, sets up the survey design, computes
all indicators, generates publication-ready tables and plots, and
optionally renders Word reports.
}
\details{
This is the main entry point for end-to-end STEPS analysis.
}
\examples{
\dontrun{
# Auto-detect columns
result <- run_steps_pipeline("data/raw/steps_data.csv",
                             country_name = "Senegal",
                             survey_year = 2023)
result$key_indicators
result$plots$overview

# Use a custom column mapping
result <- run_steps_pipeline("data/raw/steps_data.csv",
                             country_name = "Senegal",
                             survey_year = 2023,
                             mapping_file = "my_mapping.xlsx")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators.R
\name{compute_all_indicators}
\alias{compute_all_indicators}
\title{Compute All STEPS Indicators}
\usage{
compute_all_indicators(design)
}
\arguments{
\item{design}{A \code{steps_designs} list from \code{\link[=setup_survey_design]{setup_survey_design()}}
(with elements \verb{$step1}, \verb{$step2}, \verb{$step3}), or a single
\link[survey:svydesign]{survey::svydesign} object for backward compatibility.}
}
\value{
A list with two elements:
\itemize{
\item \code{results}: a named list containing indicator results grouped by domain
(tobacco, alcohol, diet_pa, anthropometry, blood_pressure, biochemical)
\item \code{key_indicators}: a tibble with columns domain, indicator, estimate,
lower, and upper, summarising headline estimates across all domains
}
}
\description{
Runs all indicator modules (tobacco, alcohol, diet & physical activity,
anthropometry, blood pressure, and biochemical), using the appropriate
step-specific survey design for each domain per WHO STEPS methodology:
\itemize{
\item Step 1 (behavioural): tobacco, alcohol, diet & physical activity
\item Step 2 (physical):    anthropometry, blood pressure
\item Step 3 (biochemical): biochemical measures
}
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  all_indicators <- compute_all_indicators(design)
  names(all_indicators$results)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_clean.R
\name{mod_clean_server}
\alias{mod_clean_server}
\title{Clean Module — Server}
\usage{
mod_clean_server(id, upload_out)
}
\arguments{
\item{id}{Module namespace id.}

\item{upload_out}{Reactive list returned by \code{\link[=mod_upload_server]{mod_upload_server()}}.}
}
\value{
A \link[shiny:reactive]{shiny::reactive} returning the cleaned data frame.
}
\description{
Clean Module — Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey_design.R
\name{.make_design}
\alias{.make_design}
\title{Create a single survey design for a given weight column}
\usage{
.make_design(data, wt_col, label = "")
}
\arguments{
\item{data}{A data frame (typically from \code{\link[=clean_steps_data]{clean_steps_data()}}).}

\item{wt_col}{Character name of the weight column to use.}

\item{label}{Label for messages (e.g. "Step 1").}
}
\value{
A \link[survey:svydesign]{survey::svydesign} object.
}
\description{
Internal helper that builds one \link[survey:svydesign]{survey::svydesign} object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{plot_completeness}
\alias{plot_completeness}
\title{Plot completeness heatmap across STEPS domains}
\usage{
plot_completeness(dq)
}
\arguments{
\item{dq}{A \code{steps_quality} object from \code{\link[=steps_data_quality]{steps_data_quality()}}.}
}
\value{
A ggplot object.
}
\description{
Creates a tile heatmap showing missingness percentage by variable,
grouped by STEPS domain.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/globals.R
\docType{package}
\name{stepssurvey-package}
\alias{stepssurvey}
\alias{stepssurvey-package}
\title{stepssurvey: Analyse WHO STEPS Survey Data}
\description{
A complete analysis pipeline for the WHO STEPwise Approach to NCD Risk
Factor Surveillance (STEPS).
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/drpakhare/stepssurvey}
  \item Report bugs at \url{https://github.com/drpakhare/stepssurvey/issues}
}

}
\author{
\strong{Maintainer}: Abhijit Pakhare \email{abhijit.cfm@aiimsbhopal.edu.in}

Authors:
\itemize{
  \item Ankur Joshi
  \item Lena Charlette
}

Other contributors:
\itemize{
  \item WHO STEPS R Pipeline Contributors [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{build_all_tables}
\alias{build_all_tables}
\title{Build all tables from computed results}
\usage{
build_all_tables(results)
}
\arguments{
\item{results}{A named list of results from \code{\link[=compute_all_tables]{compute_all_tables()}}.}
}
\value{
A named list of flextable objects. NULL entries excluded.
}
\description{
Build all tables from computed results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators_tobacco.R
\name{compute_tobacco_indicators}
\alias{compute_tobacco_indicators}
\title{Compute Tobacco Use Indicators}
\usage{
compute_tobacco_indicators(design)
}
\arguments{
\item{design}{A survey design object from \code{\link[=setup_survey_design]{setup_survey_design()}}.}
}
\value{
A named list of survey estimates. Each element contains proportion
estimates (as tibble with columns: estimate, lower, upper, etc.) for:
\itemize{
\item \code{current_tobacco_any_total/by_sex/by_age}: any current tobacco use
(smoking or smokeless) -- preferred headline variable
\item \code{current_tobacco_total/by_sex/by_age}: current tobacco smoking
\item \code{current_smoker_total/by_sex/by_age}: current smoker
\item \code{current_smokeless_total/by_sex/by_age}: current smokeless tobacco
\item \code{daily_tobacco_total/by_sex/by_age}: daily tobacco use
(only elements for variables present in design are returned)
}
}
\description{
Calculates prevalence of tobacco use from a survey design object.
Computes proportions of current and daily tobacco use, stratified by
sex and age group where available.
}
\details{
When both smoking and smokeless tobacco variables are present,
\code{current_tobacco_any} (either smoking or smokeless) is preferred
as the headline tobacco indicator.  The function also reports
\code{current_smoker} and \code{current_smokeless} separately if available.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  tobacco_results <- compute_tobacco_indicators(design)
}
}
\seealso{
\code{\link[=compute_all_indicators]{compute_all_indicators()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{theme_steps}
\alias{theme_steps}
\title{WHO STEPS ggplot2 theme}
\usage{
theme_steps(base_size = 11)
}
\arguments{
\item{base_size}{Base font size (default 11).}
}
\value{
A \link[ggplot2:theme]{ggplot2::theme} object.
}
\description{
A clean, minimal ggplot2 theme styled with WHO STEPS colours.
}
\examples{
library(ggplot2)
ggplot(mtcars, aes(wt, mpg)) + geom_point() + theme_steps()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indicators_biochemical.R
\name{compute_biochemical_indicators}
\alias{compute_biochemical_indicators}
\title{Compute Biochemical Indicators}
\usage{
compute_biochemical_indicators(design)
}
\arguments{
\item{design}{A survey design object from \code{\link[=setup_survey_design]{setup_survey_design()}}.}
}
\value{
A named list of survey estimates. Each element contains estimates
(as tibble with columns: estimate, lower, upper, etc.) for:
\itemize{
\item \code{raised_glucose_total}: raised fasting glucose, overall
\item \code{raised_glucose_by_sex}: raised fasting glucose, by sex
\item \code{raised_glucose_by_age}: raised fasting glucose, by age group
\item \code{diabetes_total}: diabetes, overall
\item \code{diabetes_by_sex}: diabetes, by sex
\item \code{diabetes_by_age}: diabetes, by age group
\item \code{impaired_glucose_total}: impaired fasting glucose, overall
\item \code{impaired_glucose_by_sex}: impaired fasting glucose, by sex
\item \code{impaired_glucose_by_age}: impaired fasting glucose, by age group
\item \code{raised_chol_total}: raised total cholesterol, overall
\item \code{raised_chol_by_sex}: raised total cholesterol, by sex
\item \code{raised_chol_by_age}: raised total cholesterol, by age group
\item \code{fasting_glucose_mean_total}: mean fasting glucose, overall
\item \code{fasting_glucose_mean_by_sex}: mean fasting glucose, by sex
\item \code{total_chol_mean_total}: mean total cholesterol, overall
\item \code{total_chol_mean_by_sex}: mean total cholesterol, by sex
(if the corresponding variables exist in design)
}
}
\description{
Calculates prevalence of raised glucose, diabetes, impaired glucose
tolerance, and raised cholesterol, plus mean fasting glucose and total
cholesterol from a survey design object.
}
\examples{
\donttest{
  test_data <- generate_test_data(n = 500, seed = 42)
  cols <- detect_steps_columns(test_data)
  clean <- clean_steps_data(test_data, cols)
  design <- setup_survey_design(clean)
  biochemical_results <- compute_biochemical_indicators(design)
}
}
\seealso{
\code{\link[=compute_all_indicators]{compute_all_indicators()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_data.R
\name{generate_test_data}
\alias{generate_test_data}
\title{Generate simulated STEPS test data}
\usage{
generate_test_data(n = 3000, seed = 42)
}
\arguments{
\item{n}{Number of observations (default 3000).}

\item{seed}{Random seed for reproducibility (default 42).}
}
\value{
A data frame with \code{n} rows and the following columns:
\itemize{
\item \code{stratum}: Strata identifier (S1-S5)
\item \code{psu}: Primary sampling unit (PSU1-PSU40)
\item \code{wt_final}: Final analysis weight
\item \code{sex}: Sex (1=Male, 2=Female)
\item \code{age}: Age in years (18-69)
\item Step 1 (behavioural): \code{t1}, \code{t2} (tobacco), \code{a1}, \code{a5} (alcohol),
\code{met_total} (physical activity), \code{d1}-\code{d4} (diet)
\item Step 2 (physical): \code{m1} (height), \code{m2} (weight), \code{m3} (waist),
\code{b1}-\code{b6} (blood pressure), \code{b7} (BP medication)
\item Step 3 (biochemical): \code{c1_mmol} (glucose), \code{c5} (DM meds),
\code{c6} (cholesterol), \code{c10} (cholesterol meds)
}
}
\description{
Creates a realistic simulated dataset matching WHO STEPS survey structure.
Includes sampling design variables, demographics, and measures from all
three steps (behavioural, physical, biochemical).
}
\details{
Simulation parameters are realistic for low-middle income settings:
\itemize{
\item Tobacco prevalence: 32\% males, 8\% females
\item Alcohol current use: 55\% males, 28\% females
\item Heavy episodic drinking: 35\% of drinkers
\item Physical activity: MET-minutes/week, mean 1800, SD 1200
\item Diet: Fruit/veg days and servings per day (0-7, 1-5)
\item BP increases with age; medication prevalence 12\%
\item Glucose: mean 5.2 mmol/L, increases with age
\item Total cholesterol: mean 4.8 mmol/L
}

Use this function for:
\itemize{
\item Testing the STEPS pipeline
\item Developing reports before real data arrives
\item Training analysts on the analysis system
}
}
\examples{
\donttest{
  # Generate smaller dataset for quick testing
  test_data <- generate_test_data(n = 500, seed = 123)
  head(test_data)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_indicators.R
\name{mod_indicators_server}
\alias{mod_indicators_server}
\title{Indicators Module — Server}
\usage{
mod_indicators_server(id, design_out)
}
\arguments{
\item{id}{Module namespace id.}

\item{design_out}{Reactive returning a survey::svydesign object.}
}
\value{
A list with two reactive elements:
\itemize{
\item \code{results}: reactive list of indicator results by domain
\item \code{key_indicators}: reactive tibble of headline key indicators
}
}
\description{
Indicators Module — Server
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme.R
\name{steps_colors}
\alias{steps_colors}
\title{WHO STEPS colour palette}
\usage{
steps_colors()
}
\value{
A named list of hex colour codes.
}
\description{
A named list of colours used in WHO STEPS reports and visualisations.
}
\examples{
steps_colors()$blue
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app_ui.R
\name{app_ui}
\alias{app_ui}
\title{Main Shiny UI}
\usage{
app_ui()
}
\value{
A Shiny UI definition.
}
\description{
Builds the top-level UI using bslib sidebar page.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/steps_registry.R
\name{steps_table_registry}
\alias{steps_table_registry}
\title{WHO STEPS Data Book Table Registry}
\usage{
steps_table_registry()
}
\value{
A list of table specification lists.
}
\description{
Defines all standard tables from the WHO STEPS Epi Info report template.
Each entry specifies the table metadata; generic compute and formatting
functions use this registry to produce the full data book automatically.
}
\section{Table types}{

\describe{
\item{proportion}{Single binary indicator: \% (95\% CI) by age × sex.
Most common type. Example: "Current smokers among all respondents."}
\item{mean}{Continuous variable: mean (95\% CI) by age × sex.
Example: "Mean BMI (kg/m²)."}
\item{category}{Multi-level factor: \% per level (95\% CI) by age × sex.
Example: "BMI classifications (Underweight / Normal / Overweight / Obese)."}
\item{cascade}{Diagnosis → treatment → control chain: multiple proportions
with nested denominators. Example: "Raised BP diagnosis, treatment and control."}
\item{combined}{Summary of combined risk factors: 0, 1-2, 3-5 risk factors.}
}
}

\section{Registry fields}{

\describe{
\item{id}{Unique short identifier (e.g., "T_smoking_current").}
\item{section}{Data book section (e.g., "Tobacco Use", "Blood Pressure").}
\item{step}{STEPS step number (1, 2, or 3).}
\item{title}{Table title as shown in the data book.}
\item{description}{One-line description from the WHO template.}
\item{type}{One of: "proportion", "mean", "category", "cascade", "combined".}
\item{variable}{Column name(s) in the cleaned data frame to analyse.
For proportion: single logical variable.
For mean: single numeric variable.
For category: single factor variable.
For cascade: named list of logical variables.}
\item{denominator}{NULL (= all respondents) or column name for subsetting
(e.g., "current_alcohol" to restrict to drinkers).}
\item{levels}{For category type: named character vector of level labels.}
\item{epi_info}{Epi Info program name(s) for reference.}
\item{unit}{Display unit (e.g., "\%", "mmHg", "cm", "kg/m²", "mmol/L").}
\item{questions}{STEPS instrument question codes used.}
\item{sex_panels}{Logical. TRUE = 3 panels (Men/Women/Both);
FALSE = 2 panels (Men/Women only, e.g., height/weight means).
Default TRUE.}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/table_builder.R
\name{.style_steps_table}
\alias{.style_steps_table}
\title{Apply WHO STEPS styling to a flextable}
\usage{
.style_steps_table(ft, title, description = NULL)
}
\arguments{
\item{ft}{A flextable object.}

\item{title}{Table title.}

\item{description}{Optional description text.}
}
\value{
Styled flextable.
}
\description{
Apply WHO STEPS styling to a flextable
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reports.R
\name{steps_config}
\alias{steps_config}
\title{Create STEPS analysis configuration}
\usage{
steps_config(
  data_path,
  country_name = "Country Name",
  survey_year = 2024,
  age_min = 18,
  age_max = 69,
  weight_var = "wt_final",
  strata_var = "stratum",
  cluster_var = "psu",
  bp_sbp_threshold = 140,
  bp_dbp_threshold = 90,
  bmi_overweight = 25,
  bmi_obese = 30,
  glucose_threshold = 7,
  glucose_impaired_threshold = 6.1,
  chol_threshold = 5
)
}
\arguments{
\item{data_path}{Path to raw STEPS data file (CSV or Excel).}

\item{country_name}{Country name for reports (default "Country Name").}

\item{survey_year}{Survey year (default 2024).}

\item{age_min}{Minimum age (default 18).}

\item{age_max}{Maximum age (default 69).}

\item{weight_var}{Weight variable name (default "wt_final", set NULL if none).}

\item{strata_var}{Strata variable name (default "stratum", set NULL if none).}

\item{cluster_var}{Cluster variable name (default "psu", set NULL if none).}

\item{bp_sbp_threshold}{SBP threshold for raised BP (default 140).}

\item{bp_dbp_threshold}{DBP threshold for raised BP (default 90).}

\item{bmi_overweight}{BMI threshold for overweight (default 25.0).}

\item{bmi_obese}{BMI threshold for obesity (default 30.0).}

\item{glucose_threshold}{Fasting glucose threshold in mmol/L (default 7.0).}

\item{glucose_impaired_threshold}{Impaired fasting glucose threshold in mmol/L (default 6.1).}

\item{chol_threshold}{Total cholesterol threshold in mmol/L (default 5.0).}
}
\value{
A list with elements:
\itemize{
\item \code{data_path}: Input file path
\item \code{country_name}: Country name
\item \code{survey_year}: Survey year
\item \code{age_min}, \code{age_max}: Age range
\item \code{weight_var}, \code{strata_var}, \code{cluster_var}: Design variable names
\item Threshold parameters for BP, BMI, glucose, cholesterol
}
}
\description{
Builds a configuration list that specifies data paths, design variables,
and report parameters for the STEPS pipeline.
}
\examples{
\dontrun{
  cfg <- steps_config("data/steps_2023.csv", "Senegal", 2023)
  cfg <- steps_config("data/steps.csv", "Mongolia", 2019,
                      bp_sbp_threshold = 130, bp_dbp_threshold = 80)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{detect_steps_columns}
\alias{detect_steps_columns}
\title{Auto-detect all standard STEPS columns}
\usage{
detect_steps_columns(data)
}
\arguments{
\item{data}{A data frame (typically from \code{\link[=import_steps_data]{import_steps_data()}}).}
}
\value{
A named list of detected column names (or \code{NULL} for missing).
}
\description{
Scans a data frame for standard WHO STEPS variable names across
versions 3.1 and 3.2.  Aliases are listed in priority order: the
first match wins, so put the most specific / unambiguous name first.
}
\details{
WHO STEPS reorganised variable codes between v3.1 and v3.2:

\strong{v3.1 / Epi Info codes} (still common in many country datasets):
B1-B6 = blood-pressure readings, B7 = BP meds,
C1 = fasting glucose, C5 = DM meds, C6 = total cholesterol, C10 = chol meds,
M1 = height, M2 = weight, M3 = waist.

\strong{v3.2 instrument codes}:
M4a/M5a/M6a = SBP readings, M4b/M5b/M6b = DBP readings, M7 = BP meds,
M11 = height, M12 = weight, M14 = waist, M15 = hip,
B5 = fasting glucose, B6 = DM meds, B8 = total cholesterol, B9 = chol meds,
B16 = triglycerides, B17 = HDL cholesterol,
C1 = sex, C3 = age.

The function includes aliases for both versions so datasets from
either instrument version are detected automatically.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.compute_proportion}
\alias{.compute_proportion}
\title{Compute a proportion table (type = "proportion")}
\usage{
.compute_proportion(entry, design, data)
}
\description{
Produces total, by_sex, by_age data frames with estimate, lower, upper.
Values are in percentage (0-100).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survey_design.R
\name{svyprop}
\alias{svyprop}
\title{Weighted proportion estimation with 95\% CI}
\usage{
svyprop(formula, design, by = NULL, na.rm = TRUE)
}
\arguments{
\item{formula}{A formula (e.g., \code{~variable} or using binary variables).}

\item{design}{A survey design object (from \code{\link[=setup_survey_design]{setup_survey_design()}}).}

\item{by}{Optional formula for stratification (e.g., \code{~sex}).}

\item{na.rm}{Logical; if TRUE (default), omit NA values.}
}
\value{
A data frame with columns:
\itemize{
\item \code{estimate}: estimated proportion (\%)
\item \code{lower}: 95\% CI lower bound (\%)
\item \code{upper}: 95\% CI upper bound (\%)
\item \code{se}: standard error (\%)
\item If \code{by} is specified: grouping column(s) prepended
}
}
\description{
Calculates weighted proportions (as percentages) with 95\% confidence
intervals for a yes/no variable, optionally stratified by a grouping
variable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualizations.R
\name{build_radar_plot}
\alias{build_radar_plot}
\title{Build radar / spider chart of NCD risk factor profile}
\usage{
build_radar_plot(key_indicators, country_name, survey_year)
}
\arguments{
\item{key_indicators}{A data frame with domain, indicator, estimate.}

\item{country_name}{Country name for title.}

\item{survey_year}{Survey year for title.}
}
\value{
A ggplot2 object.
}
\description{
Creates a radar-style chart showing prevalence of key risk factors
on a polar coordinate system for quick visual comparison.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_engine.R
\name{.compute_combined}
\alias{.compute_combined}
\title{Compute combined risk factors (type = "combined")}
\usage{
.compute_combined(entry, design, data)
}
\description{
Counts how many of 5 key risk factors each respondent has, then computes
the proportion with 0, 1-2, and 3-5 risk factors.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mod_reports.R
\name{mod_reports_server}
\alias{mod_reports_server}
\title{Reports Module — Server}
\usage{
mod_reports_server(id, upload_out)
}
\arguments{
\item{id}{Module namespace id.}

\item{upload_out}{Reactive list returned by \code{\link[=mod_upload_server]{mod_upload_server()}}, containing
\code{raw} (data frame), \code{cols} (column mapping), and \code{config} (survey configuration).}
}
\value{
A list with reactive elements:
\itemize{
\item \code{report_path}: Path to generated Word document
\item \code{output_dir}: Directory containing all outputs
\item \code{generation_status}: Current status string
}
}
\description{
Reports Module — Server
}
\details{
The full pipeline is executed internally:
\enumerate{
\item Cleans data using \code{\link[=clean_steps_data]{clean_steps_data()}}
\item Sets up survey design with \code{\link[=setup_survey_design]{setup_survey_design()}}
\item Computes all indicators with \code{\link[=compute_all_indicators]{compute_all_indicators()}}
\item Builds tables with \code{\link[=build_steps_tables]{build_steps_tables()}}
\item Builds plots with \code{\link[=build_steps_plots]{build_steps_plots()}}
\item Saves all outputs to temp directory
\item Renders report using \code{\link[=render_country_report]{render_country_report()}}
}

Progress is shown via status text and progress bar during generation.
Errors are caught and displayed to the user via notifications.
}
\keyword{internal}
